--[[
Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-1-27

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类 -- 三角形骨骼
----------------------
local M    = class("Shape1Triangle", require("app.common.shape.ShapeBase"))
M.TAG    = "Shape1Triangle"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 主层
    self._layer        = params.layer
    -- 形状ID
    self._id            = 1
    -- 字段[骨骼资源文件名，不带文件后缀名]
    self._file            = "bone/sanjiao"
    -- 字段[骨架名称]
    self._armatureName    = "sanjiao"
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()
end

-- 图片渲染
function M:loadDisplay()

end

-- 初始化动画名
function M:initAnimationNames()
    self._breathAni1        = "sanjiao_huxi_1" 		-- 呼吸
    self._breathAni2        = "sanjiao_huxi_2" 		-- 呼吸眨眼
    self._rhythmAni1        = "sanjiao_lvdong" 		-- 律动1
    self._rhythmAni2        = "sanjiao_lvdong_2" 	-- 律动2
    self._spitOutAni1        = "sanjiao_tu_1" 		-- 吐出物体
    self._spitOutAni2        = "sanjiao_tu_2" 		-- 吐出物体2
    self._transformationAni = "sanjiao_zuanchu" 	-- 变成小精灵
    self._openMouthAni1    = "sanjiao_zhangzui" 	-- 张嘴
    self._openMouthAni2    = "sanjiao_zhangzui_2" 	-- 张嘴持续
    self._eatAni1            = nil 					-- 吃东西开始
    self._eatAni2            = "sanjiao_chi" 		-- 吃东西循环
    self._eatAni3            = "sanjiao_chi_2" 		-- 吃东西咽下
    self._unhappyAni        = "sanjiao_bugaoxing" 	-- 不高兴

    -- 新增动作
    self._standbyAni        = "sanjiao_daiji"       -- 待机动作
    self._unhappyAni2      = "sanjiao_bugaoxing_1"  -- 不高兴2和不高兴1随机切换使用
    self._eatAni2_2         = "sanjiao_chi_3"       -- 吃东西循环2和吃东西循环1随机切换使用
end

-- 物品纹理名称顺序洗牌
function M:ranTextureName()
    local textureNameTab = {
        "sanjiao_wupin_1", -- 三明治
        "sanjiao_wupin_2", -- 饭团
        "sanjiao_wupin_3", -- 帽子
        "sanjiao_wupin_4", -- 山
        "sanjiao_wupin_5", -- 粽子
    }
    local itemIdTab = {1, 2, 3, 4, 5}

    T.shuffle(itemIdTab)
    self._itemIdTab = itemIdTab
    self._textureNameTab = textureNameTab
end

-- 播放物品台词
function M:playItemVoice()
    soundVoice:stopNowSound()
    if self._itemId == 1 then
        -- 播放配音[v147091]三明治
        soundVoice:playEffectv147091()
    elseif self._itemId == 2 then
        -- 播放配音[v147092]饭团
        soundVoice:playEffectv147092()
    elseif self._itemId == 3 then
        -- 播放配音[v147095]圣诞帽
        soundVoice:playEffectv147095()
    elseif self._itemId == 4 then
        -- 播放配音[v147094]山
        soundVoice:playEffectv147094()
    elseif self._itemId == 5 then
        -- 播放配音[v147093]粽子
        soundVoice:playEffectv147093()
    end
end

-- 吐出物体动作随机播放
function M:ranSpitOutAct()
    local ranNum = math.random(1, 2)
    -- 吐物骨骼延迟显示时间, 落地粒子出现时间
    local itemTime, parTime
    if ranNum == 1 then
        itemTime = 0.3
		parTime = 0.7
        self:spitOutAct1()
    else
        itemTime = 0.3
		parTime = 0.7
        self:spitOutAct2()
    end

    -- 避免星形物品图片穿帮，延迟显示
    self:line({                    
        {"delay", itemTime}, 
        {"fn", function ()
            self:showItemBone()
        end}
    })

    -- 提前显示粒子
    self:line({
        {"delay", parTime}, 
        {"fn", function ()
            local item = self._parent:createItem():hide()
            -- 当前展示的实物
            self._curItem = item
            -- 粒子
            local pos = item:worldpoint()
            pos.y = pos.y - item:ch()/2.5
            local particle = CUtil:newParticle("particle/twdl.plist"):to(self._layer, 100):p(pos)
        end}
    })
    -- self:spitOutChangeScale()

    if self._effect then sound.stopSound(self._effect) end
    -- 播放音效[sfx14706014]点击三角形吐物/两个骨骼都用这个音效
    self._effect = soundEffect:playEffectsfx14706014()
end

-- 动作回调
function M:handlerOnDerived(__evtType, __moveId)
    if __evtType == 0 then
        if __moveId == self._standbyAni then
            self:playFeedAniSound("sfx14705004")
        elseif __moveId == self._unhappyAni then
            self:playFeedAniSound("sfx14705005")
        elseif __moveId == self._unhappyAni2 then
            self:playFeedAniSound("sfx14705006")
        elseif __moveId == self._openMouthAni1 then
            self:playFeedAniSound("sfx14705007")
        elseif __moveId == self._eatAni2 or __moveId == self._eatAni2_2 then
            self:playFeedAniSound("sfx14705008")
        end
    end
end

-- 喂食退场
function M:feedQuit(dir, callback)
    -- -- 向前青蛙跳出场
    -- soundEffect:playEffectsfx14700004()
    dir = ifnil(dir, (self:x() < V.w_2 - 10) and -1 or (self:x() > V.w_2 + 10) and 1 or N.randomExists(1, -1))
    self:breathAct1()

    self:line({
        A.union({
            {"jumpBy", 0.4, cc.p(100, -85 * Y_FACTOR), 100, 1},
            A.line({
                {"scaleTo", 0.15, 0.9 * self._scale * 1, 1.15 * self._scale * 1},
                {"scaleTo", 0.2, 1.15 * self._scale * 1, 0.9 * self._scale * 1},
                {"scaleTo", 0.1, 1 * self._scale * 1},
            })
        }),
        {"delay", 0.05},
        A.union({
            {"jumpBy", 0.4, cc.p(-160, -85 * Y_FACTOR), 120, 1},
            A.line({
                {"scaleTo", 0.15, 0.9 * self._scale * 1.13, 1.15 * self._scale * 1.13},
                {"scaleTo", 0.2, 1.15 * self._scale * 1.13, 0.9 * self._scale * 1.13},
                {"scaleTo", 0.1, 1 * self._scale * 1.13},
            })
        }),
        {"delay", 0.05},
        A.union({
            {"jumpBy", 0.4, cc.p(160, -210 * Y_FACTOR), 100, 1},
            A.line({
                {"scaleTo", 0.15, 0.9 * self._scale * 1.3, 1.15 * self._scale * 1.3},
                {"scaleTo", 0.2, 1.15 * self._scale * 1.3, 0.9 * self._scale * 1.3},
                {"scaleTo", 0.1, 1 * self._scale * 1.3},
            })
        }),
        {"remove"},
    })

    -- 返回音效编号
    return "sfx14700004"
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end

return M