--[[
Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-1-27

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类 -- 正方形骨骼
----------------------
local M    = class("Shape2Square", require("app.common.shape.ShapeBase"))
M.TAG    = "Shape2Square"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 主层
    self._layer         = params.layer
    -- 形状ID
    self._id            = 2
    -- 字段[骨骼资源文件名，不带文件后缀名]
    self._file          = "bone/fang"
    -- 字段[骨架名称]
    self._armatureName  = "fang"
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()
    -- 物品纹理名称顺序洗牌
    self:ranTextureName()
end

-- 图片渲染
function M:loadDisplay()

end

-- 初始化动画名
function M:initAnimationNames()
    self._breathAni1        = "fang_huxi" 		-- 呼吸
    self._breathAni2        = "fang_huxi_2" 	-- 呼吸眨眼
    self._rhythmAni1        = "fang_lvdong" 	-- 律动1
    self._rhythmAni2        = "fang_lvdong_2" 	-- 律动2
    self._spitOutAni1       = "fang_tu" 		-- 吐出物体
    self._spitOutAni2       = "fang_tu_2" 		-- 吐出物体2
    self._transformationAni = "fang_zuanchu" 	-- 变成小精灵
    self._openMouthAni1     = "fang_zhangzui_1" -- 张嘴
    self._openMouthAni2     = "fang_zhangzui_2" -- 张嘴持续
    self._eatAni1           = "fang_chi" 		-- 吃东西开始
    self._eatAni2           = "fang_chi_1" 	    -- 吃东西循环
    self._eatAni3           = "fang_chi_2" 	    -- 吃东西咽下
    self._unhappyAni        = "fang_bugaoxing" 	-- 不高兴

    -- 新增动作
    self._standbyAni        = "fang_daiji"        -- 待机动作
    self._unhappyAni2       = "fang_bugaoxing_1"  -- 不高兴
    self._openMouthAni2_1   = "fang_zhangzui_3"   -- 张嘴动作2  -- 两套张嘴轮流
    self._openMouthAni2_2   = "fang_zhangzui_4"   -- 张嘴持续2
end

-- 物品纹理名称顺序洗牌
function M:ranTextureName()
    local textureNameTab = {
        "fang_wupin_1", -- 抱枕
        "fang_wupin_2", -- 吐司面包
        "fang_wupin_3", -- 骰子
        "fang_wupin_4", -- 魔方
        "fang_wupin_5", -- 窗户
       
    }
    local itemIdTab = {1, 2, 3, 4, 5}
    T.shuffle(itemIdTab)
    self._itemIdTab = itemIdTab
    self._textureNameTab = textureNameTab
end

-- 播放物品台词
function M:playItemVoice()
    soundVoice:stopNowSound()
    if self._itemId == 1 then
        -- 播放配音[v147098]抱枕
        soundVoice:playEffectv147098()
    elseif self._itemId == 2 then
        -- 播放配音[v147096]土司片
        soundVoice:playEffectv147096()
    elseif self._itemId == 3 then
        -- 播放配音[v147100]方巾
        soundVoice:playEffectv147100()
    elseif self._itemId == 4 then
        -- 播放配音[v147099]魔方
        soundVoice:playEffectv147099()
    elseif self._itemId == 5 then
        -- 播放配音[v147097]窗户
        soundVoice:playEffectv147097()
    end
end

-- 吐出物体动作随机播放
function M:ranSpitOutAct()
    local ranNum = math.random(1, 2)
    -- 吐物骨骼延迟显示时间, 落地粒子出现时间
    local itemTime, parTime
    if ranNum == 1 then
        itemTime = 0.3
		parTime = 0.7
        self:spitOutAct1()
    else
        itemTime = 0.3
		parTime = 0.7
        self:spitOutAct2()
    end

    -- 避免星形物品图片穿帮，延迟显示
    self:line({                    
        {"delay", itemTime}, 
        {"fn", function ()
            self:showItemBone()
        end}
    })

    -- 提前显示粒子
    self:line({
        {"delay", parTime}, 
        {"fn", function ()
            local item = self._parent:createItem():hide()
            -- 当前展示的实物
            self._curItem = item
            -- 粒子
            local pos = item:worldpoint()
            pos.y = pos.y - item:ch()/2.4
            local particle = CUtil:newParticle("particle/twdl.plist"):to(self._layer, 100):p(pos)
        end}
    })
    -- self:spitOutChangeScale()

    if self._effect then sound.stopSound(self._effect) end
    -- 播放音效[sfx14706018]	点击方形吐物/两个骨骼都用这个音效
    self._effect = soundEffect:playEffectsfx14706018()
end

-- 动作回调
function M:handlerOnDerived(__evtType, __moveId)
    if __evtType == 0 then
        if __moveId == self._standbyAni then
            self:playFeedAniSound("sfx14705019")
        elseif __moveId == self._unhappyAni then
            self:playFeedAniSound("sfx14705020")
        elseif __moveId == self._unhappyAni2 then
            self:playFeedAniSound("sfx14705021")
        elseif __moveId == self._openMouthAni1 then
            self:playFeedAniSound("sfx14705022")
        elseif __moveId == self._eatAni1 then
            self:playFeedAniSound("sfx14705023")
        end
    end
end



-- 喂食退场
function M:feedQuit(dir, callback)
    -- 向前青蛙跳出场
    -- soundEffect:playEffectsfx14700004()
    dir = ifnil(dir, (self:x() < V.w_2 - 10) and -1 or (self:x() > V.w_2 + 10) and 1 or N.randomExists(1, -1))
    self:breathAct1()

    self:line({
        A.union({
            {"jumpBy", 0.4, cc.p(100, -85 * Y_FACTOR), 100, 1},
            A.line({
                {"scaleTo", 0.15, 0.9 * self._scale * 1, 1.15 * self._scale * 1},
                {"scaleTo", 0.2, 1.15 * self._scale * 1, 0.9 * self._scale * 1},
                {"scaleTo", 0.1, 1 * self._scale * 1},
            })
        }),
        {"delay", 0.05},
        A.union({
            {"jumpBy", 0.4, cc.p(-160, -85 * Y_FACTOR), 120, 1},
            A.line({
                {"scaleTo", 0.15, 0.9 * self._scale * 1.13, 1.15 * self._scale * 1.13},
                {"scaleTo", 0.2, 1.15 * self._scale * 1.13, 0.9 * self._scale * 1.13},
                {"scaleTo", 0.1, 1 * self._scale * 1.13},
            })
        }),
        {"delay", 0.05},
        A.union({
            {"jumpBy", 0.4, cc.p(160, -210 * Y_FACTOR), 100, 1},
            A.line({
                {"scaleTo", 0.15, 0.9 * self._scale * 1.3, 1.15 * self._scale * 1.3},
                {"scaleTo", 0.2, 1.15 * self._scale * 1.3, 0.9 * self._scale * 1.3},
                {"scaleTo", 0.1, 1 * self._scale * 1.3},
            })
        }),
        {"remove"},
    })

    return "sfx14700004"
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end


return M