--[[Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-1-27

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类 -- 圆形骨骼
----------------------
local M    = class("Shape3Circle", require("app.common.shape.ShapeBase"))
M.TAG    = "Shape3Circle"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 主层
    self._layer        = params.layer
    -- 形状ID
    self._id            = 3
    -- 字段[骨骼资源文件名，不带文件后缀名]
    self._file            = "bone/yuan"
    -- 字段[骨架名称]
    self._armatureName    = "yuan"
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()
    -- 物品纹理名称顺序洗牌
    self:ranTextureName()
end


-- 图片渲染
function M:loadDisplay()

end

-- 初始化动画名
function M:initAnimationNames()
    self._breathAni1        = "yuan_huxi_1" 	-- 呼吸
    self._breathAni2        = "yuan_huxi_2" 	-- 呼吸眨眼
    self._rhythmAni1        = "yuan_lvdong" 	-- 律动1
    self._rhythmAni2        = "yuan_lvdong_2" 	-- 律动2
    self._spitOutAni1       = "yuan_tu_1" 		-- 吐出物体
    self._spitOutAni2       = "yuan_tu_2" 		-- 吐出物体2
    self._transformationAni = "yuan_zuanchu" 	-- 变成小精灵
    self._openMouthAni1     = "yuan_zhangzui_1" -- 张嘴
    self._openMouthAni2     = "yuan_zhangzui_2" -- 张嘴持续
    self._eatAni1           = nil 				-- 吃东西开始
    self._eatAni2           = "yuan_chi_1" 		-- 吃东西循环
    self._eatAni3           = "yuan_chi_2" 		-- 吃东西咽下
    self._unhappyAni        = "yuan_bugaoxing" 	-- 不高兴
    self._spitBubbleAni     = "yuan_tu_3"       -- 吐出泡泡专用

    -- 新增动作
    self._standbyAni        = "yuan_daiji"      -- 待机动作
    self._unhappyAni2       = "yuan_bugaoxing_1"  -- 不高兴2和不高兴1随机切换使用
    self._eatAni2_2         = "yuan_chi_3"       -- 吃东西循环2和吃东西循环1随机切换使用
end

-- 物品纹理名称顺序洗牌[aniName为特殊物品，不换纹理，单独播放动画]
function M:ranTextureName()
    local textureNameTab = {
        "yuan_wupin_1",                  -- 西瓜
        "yuan_wupin_2",                  -- 钟表
        "yuan_wupin_3",                  -- 冰淇淋球
        "yuan_wupin_4",                  -- 球
        {aniName = self._spitBubbleAni}, -- 吐泡泡
    }

    local itemIdTab = {1, 2, 3, 4, 5}
    T.shuffle(itemIdTab)
    self._itemIdTab = itemIdTab
    self._textureNameTab = textureNameTab
end

-- 播放物品台词
function M:playItemVoice()
    soundVoice:stopNowSound()
    if self._itemId == 1 then
        -- 播放配音[v147103]西瓜
        soundVoice:playEffectv147103()
    elseif self._itemId == 2 then
        -- 播放配音[v147105]时钟
        soundVoice:playEffectv147105()
    elseif self._itemId == 3 then
        -- 播放配音[v147104]冰淇淋球
        soundVoice:playEffectv147104()
    elseif self._itemId == 4 then
        -- 播放配音[v147101]球
        soundVoice:playEffectv147101()
    elseif self._itemId == 5 then
        -- 播放配音[v147102]泡泡
        soundVoice:playEffectv147102()
    end
end

-- 吐出物体动作随机播放
function M:ranSpitOutAct()
    local ranNum = math.random(1, 2)
    -- 吐物骨骼延迟显示时间, 落地粒子出现时间
    local itemTime, parTime
    if ranNum == 1 then
        itemTime = 0.5
		parTime = 0.8
        self:spitOutAct1()
    else
        itemTime = 0.5
		parTime = 0.8
        self:spitOutAct2()
    end

    -- 避免星形物品图片穿帮，延迟显示
    self:line({                    
        {"delay", itemTime}, 
        {"fn", function ()
            self:showItemBone()
        end}
    })

    -- 提前显示粒子
    self:line({
        {"delay", parTime}, 
        {"fn", function ()
            local item = self._parent:createItem():hide()
            -- 当前展示的实物
            self._curItem = item
            -- 粒子
            local pos = item:worldpoint()
            if self._itemId == 1 then  -- 西瓜
                pos.y = pos.y - item:ch()/3.5
            elseif self._itemId == 5 then -- 泡泡
                pos.y = pos.y - item:ch()/2
            else
                pos.y = pos.y - item:ch()/2.5
            end
            local particle = CUtil:newParticle("particle/twdl.plist"):to(self._layer, 100):p(pos)
        end}
    })
    -- self:spitOutChangeScale()

    if self._effect then sound.stopSound(self._effect) end
    -- 播放音效[sfx14706023]点击圆形吐物/两个骨骼都用这个音效
    self._effect = soundEffect:playEffectsfx14706023()
end

-- 吐泡泡动作
function M:spitBubbleAct()
    self:playAniWithCheck(self._spitBubbleAni)
    -- self:spitOutChangeScale()

    -- 显示粒子的时间
    local parTime = 1.7
    self:line({
        {"delay", parTime}, 
        {"fn", function ()
            local item = self._parent:createItem():hide()
            -- 当前展示的实物
            self._curItem = item
            -- 粒子
            local pos = item:worldpoint()
            pos.y = pos.y - item:ch()/2.5
            local particle = CUtil:newParticle("particle/twdl.plist"):to(self._layer, 100):p(pos)
        end}
    })

    -- 播放音效[sfx14706023]点击圆形吐物/两个骨骼都用这个音效
    soundEffect:playEffectsfx14706023()
end

-- 动作回调
function M:handlerOnDerived(__evtType, __moveId)
    if __evtType == 1 then
        if __moveId == self._spitBubbleAni then
            self:onSpitOutHandler()
        end

    elseif __evtType == 0 then
        if __moveId == self._standbyAni then
            self:playFeedAniSound("sfx14705014")
        elseif __moveId == self._unhappyAni then
            self:playFeedAniSound("sfx14705015")
        elseif __moveId == self._unhappyAni2 then
            self:playFeedAniSound("sfx14705016")
        elseif __moveId == self._openMouthAni1 then
            self:playFeedAniSound("sfx14705017")
        elseif __moveId == self._eatAni2 or __moveId == self._eatAni2_2 then
            self:playFeedAniSound("sfx14705018")
        end
    end
end



-- 喂食退场
function M:feedQuit(dir, callback)
    -- 滚动出场
    -- soundEffect:playEffectsfx14700005()
    
    dir = ifnil(dir, (self:x() < V.w_2 - 10) and -1 or (self:x() > V.w_2 + 10) and 1 or N.randomExists(1, -1))
    self:breathAct1()

    self:one({"rotateBy", 1.6, dir * 720})
    self:line({
        {"moveBy", 1.6, cc.p(800 * dir, 0)},
        {"remove"},
    })

    return "sfx14700005"
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end


return M