--[[Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-1-27

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类 -- 五角星形骨骼
----------------------
local M    = class("Shape6Star", require("app.common.shape.ShapeBase"))
M.TAG    = "Shape6Star"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 主层
    self._layer        = params.layer
    -- 形状ID
    self._id            = 6
    -- 字段[骨骼资源文件名，不带文件后缀名]
    self._file            = "bone/xing"
    -- 字段[骨架名称]
    self._armatureName    = "xing"
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()
    -- 物品纹理名称顺序洗牌
    self:ranTextureName()
end


-- 图片渲染
function M:loadDisplay()

end

-- 初始化动画名
function M:initAnimationNames()
    self._breathAni1        = "huxi" 			-- 呼吸
    self._breathAni2        = "xing_huxi_2" 	-- 呼吸眨眼
    self._rhythmAni1        = "xing_lvdong" 	-- 律动1
    self._rhythmAni2        = "xing_lvdong_2" 	-- 律动2
    self._spitOutAni1        = "xing_tu_1" 		-- 吐出物体
    self._spitOutAni2        = "xing_tu_2" 		-- 吐出物体2
    self._transformationAni = "xing_zuanchu" 	-- 变成小精灵
    self._openMouthAni1    = "xing_zhangzui" 	-- 张嘴
    self._openMouthAni2    = "xing_zhangzui_2" -- 张嘴持续
    self._eatAni1            = nil 				-- 吃东西开始
    self._eatAni2            = "xing_chi_1" 		-- 吃东西循环
    self._eatAni3            = "xing_chi_2" 		-- 吃东西咽下
    self._unhappyAni        = "xing_bugaoxing"  -- 不高兴

    -- 新增动作
    self._standbyAni        = "xing_daiji"        -- 待机动作
    self._unhappyAni2       = "xing_bugaoxing_1"  -- 不高兴2和不高兴1随机切换使用
    self._eatAni2_2         = "xing_chi_3"        -- 吃东西循环2和吃东西循环1随机切换使用
end

-- 物品纹理名称顺序洗牌
function M:ranTextureName()
    local textureNameTab = {
        "xing_wupin_1", -- 饼干
        "xing_wupin_2", -- 杨桃
        "xing_wupin_3", -- 海星
        "xing_wupin_4", -- 宝石
        "xing_wupin_5", -- 星星
    }
    local itemIdTab = {1, 2, 3, 4, 5}

    T.shuffle(itemIdTab)
    self._itemIdTab = itemIdTab
    self._textureNameTab = textureNameTab
end

-- 播放物品台词
function M:playItemVoice()
    soundVoice:stopNowSound()
    if self._itemId == 1 then
        -- 播放配音[v147119]饼干
        soundVoice:playEffectv147119()
    elseif self._itemId == 2 then
        -- 播放配音[v147116]杨桃
        soundVoice:playEffectv147116()
    elseif self._itemId == 3 then
        -- 播放配音[v147118]海星
        soundVoice:playEffectv147118()
    elseif self._itemId == 4 then
        -- 播放配音[v147120]宝石
        soundVoice:playEffectv147120()
    elseif self._itemId == 5 then
        -- 播放配音[v147117]星星
        soundVoice:playEffectv147117()
    end
end

-- 吐出物体动作随机播放
function M:ranSpitOutAct()
    local ranNum = math.random(1, 2)
    -- 吐物骨骼延迟显示时间, 落地粒子出现时间
    local itemTime, parTime
    if ranNum == 1 then
        itemTime = 0.4
		parTime = 0.7
        self:spitOutAct1()
    else
        itemTime = 0.5
		parTime = 0.75
        self:spitOutAct2()
    end

    -- 避免星形物品图片穿帮，延迟显示
    self:line({                    
        {"delay", itemTime}, 
        {"fn", function ()
            self:showItemBone()
        end}
    })
    
    -- 提前显示粒子
    self:line({
        {"delay", parTime}, 
        {"fn", function ()
            local item = self._parent:createItem():hide()
            -- 当前展示的实物
            self._curItem = item
            -- 粒子
            local pos = item:worldpoint()
            pos.y = pos.y - item:ch()/2.3
            local particle = CUtil:newParticle("particle/twdl.plist"):to(self._layer, 100):p(pos)
        end}
    })
    -- self:spitOutChangeScale()

    if self._effect then sound.stopSound(self._effect) end
    -- 播放音效[sfx14706027]点击星形吐物/两个骨骼都用这个音效
    self._effect = soundEffect:playEffectsfx14706027()
end

-- 动作回调
function M:handlerOnDerived(__evtType, __moveId)
    if __evtType == 0 then
        if __moveId == self._standbyAni then
            self:playFeedAniSound("sfx14705029")
        elseif __moveId == self._unhappyAni then
            self:playFeedAniSound("sfx14705030")
        elseif __moveId == self._unhappyAni2 then
            self:playFeedAniSound("sfx14705031")
        elseif __moveId == self._openMouthAni1 then
            self:playFeedAniSound("sfx14705032")
        elseif __moveId == self._eatAni2 or __moveId == self._eatAni2_2 then
            self:playFeedAniSound("sfx14705033")
        end
    end
end


-- 喂食退场
function M:feedQuit(dir, callback)
    -- soundEffect:playEffectsfx14700010()
    -- 摇晃出场
    
    dir = ifnil(dir, (self:x() < V.w_2 - 10) and -1 or (self:x() > V.w_2 + 10) and 1 or N.randomExists(1, -1))
    CUtil:changeAnchorP(self, cc.p(0.5, 0.2))
    self:line({
        {"moveBy", 1.6, cc.p(700 * dir, 0)},
        {"remove"},
    })

    self:cycle({
        {"easing", "SINEOUT", {"rotateBy", 0.2, 16}},
        {"easing", "SINEIN", {"rotateBy", 0.2, -16}},
        {"easing", "SINEOUT", {"rotateBy", 0.2, -16}},
        {"easing", "SINEIN", {"rotateBy", 0.2, 16}},
    })

    self:cycle({
        {"scaleTo", 0.2, 0.95 * self._scale},
        {"scaleTo", 0.2, 1 * self._scale},
    })

    return "sfx14700010"
end




----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end


return M