--[[
Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-1-27

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类 -- 形状骨骼基类
----------------------
local M     = classArmatureTouch("ShapeBase")
M.TAG       = "ShapeBase"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 各个形状嘴的位置
local mouthPoints   = {cc.p(158, 56), cc.p(131, 82), cc.p(149, 92), cc.p(113, 121), cc.p(153, 71), cc.p(164, 115)}



----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 主层
    self._layer        = params.layer
    -- 形状ID
    self._id            = 0
    -- 当前场景名
    self._sceneName    = params.sceneName or self:getScene().name
    -- 在场景中的大小
    self._scale        = params.scale or 0.8


    -- 连点吐物场景时的父节点
    self._parent = params.parent
    -- 预先准备物品的序数[未吐物时，最高为3]
    self._preItemIndex    = 0
    -- 吐出物品的序数[最高3次]
    self._curItemIndex    = 0
    -- 形状点连点吐物场景的形状ID,与其他场景区分 
    self._shapeId = params.shapeId
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 初始化动画名
    self:initAnimationNames()

    -- 物品纹理名称顺序洗牌
    self:ranTextureName()

    -- 播放呼吸动作1-1
    if S.contains(self._sceneName, "Shapefeed") then
        self:breathAct1()
        -- self:openMouthAct1()
        -- local node = D.img("common/help.png"):to(self, 100):bindTouchLocate()
    end
end

-- 初始化动画名
function M:initAnimationNames()
    self._breathAni1        = nil 		-- 呼吸
    self._breathAni2        = nil 		-- 呼吸眨眼
    self._rhythmAni1        = nil 		-- 律动1
    self._rhythmAni2        = nil 		-- 律动2
    self._spitOutAni1        = nil 		-- 吐出物体
    self._spitOutAni2        = nil 		-- 吐出物体2
    self._transformationAni = nil 		-- 变成小精灵
    self._openMouthAni1    = nil 		-- 张嘴
    self._openMouthAni2    = nil 		-- 张嘴持续
    self._eatAni1            = nil 		-- 吃东西开始
    self._eatAni2            = nil 		-- 吃东西循环
    self._eatAni3            = nil 		-- 吃东西咽下
    self._unhappyAni        = nil 		-- 不高兴

    -- 新增动作
    self._standbyAni        = nil       -- 待机动作 
    self._unhappyAni2       = nil       -- 不高兴2    --（两套不高兴随机轮流）
    self._eatAni2_2         = nil       -- 吃东西循环2 --（两套吃东西随机轮流，部分有）
    self._openMouthAni2_1   = nil       -- 张嘴       --（两套张嘴随机轮流，部分有）
    self._openMouthAni2_2   = nil       -- 张嘴持续
end


-- 吃食物
function M:eatFood()
    self._layer:playShapeVoice(self._id)
    self:loadEatParticle()
    if self._eatAni1 then
        self:eatAct1()
    else
        self:eatAct2()
    end

    self:line({
        { "delay", 1 },
        { "fn", function()
            self:eatAct3()
        end },
    })
end

-- 拒绝吃食
function M:refuseEat()
    self:unhappyAct()
end

-- 进场
function M:enter(callback)
    self:scale(0):line({
        {"easing", "backOut", {"scaleTo", 0.5, self._scale}},
        {"fn", function()
            if callback then
                callback()
            end
        end},
    })
end


-- 进场
function M:feedEnter(callback)
    local dis = -600
    -- 在屏幕中间一般概率在右边，在屏幕右边则一定在右边
    if (math.abs(self:px() - V.w_2) < 1 and math.random() < 0.5) or self:px() - V.w_2 > 1 then
        dis = 600
    end
    self:scale(self._scale):px(self:px() + dis)
    self:line({
        -- {"easing", "IN", {"moveBy", 0.5, cc.p(-dis, 0)}, 1},
        {"jumpBy", 0.5, cc.p(-dis, 0), 50, 1},
        {"fn", function()
            CAction:feedback(self)
            if callback then
                callback()
            end
        end},
    })
    -- self:scale(self._scale):py(self:py() + 500)
    -- self:line({
    --     {"easing", "IN", {"moveBy", 0.5, cc.p(0, -500)}, 1},
    --     {"fn", function()
    --         CAction:feedback(self)
    --         if callback then
    --             callback()
    --         end
    --     end},
    -- })
end

-- 喂食退场
function M:feedQuit(dir, callback)
    print("未重写退场方法")
    -- dir = ifnil(dir, (self:x() < V.w_2 - 10) and -1 or (self:x() > V.w_2 + 10) and 1 or N.randomExists(1, -1))
    -- self:breathAct1()

    -- if self._id == 3 then
    --     soundEffect:playEffectsfx14700005()
    --     self:one({"rotateBy", 1.6, dir * 720})
    -- else
    --     soundEffect:playEffectsfx14700004()
    --     self:line({
    --         A.union({
    --             {"jumpBy", 0.4, cc.p(0, -30), 50, 1},
    --             A.line({
    --                 {"scaleTo", 0.1, 0.9 * self._scale, 1.15 * self._scale},
    --                 {"scaleTo", 0.2, 1.1 * self._scale, 0.95 * self._scale},
    --                 {"scaleTo", 0.1, 1 * self._scale},
    --             })
    --         }),
    --         A.cycle({
    --             A.union({
    --                 {"jumpBy", 0.4, cc.p(0, 0), 50, 1},
    --                 A.line({
    --                     {"scaleTo", 0.1, 0.9 * self._scale, 1.15 * self._scale},
    --                     {"scaleTo", 0.2, 1.1 * self._scale, 0.95 * self._scale},
    --                     {"scaleTo", 0.1, 1 * self._scale},
    --                 })
    --             }),
    --         }, 5)
    --     })
    -- end

    -- self:line({
    --     {"moveBy", 1.6, cc.p(800 * dir, 0)},
    --     {"remove"},
    -- })
end

-- 播放喂食场景骨骼音效
function M:playFeedAniSound(soundIndex)
    sound.stopSound(self._feedAniSound)
    self._feedAniSound = soundEffect["playEffect"..soundIndex](soundEffect)
end

-- 获取嘴的位置
function M:getMouthPoint()
    return mouthPoints[self._id]
end

-- 吃东西粒子
function M:loadEatParticle()
    local particle  = P.new("particle/cbg.plist"):to(self, 10):p(self:getMouthPoint())
    CUtil:changeParticleColor(particle, cc.c3b(255, 234, 110))
    self._eatPar    = particle
end

-- 检查参数播放动画
function M:playAniWithCheck(animName, fps, animOption)
    if animName then
        self:play(animName, fps, animOption)
    else
        print("这个动作值为nil")
    end
end

-- 呼吸动作1-1
function M:breathAct1()
    -- 记录呼吸动作播放次数，到达指定次数眨一下眼
    self._breathCount    = 0
    self:playAniWithCheck(self._breathAni1)
end

-- 呼吸动作2(眨眼)-1
function M:breathAct2()
    self:playAniWithCheck(self._breathAni2, nil, { loop = 0 })
end

-- 律动1-1
function M:rhythmAct1()
    self._touchLock = false
    self:playAniWithCheck(self._rhythmAni1, nil, { loop = 0 })

    if self._effect then sound.stopSound(self._effect) end
    -- 播放音效/律动1
    if self._shapeId == 1 then
        -- 播放音效[sfx14706012]sanjiao_lvdong/三角律动
        self._effect = soundEffect:playEffectsfx14706012()
    elseif self._shapeId == 2 then
        -- 播放音效[sfx14706016]fang_lvdong/方形律动
        self._effect = soundEffect:playEffectsfx14706016()
    elseif self._shapeId == 3 then
        -- 播放音效[sfx14706019]jv_lvdong/矩形律动
        self._effect = soundEffect:playEffectsfx14706019()
    elseif self._shapeId == 4 then
        -- 播放音效[sfx14706024]yuan_lvdong/圆形律动，仅第一次的时候播放
        self._effect = soundEffect:playEffectsfx14706024()
    elseif self._shapeId == 5 then
        -- 播放音效[sfx14706025]xing_lvdong/星形律动，仅第一次的时候播放
        self._effect = soundEffect:playEffectsfx14706025()
    elseif self._shapeId == 6 then
        -- 播放音效[sfx14706028]xin_lvdong/心形律动，仅第一次的时候播放
        self._effect = soundEffect:playEffectsfx14706028()
    end
end

-- 律动2-1
function M:rhythmAct2()
    self._touchLock = false -- 解锁触控
    self:playAniWithCheck(self._rhythmAni2, nil, { loop = 0 })

    if self._effect then sound.stopSound(self._effect) end
    -- 播放音效/律动2
    if self._shapeId == 1 then
        -- 播放音效[sfx14706013]sanjiao_lvdong2
        self._effect = soundEffect:playEffectsfx14706013()
    elseif self._shapeId == 2 then
        -- 播放音效[sfx14706017]fang_lvdong2
        self._effect = soundEffect:playEffectsfx14706017()
    elseif self._shapeId == 3 then
        -- 播放音效[sfx14706020]jv_lvdong2
        self._effect = soundEffect:playEffectsfx14706020()
    elseif self._shapeId == 4 then
        -- 播放音效[sfx14706022]yuan_lvdong2
        self._effect = soundEffect:playEffectsfx14706022()
    elseif self._shapeId == 5 then
        -- 播放音效[sfx14706026]xing_lvdong2
        self._effect = soundEffect:playEffectsfx14706026()
    elseif self._shapeId == 6 then
        -- 播放音效[sfx14706029]xin_lvdong2
        self._effect = soundEffect:playEffectsfx14706029()
    end
end

-- 吐出物体1-0
function M:spitOutAct1()
    self:playAniWithCheck(self._spitOutAni1)
    print("spitOutAct1")
end

-- 吐出物体2-0
function M:spitOutAct2()
    self:playAniWithCheck(self._spitOutAni2)
    print("spitOutAct2")
end

-- 变成小精灵-0
function M:transformationAct()
    self:playAniWithCheck(self._transformationAni)
    
    if self._effect then sound.stopSound(self._effect) end
    -- 播放音效[sfx14706011]形状五官/形状五官冒出的时候播放
    self._effect = soundEffect:playEffectsfx14706011()
end

-- 张嘴1-0
function M:openMouthAct1()
    self:playAniWithCheck(self._openMouthAni1)
end
-- 张嘴持续-1
function M:openMouthAct2()
    self:playAniWithCheck(self._openMouthAni2)
end

-- 吃东西动画开始-0
function M:eatAct1()
    self:playAniWithCheck(self._eatAni1)
end
-- 吃东西动画（持续）-1
function M:eatAct2()
    -- 两种循环吃东西(如果有)动画随机播放
    local aniName = self._eatAni2
    if self._eatAni2_2 and math.random() < 0.5 then
        aniName = self._eatAni2_2
    end
    self:playAniWithCheck(aniName)
end
-- 吃东西动画，咽下-0
function M:eatAct3()
    self._eatPar:remove()
    self:playAniWithCheck(self._eatAni3)
end

-- 喂错食物，不高兴-0
function M:unhappyAct()
    -- 两种(如果有)动画随机轮流
    self._unhappyIndex  = self._unhappyIndex and (3 - self._unhappyIndex) or math.random(1, 2)
    local aniName = self._unhappyAni
    if self._unhappyIndex == 2 and self._unhappyAni2 then
        aniName = self._unhappyAni2
    end
    self:playAniWithCheck(aniName, nil, { loop = 0 })
end

-- 待机-0
function M:standbyAct()
    self:playAniWithCheck(self._standbyAni)
end

-- 动画回调动作
function M:movementHandler(__evtType, __moveId)
    -- 派生类回调方法（如果返回值为true 此处的逻辑不再执行）
    local isReturn = self:handlerOnDerived(__evtType, __moveId)
    if isReturn then return end

    -- 根据场景名判断执行不同的动作回调
    if S.contains(self._sceneName, "Shapefeed") then
        self:movementHandlerFeed(__evtType, __moveId)
    elseif S.contains(self._sceneName, "Shapeline") then
        self:movementHandlerLine(__evtType, __moveId)
    end
end

-- 形状匹配喂食场景回调
function M:movementHandlerFeed(__evtType, __moveId)
    if __evtType == 1 then
        -- 眨眼呼吸后播待机动画
        if __moveId == self._breathAni2 then
            self:standbyAct()
        -- 待机后切回正常呼吸
        elseif __moveId == self._standbyAni then
            self:breathAct1()
            -- 吃动作开始接循环吃
        elseif __moveId == self._eatAni1 then
            self:eatAct2()
            -- 吃食物咽下调用layer方法记录数量
        elseif __moveId == self._eatAni3 then
            self:breathAct1()
            self._layer:eatOneFood()
            -- 张嘴开始接张嘴循环
        elseif __moveId == self._openMouthAni1 then
            self:openMouthAct2()
            -- 喂错不高兴接呼吸
        elseif __moveId == self._unhappyAni or __moveId == self._unhappyAni2 then
            self:breathAct1()
        end

    elseif __evtType == 2 then
        -- 每呼吸二下 眨一下眼睛，然后接待机动作
        if __moveId == self._breathAni1 then
            self._breathCount = self._breathCount + 1
            if self._breathCount == 2 then
                self:breathAct2()
            end
        end
    end
end

----------------------- 形状连点吐物场景的方法 ---------------------------

-- 当变为骨骼精灵的回调
function M:onBecomeBoneHandler()
    self._touchLock = false
    local isFirst = true
    self:openGuide(isFirst)
end

-- 播放随机律动
function M:ranRhythmAct()
    if not self._rhythmCount then
        self._rhythmCount = 1
    else 
        self._rhythmCount = self._rhythmCount + 1
    end

    -- 律动两次之后
    if self._rhythmCount > 2 then
        self._rhythmCount = nil
        self:breathAct1()
        self:stopGuide()
        self:openGuide()
        return
    end

    local ranNum = math.random(1, 2)
    if ranNum == 1 then
        self:rhythmAct1()
    elseif ranNum == 2 then
        self:rhythmAct2()
    end
end

-- 无操作律动引导提示(首次引导2秒, 后面4秒一次)
function M:openGuide(isFirst)
    if self._openGuideAct then return end

    local delayTime
    if isFirst then
        delayTime = 2
    else
        delayTime = 4
    end

    self._openGuideAct = A.line({
        {"delay", delayTime},
        {"fn", function ()
            self:ranRhythmAct()
            self:playGuideVoice()
            self._layer:playFingerClickGuide(self)
        end},
    }):at(self)
end

-- 播放引导台词
function M:playGuideVoice()
	if self._guideVoiceTag then
		self._guideVoiceTag = false
        -- 播放配音[v147153]精灵会吐出什么呢？
        soundVoice:playEffectv147153()
	else
		self._guideVoiceTag = true

        if self._shapeId == 1 then
            -- 播放配音[v147203]点一点精灵～看看三角形的有什么吧！
            soundVoice:playEffectv147203()
        elseif self._shapeId == 2 then
            -- 播放配音[v147152]点一点精灵～看看正方形的有什么吧！
            soundVoice:playEffectv147152()
        elseif self._shapeId == 3 then
            -- 播放配音[v147201]点一点精灵～看看长方形的有什么吧！
            soundVoice:playEffectv147201()
        elseif self._shapeId == 4 then
            -- 播放配音[v147202]点一点精灵～看看圆形的有什么吧！
            soundVoice:playEffectv147202()
        elseif self._shapeId == 5 then
            -- 播放配音[v147204]点一点精灵～看看星形的有什么吧！
            soundVoice:playEffectv147204()
        elseif self._shapeId == 6 then
            -- 播放配音[v147205]点一点精灵～看看爱心形的有什么吧！
            soundVoice:playEffectv147205()
        end
	end
end

-- 操作后 取消律动引导提示
function M:stopGuide()
    if not self._openGuideAct then return end
    self:stopAction(self._openGuideAct)
    self._layer:stopFingerGuide()
    self._openGuideAct = nil
end

-- 吐物时形状骨骼缩放
function M:spitOutChangeScale()
    self:line({
        {"delay", 0.2},
        {"scaleTo", 0.1, 1.5}, 
        {"scaleTo", 0.15, 1}
    })
end

-- 当吐完物品的回调
function M:onSpitOutHandler()
    self:breathAct1()
    local isSpitAllOver = self:checkIsSpitAllOver()
    
    if isSpitAllOver then
        self._touchLock = true
    else
        self._touchLock = false
    end

    if not tolua.isnull(self._curItem) then
        self._curItem:show()
    end
end

-- 检查是否完成三次吐物
function M:checkIsSpitAllOver()
    -- 引入显示的物品ID（解决物品ID变更导致退场方法错误的问题）
    self._showItemId = self._itemId
    if self._curItemIndex >= 3 then
        self:line({
            {"delay", 0.5},
            {"fn", function ()
                local callback = function ()
                    self._layer:quitAndEnterNextProp()
                end
                self._parent:itemGetOutAction(self._curItem, callback)
            end}
        })
        return true
    else
        self:changeItemSkin()
        self:line({
            {"delay", 0.5},
            {"fn", function ()
                self._parent:itemGetOutAction(self._curItem)
                self:openGuide()
            end}
        })
        return false
    end
end

-- 给物品换装(提前换装)
function M:changeItemSkin()
    if self._preItemIndex >= 3 then return end
	self._preItemIndex = self._preItemIndex + 1

    local itemId = self._itemIdTab[self._preItemIndex]
    self._textureName = self._textureNameTab[itemId]          -- 物品纹理名称/动画名称
    self._itemId = itemId                                     -- 物品ID
    if not istable(self._textureName) then
        local shapeName = self:getShapeName()
        self._armature:change(shapeName.."_wupin", self._textureName)
        self:hideItemBone()
    end
end

-- 检查是换装吐物，还是播放特定吐物动画
function M:checkAndSpitOutItem()
	if self._curItemIndex >= 3 then return end
	self._curItemIndex = self._curItemIndex + 1

    if istable(self._textureName) then
        local aniName = self._textureName.aniName
        if aniName == self._spitBubbleAni then
            self:spitBubbleAct()
        elseif aniName == self._spitLeafAni then
            self:spitLeafAct()
        elseif aniName == self._spitCardAni then
            self:spitCardAct()
        elseif aniName == self._spitTissueAni then
            self:spitTissueAct()
        end
    else
        self:ranSpitOutAct()
    end

    self:line({
        {"delay", 0.5},
        {"fn", function ()
            self:playItemVoice()
        end}
    })
end

-- 获取美术的形状的名称
function M:getShapeName()
    local shapeName
    if self._shapeId == 1 then
        shapeName = "sanjiao"
    elseif self._shapeId == 2 then
        shapeName = "fang"
    elseif self._shapeId == 3 then
        shapeName = "jv"
    elseif self._shapeId == 4 then
        shapeName = "yuan"
    elseif self._shapeId == 5 then
        shapeName = "xing"
    elseif self._shapeId == 6 then
        shapeName = "xin"
    end
    return shapeName
end

-- 隐藏物品骨骼
function M:hideItemBone()
    local shapeName = self:getShapeName()
    self._armature:getBone(shapeName.."_wupin"):hide()
end

-- 显示物品骨骼
function M:showItemBone()
    local shapeName = self:getShapeName()
    self._armature:getBone(shapeName.."_wupin"):show()
end

-- 形状连点吐物 场景回调
function M:movementHandlerLine(__evtType, __moveId)
    if __evtType == 1 then
        -- 眨眼呼吸后切回正常呼吸
        if __moveId == self._breathAni2 then
            self:breathAct1()
        -- 出现小精灵接正常呼吸
        elseif __moveId == self._transformationAni then
            self:breathAct1()
        -- 当吐完物品的回调
        elseif __moveId == self._spitOutAni1 or __moveId == self._spitOutAni2 then
            self:onSpitOutHandler()
        -- 律动1接着随机律动
        elseif __moveId == self._rhythmAni1 then
            self:ranRhythmAct()
        -- 律动2接着随机律动
        elseif __moveId == self._rhythmAni2 then
            self:ranRhythmAct()
        end
    elseif __evtType == 2 then
        -- 每呼吸达到次数 眨一下眼睛
        if __moveId == self._breathAni1 then
            self._breathCount = self._breathCount + 1
            if self._breathCount == 1 then
                self:breathAct2()
            end
        end
    end
end

-- 派生类重写这个方法执行单独的回调逻辑
function M:handlerOnDerived(__evtType, __moveId)
    return false
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end


return M