--[[Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

场景层类，定义层相关操作方法及逻辑实现。

-   定义场景层功能方法。

]]
----------------------
-- 类
----------------------
local M = classLayerTouch("Main")


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 配置表
local Config = require("app.numline.Config")
-- 轮廓线                    
local Outline = require("app.numline.node.Outline")
-- 物品所有点的父节点
local Spots = require("app.numline.node.Spots")
-- 实物item的类
local ItemWatermelon = require("app.numline.node.ItemWatermelon")
local ItemGift       = require("app.numline.node.ItemGift")
local ItemBall       = require("app.numline.node.ItemBall")
local ItemButterfly  = require("app.numline.node.ItemButterfly")
local ItemCar        = require("app.numline.node.ItemCar")
local ItemPlane      = require("app.numline.node.ItemPlane")
local ItemCake       = require("app.numline.node.ItemCake")
local ItemTree       = require("app.numline.node.ItemTree")
local ItemCrown      = require("app.numline.node.ItemCrown")
local ItemClassTab = {
    false, ItemWatermelon, ItemGift, ItemBall, ItemButterfly, ItemCar,
    ItemPlane, ItemCake, ItemTree, ItemCrown
}
-- 菜单栏
local Menu = require("app.numline.node.Menu")
-- 手指引导
local GuideFinger = require("app.common.GuideFinger")

-- [操作变量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义视图实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
end

----------------------
-- 结点渲染
----------------------
--[[--

视图渲染，处理视图结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- 加载其他
    self:loadRenderOtherwise()
    -- 加载背景图
    self:loadBg()
    -- 加载手指引导
    self:loadGuideFinger()
    -- 获取当前轮次的关卡组合表
    self:getCurLevelTab()
    -- 加载菜单栏
    self:loadMenu()
    -- 加载物品关卡
    self:loadLevel()
end

-- 加载其他
function M:loadRenderOtherwise()

end

function M:onEnterTransitionFinish()
    -- 播放背景音乐[bgm14708]数字连线
    sound:playbgm14708()
end

-- 加载背景图
function M:loadBg()
    local bg = U.loadBg("num/line/bg.png"):to(self)
end

-- 获取当前轮次的关卡组合表
function M:getCurLevelTab()
	if (not G_numline_turnIndex) or G_numline_turnIndex == 0 or G_numline_turnIndex >= #G_numlineLevelTab then
		G_numline_turnIndex = 1
	else
		G_numline_turnIndex = G_numline_turnIndex + 1
	end
	-- 关卡组合表
	self._levelTab = G_numlineLevelTab[G_numline_turnIndex]
	-- 关卡索引
	self._levelIndex = 1
	-- 初始化关卡ID, 同时也是物品ID
	self._curLevel = self._levelTab[self._levelIndex]
end

-- 加载菜单栏
function M:loadMenu()
    local menu = Menu.new({
        main = self,
        num  = self._curLevel
    }):to(self, 400):p(90, 80)

    self._menu = menu
end

-- 加载物品关卡
function M:loadLevel()
    self._outline = self:createOutline()
    self._item = self:createItem()
    self._spots = self:createSpots()
end

-- 进入下一关(按顺序操作)
function M:enterNextLevel()
    if self._levelIndex >= #self._levelTab then
        self:enterNextScene()
        return
    end

    self._levelIndex = self._levelIndex + 1
    -- 菜单栏手动切换形状后，则不按照预设的形状ID表顺序进行
    if self._isOperateByHand then
        if self._curLevel <10 then
            self._curLevel = self._curLevel + 1
        else
            self._curLevel = 2
        end
    else
        self._curLevel = self._levelTab[self._levelIndex]
    end

	self._menu:setBtnCurNum(self._curLevel)
	self._menu:tickSelect(self._curLevel)
    self:loadLevel()
end

-- 退出当前关卡进入指定关卡（菜单栏手动切换关卡）
function M:quitAndEnterLevel(level)
    if not tolua.isnull(self._outline) then
        self._outline:remove()
    end

    if not tolua.isnull(self._item) then
        self._item:remove()
    end

    if not tolua.isnull(self._spots) then
        self._spots:remove()
    end
    self:clearActiveParticle()

    -- 关闭数字播报
    if self._numSpotAct then
        self:stopAction(self._numSpotAct)
        self._numSpotAct = nil
        self._numIndex = 0
    end

    -- 是否手动操作
    self._isOperateByHand = true
    self._curLevel = level
    self:loadLevel()
end

-- 进入下一个场景
function M:enterNextScene()
    local encourage = CUtil:loadEncourage(self, 3, function ()
        G_addFinish("numline")
        SceneController:enterNextMoudleScene()
    end)
end

-- 创建物品
function M:createItem()
    local item = ItemClassTab[self._curLevel].new({
        main   = self,
        itemId = self._curLevel,
        scale  = 1,
    }):to(self):p(480 + X_OFFSET, 270 + Y_OFFSET - 40):hide()

    -- 蛋糕场景扩大触控范围
    if self._curLevel == 8 then
        item:bindTouch({
			rectFactorX = 1,
			rectFactorY = 1.5
        })
    end
    return item
end

-- 创建线框（线集合）
function M:createOutline()
    local outline = Outline.new({
        main = self,
        itemId = self._curLevel
    }):to(self):p(480 + X_OFFSET, 270 + Y_OFFSET - 40)
    return outline
end

-- 创建引导点集合
function M:createSpots()
    local spots = Spots.new({
        main = self,
        itemId = self._curLevel
    }):to(self):p(480 + X_OFFSET, 270 + Y_OFFSET - 40)
    return spots
end

-- 点击数字点后的回调（生成线并激活下一个待点击的点）
function M:onClickSpotHandler(num)
    local callback = function()
        -- 激活下一个待点击的点
        self._spots:activeNextSpot()
        sound.voiceFadeOut(soundEffect._nowSound, 0.2)
    end
    self._outline:showCurLine(num, callback)
end

-- 重新播报数字
function M:rePlayNumber()
    -- 退出当前物品,进入下一关的方法
    local callback = function ()
        self:line({
            { "fn", function()
                self._spots:quitEffect()
            end },
            {"delay", 0.3},
            {"fn", function ()
                self._outline:quitEffect()
            end},
            { "delay", 0.3 },
            { "fn", function()
                self._item:show()
                self._item:enterEffect()
            end }
        })
    end

    local numSpotTab = self._spots._numSpotTab
    -- self._numSpotAct = A.cycle({
    --         if not self._numIndex then
    --             self._numIndex = 0
    --         elseif self._numIndex >= #numSpotTab then
    --             callback()
    --             self:stopAction(self._numSpotAct)
    --             self._numSpotAct = nil
    --             self._numIndex = 0
    --             return
    --         end

    --         self._numIndex = self._numIndex + 1
    --         numSpotTab[self._numIndex]:onRePlayHandler()
    --     end},
    --     {"delay", 0.85}
    -- }):at(self)

    for k, v in pairs(numSpotTab) do
        if v and not tolua.isnull(v) then

            local time = 0.85 * k

            if device.language == "ru" and k >= 5 then
                time = time - 0.85 + 1.3 
            end

            v:plan(function() 
                v:onRePlayHandler()

                v:plan(function() 
                    if k == #numSpotTab and callback then
                        callback()
                    end
                end, 0.85)

            end, time)
        end
    end
end

-- 播放激励粒子
function M:playActiveParticle()
	-- 粒子（播放物品激励动画时）
	local particle = CUtil:newParticle("particle/ptjl.plist"):to(self, 1000):p(V.w_2, V.h_2)
	self._activeParticle = particle

    -- 播放音效[sfx14712018]变身后全屏粒子
    soundEffect:playEffectsfx14712018()
end

-- 清除激励粒子
function M:clearActiveParticle()
	if not tolua.isnull(self._activeParticle) then
		CUtil:clearParticle(self._activeParticle, 1)
	end
end

-- 加载手指引导
function M:loadGuideFinger()
    if not self._finger then
        local finger    = GuideFinger.new({
        }):to(self, 10000):p(0, 0)
        self._finger = finger
    end
end

-- 播放手指点击引导（跟随待机引导的line动作，非cycle）
function M:playFingerClickGuide(node)
	local wpos = node:worldpoint()
	local anchorY = node:getAnchorPoint().y
	wpos.y = wpos.y + node:ch()/2 - node:ch()*anchorY + 30
	self._finger:playFingerClickGuide(wpos)
end

-- 停止手指引导(拖动/点击通用)
function M:stopFingerGuide()
	self._finger:closeHandGuide()
end

----------------------
-- 结点析构
----------------------
--[[--

视图析构，处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

function M:onExitTransitionStart()

end

return M