

--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("Apple")
M.TAG       = "Apple"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	-- 父节点
	self._parent = params.parent
	-- 主层级
	self._main   = params.main
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end

-- 图片渲染
function M:loadDisplay()
	self:display("num/line/item/item9/apple.png")
end

-- 苹果进场的方法
function M:getInto()
	self:line({
		{"easing", "backOut", { "scaleTo", 0.5, 0.8 }},
		{"fn", function ()
			self:playBreath()
		end}
	})
end

-- 苹果呼吸方法
function M:playBreath()
	CUtil:changeAnchorP(self, ccp(0.5, 1))
	local dir = T.shuffle({1, -1})[1]
	local anchor = dir * math.random(2, 5)
	local time = math.random(2, 5)/10
	self._breathAct = A.line({
		{"rotateBy", time, anchor},
		{"rotateBy", 2*time, -2*anchor},
		{"rotateBy", time, anchor},
		{"fn", function ()
			self:playBreath()
		end}
	}):at(self)
end

-- 苹果停止呼吸方法
function M:stopBreath()
	if self._breathAct then
		self:stopAction(self._breathAct)
	end
end

-- 苹果掉落的方法
function M:appleDrop(event)
	self._parent:playClickScale()
	self._parent._autoEvent:stopGuide()
	self._parent._autoEvent:openGuide()
	T.removeOrder(self._parent._appleTab, self)
	CUtil:resetParent(self, self._main, 1000)

	self:stopBreath()
	self:playAppleDropEffect()
	self._parent._appleDir = -1*self._parent._appleDir
	local divX = self._parent._appleDir * math.random(50, 40)
	local divY = 100 - self:py()
	local time = divY/500
	--粒子
	local particle = CUtil:newParticle("particle/zhaipg.plist"):to(self._parent._main, 100):p(self:point())
	particle:cycle({
		{"delay", 1/60},
		{"fn", function ()
			particle:p(self:point())
		end}
	})

	self:line({
		{"jumpBy", 0.3, ccp(divX, divY), 100, 1},
		{"fn", function ()
			CUtil:clearParticle(particle, 1)
			CUtil:changeAnchorP(self, ccp(0.5, 0))
			CAction:feedback2(self, 1)
			-- 检查苹果是否全部掉完
			self._parent:checkIsAppleAllDropOut(event, "apple")
		end},
		{"delay", 0.15},
		{"fn", function ()
			-- 苹果滚动出去
			CUtil:changeAnchorP(self, ccp(0.5, 0.5))
			self:union({
				{"moveBy", 2, cc.p(800*self._parent._appleDir, 0)},
				{"rotateBy", 2, 3*360*self._parent._appleDir},
			})
		end}
	})
end

-- 播放苹果掉落音效
function M:playAppleDropEffect()
	if not self._appleDropIndex then
		self._appleDropIndex = 1
	else
		self._appleDropIndex = self._appleDropIndex + 1
	end

	if self._appleDropIndex == 1 then
		-- 播放音效[sfx14712028]点击苹果1
		soundEffect:playEffectsfx14712028()
	elseif self._appleDropIndex == 2 then
		-- 播放音效[sfx14712029]点击苹果2
		soundEffect:playEffectsfx14712029()
	elseif self._appleDropIndex == 3 then
		-- 播放音效[sfx14712030]点击苹果3
		soundEffect:playEffectsfx14712030()
	end
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)
	if self._parent._touchLock then return end
	self:appleDrop()
	return true
end

function M:onTouchMoved(x, y, touches)

end

function M:onTouchEnded(x, y, touches)
	
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M