--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classArmatureTouch("ItemBone")
M.TAG       = "ItemBone"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
local aniNameTab = {
	"xigua_dong", "liwuhe_dong", "qiu_tiao",
	"hudie_dong", "che_dong", "feiji_fei",
	"dangao", "shu_dong", "huangguan",
}

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._parent = params.parent
	self._itemId = params.itemId
end


----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
end

-- 骨骼动画衔接
function M:movementHandler( __evtType, __moveId )
	if __evtType == 1 then
		-- 西瓜虫子张嘴后播放咀嚼
		if __moveId == "chi1" then
			self:play("chi2", nil, {loop = 0})
		-- 西瓜虫子咀嚼后播放吞下
		elseif __moveId == "chi2" then
			self:play("chi3")
		end
	end
end

-- 播放虫子张嘴吃动画
function M:playEatAni()
	self:play("chi1")
	self:playEatWatermelonSoundEffect()
	-- 粒子
	local particle = CUtil:newParticle("particle/chixigua.plist"):to(self, 100):color(cc.c3b(239, 86, 81)):p(self:cw()/2, 0)
	CUtil:changeParticleColor(particle, cc.c3b(239, 86, 81))
	self:line({
		{"delay", 1},
		{"fn", function ()
			CUtil:clearParticle(particle, 0.5)
		end}
	})

	self:stopCrawlSoundEffect()
end

-- 播放毛毛虫爬进场
function M:playCrawlEnter()
	self:play("pa")
	self:stopCrawlSoundEffect()
	-- 播放音效[sfx14712010]毛虫蠕动
	self._crawlSoundEffect = soundEffect:playEffectsfx14712010()
	-- 播放音效[sfx14712011]毛虫入场语气词
	soundEffect:playEffectsfx14712011()
end

-- 播放虫子爬行动画
function M:playCrawl()
	self:play("pa")
	self:stopCrawlSoundEffect()
	-- 播放音效[sfx14712010]毛虫蠕动
	self._crawlSoundEffect = soundEffect:playEffectsfx14712010()
end

-- 播放毛毛虫爬退场
function M:playCrawlOut()
	self:play("pa")
	self:stopCrawlSoundEffect()
	-- 播放音效[sfx14712010]毛虫蠕动
	self._crawlSoundEffect = soundEffect:playEffectsfx14712010()
	-- 播放音效[sfx14712015]毛虫出场
	soundEffect:playEffectsfx14712015()
end

-- 爬行音效销毁
function M:stopCrawlSoundEffect()
	if self._crawlSoundEffect then
		sound.stopSound(self._crawlSoundEffect)
	end
end

-- 播放毛毛虫吃西瓜音效
function M:playEatWatermelonSoundEffect()
	if self._eatEffect then
		sound.stopSound(self._eatEffect)
	end

	local eatEffectTab = {1, 2, 3}
	if not self._eatEffectTab then
		self._eatEffectIndex = T.shuffle(eatEffectTab)[1]
	else
		T.removeOrder(eatEffectTab, self._eatEffectIndex, true)
		self._eatEffectIndex = T.shuffle(eatEffectTab)[1]
	end

	if self._eatEffectIndex == 1 then
		-- 播放音效[sfx14712012]毛虫吃西瓜1
		self._eatEffect = soundEffect:playEffectsfx14712012()
	elseif self._eatEffectIndex == 2 then
		-- 播放音效[sfx14712013]毛虫吃西瓜2
		self._eatEffect = soundEffect:playEffectsfx14712013()
	elseif self._eatEffectIndex == 3 then
		-- 播放音效[sfx14712014]毛虫吃西瓜3
		self._eatEffect = soundEffect:playEffectsfx14712014()
	end
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M