

--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("ItemCar")
M.TAG       = "ItemCar"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 配置表
local Config = require("app.numline.Config")
-- 物品骨骼的类
local ItemBone = require("app.numline.node.ItemBone")
-- 公用实物动作类
local PublicAct = require("app.numline.node.PublicAct")
-- 自动事件类
local AutoEvent = require("app.numline.node.AutoEvent")
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._main   	= params.main
	self._itemId 	= params.itemId
	self._scale  	= params.scale
	self._touchLock = true
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
	-- 加载实物的骨骼
	self:loadItemBone()
	-- 加载自动事件
	self:loadAutoEvent()
end

-- 图片渲染
function M:loadDisplay()
	self:display("num/line/item/item"..self._itemId.."/item.png"):opacity(0)
end

-- 加载自动事件
function M:loadAutoEvent()
	local autoEvent = AutoEvent.new({
		itemId = self._itemId,	-- 实物ID
		delayTime = 2,		-- 自动事件的间隔时间
		itemNode = self			-- 物品节点
	}):to(self):p(10000, 10000):unbindTouch()

	self._autoEvent = autoEvent
end

-- 加载实物的骨骼
function M:loadItemBone()
	local itemBone = ItemBone.new({
		parent       = self,
		itemId       = self._itemId,
		file         = "bone/".."kaiche",
		armatureName = "qiche_dong",
		-- animName = ""
	}):to(self, 100):p(self:cw()/2, self:ch()/2):unbindTouch():scaleX(-1)
	self._itemBone = itemBone
end

-- 点击缩放效果
function M:playClickScale()
	self:line({
		{"scaleTo", 0.05, 0.9},
		{"scaleTo", 0.15, 1.1},
		{"scaleTo", 0.1, 1},
	})
end

-- 进场效果
function M:enterEffect()
	self:line({
		{"delay", 0.2},
		{"fn", function ()
			soundVoice:stopNowSound()
			-- 播放配音[v147196]小汽车嘟嘟嘟
			soundVoice:playEffectv147196()
			self:playCarActive()
		end}
	})
end

-- 播放车子启动动画
function M:playCarActive()
	CUtil:changeAnchorP(self, ccp(0.5, 0))
	self._itemBone:play("che_dong")
	-- 播放音效[sfx14712021]汽车启动
	soundEffect:playEffectsfx14712021()
	
	self:line({
		{"delay", 1},
		{"fn", function ()
			self:playCarShake()
			self:activeAutoEvent()
		end},
	})
end

-- 激活自动事件
function M:activeAutoEvent()
	-- 开启自动事件
	self._autoEvent:openAutoEvent()
	self._main:playActiveParticle()
end

-- 退场效果
function M:quitEffect()
	self:line{
		{"delay", 0.1},
		{"fn", function ()
			-- 粒子
			local particle = CUtil:newParticle("particle/chetc.plist"):to(self, 100):p(self:cw()/2, self:ch()*0.1)
			self:playCarShake()

			-- 汽车退场的方向
			local div = self:getCarQuitDiv()
			self:line({
				{"moveBy", 3, ccp(div*800+X_OFFSET, 0)},
				{"fn", function ()
					self._main:clearActiveParticle()
					self._main:enterNextLevel()
				end},
				{"remove"}
			})
		end}
	}
end

-- 获取汽车退场的方向
function M:getCarQuitDiv()
	local div
	if self._startPos then
		if self._endPos.x - self._startPos.x < 0 then
			div = -1
		else
			div = 1
		end
	else
		div = 1
	end

	return div
end

-- 播放汽车颠簸抖动
function M:playCarShake()
	local callback = function ()
		self:playCarShake()
	end
	local strength = math.random(20, 40)/100
	CAction:feedback2(self, strength, callback)
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end

----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()

	self._startPos = ccp(x, y)
	self._endPos   = ccp(x, y)
	self:playClickScale()
	self._autoEvent:clickEvent()
	return true
end

function M:onTouchMoved(x, y, touches)
	self._endPos = ccp(x, y)
end

function M:onTouchEnded(x, y, touches)
	self._endPos = ccp(x, y)
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M