

--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("ItemCrown")
M.TAG       = "ItemCrown"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 配置表
local Config = require("app.numline.Config")
-- 公用实物动作类
local PublicAct = require("app.numline.node.PublicAct")
-- 自动事件类
local AutoEvent = require("app.numline.node.AutoEvent")
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._main      = params.main
	self._itemId    = params.itemId
	self._scale     = params.scale
	self._touchLock = true
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
	-- 加载实物
	self:loadItemNode()
	-- 加载自动事件
	self:loadAutoEvent()
end

-- 图片渲染
function M:loadDisplay()
	self:display("num/line/item/item"..self._itemId.."/item.png"):opacity(0)
end

-- 加载自动事件
function M:loadAutoEvent()
	local autoEvent = AutoEvent.new({
		itemId = self._itemId,	-- 实物ID
		delayTime = 2,		    -- 自动事件的间隔时间
		itemNode = self			-- 物品节点
	}):to(self):p(10000, 10000):unbindTouch()

	self._autoEvent = autoEvent
end

-- 加载实物
function M:loadItemNode()
	local itemNode = D.img("num/line/item/item"..self._itemId.."/item.png"):to(self):p(self:cw()/2, self:ch()/2)
	self._itemNode = itemNode
end

-- 点击缩放效果
function M:playClickScale()
	self:line({
		{"scaleTo", 0.05, 0.7*0.9},
		{"scaleTo", 0.15, 0.7*1.1},
		{"scaleTo", 0.1, 0.7*1},
	})
end

-- 进场效果
function M:enterEffect()
	self:activeAutoEvent()
	self:line({
		{"delay", 0.2},
		{"fn", function ()
			soundVoice:stopNowSound()
			-- 播放配音[v147200]大皇冠亮晶晶
			soundVoice:playEffectv147200()
		end}
	})
end

-- 激活自动事件
function M:activeAutoEvent()
	self:line({
		{"delay", 0.2},
		{"fn", function ()
			-- 播放音效[sfx14712008]变形后缩小
			soundEffect:playEffectsfx14712008()
		end},
		{"scaleTo", 0.5, 0.7},
		{"fn", function ()
			-- 开启自动事件
			self._autoEvent:openAutoEvent()
			self._main:playActiveParticle()
		end}
	})
end

function M:quitEffect()
	local callback = function ()
		self._main:clearActiveParticle()
		self._main:enterNextLevel()
	end

	PublicAct:rollOutAction(self, 0, callback)
	-- 播放音效[sfx14712032]皇冠移出
	soundEffect:playEffectsfx14712032()
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()
	
	self:playClickScale()
	self._autoEvent:clickEvent()
	return true
end

function M:onTouchMoved(x, y, touches)

end

function M:onTouchEnded(x, y, touches)
	
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M