

--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("Outline")
M.TAG       = "Outline"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
local Config = require("app.numline.Config")

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._main    = params.main
	self._itemId  = params.itemId
	self._lineTab = {}
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()

	-- 加载线条
	self:loadLine()
end

-- 图片渲染
function M:loadDisplay()
	self:display("num/line/item/item"..self._itemId.."/outline.png"):opacity(0)
end

-- 加载线条
function M:loadLine()
	local linePosTab = Config.itemInfo[self._itemId].linePosTab

	for i, v in pairs(linePosTab) do
		local path = "num/line/item/item"..self._itemId.."/line"..i..".png"
		-- local opacity
		-- if i%2 == 1 then
		-- 	opacity = 255
		-- else
		-- 	opacity = 100
		-- end
		local line = self:createLine(path, v) -- :opacity(opacity)
		table.insert(self._lineTab, line)
	end
end

-- 创建线条
function M:createLine(path, pos)
	local line = D.img(path):to(self):p(pos):opacity(0)
	return line
end

-- 显示当前对应的线条
function M:showCurLine(num, callback)
	local lineDir = Config.itemInfo[self._itemId].lineDirTab[num]
	self._lineTab[num]:opacity(255)
	self:playRanLineSoundEffect()
	CUtil:dirShow(self._lineTab[num], lineDir, 0.5, callback)

	-- 点击最后一个数字时,要连接最后两条线
	if num == #self._lineTab-1 then
		self._lineTab[num+1]:line({
			{"delay", 0.5},
			{"fn", function ()
				local lineDir = Config.itemInfo[self._itemId].lineDirTab[num+1]
				self._lineTab[num+1]:opacity(255)
				-- 播放音效[sfx14712003]最后一个节点连线
				soundEffect:playEffectsfx14712003()

				local fn = function ()
					-- 播放音效[sfx14712009]连到星星
					soundEffect:playEffectsfx14712009()
					local fn = function ()
						-- 播完星星动画后，重新播放数字
						self._main:rePlayNumber()
					end
					-- 点击最后一个数字点时，播放首个点星星的动画
					self._main._spots._firstSpot:playFirstSpotAni(fn)
				end
				CUtil:dirShow(self._lineTab[num+1], lineDir, 0.5, fn)
			end}
		})
	end
end

-- 播放随机的连线音效
function M:playRanLineSoundEffect()
	if not self._ranTag then
		self._ranTag = math.random(1, 2)
	else
		if self._ranTag == 1 then
			self._ranTag = 2
		elseif self._ranTag == 2 then
			self._ranTag = 1
		end
	end
	
	soundEffect:stopNowSound()
	if self._ranTag == 1 then
		-- 播放音效[sfx14712001]连线1
		soundEffect:playEffectsfx14712001()
	elseif self._ranTag == 2 then
		-- 播放音效[sfx14712002]连线2
		soundEffect:playEffectsfx14712002()
	end
end

-- 退场效果
function M:quitEffect()
	-- 播放音效[sfx14712007]形状变身
	soundEffect:playEffectsfx14712007()
	self:line({
		{"jumpBy", 0.3, ccp(0, 0), 50, 1},
		{"fn", function ()
			-- 粒子（出现实物时）
			local particle = CUtil:newParticle("particle/bianshen.plist"):to(self._main, 1000):p(self:point())
		end},
		{"remove"},
	})
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M