

--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("PublicAct")
M.TAG       = "PublicAct"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
end

-- 缩放退场
function M:scaleOut(node, delayTime, callback)
	node:line({
		{"delay", delayTime},
		{"easing", "backIn", { "scaleTo", 0.5, 0 }},
		{"fn", function ()
            node:hide()
            if callback then
                callback()
            end
		end},
        {"delay", 0.2},
		{"remove"}
	})
end

-- 弹跳退场
function M:bounceOut(node, delayTime, callback)
	CUtil:changeAnchorP(node, ccp(0.5, 0))
	node:line({
		{"delay", delayTime},
		A.cycle({
			A.union({
				{"jumpBy", 0.4, cc.p(200, 0), 50, 1},
				A.line({
					{"scaleTo", 0.1, node:scale()*0.9, node:scale()*1.1},
					{"scaleTo", 0.2, node:scale()*1.1, node:scale()*0.8},
					{"scaleTo", 0.1, node:scale()},
				})
			})
		}, 4),
		{"fn", function()
            if callback then
                callback()
            end
		end},
		{"remove"},
	})
end

-- 滚动出去退场
function M:rollOutAction(node, delayTime, callback)
	node:line({
		{"delay", delayTime},
		A.union({
			{"moveBy", 1.6, cc.p(800, 0)},
			{"rotateBy", 1.6, 720},
		}),
		{"fn", function ()
			node:hide()

			if callback then
				callback()
			end
		end},
		{"remove"},
	})
end

-- 绕圈飞出去动作 #节点 #方向是否翻转 #回调
function M:flyOut(node, isFlip, callback)
    local points1 = {
        cc.p(586, 267), cc.p(613, 224), cc.p(644, 195), cc.p(687, 166), cc.p(733, 145), cc.p(785, 131), cc.p(839, 125),
        cc.p(911, 123), cc.p(978, 132), cc.p(1037, 144), cc.p(1101, 162), cc.p(1170, 188), cc.p(1270, 200)
    }
    if isFlip then       -- 如果方向相反, 每一个点的X轴以第一个点为镜像翻转
        for i=2,#points1 do
            points1[i].x = points1[1].x - (points1[i].x - points1[1].x)
        end
    end
    local points2 = CatmullRomeCurvePoints(points1, 8)
    local index     = 0
    local actionTab = {} -- 生成动作表
    local speed     = 600
    for i,v in ipairs(points2) do
        local sPos  = points2[i-1] and cc.pAdd(points2[i-1], cc.p(-105 + X_OFFSET, Y_OFFSET)) or node:point()
        local ePos  = cc.pAdd(points2[i], cc.p(-105 + X_OFFSET, Y_OFFSET))
        speed       = speed + 5
        table.insert(actionTab, {"moveTo", PT.d(sPos, ePos) / speed, ePos})
    end
    table.insert(actionTab, {"fn", function()
        if callback then callback() end
        node:remove()
    end})
    node:line(actionTab)
    local pos1 = node:point()
    local pos2 = node:point()
    local endAngle = node:rotate()
    local angleDis = 180
    node:scheduleUpdateWithPriorityLua(function (dt)
        pos2 = node:point()
        if pos1.x ~= pos2.x or pos1.y ~= pos2.y then
            local angle = -PT.quadAngle(pos1, pos2) + angleDis
            endAngle = angle
            pos1 = pos2
        end
        local angDis = endAngle - node:rotate()
        angDis = angDis % 360
        if angDis < -180 then
            angDis = 360 + angDis
        elseif angDis > 180 then
            angDis = angDis - 360
        end
        node:rotate(node:rotate() + angDis * dt * 5)
    end, 1)
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M