

--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("Spot")
M.TAG       = "Spot"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 点的入场时间
local enterTime = 0.5

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._itemId = params.itemId
	self._num    = params.num
	self._scale  = params.scale
	self._parent = params.parent
	self._main   = params.main
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end

-- 图片渲染
function M:loadDisplay()
	if self._num then
		self:display("num/line/numspot/grey/"..self._num..".png"):opacity(0)
		self:loadClickStar()
		self:loadSpotState()
	else
		self._touchLock = true
		self:loadStar()
	end
end

-- 加载点的各种状态
function M:loadSpotState()
	local greySpot = D.img("num/line/numspot/grey/"..self._num..".png"):to(self, 10):p(self:cw()/2,self:ch()/2):hide()
	local clickSpot = D.img("num/line/numspot/click/"..self._num..".png"):to(self, 20):p(self:cw()/2,self:ch()/2):hide()
	local clickedSpot = D.img("num/line/numspot/clicked/"..self._num..".png"):to(self, 30):p(self:cw()/2,self:ch()/2):hide()
	-- 未点击灰色点
	self._greySpot = greySpot
	-- 待点击状态红色点
	self._clickSpot = clickSpot
	-- 已点击状态彩色点
	self._clickedSpot = clickedSpot

	if self._num == 1 then
		self:showClickSpot()
	else
		self:showGreySpot()
	end
end

-- 加载首个点显示的星星
function M:loadStar()
	local star = D.img("num/line/star.png"):to(self):p(self:cw()/2+2, self:ch()/2+2):scale(1.5)
	self._star = star
end

-- 加载待点击数字的星星底图
function M:loadClickStar()
	local clickStar = D.img("num/line/star2.png"):to(self):p(self:cw()/2, self:ch()/2):anchor(ccp(0.495, 0.51)):hide()
	clickStar:cycle({
		{"rotateBy", 1, 180}
	})
	self._clickStar = clickStar
end

-- 入场效果
function M:enterEffect()
	self:line({
		{"easing", "backOut", {"scaleTo", enterTime, self._scale}},
	})
end

-- 退场效果
function M:quitEffect()
	self:line({
		{"easing", "backIn", {"scaleTo", enterTime, 0}},
	})
end

-- 播放首个点动画效果
function M:playFirstSpotAni(callback)
	CUtil:newParticle("particle/xingxing.plist"):to(self._parent):p(self:point())
	self._firstSpotAct = A.line({
		{"easing", "backOut", {"rotateBy", 0.5, -180}},
		{"easing", "backOut", {"rotateBy", 0.5, 180}},
		{"fn", function ()
			if callback then
				callback()
			end
		end}
	}):at(self)
end

-- 显示未点击灰色点
function M:showGreySpot()
	self:z(100)
	self._touchLock = true
	self._greySpot:hide()
	self._clickSpot:hide()
	self._clickedSpot:hide()
	self._clickStar:hide()
end

-- 显示待点击状态红色点
function M:showClickSpot()
	--粒子
	local clickSpotPar = CUtil:newParticle("particle/lsz.plist"):to(self):p(self:cw()/2, self:ch()/2):scale(1.3)
	self._clickSpotPar = clickSpotPar 

	self:z(1000)
	self._touchLock = false
	self._greySpot:hide()
	self._clickSpot:show()
	self._clickedSpot:hide()
	self._clickStar:show()

	-- 第一个点入场的时候有延迟时间,入场后再播放
	local delayTime
	if self._num == 1 then
		delayTime = enterTime
	else
		delayTime = 0
	end

	self:line({
		{"delay", delayTime},
		{"scaleTo", 0.5, self._scale*1.3},
		{"fn", function ()
			self:openGuide()
		end}
	})
end

-- 显示已点击状态彩色点
function M:showClickedSpot()
	self:z(100)
	self._greySpot:hide()
	self._clickSpot:hide()
	self._clickedSpot:show()
	self._clickStar:hide()
end

-- 点击的回调（待点击状态的红色点）
function M:onClickSpot()
	self._touchLock = true
	self._main._menu:barChange(false)
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()
	self:playClickSoundEffect(self._num)
	self:playNumberVoice(self._num)
	self:stopAllActions()
	if self._clickSpotPar then
		self._clickSpotPar:remove()
	end
	CUtil:newParticle("particle/lszdj.plist"):to(self):p(self:cw()/2, self:ch()/2):scale(2)

	self:stopGuide()
	-- 新版点击效果
	self:line({
		{"scaleTo", 0.1, self._scale},
		{"scaleTo", 0.2, self._scale*1.3},
		{"scaleTo", 0.1, self._scale},
		{"fn", function ()
			self:showClickedSpot()
			-- 点击后的回调
			self._parent._main:onClickSpotHandler(self._num)
		end}
    })
end

-- 开启缩放引导
function M:openGuide()
	if self._isOpenGuide then return end
	self._isOpenGuide = true
	self:openScaleGuide(self, self._scale*1.3, 1.2, 3, function ()
		self:playGuideVoice()
		self._main:playFingerClickGuide(self)
	end)
end

-- 播放引导台词
function M:playGuideVoice()
	soundVoice:stopNowSound()
	-- 播放配音[v147191]点一点
	soundVoice:playEffectv147191()
end

-- 关闭缩放引导
function M:stopGuide()
	if self._isOpenGuide then
		self._isOpenGuide = false
		self:stopScaleGuide(self)
		self._main:stopFingerGuide()
	end
end

-- 开启缩放引导 #精灵，缩放倍数
function M:openScaleGuide(node, scaleSize, scaleMultiple, time, callback)
    if not node._commonAction_guideAct then
        scaleMultiple   = scaleMultiple or 1.2
        local action    = A.cycle({
            {"delay", time or 3},
            {"fn", function()
                soundEffect:playEffectsfx14700011()
                if callback then
                    callback()
                end
            end},
            {"scaleto", 0.2, scaleSize * scaleMultiple},
            {"scaleto", 0.1, scaleSize},
            {"scaleto", 0.2, scaleSize * scaleMultiple},
            {"scaleto", 0.1, scaleSize},
        })
        node._commonAction_guideAct = action
        node._commonAction_iniScale = scaleSize
        node:runAction(node._commonAction_guideAct)
    end
end

-- 关闭缩放引导
function M:stopScaleGuide(node)
    if node._commonAction_guideAct then
        node:stopAction(node._commonAction_guideAct)
        node._commonAction_guideAct = nil
        node:scale(node._commonAction_iniScale)
    end
end

-- 当重新播报数字的回调
function M:onRePlayHandler()
	self:z(1000)
	-- 粒子
	local particle = CUtil:newParticle("particle/lszzs.plist"):to(self._parent, 1000):p(self:point()):scale(0.8)
	--获取语音的id
	self:line({
		{"delay", 0.35},
		{"fn", function ()
			self:playNumberVoice(self._num)
			self:playClickSoundEffect(self._num)
		end},
		{"scaleTo", 0.25, self._scale*1.3},
		{"scaleTo", 0.25, self._scale},
	})
end

--获取语音的id
function M:getNumVoiceId(num)
	local voiceIndex = totable({
		"v147158", "v147159", "v147160", "v147161", "v147162",
		"v147163", "v147164", "v147165", "v147167", "v147168",
	})[num]
	return voiceIndex
end

-- 播放数字台词
function M:playNumberVoice(num)
	soundVoice:stopNowSound()
	local voiceIndex = self:getNumVoiceId(num)
	self._numVoice = soundVoice["playEffect"..voiceIndex](soundVoice)
end

-- 播放连点音效
function M:playClickSoundEffect(num)
	local effectIndex = totable({
		"sfx14706001", "sfx14706002", "sfx14706003", "sfx14706004", "sfx14706005",
		"sfx14706006", "sfx14706031", "sfx14712004", "sfx14712005", "sfx14712006",
	})[num]

	soundEffect["playEffect"..effectIndex](soundEffect)
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	self:onClickSpot()
	return true
end

function M:onTouchMoved(x, y, touches)

end

function M:onTouchEnded(x, y, touches)
	
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M