

--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("Spots")
M.TAG       = "Spots"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
local Config = require("app.numline.Config")
local Spot = require("app.numline.node.Spot")

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._main       = params.main
	-- 物品ID
	self._itemId     = params.itemId
	-- 当前的数字
	self._curNum     = 1
	-- 数字点的集合
	self._numSpotTab = {}
	-- 点缩放倍数
	self._spotScale = 0.5
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
	-- 加载数字点集合
	self:loadSpots()
end

-- 图片渲染
function M:loadDisplay()
	self:display("num/line/item/item"..self._itemId.."/item.png"):opacity(0)
end

-- 加载点位集合
function M:loadSpots()
	local spotPosTab = Config.itemInfo[self._itemId].spotPosTab

	for i, v in pairs(spotPosTab) do
		local pos = spotPosTab[i]
		if i == 1 then
			self._firstSpot = self:createFirstSpot(pos)
			self._firstSpot:enterEffect()
		else
			-- 点位表首位是非数字的点，数字点从第二个开始
			local num = i - 1
			local numSpot = self:createNumSpot(num, pos)
			table.insert(self._numSpotTab, numSpot)
			numSpot:enterEffect()
		end
	end
end

-- 创建首个点
function M:createFirstSpot(pos)
	local firstSpot = Spot.new({
		itemId = self._itemId,
		num    = nil,
		scale  = self._spotScale,
		parent = self,
		main   = self._main,
	}):to(self):p(pos):scale(0)
	return firstSpot
end

-- 创建数字提示点
function M:createNumSpot(num, pos)
	local numSpot = Spot.new({
		itemId = self._itemId,
		num    = num,
		scale  = self._spotScale,
		parent = self,
		main   = self._main,
	}):to(self):p(pos):scale(0)
	return numSpot
end

-- 激活下一个待点击的点
function M:activeNextSpot()
	if self._curNum >= #self._numSpotTab then
		print("连线完成了")
		return 
	end
	self._curNum = self._curNum + 1
	local nextSpot = self._numSpotTab[self._curNum]
	nextSpot:showClickSpot()
end

-- 退场效果
function M:quitEffect()
	self:line({
		{"delay", 0.3},
		{"remove"}
	})

	for i, v in pairs(self:getChildren()) do
		v:line({
			{"easing", "backIn", {"scaleTo", 0.3, 0}},
		})
	end
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M