--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]

--[[!--

场景层类，定义层相关操作方法及逻辑实现。

-   定义场景层功能方法。

]]

----------------------
-- 类
----------------------
local M = classLayerTouch("Main")


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
local Config = require("app.numpuzzle.Config")     	 	-- 配置表
local Menu = require("app.numpuzzle.node.Menu")         -- 菜单栏
local Car = require("app.numpuzzle.node.Car")       	-- 载数字的车
local Number = require("app.numpuzzle.node.Number") 	-- 待拼的数字
local Outline = require("app.numpuzzle.node.Outline")   -- 虚线框
local Item = require("app.numpuzzle.node.Item")         -- 虚线框
local GuideFinger = require("app.common.GuideFinger")   -- 手指引导

-- [操作变量]
-- ..

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义视图实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
end

----------------------
-- 结点渲染
----------------------
--[[--

视图渲染，处理视图结点加载、事件绑定等相关操作
]]

function M:onRender()
	-- 加载渲染以外的其他操作
	self:loadRenderOtherwise()
	-- 加载背景图
	self:loadBg()
	-- 获取当前轮次的关卡组合表
	self:getCurLevelTab()
	-- 加载手指引导
	self:loadGuideFinger()
	-- 初始化数据
	self:initData()
	-- 加载菜单栏
	self:loadMenu()
	-- 加载数字关卡
	self:loadNumLevel()

	-- CUtil:loadSubtitle():to(self, 9999)
end

-- 加载其他
function M:loadRenderOtherwise()

end

function M:onEnterTransitionFinish()
	-- 播放背景音乐[bgm14709]拼数字
	sound:playbgm14709()
end

-- 	加载背景图
function M:loadBg()
	local bg = U.loadBg("num/puzzle/bg.png"):to(self)
end

-- 获取当前轮次的关卡组合表
function M:getCurLevelTab()
	if (not G_numpuzzle_turnIndex) or G_numpuzzle_turnIndex == 0 or G_numpuzzle_turnIndex >= #G_numpuzzleLevelTab then
		G_numpuzzle_turnIndex = 1
	else
		G_numpuzzle_turnIndex = G_numpuzzle_turnIndex + 1
	end
	-- 数字关卡组合表
	self._numTab = G_numpuzzleLevelTab[G_numpuzzle_turnIndex]
	-- 数字关卡索引
	self._numIndex = 1
	-- 初始化数字关卡ID
	self._curNum = self._numTab[self._numIndex]
end

-- 初始化数据
function M:initData()
	-- 运字车的ID
	self._carId     = 0
	-- 存放当前数字的实物节点
	self._itemTab = {}
	-- 物品ID表
	self._itemIdTab = {}
	T.copyTable(self._itemIdTab, Config.itemInfo.itemIdTab[self._curNum])
end

-- 加载菜单栏
function M:loadMenu(isExtend)
    local menu = Menu.new({
		main = self,
        num = self._curNum,
        isExtend = isExtend
	}):to(self, 400):p(90, 80)
	
	self._menu = menu
end

-- 加载数字关卡
function M:loadNumLevel()
	-- 虚线框出场后的回调(虚线框进场后，车再带着数字进场)
	local callback = function ()
		local car = self:createCar()
		local number = self:createNumber(car)
		car:enterEffect()
		number:enterEffect()
		self._car = car
		
		self._number = number
		self._number:openGuideOnCar()
	end
	self._outline = self:createOutline()
	self._outline:enterEffect(callback)
end

-- 进入下一数字关卡（正常按顺序操作）
function M:enterNextNumLevel()
	if self._numIndex >= #self._numTab then
		self:enterNextScene()
		return
	end

	self._numIndex = self._numIndex + 1
	-- 菜单栏手动切换形状后，则不按照预设的形状ID表顺序进行
    if self._isOperateByHand then
        if self._curNum <10 then
            self._curNum = self._curNum + 1
        else
            self._curNum = 1
        end
    else
        self._curNum = self._numTab[self._numIndex]
    end

	self._isShownFirstItem = nil
	self._menu:setBtnCurNum(self._curNum)
	self._menu:tickSelect(self._curNum)
	T.copyTable(self._itemIdTab, Config.itemInfo.itemIdTab[self._curNum])
	self._itemTab = {}
	self:loadNumLevel()
end

-- 进入下一个场景
function M:enterNextScene()
    local encourage = CUtil:loadEncourage(self, 3, function ()
    	G_addFinish("numpuzzle")
        SceneController:enterNextMoudleScene()
    end)
end

-- 退出当前关卡，并且进入指定数字关卡（菜单栏点击指定数字的操作）
function M:quitAndEnterNextNum(nextNum)
	soundEffect:stopNowSound()
	self:stopFingerGuide()
	-- 移除汽车
	if not tolua.isnull(self._car) then
		self._car:remove()
	end
	-- 移除虚线框
	if not tolua.isnull(self._outline) then
		self._outline:remove()
	end
	-- 移除实物
	for i,v in pairs(self._itemTab) do
		if not tolua.isnull(v) then
			v:remove()
		end
	end
	-- 移除数字
	if not tolua.isnull(self._number) then
		self._number:remove()
	end

	self._isShownFirstItem = nil
	-- 手动点击菜单栏数字的操作标识
	self._isOperateByHand = true
	self._curNum = nextNum
	self._menu:tickSelect(self._curNum)
	T.copyTable(self._itemIdTab, Config.itemInfo.itemIdTab[self._curNum])
	self._itemTab = {}
	self:loadNumLevel()
end

-- 创建虚线框
function M:createOutline()
	local pos = Config.outlineInfo.pos
	local scale = Config.outlineInfo.scale
	local outline = Outline.new({
		main  = self,
		num   = self._curNum,    -- 显示的数字
		scale = scale,           -- 进场后的显示倍数
		pos   = pos,
	}):to(self, 5):p(pos):scale(0)
	return outline
end

-- 创建载数字的汽车
function M:createCar()
	local carId = self:getRandomCarId()
	local scale = Config.carInfo[carId].scale
	local pos = Config.carInfo[carId].pos
	
	local car = Car.new({
		main  = self,
		carId = carId,
		scale = scale,
		pos   = pos,
	}):to(self, 20):p(pos.x+400, pos.y):scale(scale):flipX(true):unbindTouch()
	return car
end

-- 获取随机的汽车ID
function M:getRandomCarId()
	-- 可选运字车的ID表
	local carIdTab = {1, 2, 3}
	local carId = self._carId
	-- carId == 0 说明是第一次玩,那就随机选一个
	if carId == 0 then
		carId = T.shuffle(carIdTab)[1]   	
	else
		for i, v in pairs(carIdTab) do
			-- 遍历找出上一回用的车子,从可选表中移除该ID
			if carId == v then
				table.remove(carIdTab, i)
			end
		end
		-- 从可选表剩下的ID随机选一个
		carId = T.shuffle(carIdTab)[1]
	end
	self._carId = carId
	return carId
end

-- 创建数字
function M:createNumber(car)
	local scale = Config.numberInfo[car._carId].scale
	local moveScale = Config.numberInfo[car._carId].moveScale
	local pos = Config.numberInfo[car._carId].pos
	
	local number = Number.new({
		main  	  = self,
		num   	  = self._curNum,  -- 当前的数字
		car   	  = car,           -- 载数字的车节点
		scale 	  = scale,         -- 入场时缩放的倍数
		moveScale = moveScale,     -- 被拖动时的倍数
		pos	  	  = pos,           -- 初始位置的世界坐标
	}):to(self, 10):p(pos.x + 400, pos.y):scale(scale)
	
	car._number = number
	return number
end

-- 蹦出实物(实物出现)
function M:showItemJumpOut()
	if #self._itemIdTab <1 then return end
	self._number._numBone:clickAct()
	local itemId = T.shuffle(self._itemIdTab)[1]
	self._item = self:createItem(itemId)
	table.insert(self._itemTab, self._item)
	table.remove(self._itemIdTab, 1)

	if not self._isShownFirstItem then
		self:playFirstClickNumSoundEffect()
		self._item:jumpToLeft()
		-- 是否点击数字显示了第一个物品
		self._isShownFirstItem = true
	else
		-- 实物全部退场后的回调
		local callback = function ()
			self:enterNextNumLevel()
		end
		-- 物品和数字一起跳出去
		self._item:jumpToRight(self._number, callback)
		self:playSecondClickNumSoundEffect()
	end
end

-- 创建实物
function M:createItem(itemId)
	local scale = Config.itemInfo.scaleTab[self._curNum][itemId]
	local anchor = Config.itemInfo.anchorTab[self._curNum][itemId]
	local item = Item.new({
		main    = self,
		num     = self._curNum,    -- 数字
		itemId  = itemId,          -- 物品ID
		scale   = scale,           -- 进场后显示的倍数
	}):to(self, 100):p(480+X_OFFSET, 146+Y_OFFSET):scale(0):anchor(anchor)
	return item
end

-- 获取数字到虚线框到距离（世界坐标）
function M:getNumToOutlineDist(numPos)
	local outlinePos = self._outline:worldpoint()
	local dist = ccpDistance(numPos, outlinePos)
	return dist
end

-- 播放音效第一次点击屏幕上的数字
function M:playFirstClickNumSoundEffect()
	if self._clickEff then
		sound.stopSound(self._clickEff)
	end

    if self._clickEff then
        sound.stopSound(self._clickEff)
    end
	local effectIndex = totable({
		"sfx14713008", "sfx14713013", "sfx14713017", "sfx14713021", "sfx14713025",
        "sfx14713028", "sfx14713032", "sfx14713035", "sfx14713038", "sfx14713041",
	})[self._curNum]
	self._clickEff = soundEffect["playEffect"..effectIndex](soundEffect)
end

-- 播放语音第二次点击屏幕上的数字
function M:playSecondClickNumSoundEffect()
	if self._clickEff then
		sound.stopSound(self._clickEff)
	end

	local effectIndex = totable({
		"sfx14713009", "sfx14713043", "sfx14713018", "sfx14713022", "sfx14713026",
        "sfx14713029", "sfx14713033", "sfx14713036", "sfx14713039", "sfx14713042",
	})[self._curNum]
	self._clickEff = soundEffect["playEffect"..effectIndex](soundEffect)
end

-- 加载手指引导
function M:loadGuideFinger(pos1, pos2)
    if not self._finger then
        local finger    = GuideFinger.new({
            startPos    = pos1,
            endPos      = pos2,
        }):to(self, 10000):p(0, 0)
        self._finger = finger
    end
end

-- 播放手指拖动引导（跟随缩放引导的line动作，非cycle）
function M:playFingerMoveGuide()
	local startPos = ccp(self._number:px(), self._number:py() + self._number:ch()/2*self._number._scale)
	local endPos = self._outline:point()
	self._finger:playFingerMoveGuide(startPos, endPos)
end

-- 播放手指点击引导（跟随待机引导的line动作，非cycle）
function M:playFingerClickGuide()
	local startPos = self._number:point()
	self._finger:playFingerClickGuide(startPos)
end

-- 停止手指引导(拖动/点击通用)
function M:stopFingerGuide()
	self._finger:closeHandGuide()
end

----------------------
-- 结点析构
----------------------
--[[--

视图析构，处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end

function M:onExitTransitionStart()

end

return M