--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("Car")
M.TAG       = "Car"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
	
	self._main 	  = params.main
	-- 运字车的Id
	self._carId   = params.carId
	-- 缩放倍数
	self._scale   = params.scale
	-- 进场后的坐标
	self._pos = params.pos
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end

-- 图片渲染
function M:loadDisplay()
	self:display("num/puzzle/car/car"..self._carId..".png")
end

-- 车子进场效果
function M:enterEffect(callback)
	self:playCarEnterSoundEffect()
	self:union({
		{"easing", "backOut", { "moveTo", 0.3, ccp(self._pos.x, self:point().y)}},
		{ "fn", function()
			self:playCarShake1(callback)
		end }
	})
end

-- 播放车子驶入音效
function M:playCarEnterSoundEffect()
	if self._carId == 1 then
		-- 播放音效[sfx14713005]火车驶入
		soundEffect:playEffectsfx14713005()
	elseif self._carId == 2 then
		-- 播放音效[sfx14713003]货车驶入
		soundEffect:playEffectsfx14713003()
	elseif self._carId == 3 then
		-- 播放音效[sfx14713004]轿车驶入
		soundEffect:playEffectsfx14713004()
	end
end

-- 播放车子刹车抖动
function M:playCarShake1(callback)
	CUtil:changeAnchorP(self, ccp(0.5, 0))
    self:line({
        { "scaleTo", 0.1, self:scaleX() * 1.1, self:scaleY() * 0.9 },
		{"fn", function ()
			self._number:playNumJumpAct()
		end},
        { "scaleTo", 0.2, self:scaleX() * 0.9, self:scaleY() * 1.1 },
        { "scaleTo", 0.1, self:scale() },
        { "fn", function()
			CUtil:changeAnchorP(self, ccp(0.5, 0.5))
			if callback then
				callback()
			end
        end }
    })
end

-- 播放数字装回车子抖动
function M:playCarShake2(callback)
	CUtil:changeAnchorP(self, ccp(0.5, 0))
	self._number:playNumJumpAct()
    self:line({
        { "scaleTo", 0.1, self:scaleX() * 1.1, self:scaleY() * 0.9 },
        { "scaleTo", 0.2, self:scaleX() * 0.9, self:scaleY() * 1.1 },
        { "scaleTo", 0.1, self:scale() },
        { "fn", function()
            CUtil:changeAnchorP(self, ccp(0.5, 0.5))
			if callback then
				callback()
			end
        end }
    })
end

-- 车子退场效果
function M:quitEffect(callback)
	self:z(100)
	self:line({
		{"easing", "backin", {"moveBy", 0.3, ccp(400, 0)}},
		{"fn",function ()
			if callback then
				callback()
			end
		end},
		{"remove"}
	})
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M