--[[
Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类
----------------------
local M    = classSpriteTouch("Item")
M.TAG    = "Item"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)

    self._main 		= params.main
	-- 数字
	self._num 		= params.num
	-- 象形物品ID
	self._itemId    = params.itemId
	-- 进场后显示的倍数
	self._scale		= params.scale
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()
end

-- 图片渲染
function M:loadDisplay()
    self:display("num/puzzle/item/item"..self._num.."_"..self._itemId..".png")
end

-- 物品往左蹦出来效果
function M:jumpToLeft()
	self._main._number._touchLock = false
	self._main._number:openGuideOnMain()
	self:playItemVoice()
	-- 播放音效[sfx14713006]点击道具蹦出1
	soundEffect:playEffectsfx14713006()
	local devX
	if self._num == 10 then
		devX = 260 + X_OFFSET
	else
		devX = 230 + X_OFFSET
	end
	self:line({
		A.union({
			{"scaleTo", 0.3, self._scale},
			CUtil:jumpToLimitHeight(self, 0.3, ccp(self:px() - devX, self:py()), 100, 1),
		}),
		{"fn", function ()
			--粒子
			local particle = CUtil:newParticle("particle/psztc.plist"):to(self, 100):p(self:cw()/2, self:ch()/2)
			CAction:feedback2(self, 1)
		end},
		{"delay", 0.5},
		-- 物品摇晃
		{"fn", function ()
			self:playItemSwaySoundEffect()
		end},
		{"rotateBy", 0.3, -20},
		{"rotateBy", 0.6, 40},
		{"rotateBy", 0.6, -40},
		{"rotateBy", 0.6, 40},
		{"rotateBy", 0.3, -20},
		{"fn", function ()
			CAction:feedback2(self,0.5)
		end},
		{"delay", 1},
		{"fn", function ()
			self:quitToLeft()
		end}
	})
end

-- 物品往右蹦出来效果
function M:jumpToRight(node, callback)
	self:playItemVoice()
	-- 播放音效[sfx14713007]点击道具蹦出2
	soundEffect:playEffectsfx14713007()

	local devX
	if self._num == 10 then
		devX = 260 + X_OFFSET
	else
		devX = 230 + X_OFFSET
	end

	self:line({
		A.union({
			{"scaleTo", 0.3, self._scale},
			CUtil:jumpToLimitHeight(self, 0.3, ccp(self:px() + devX, self:py()), 100, 1),
		}),
		{"fn", function ()
			--粒子
			local particle = CUtil:newParticle("particle/psztc.plist"):to(self, 100):p(self:cw()/2, self:ch()/2)
			CAction:feedback2(self, 1)
		end},
		{"delay", 0.5},
		-- 物品摇晃
		{"fn", function ()
			self:playItemSwaySoundEffect()
		end},
		{"rotateBy", 0.3, 20},
		{"rotateBy", 0.6, -40},
		{"rotateBy", 0.6, 40},
		{"rotateBy", 0.6, -40},
		{"rotateBy", 0.3, 20},
		{"fn", function ()
			CAction:feedback2(self,0.5)
		end},
		{"delay", 1},
		{"fn", function ()
			self:quitToRight(node, callback)
		end}
	})
end

-- 物品从屏幕左边蹦出去（退场）
function M:quitToLeft(callback)
	-- 播放音效[sfx14713011]跳着离场
	self._jumpEffect = soundEffect:playEffectsfx14713011()
	self:line({
		A.cycle({
			A.union({
				{"jumpBy", 0.4, cc.p(-150, 0), 50, 1},
				A.line({
					{"scaleTo", 0.1, 0.9*self._scale, 1.15*self._scale},
					{"scaleTo", 0.2, 1.1*self._scale, 0.95*self._scale},
					{"scaleTo", 0.1, 1*self._scale},
				})
			})
		}, 4),
		{"fn", function()
			if self._jumpEffect then
				sound.stopSound(self._jumpEffect)
			end

			if callback then
				callback()
			end
		end},
		{"remove"},
	})
end

-- 物品从屏幕右边蹦出去（退场）
function M:quitToRight(node, callback)
	soundEffect:playEffectsfx14713016()
	local particle = P.new("particle/bingganjl.plist"):to(self._main, 999):p(V.w_2, 20)
	particle:setAutoRemoveOnFinish(true)
	for i=2,4 do
		local particle = P.new("particle/bingganjl.plist"):to(self._main, 999):p(V.w_2, 20)
		particle:setTexture(R.loadTexture("particle/bingganjl"..i..".png"))
		particle:setAutoRemoveOnFinish(true)
	end

	if self._jumpEffect then
		sound.stopSound(self._jumpEffect)
	end
	-- 播放音效[sfx14713011]跳着离场
	self._jumpEffect = soundEffect:playEffectsfx14713011()
	self:line({
		A.cycle({
			A.union({
				{"jumpBy", 0.4, cc.p(150, 0), 50, 1},
				A.line({
					{"scaleTo", 0.1, 0.9*self._scale, 1.15*self._scale},
					{"scaleTo", 0.2, 1.1*self._scale, 0.95*self._scale},
					{"scaleTo", 0.1, 1*self._scale},
				})
			})
		}, 4),
		{"delay", 0.5},
		{"fn", function()
			if self._jumpEffect then
				sound.stopSound(self._jumpEffect)
			end		
			if callback then callback() end
		end},
		{"remove"},
	})
	-- 数字跟随跳跃退场
	node:line({
		A.cycle({
			A.union({
				{"jumpBy", 0.4, cc.p(150, 0), 50, 1},
				A.line({
					{"scaleTo", 0.1, 0.9*node._moveScale, 1.15*node._moveScale},
					{"scaleTo", 0.2, 1.1*node._moveScale, 0.95*node._moveScale},
					{"scaleTo", 0.1, 1*node._moveScale},
				})
			})
		}, 4),
		{"remove"},
	})
end

-- 播放物品语音
function M:playItemVoice()
	soundVoice:stopNowSound()
	local voiceIndex = totable({
		{"v147171", "v147172"},
		{"v147173", "v147174"},
		{"v147176", "v147175"},
		{"v147178", "v147177"},
		{"v147179", "v147180"},
		{"v147182", "v147181"},
		{"v147184", "v147183"},
		{"v147186", "v147185"},
		{"v147188", "v147187"},
		{"v147190", "v147189"},
	})[self._num][self._itemId]

	self._itemVoice = soundVoice["playEffect"..voiceIndex](soundVoice)
end

-- 播放物品摇晃音效
function M:playItemSwaySoundEffect()
	local effectIndex = totable({
		{"sfx14713012", "sfx14713010"},
		{"sfx14713014", "sfx14713015"},
		{"sfx14713019", "sfx14713020"},
		{"sfx14713024", "sfx14713023"},
		{"sfx14713027", "sfx14713014"},
		{"sfx14713031", "sfx14713030"},
		{"sfx14713012", "sfx14713034"},
		{"sfx14713012", "sfx14713037"},
		{"sfx14713044", "sfx14713040"},
		{"sfx14713010", "sfx14713010"},
	})[self._num][self._itemId]

	self._itemEffect = soundEffect["playEffect"..effectIndex](soundEffect)
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end


return M