--[[
Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类
----------------------
local M    = classSpriteTouch("Number")
M.TAG    = "Number"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
local Config = require("app.numpuzzle.Config")

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)

    self._main 		= params.main
	-- 车节点
	self._car		= params.car
	-- 数字
	self._num 		= params.num
	-- 进场后显示的倍数
	self._scale		= params.scale
	-- 被拖动时的倍数
	self._moveScale = params.moveScale
	-- 初始位置的世界坐标
	self._pos  = params.pos
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()
end

-- 图片渲染
function M:loadDisplay()
    self:display("num/puzzle/number/"..self._num..".png"):scale(0.25*1.8)

	-- 扩大触控范围
	self:bindTouch({
		rectFactorX = 1.5,
		rectFactorY = 1.5
	})
end

-- 	加载骨骼
function M:loadBone()
	local scale = Config.boneInfo.scale
	local pos = Config.boneInfo.posTab[self._num]

	local numBone = require("app.numwrite.numbone.NumBone"..self._num).new({
		layer  = self._main,
		parant = self,
		number    = self._num,
	}):to(self):p(pos):scale(scale):unbindTouch():hide()

	self._numBone = numBone
	numBone:breathAct1()
end

-- 进场效果
function M:enterEffect()
	self:line({
		{"easing", "backOut", { "moveTo", 0.3, self._pos}},
	})
end

-- 退场效果
function M:quitEffect()
	self:line({
		{"easing", "backin", {"moveBy", 0.3, ccp(400, 0)}},
		{"remove"}
	})
end

-- 检查数字是否靠近当前虚线框
function M:checkIsNearOutline()
    self._touchLock = true

    local dist = self._main:getNumToOutlineDist(self:worldpoint())
    if dist <= 100 then
        self:onSuccessHandler()
    else
        self:onFailHandler()
    end
end

-- 拼放正确的回调
function M:onSuccessHandler()
	self:loadBone()
	self:playNumberVoice(self._num)
	-- 播放音效[sfx14707004]数字拼入（复用）
	soundEffect:playEffectsfx14707004()
	local pos = self._main._outline:worldpoint()
	self:line({
		A.union({
			{"moveTo", 0.25, ccp(pos.x, pos.y+35)},
			{"scaleTo", 0.25, self._moveScale * 1.5}
		}),
		A.union({
			{"moveTo", 0.02, pos},
			{"scaleTo", 0.02, self._moveScale}
		}),
		{"fn", function ()
			--粒子
			local particle = CUtil:newParticle("particle/psz.plist"):to(self, 100):p(self:cw()/2, self:ch()/2)

			local callback = function ()
				self:shapeShaking()
			end
			self._main._outline:quitEffect(callback)
		end}
	})
end

-- 形状晃动再变为骨骼
function M:shapeShaking()
	self:line({
		{"moveBy", 0.1, ccp(self:cw()/5, 0)},
		{"moveBy", 0.2, ccp(-2*self:cw()/5, 0)},
		{"moveBy", 0.1, ccp(self:cw()/5, 0)},
		{"fn", function ()
			self._numBone:show()
			self:opacity(0)
			self:openGuideOnMain()
			-- 数字是否离开车在mainlayer上
			self._isOnMain = true
			self._touchLock = false
			
			self:line({
				{"delay", 0.3}, 
				{"fn", function ()
					self._car:quitEffect()
				end}
			})
		end}
	})
end

-- 数字跳起动作
function M:playNumJumpAct(callback)
	local time = 1
	self:line({
		A.union({
			A.line({
				{"scaleTo", 0.1 * time, self._scale*0.8, self._scale*1.2},
				{"scaleTo", 0.1 * time, self._scale, self._scale},
			}),
			A.line({
				{"jumpBy", 0.25 * time, cc.p(0, 0), 60, 1},
				{"jumpBy", 0.15 * time, cc.p(0, 0), 20, 1},
			}),
			{"fn", function ()
				if callback then
					callback()
				end
			end}
		}),
	})
end

-- 播放数字台词
function M:playNumberVoice(num)
	soundVoice:stopNowSound()
	local voiceIndex = totable({
		"v147158", "v147159", "v147160", "v147161", "v147162",
		"v147163", "v147164", "v147165", "v147167", "v147168",
	})[num]

	self._numVoice = soundVoice["playEffect"..voiceIndex](soundVoice)
end

-- 拼放错误的回调(回到原来位置)
function M:onFailHandler()
	-- 播放音效[sfx14700002]错误提示
	soundEffect:playEffectsfx14700002()

	self:line({
		{"rotateBy", 0.05, 15},
		{"rotateBy", 0.1, -30},
		{"rotateBy", 0.05, 15},
		{"fn", function ()
			-- 播放音效[sfx14700003]错误弹回
			soundEffect:playEffectsfx14700003()
		end},
		A.union({
			CUtil:jumpToLimitHeight(self, 0.3, self._pos, 150, 1),
			{"scaleTo", 0.3, self._scale}
		}),
		{"fn", function ()
			self._touchLock = false
			self:z(1)
			self._car:playCarShake2()
			self:openGuideOnCar()
		end}
	})
end

-- 点击数字缩放效果
function M:playClickScale()
	self:line({
		{"scaleTo", 0.1, self._moveScale*0.85},
		{"scaleTo", 0.2, self._moveScale*1.15},
		{"scaleTo", 0.1, self._moveScale},
	})
end

-- 开启引导(数字在车上时)
function M:openGuideOnCar()
	if self._isOpenGuideOnCar then return end
	self._isOpenGuideOnCar = true
	self:openScaleGuide(self, self._scale, nil, 4, function ()
		self:playGuideVoice1()
		self._main:playFingerMoveGuide()
	end)
end

-- 播放引导台词
function M:playGuideVoice1()
	soundVoice:stopNowSound()
	-- 播放配音[v147170]拼一拼
	soundVoice:playEffectv147170()
end

-- 关闭引导(数字在车上时)
function M:stopGuideOnCar()
	if self._isOpenGuideOnCar then
		self._isOpenGuideOnCar = false
		self:stopScaleGuide(self)
		self._main:stopFingerGuide()
	end
end

-- 开启引导(数字在mainLayer时)
function M:openGuideOnMain()
	print("开启引导")
	self:openStandbyGuide(self, 4, function ()
		self:playGuideVoice2()
		self._main:playFingerClickGuide()
	end)
end

-- 播放引导台词
function M:playGuideVoice2()
	soundVoice:stopNowSound()
	-- 播放配音[v147191]点一点
	soundVoice:playEffectv147191()
end

-- 关闭引导(数字在mainLayer时)
function M:stopGuideOnMain()
	self:stopStandbyGuide(self)
	print("关闭引导")
	self._main:stopFingerGuide()
end

-- 开启缩放引导 #精灵，缩放倍数
function M:openScaleGuide(node, scaleSize, scaleMultiple, time, callback)
    if not node._commonAction_guideAct then
        scaleMultiple   = scaleMultiple or 1.2
        local action    = A.cycle({
            {"delay", time or 3},
            {"fn", function()
				CUtil:changeAnchorP(self, ccp(0.5, 0))
                soundEffect:playEffectsfx14700011()
                if callback then
                    callback()
                end
            end},
            {"scaleto", 0.2, scaleSize * scaleMultiple},
            {"scaleto", 0.1, scaleSize},
            {"scaleto", 0.2, scaleSize * scaleMultiple},
            {"scaleto", 0.1, scaleSize},
			{"fn", function ()
				CUtil:changeAnchorP(self, ccp(0.5, 0.5))
			end}
        })
        node._commonAction_guideAct = action
        node._commonAction_iniScale = scaleSize
        node:runAction(node._commonAction_guideAct)
    end
end

-- 关闭缩放引导
function M:stopScaleGuide(node)
    if node._commonAction_guideAct then
		CUtil:changeAnchorP(self, ccp(0.5, 0.5))
        node:stopAction(node._commonAction_guideAct)
        node._commonAction_guideAct = nil
        node:scale(node._commonAction_iniScale)
    end
end

-- 开启数字骨骼的待机引导
function M:openStandbyGuide(node, time, callback)
    if not node._commonAction_guideAct then
        local action    = A.cycle({
            {"delay", time or 3},
            {"fn", function()
                if callback then
                    callback()
                end
				node._numBone:standbyAct()
            end}
        })
        node._commonAction_guideAct = action
        node:runAction(node._commonAction_guideAct)
    end
end

-- 关闭数字骨骼的待机引导
function M:stopStandbyGuide(node)
    if node._commonAction_guideAct then
		node:stopAction(node._commonAction_guideAct)
        node._commonAction_guideAct = nil
		node._numBone:breathAct1()
    end
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)
    if self._touchLock then return end

	self._touchLock = true
	self._touchMoveLock = true
	
	soundVoice:stopNowSound()
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()
	self._main._menu:barChange(false)
	
	-- 数字已经拼好
	if self._isOnMain then
		self:stopGuideOnMain()
		self:playClickScale()
		self._main:showItemJumpOut()
	else
		--数字还在车上
		self._touchMoveLock = false

		self:line({
			{"delay", 0.02},
			{"fn", function ()
				self:p(x - 30, y + 30):scale(self._moveScale):z(50)
			end}
		})
		self:stopGuideOnCar()
	end

    return true
end

function M:onTouchMoved(x, y, touches)
	if self._touchMoveLock then return end
	self:p(x - 30, y + 30)
	CUtil:limitRectMove({node = self})
	return true
end

function M:onTouchEnded(x, y, touches)
	if self._touchMoveLock or self._isOnMain then return end
	self:checkIsNearOutline()
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end


return M