local M = {}

-- 获取圆上等距离的点
function M:getCirclePoints(params)
    local pointTab      = {}
    local oriPos        = ifnil(params.oriPos, cc.p(0, 0))      -- 圆心
    local radius        = ifnil(params.radius, 50)              -- 半径
    local startAngle    = ifnil(params.startAngle, 0)           -- 起始角度
    local endAngle      = ifnil(params.endAngle, 360)           -- 结束角度
    local intervalDis   = ifnil(params.intervalDis, 1)          -- 距离间隔
    local isReverse     = ifnil(params.isReverse, false)        -- 是否逆时针
    local ryFactor      = ifnil(params.ryFactor, 1)             -- Y轴半径的修正系数
    -- 角度差
    local deltaAngle    = math.abs(endAngle - startAngle)
    -- 弧长
    local circleLen     = 2 * radius * math.pi * deltaAngle / 360
    -- 总计点的数量
    local count         = math.floor(circleLen / intervalDis)
    -- 每两点之间间隔角度
    local intervalAngle = isReverse and -deltaAngle / count or deltaAngle / count
    -- 计算坐标点
    for i=1,count do
        local angle = startAngle + intervalAngle * i
        local ax = radius * math.cos(math.rad(angle))
        local ay = radius * math.sin(math.rad(angle)) * ryFactor
        local bx = ax + oriPos.x
        local by = ay + oriPos.y
        table.insert(pointTab, ccp(bx, by))
    end

    if params.showNode then
        for i,v in ipairs(pointTab) do
            local node = D.img("common/help.png"):to(params.showNode, 2000):p(v)
        end
    end
    return pointTab
end

-- 获取直线上等距离的点
function M:getLinePoints(params)
    local startPoint    = ifnil(params.startPoint, cc.p(100, 100))
    local endPoint      = ifnil(params.endPoint, cc.p(0, 0))
    local intervalDis   = ifnil(params.intervalDis, 1)

    local distance      = ccpDistance(startPoint, endPoint)
    local count         = distance / intervalDis
    local deltaX        = endPoint.x - startPoint.x
    local deltaY        = endPoint.y - startPoint.y
    local aTable        = {}
    for i = 1, count do
        local delta = i / count
        local posX  = startPoint.x + delta * deltaX
        local posY  = startPoint.y + delta * deltaY
        local point = ccp(posX, posY)
        table.insert(aTable, point)
    end
    if params.showNode then
        for i,v in ipairs(aTable) do
            local node = D.img("common/help.png"):to(params.showNode, 2000):p(v)
        end
    end
    return aTable
end

-- 合并向量表
function M:mergeVectorTable(tables)
    local allTable = {}
    for i1,v1 in ipairs(tables) do
        for i2,v2 in ipairs(v1) do
            table.insert(allTable, v2)
        end
    end

    return allTable
end

-- 数字一轨迹点
function M:calculatePoint1(node)
    local tab1 = self:getLinePoints({
        showNode    = node,
        startPoint  = cc.p(31, 237),
        endPoint    = cc.p(31, 20),
    })
    return {tab1}
end

-- 数字二轨迹点
function M:calculatePoint2(node)
    local oriPos    = cc.p(96, 210)
    -- local node1  = D.img("common/help.png"):to(node, 9999):p(oriPos):bindTouchLocate()
    -- CUtil:setRotate(node1)

    -- 圆弧终点角度
    local angle     = -43
    -- 第一部分圆弧
    local tab1 = self:getCirclePoints({
        oriPos      = oriPos,
        startAngle  = 200,
        endAngle    = angle,
        radius      = 75,
        intervalDis = 1,
        showNode    = node,
        isReverse   = true
    })

    -- 衔接圆弧的直线，角度与圆弧终点相同
    local pos1  = tab1[#tab1]
    local rot   = angle - 90
    local lenth = 190
    local px2   = pos1.x + lenth * math.cos(math.rad(rot))
    local py2   = pos1.y + lenth * math.sin(math.rad(rot))

    local tab2 = self:getLinePoints({
        showNode    = node,
        startPoint  = tab1[#tab1],
        endPoint    = cc.p(px2, py2),
    })

    local tab3 = self:getLinePoints({
        showNode    = node,
        startPoint  = tab2[#tab2],
        endPoint    = cc.p(184, tab2[#tab2].y),
    })

    -- 数字2延长线
    local tab4 = self:getLinePoints({
        showNode    = node,
        startPoint  = tab3[#tab3],
        endPoint    = cc.p(tab3[#tab3].x + 50, tab3[#tab3].y),
    })
    return {tab1, tab2, tab3, tab4}
end


-- 数字三轨迹点
function M:calculatePoint3(node)
    -- local node1  = D.img("common/help.png"):to(node, 9999):p(oriPos):bindTouchLocate()
    -- CUtil:setRotate(node1)
    local oriPos1   = cc.p(109, 242)
    local radius1   = 64
    local radius2   = 76
    local oriPos2   = cc.p(oriPos1.x, oriPos1.y - radius1 - radius2)

    -- 第一部分圆弧
    local tab1 = self:getCirclePoints({
        oriPos      = oriPos1,
        startAngle  = 182,
        endAngle    = -90,
        radius      = radius1,
        intervalDis = 1,
        showNode    = node,
        isReverse   = true
    })

    local tab2 = self:getLinePoints({
        showNode    = node,
        startPoint  = tab1[#tab1],
        endPoint    = cc.p(tab1[#tab1].x - 10, tab1[#tab1].y),
    })

    local tab3 = self:getLinePoints({
        showNode    = node,
        startPoint  = tab2[#tab2],
        endPoint    = tab1[#tab1],
    })

    local tab4 = self:getCirclePoints({
        oriPos      = oriPos2,
        startAngle  = 90,
        endAngle    = -182,
        radius      = radius2,
        intervalDis = 1,
        showNode    = node,
        isReverse   = true
    })
    return {tab1, tab2, tab3, tab4}
end

-- 数字四轨迹点
function M:calculatePoint4(node)

    -- 第一笔斜的直线
    local starPos  = cc.p(172, 350)
    local rot       = 32
    local lenth     = 276
    local px2       = starPos.x - lenth * math.sin(math.rad(rot))
    local py2       = starPos.y - lenth * math.cos(math.rad(rot))

    local tab1 = self:getLinePoints({
        showNode    = node,
        startPoint  = starPos,
        endPoint    = cc.p(px2, py2),
    })

    local tab2 = self:getLinePoints({
        showNode    = node,
        startPoint  = tab1[#tab1],
        endPoint    = cc.p(240, tab1[#tab1].y),
    })

    local tab3 = self:getLinePoints({
        showNode    = node,
        startPoint  = cc.p(196, starPos.y),
        endPoint    = cc.p(196, 24),
    })

    return {tab1, tab2, tab3}
end

-- 数字五轨迹点
function M:calculatePoint5(node)
    if node then
        local node1     = D.img("common/help.png"):to(node, 9999):p(109, 242):bindTouchLocate()
        CUtil:setRotate(node1)
    end

    -- 下面圆弧部分
    local tab2 = self:getCirclePoints({
        oriPos      = cc.p(108, 113),
        startAngle  = 124,
        endAngle    = -152,
        radius      = 87,
        intervalDis = 1,
        showNode    = node,
        isReverse   = true
    })

    -- 第一笔斜的直线,以圆弧的起点为终点
    local endPos  = tab2[1]
    local rot     = 80
    local lenth   = 126
    local px2     = endPos.x + lenth * math.cos(math.rad(rot))
    local py2     = endPos.y + lenth * math.sin(math.rad(rot))
    local tab1    = self:getLinePoints({
        showNode    = node,
        startPoint  = cc.p(px2, py2),
        endPoint    = endPos,
    })

    local tab3 = self:getLinePoints({
        showNode    = node,
        startPoint  = tab1[1],
        endPoint    = cc.p(tab1[1].x + 110, tab1[1].y),
    })

    return {tab1, tab2, tab3}
end

-- 数字六轨迹点
function M:calculatePoint6(node)
    -- local oriPos    = cc.p(109, 242)
    -- local node1     = D.img("common/help.png"):to(node, 9999):p(oriPos):bindTouchLocate()
    -- CUtil:setRotate(node1)

    -- 大圆弧与小圆弧衔接处的角
    local sRot      = -170
    -- 6字下方小圆
    local tab2 = self:getCirclePoints({
        oriPos      = cc.p(112, 113),
        startAngle  = sRot,
        endAngle    = 120,
        radius      = 80,
        intervalDis = 1,
        showNode    = node,
        isReverse   = false
    })

    -- 上方大圆结束点，即下方小圆起始点
    local endPos    = tab2[1]
    -- 依据已知结束点、衔接处角度，及预设半径计算得圆心坐标
    local radius    = 200
    local rx    = endPos.x - math.cos(math.rad(sRot)) * radius
    local ry    = endPos.y - math.sin(math.rad(sRot)) * radius

    -- 第一部分圆弧
    local tab1 = self:getCirclePoints({
        oriPos      = cc.p(rx, ry),
        startAngle  = 110,
        endAngle    = 360 + sRot,
        radius      = radius,
        intervalDis = 1,
        showNode    = node,
        isReverse   = false
    })

    return {tab1, tab2}
end


-- 数字七轨迹点
function M:calculatePoint7(node)
    -- local oriPos    = cc.p(109, 242)
    -- local node1     = D.img("common/help.png"):to(node, 9999):p(oriPos):bindTouchLocate()
    -- CUtil:setRotate(node1)

    -- 交点
    local point = cc.p(196, 304)

    local tab1 = self:getLinePoints({
        showNode    = node,
        startPoint  = cc.p(30, point.y),
        endPoint    = point,
    })

    -- 第一笔斜的直线
    local rot   = 25
    local lenth = 304
    local px2   = point.x - lenth * math.sin(math.rad(rot))
    local py2   = point.y - lenth * math.cos(math.rad(rot))
    local tab2 = self:getLinePoints({
        showNode    = node,
        startPoint  = point,
        endPoint    = cc.p(px2, py2),
    })

    return {tab1, tab2, tab3, tab4}
end

-- 数字八轨迹点
function M:calculatePoint8(node)
    -- local oriPos    = cc.p(109, 242)
    -- local node1     = D.img("common/help.png"):to(node, 9999):p(oriPos):bindTouchLocate()
    -- CUtil:setRotate(node1)

    -- 8的上半圆
    local radius1   = 65
    local oriPos1   = cc.p(118, 256)
    -- 8的下半圆
    local radius2   = 80
    local oriPos2   = cc.p(oriPos1.x, oriPos1.y - radius1 - radius2)

    -- 第一部分圆弧
    local tab1 = self:getCirclePoints({
        oriPos      = oriPos1,
        startAngle  = 0,
        endAngle    = 270,
        radius      = radius1,
        intervalDis = 1,
        showNode    = node,
        isReverse   = false
    })

    -- 第一部分圆弧
    local tab2 = self:getCirclePoints({
        oriPos      = oriPos2,
        startAngle  = 90,
        endAngle    = -270,
        radius      = radius2,
        intervalDis = 1,
        showNode    = node,
        isReverse   = true
    })

    -- 第一部分圆弧
    local tab3 = self:getCirclePoints({
        oriPos      = oriPos1,
        startAngle  = -90,
        endAngle    = 0,
        radius      = radius1,
        intervalDis = 1,
        showNode    = node,
        isReverse   = false
    })

    return {tab1, tab2, tab3}
end

-- 数字九轨迹点
function M:calculatePoint9(node)
    -- 大圆弧与小圆弧衔接处的角
    local sRot      = 10
    -- 9字上方小圆
    local tab1 = self:getCirclePoints({
        oriPos      = cc.p(112, 228),
        startAngle  = sRot,
        endAngle    = -350,
        radius      = 80,
        intervalDis = 1,
        showNode    = node,
        isReverse   = false
    })

    -- 下方大圆开始点，即上方小圆结束点
    local startPos  = tab1[#tab1]
    -- 依据已知结束点、衔接处角度，及预设半径计算得圆心坐标
    local radius    = 200
    local rx    = startPos.x - math.cos(math.rad(sRot)) * radius
    local ry    = startPos.y - math.sin(math.rad(sRot)) * radius

    -- 第一部分圆弧
    local tab2 = self:getCirclePoints({
        oriPos      = cc.p(rx, ry),
        startAngle  = sRot,
        endAngle    = -70,
        radius      = radius,
        intervalDis = 1,
        showNode    = node,
        isReverse   = true
    })

    for i=1,16 do
        table.remove(tab1)
        table.remove(tab2, 1)
    end

    return {tab1, tab2}
end

-- 数字十轨迹点
function M:calculatePoint10(node)
    -- local oriPos    = cc.p(109, 242)
    -- local node1     = D.img("common/help.png"):to(node, 9999):p(oriPos):bindTouchLocate()
    -- CUtil:setRotate(node1)


    local tab1 = self:getLinePoints({
        showNode    = node,
        startPoint  = cc.p(26, 250),
        endPoint    = cc.p(26, 30),
    })


    -- 右边0
    local tab2 = self:getCirclePoints({
        oriPos      = cc.p(200, 140),
        startAngle  = 90,
        endAngle    = -270,
        radius      = 75,
        intervalDis = 1,
        showNode    = node,
        isReverse   = false,
        ryFactor    = 1.58
    })

    -- 插值算法
    local allDis    = 0
    tab2[1].dis     = allDis
    for i=2, #tab2 do
        local dis   = PT.d(tab2[i-1], tab2[i])
        allDis      = allDis + dis
        tab2[i].dis = allDis
    end
    local newTab = {}
    for i,v in ipairs(tab2) do
        table.insert(newTab, v)
        if #newTab < v.dis then
            table.insert(newTab, v)
        end
    end
    -- print(#tab2, allDis, #newTab)
    -- for i,v in ipairs(newTab) do
    --     print(v.x, v.y) 
    -- end

    return {tab1, newTab}
end



-- -- 获取椭圆线上点
-- function M:getPointPosOfShape(params) 
--  local startAngle    = ifnil(params.startAngle, 0) 
--  local endAngle      = ifnil(params.endAngle, 360)
--  local clockwise     = ifnil(params.clockwise, true)
--  local intervalAngle = ifnil(params.intervalAngle, 5)                -- 间隔角度(暂不支持传参) 
--  local oriPos        = ifnil(params.oriPos, ccp(0, 0))       -- 原始位置 
--  local aValue        = ifnil(params.aValue, 50)                      -- A线段长度 
--  local bValue        = ifnil(params.bValue, aValue)                      -- B线段长度 
--  local inctAng       = ifnil(params.InclinationAngle, 0)             -- 倾斜角 
--  local pointTab      = {}                                            -- 返回点的位置集合

--  local count         = 1
--  local deltaAngle    = math.abs(startAngle - endAngle)
--  if not clockwise then count = -1 end

--  for i = 0, deltaAngle do
--      local rad = count * i % intervalAngle
--      if rad == 0 then
--          local ax = aValue * math.sin( math.rad(startAngle + count * i) )
--          local ay = bValue * math.cos( math.rad(startAngle + count * i) )
--          local bx =     ax * math.cos( math.rad(inctAng) ) - ay * math.sin( math.rad(inctAng) ) + oriPos.x 
--          local by =     ax * math.sin( math.rad(inctAng) ) + ay * math.cos( math.rad(inctAng) ) + oriPos.y
--          table.insert(pointTab, ccp(bx, by)) 
--      end
--  end

--  return pointTab 
-- end


-- 计算2点的斜度
function M:calculateSkewByTwoPoint(pointA, pointB)
    local skew   = nil
    local deltaX = pointB.x - pointA.x
    local deltaY = pointB.y - pointA.y
    if deltaX ~= 0 and deltaY ~= 0 then
        skew = deltaY / deltaX
    end
    return skew
end

return M