--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-19

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字10在门内的礼物盒
----------------------
local M     = classSprite("N10Box")
M.TAG       = "N10Box"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)

    self._layer     = params.layer

    self._id        = params.id or 1

    self._door      = params.door
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
    self:display("num/write/door/n10box/b"..self._id..".png")
end

function M:onEnterTransitionFinish()
    self._iniPoint = self:point()
end

-- 变成玩具
function M:transformToToy()
    self._isClicked = true
    -- 完成数+1
    self._door._finishCount = self._door._finishCount + 1
    -- 记录下当前完成数，避免延时动作执行完后值发生变化
    local nowCount  = self._door._finishCount
    self._layer:playNumberVoice(nowCount)
    self:line({
        {"delay", 0.15},
        {"fn", function()
            -- 粒子
            local particle = P.new("particle/liwubianshen.plist"):to(self:getParent(), self:z() + 100):p(self:p())
            particle:setAutoRemoveOnFinish(true) 
        end}, 
    })

    local sx, sy = self:scaleX(), self:scaleY()
    self:z(1000):line({
        {"scaleTo", 0.15, sx * 1.2, sy * 0.8},
        {"fn", function()
            self:display("num/write/door/n10box/toy"..self._id..".png")
            :scale(0.4 * 1.2, 0.4 * 0.8)
        end},
        {"scaleTo", 0.15, 0.4, 0.4},
        {"fn", function()
            local time      = math.random(60, 90) / 100
            self:cycle({
                {"moveBy", time,     cc.p(0, time * 20)},
                {"moveBy", time * 2, cc.p(0, -time * 40)},
                {"moveBy", time,     cc.p(0, time * 20)},
            })

            if nowCount == 10 then
                self._door:stopGuide()
                self._door:numBoneHappyOut()
            else
                self._door:openGuide()
            end
        end},
    })
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
    self:unbindTouch()
    self._door:stopGuide()
    CAction:stopBreathing(self)
    self:getScene()._menuBtn:barChange(false)

    soundEffect:playEffectsfx14711098()
    local scale = self:scale()
    self:scale(0.6 * scale)
    self:line({
        {"scaleTo", 0.2, scale},
        {"fn", function()
            self:transformToToy()
        end},
    })

    -- return true
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end


return M