--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-1-21

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字1在门内的面包
----------------------
local M     = classSprite("N1Bread")
M.TAG       = "N1Bread"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._layer 	= params.layer

	self._door 		= params.door
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
	self:display("num/puzzle/item/item1_2.png")
end

function M:onEnterTransitionFinish()
	self._iniPoint = self:point()
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	self._touchLock = true
	self._door:stopGuide()
    CAction:stopBreathing(self)
    self:getScene()._menuBtn:barChange(false)
	
	soundEffect:playEffectsfx14700001()
    local scale = self:scale()
    self:scale(0.6 * scale)
    self:line({
        {"scaleTo", 0.2, scale},
        {"fn", function()
    		self._layer:playNumberVoice(1)
			-- 面包跳到骨骼手上
			local numBonePoint = cc.pAdd(self._layer._numBone:worldpoint(), cc.p(4, 0))
			soundEffect:playEffectsfx14711020()
			self:line({
				A.union({
					{"jumpTo", 0.5, self:getParent():convertToNodeSpace(numBonePoint), 100, 1},
					{"scaleTo", 0.5, 0.45},
					{"rotateTo", 0.5, 20},
					A.line({
						{"delay", 0.3},
						{"fn", function()
							soundEffect:playEffectsfx14711021()
							self._layer._numBone:eatBread()
						end},
					})
				}),
				{"delay", 0.1},
				{"hide"},
			})
        end},
    })

	return true
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M