--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-2-24

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类 数字3终点的蝴蝶骨骼
----------------------
local M     = classArmatureTouch("N3Butterfly")
M.TAG       = "N3Butterfly"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
	-- 层
	self._layer 		= params.layer
	-- 门
	self._door 		= params.door
    -- 字段[骨骼资源文件名，不带文件后缀名]
    self._file            = "bone/hudie"
    -- 字段[骨架名称]
    self._armatureName    = "hudie_dong"

    self._speed         = 100
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end

-- 图片渲染
function M:loadDisplay()

end

function M:fly()
    self._isClicked = true
	-- 完成数+1
	self._door._finishCount = self._door._finishCount + 1
	-- 记录下当前完成数，避免延时动作执行完后值发生变化
	local nowCount 	= self._door._finishCount
    -- 计数语音
    self._layer:playNumberVoice(nowCount)
    -- 蝴蝶飞音效
    self._door:playButterflyEffect()
    -- 蝴蝶飞动画
	self:play("hudie_dong", nil, {loop = 1})
	self._pos = self:point()
	self:moveAct()
	if nowCount == 3 then
        self._door:stopGuide()
		self:line({
			{"delay", 1},
			{"fn", function()
				self._door:butterflyOut()
			end},
		})
	else
		self._door:openGuide()	
	end
end

function M:movementHandler(__evtType, __moveId)
	
end


-- 虫子移动
function M:moveAct()
    if self._moveAct then return end
    -- local pos = cc.p(self._pos.x + math.random(-100, 100), self._pos.y + math.random(-80, 80))
    minP = self:getParent():convertToNodeSpace(cc.p(180, 40))
    maxP = self:getParent():convertToNodeSpace(cc.p(V.w - 60, V.h - 120))
    local pos = cc.p(math.random(minP.x, maxP.x), math.random(minP.y, maxP.y))

    local time = PT.distance(pos, self:point()) / self._speed

    local rot 	= -PT.quadAngle(pos, self:point()) + 90
    self:rotate(rot)
    local act = A.line({
        {"moveTo", time, pos},
        {"fn", function()
            self._moveAct = nil 
            self:moveAct()
        end},
    })
    self._moveAct = act
    self:runAction(self._moveAct)
end

-- 停止虫子移动
function M:stopMove()
    if self._moveAct then
        self:stopAction(self._moveAct)
        self._moveAct = nil
    end
end


--------------------------
-- 触控
-------------------------- 

function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	self._touchLock = true
	self._door:stopGuide()
    CAction:stopBreathing(self)
    self:getScene()._menuBtn:barChange(false)

    local scale = self:scale()
    self:scale(0.6 * scale)
    self:line({
        {"scaleTo", 0.2, scale},
        {"fn", function()
			self:fly()
        end},
    })

	return SIGN_TOUCH_BEGAN_SWALLOWS 
end







----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


return M