--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-1-21

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字4在门内的翅膀
----------------------
local M     = classSprite("N1Bread")
M.TAG       = "N1Bread"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._layer 	= params.layer
	-- ID
	self._id 		= params.id or 1

	self._door 		= params.door
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
	if self._id <= 2 then
		self:display("num/write/door/n4wing/w1.png")
	else
		self:display("num/write/door/n4wing/w2.png")
	end

	if self._id % 2 == 0 then
		self:flipX(true)
	end
end

function M:onEnterTransitionFinish()
	self._iniPoint = self:point()
end

-- 翅膀跳到骨骼身上
function M:jumpToBone()
    self._isClicked = true
	-- 完成数+1
	self._door._finishCount = self._door._finishCount + 1
	-- 记录下当前完成数，避免延时动作执行完后值发生变化
	local nowCount 	= self._door._finishCount
    self._layer:playNumberVoice(nowCount)
	-- 终点坐标
	-- local px 		= nowCount == 1 and self:x() - 200 or self:x() + 200
	local wingPos 	= self._layer._numBone:getWingPos(self._id)
	local lPos 		= self:getParent():convertToNodeSpace(wingPos)

	self:line({
		A.union({
			{"jumpTo", 0.4, lPos, 60, 1},
			{"scaleTo", 0.4, 1.3},
		}),
		{"hide"},
		{"fn", function()
			self._layer._numBone:showWing(self._id)
			if nowCount == 4 then
                self._door:stopGuide()
				self._door:numBoneFlyAway()
			else
				self._door:openGuide()
			end
		end},
	})
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	self._touchLock = true
	self._door:stopGuide()
    CAction:stopBreathing(self)
    self:getScene()._menuBtn:barChange(false)

    soundEffect:playEffectsfx14700001()
    local scale = self:scale()
    self:scale(0.6 * scale)
    self:line({
        {"scaleTo", 0.2, scale},
        {"fn", function()
			self:jumpToBone()
        end},
    })
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M