--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-18

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字6在门内的火车车厢
----------------------
local M     = classSprite("N6Train")
M.TAG       = "N6Train"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._layer 	= params.layer

	self._id 		= params.id or 1

	self._door 		= params.door
	-- 是否是火车头
	self._isLocomo 	= params.isLocomo
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
	if self._isLocomo then
		self:display("num/write/door/n6train/locomotive.png"):scale(1.5)
	else
		self:display("num/write/door/n6train/t"..self._id..".png"):scale(1.5)
	end
end

function M:onEnterTransitionFinish()
	self._iniPoint = self:point()
end

-- 火车跳到地面上
function M:jumpToTarget()
	local locomoPos = cc.p(480, 50)
    self._isClicked = true
	self._door._finishCount = self._door._finishCount + 1
	-- 记录下当前完成数，避免延时动作执行完后值发生变化
	local nowCount 	= self._door._finishCount
    self._layer:playNumberVoice(nowCount)

    if self._isLocomo then
    	self._door._isClickedLocomo = true
    end

	-- 终点坐标，火车头在固定位置，其他在火车头后按点击顺序排列
	local calNum 	= nowCount - (self._door._isClickedLocomo and 1 or 0)
	local endX 		= self._isLocomo and locomoPos.x or (locomoPos.x - calNum * 68 * 1.5)
	local endY 		= self._isLocomo and locomoPos.y or locomoPos.y - 7

	self:z(self:z() + nowCount):line({
		A.union({
			{"jumpTo", 0.4, cc.p(endX, endY), 60, 1},
			{"scaleTo", 0.4, 1.5},
		}),
		{"fn", function()
			for i,v in ipairs(self._door._items) do
				if v._isClicked then
					CAction:feedback2(v)
				end
			end

			if nowCount == 6 then
                self._door:stopGuide()
				self:line({
					{"delay", 0.5},
					{"fn", function()
						self._door:numBoneToTrain()
					end},
				})
			else
				self._door:openGuide()
			end
		end},
	})
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
	self:unbindTouch()
	self._door:stopGuide()
    CAction:stopBreathing(self)
    self:getScene()._menuBtn:barChange(false)

	local scale = self:scale()
	self:scale(0.6 * scale)
	self:line({
		{"scaleTo", 0.2, scale},
		{"fn", function()
    		soundEffect:playEffectsfx14711064()
			self:jumpToTarget()
		end},
	})
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M