--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-18

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字7在门内的萤火虫
----------------------
local M     = classArmatureTouch("N7Glowworm")
M.TAG       = "N7Glowworm"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._layer 	= params.layer

	self._door 		= params.door

    self._file            = "bone/yhc_sl"
    -- 字段[骨架名称]
    self._armatureName    = "yhc_yhc"

    self._speed         = 100
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
end

function M:onEnterTransitionFinish()
	self._iniPoint = self:point()
    self:unbindTouch()
end

-- 飞行动画
function M:flyAni()
	self:play("yhc_fei", nil, {loop = 1})
	-- self:randomFly()
end


-- 萤火虫飞到天上
function M:flyToSky()
    self._isClicked = true
	-- 完成数+1
	self._door._finishCount = self._door._finishCount + 1
	-- 记录下当前完成数，避免延时动作执行完后值发生变化
	local nowCount 	= self._door._finishCount
    self._layer:playNumberVoice(nowCount)

	self:z(self:z() + nowCount)
	self:flyAni()
	self:moveAct()

	if nowCount == 7 then
        self._door:stopGuide()
		self:line({
			{"delay", 1},
			{"fn", function()
				self._door:glowwormFlyOut()
			end},
		})
	else
		self._door:openGuide()
	end
end

-- 虫子移动
function M:moveAct()
    if self._moveAct then return end
    -- local pos = cc.p(self._pos.x + math.random(-100, 100), self._pos.y + math.random(-80, 80))
    minP = self:getParent():convertToNodeSpace(cc.p(180, 40))
    maxP = self:getParent():convertToNodeSpace(cc.p(V.w - 60, V.h - 120))
    local pos = cc.p(math.random(minP.x, maxP.x), math.random(minP.y, maxP.y))

    local time = PT.distance(pos, self:point()) / self._speed


    if pos.x < self:px() then
        self:flipX(false)
    else
        self:flipX(true)
    end
    local act = A.line({
        {"moveTo", time, pos},
        {"fn", function()
            self._moveAct = nil 
            self:moveAct()
        end},
    })
    self._moveAct = act
    self:runAction(self._moveAct)
end

-- 停止虫子移动
function M:stopMove()
    if self._moveAct then
        self:stopAction(self._moveAct)
        self._moveAct = nil
    end
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
	self:unbindTouch()
	self._door:stopGuide()
    CAction:stopBreathing(self)
    self:getScene()._menuBtn:barChange(false)

    if self._door._doorWormEffect then
        sound.voiceFadeOut(self._door._doorWormEffect)
    end
    self._door._doorWormEffect = soundEffect:playEffectsfx14711071()


    local scale = self:scale()
    self:scale(0.6 * scale)
    self:line({
        {"scaleTo", 0.2, scale},
        {"fn", function()
			self:flyToSky()
        end},
    })
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M