--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-18

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字8在门内的雪花
----------------------
local M     = classSprite("N8Snow")
M.TAG       = "N8Snow"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._layer 	= params.layer

	self._door 		= params.door
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
	self:display("num/write/door/n8snow/s"..math.random(1, 2)..".png")
end

function M:onEnterTransitionFinish()
	self._iniPoint = self:point()
end



-- 雪花飞走
function M:flyToSky()
    self._isClicked = true
	self._door:startSnow()
	-- 完成数+1
	self._door._finishCount = self._door._finishCount + 1
	-- 记录下当前完成数，避免延时动作执行完后值发生变化
	local nowCount 	= self._door._finishCount
    self._layer:playNumberVoice(nowCount)

	local endPos
	local dir =	math.random(1, 3)
	if dir == 1 then
		endPos= cc.p(-32, math.random(32, V.h - 110))
	elseif dir == 2 then
		endPos= cc.p(V.w + 32, math.random(32, V.h - 110))
	else
		endPos= cc.p(math.random(32, V.w - 32), -32)
	end	

	endPos = self:getParent():convertToNodeSpace(endPos)
	self:line({
        A.actionBezierTo({
            startPosition = self:point(),
            endPosition   = endPos,
            time          = PT.d(self:point(), endPos) / 300
        }),
        {"fn", function()
			if nowCount == 8 then
                self._door:stopGuide()
				self._door:snow()
			else
				self._door:openGuide()
			end
        end},
        {"hide"},
	})

end



----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
	self:unbindTouch()
	self._door:stopGuide()
    CAction:stopBreathing(self)
    self:getScene()._menuBtn:barChange(false)

    soundEffect:playEffectsfx14700001()
    local scale = self:scale()
    self:scale(0.6 * scale)
    self:line({
        {"scaleTo", 0.2, scale},
        {"fn", function()
			self:flyToSky()
        end},
    })
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M