--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-18

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字9在门内的气球
----------------------
local M     = classSprite("N9balloon")
M.TAG       = "N9balloon"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._layer 	= params.layer

	self._id 		= params.id or 1

	self._door 		= params.door
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
	local picIndex = self._id % 3 + 1
	self:display("num/write/door/n9balloon/b"..picIndex..".png")
end

function M:onEnterTransitionFinish()
	self._iniPoint = self:point()
end

-- 漂浮到空中
function M:jumpToTarget()
    self._isClicked = true

	self:cycle({
		{"moveBy", math.random(10, 20)/10, cc.p(0, 20)},
		{"moveBy", math.random(10, 20)/10, cc.p(0, -20)},
	})

	-- 完成数+1
	self._door._finishCount = self._door._finishCount + 1
	-- 记录下当前完成数，避免延时动作执行完后值发生变化
	local nowCount 	= self._door._finishCount
    self._layer:playNumberVoice(nowCount)
	-- 终点坐标
	local px 		= -150 + nowCount * 60
	self:z(self:z() + nowCount):line({
		A.union({
			{"jumpTo", 0.4, cc.p(px, 360), 60, 1},
		}),
		{"fn", function()
			self._door:arrangeBalloon()

			if nowCount == 9 then
                self._door:stopGuide()
				self._door:numBoneToBalloon()
			else
				self._door:openGuide()
			end
		end},
	})
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
	self:unbindTouch()
	self._door:stopGuide()
    CAction:stopBreathing(self)
    self:getScene()._menuBtn:barChange(false)

    soundEffect:playEffectsfx14700001()
    local scale = self:scale()
    self:scale(0.6 * scale)
    self:line({
        {"scaleTo", 0.2, scale},
        {"fn", function()
			self:jumpToTarget()
        end},
    })
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M