--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]

--[[!--

场景层类，定义层相关操作方法及逻辑实现。

-   定义场景层功能方法。

]]

----------------------
-- 类
----------------------
local M = classLayerTouch("Main")


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..

-- [操作变量]
-- ..








----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义视图实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
	-- 当前数字
	self._number 		= params.number or nil
end

----------------------
-- 结点渲染
----------------------
--[[--

视图渲染，处理视图结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- 加载渲染以外的其他操作
	self:loadRenderOtherwise()
	-- 背景墙
	self:loadBgWall()
	-- 加载写字路径数字
	self:loadNumberPath()

	-- self:line({
	-- 	{"delay", 2},
	-- 	{"fn", function()
	-- 		self:loadDoor()
	-- 		self._door:px(V.w_2 + 100) 
	-- 		self._numBone:moveToDoor()
	-- 	end},
	-- })
end

function M:onEnterTransitionFinish()

end

function M:enterFinish()
	-- 加载数字骨骼
	self:loadNumberBone()
end


-- 加载其他
function M:loadRenderOtherwise()
	self._number 			 = self._number or self:getParent()._number
	self:getParent()._number = self._number
end

-- 背景墙
function M:loadBgWall()
	-- local bgWall = require("app.numwrite.node.BgWall").new({
	-- 	layer 	= self,
	-- 	number 	= 1
	-- }):to(self):p(0, 0)
	-- self._bgWall = bgWall
end

-- 加载写字路径数字
function M:loadNumberPath()
	local numPath = require("app.numwrite.numpath.Num"..self._number).new({
		layer 	= self,
	}):to(self, 300):p(V.w_2, V.h_2 - 30)
	self._numPath = numPath
end

-- 加载数字骨骼
function M:loadNumberBone()
	local number 	= self._number
	local numBone 	= require("app.numwrite.numbone.NumBone"..number).new({
		layer 	= self,
		number 	= number
	}):to(self, 200):p(-200 + X_OFFSET, 120 + Y_OFFSET / 2):scale(0.75)
	numBone:enter(function ()
		self._numPath:loadAutoStroke()
	end)

	self._numBone = numBone
end

-- 场景向左移动
function M:sceneLeftMove()
	self:loadDoor()
	self:stopAllChildrenAction(self._numPath)
	self._numPath:unbindTouch()
	-- 移动动作
	local moveTime 	 = 2
	local movePoint  = cc.p(-1000, 0)
	local moveAction = A.line({
		{"moveBy", moveTime, movePoint},
	})
	self._door:runAction(moveAction:clone())
	self._numPath:z(100):runAction(moveAction:clone())

	-- 背景一起移动
	self:getParent():getBackgroundLayer():bgMoveBy(moveTime, movePoint)

	if self._numPath.moveLeft then
		self._numPath:moveLeft(moveAction)
	end

	self._numBone:line({
		{"delay", 0.5},
		{"moveTo", moveTime - 0.5, cc.p(210 + X_OFFSET, self._numBone:py())},
		{"fn", function()
			self._numBone:moveToDoor()
			self._numPath:hide()
		end},
	})
end

function M:loadDoor()
	local door = require("app.numwrite.node.Door").new({
		layer 	= self,
		number 	= self._number
	}):to(self, 900):p(V.w_2 + 80 + 1000, V.h_2 - 40)
	self._door = door
end

-- 全部结束
function M:ending()
	if self._isEnded then return end
	self._isEnded = true
	self:getParent()._finishCount = self:getParent()._finishCount + 1
	self:line({
		{"delay", 0.05},
		{"fn", function()
			if self:getParent()._finishCount < 5 then
				self:getParent():enterNextNumber()
			else
				local encourage = CUtil:loadEncourage(self, 3, function ()
					G_addFinish("numwrite")
					SceneController:enterNextMoudleScene()
				end)
			end
		end},
	})
end

-- 播报数字语音
function M:playNumberVoice(number)
	local index = 147157 + (number or self._number)
	-- 语音编号没有166 -- 没有为啥
	if index > 147165 then index = index + 1 end
	soundVoice["playEffectv"..index](soundVoice)
end

-- 节点到达屏幕外自动移除
function M:autoRemove(node)
    node:cycle({
        {"delay", 1/60},
        {"fn", function()
            if node:worldx() - node:cw()/2 > V.w then
                node:remove()
            end
        end},
    })
end

-- 停止自身及子节点的所有动作
function M:stopAllChildrenAction(node)
	node = node or self
    for i,v in ipairs(node:getChildren()) do
        self:stopAllChildrenAction(v)
    end
    node:stopAllActions()
end

----------------------
-- 结点析构
----------------------
--[[--

视图析构，处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end

function M:onExitTransitionStart()
    -- R.removeAllTextures()
    -- R.removeAllFrames()
    -- DragonBone.removeAll()
end

return M