--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类 -- 拉开窗帘的拱门
----------------------
local M     = classSpriteTouch("Door")
M.TAG       = "Door"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 主层
    self._layer     = params.layer
    -- 数字
    self._number    = params.number
    -- 门内的道具
    self._items     = {}
    -- 已完成道具数
    self._finishCount = 0
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()
    -- 箭头拖动书写
    self:loadArrowNum()
    -- 对应数字的物品
    -- self:loadItemNum1()
    self["loadItemNum"..self._number](self)
    -- 检查是否显示
    self:checkIsShow()
end

-- 图片渲染
function M:loadDisplay()
    self:setTexture("num/write/door/frame.png")
    -- 233, 233
    local center    = cc.p(self:cw()/2, self:ch()/2)
    -- 背景ID
    local bgId      = self:getBgId()
    -- 背景
    local bg        = D.img("num/write/door/bg"..bgId..".png"):to(self, -100):p(center)
    -- 裁切模板，设置位置
    local clipImg   = D.img("num/write/door/bg"..bgId..".png"):p(center)
    -- 创建遮罩
    local clip      = CCNodeExtend.extend(cc.ClippingNode:create(clipImg)):to(self, -1)
    -- 设置裁剪区域的显示，false为显示裁剪的区域，true为裁剪后的区域(很关键)
    clip:setInverted(false)
    -- 设置alpha阈值， 只有模板（stencil）的alpha像素大于alpha阈值（alphaThreshold）时内容才会被绘制。 alpha阈值（threshold）范围应是0到1之间的浮点数。 alpha阈值（threshold）默认为1。
    clip:setAlphaThreshold(0)

    -- 左右窗帘
    local curtainl  = D.img("num/write/door/curtainl.png"):a(cc.p(1, 0.5)):p(center):to(clip)
    local curtainr  = D.img("num/write/door/curtainr.png"):a(cc.p(0, 0.5)):p(center):to(clip)

    self._clip      = clip
    self._curtainl  = curtainl
    self._curtainr  = curtainr
end

function M:getBgId()
    local numberIds = {1, 1, 2, 3, 5, 1, 2, 5, 4, 3}
    return numberIds[self._number]
end

-- 当到达屏幕内的时候的才显示
function M:checkIsShow()
    self:hide()
    self._checkShowAct = A.cycle({
        {"delay", 1/60},
        {"fn", function()
            if self:px() - self:cw()/2 < V.w then
                self:show()
                self:stopAction(self._checkShowAct)
            end 
        end},
    }):at(self)
end

-- 加载箭头书写数字
function M:loadArrowNum()
    local arrowNum = require("app.numwrite.numpath.Num"..self._number).new({
        layer = self._layer,
        arrowCallback = function ()
            self:openCurtain()
        end
    }):to(self, 200):p(self:cw()/2, self:ch()/2)
    if self._number == 4 then
        arrowNum:scale(0.9)
    end

    arrowNum:loadWriteArrow()
    self._arrowNum = arrowNum
end


-- 开启引导
function M:openGuide()
    if not self._guideAct then
        local action = A.cycle({
            {"delay", 3},
            {"fn", function()
                local guideTab = {}
                for i,v in ipairs(self._items) do
                    if not v._isClicked then
                        table.insert(guideTab, v)
                    end
                end
                if #guideTab >= 1 then
                    soundEffect:playEffectsfx14700011()
                    self._guideItem    = guideTab[math.random(1, #guideTab)]
                    self._guideScale   = self._guideItem:scaleX()
                    self._guideItemAct = A.line({
                        {"scaleto", 0.2, self._guideScale * 1.3},
                        {"scaleto", 0.1, self._guideScale},
                        {"scaleto", 0.2, self._guideScale * 1.3},
                        {"scaleto", 0.1, self._guideScale},
                    }):at(self._guideItem)
                end
            end},
            {"delay", 0.5},
        })
        self._guideAct  = action
        self:runAction(self._guideAct)
    end
end

-- 关闭引导
function M:stopGuide()
    if self._guideAct then
        self:stopAction(self._guideAct)
        self._guideAct = nil

        if self._guideItem then
            self._guideItem:stopAction(self._guideItemAct):scale(self._guideScale)
            self._guideItem = nil
        end
    end
end


-- 打开窗帘
function M:openCurtain()
    soundEffect:playEffectsfx14711019()
    self._curtainl:line({
        {"moveBy", 0.5, cc.p(-220, 0)},
        {"fn", function()
            self:openedFn() 
        end},
    })
    self._curtainr:line({
        {"moveBy", 0.5, cc.p(220, 0)},
    })
end

-- 关闭窗帘
function M:closeCurtain()
    self._curtainl:line({
        {"moveBy", 0.5, cc.p(220, 0)},
    })
    self._curtainr:line({
        {"moveBy", 0.5, cc.p(-220, 0)},
    })
end

-- 打开窗帘后回调
function M:openedFn()
    self:openGuide()
    for i,v in ipairs(self._items) do
        CAction:scaleBreath(v)
        v:z(200):bindTouch()
    end
    self["fnNum"..self._number](self)
end

-- ================================ 数字1 ================================ --
-- 加载数字1道具
function M:loadItemNum1()
    -- 面包
    local bread = require("app.numwrite.dooritem.N1Bread").new({
        layer   = self._layer,
        door    = self
    }):to(self, -10):p(233, 226)
    self._items[1] = bread
end

-- 数字1回调
function M:fnNum1()
    -- self._items[1]:z(100):bindTouch()
end




-- ================================ 数字2 ================================ --
-- 加载数字2道具
function M:loadItemNum2()
    local horseCls  = require("app.numwrite.dooritem.N2Horse")
    local points    = {cc.p(233, 130), cc.p(233, 320)}
    for i,v in ipairs(points) do
        local horse = horseCls.new({
            id      = i,
            layer   = self._layer,
            door    = self
        }):to(self, -10):p(v):scale(0.5)
        CUtil:changeAnchorP(horse, cc.p(0.5, 0))
        table.insert(self._items, horse)
    end
end

-- 数字2回调
function M:fnNum2()

end

-- 数字2骑马
function M:numBoneRiding()
    self._isRided = true
    self:closeCurtain()
    self._layer._numBone:moveToHorse(self._items[math.random(1, 2)])
end




-- ================================ 数字3 ================================ --
-- 加载数字3道具
function M:loadItemNum3()
    local btFlyCls  = require("app.numwrite.dooritem.N3Butterfly")
    local points    = {cc.p(self:cw()/2, 325), cc.p(self:cw()/2 - 100, 190), cc.p(self:cw()/2 + 100, 190)}
    for i,v in ipairs(points) do
        local butterfly = btFlyCls.new({
            layer   = self._layer,
            door    = self
        }):to(self, -10):p(v):scale(0.25)
        table.insert(self._items, butterfly)
    end
end

-- 数字3回调
function M:fnNum3()
end

-- 蝴蝶飞出
function M:butterflyOut()

    self:closeCurtain()
    for i,v in ipairs(self._items) do
        v:stopMove()
        self._layer:autoRemove(v)
        v:line({
            A.union({
                {"rotateTo", 0.5, 90},
                {"moveBy", 2, cc.p(1000, 0)},
            }),
            {"remove"},
        })
    end
    self._layer._numBone:catchButterfly()


    for i,v in ipairs(self._flyEffects) do
        sound.voiceFadeOut(v, 0.5)
    end
end

-- 蝴蝶飞声音
function M:playButterflyEffect()
    -- 初始化
    if not self._flyEffects then
        self._flyEffects = {}
    end
    -- 播放音效
    local soundHandle = soundEffect:playEffectsfx14711039()
    table.insert(self._flyEffects, soundHandle)

    -- 音效最多同时存在3个
    if #self._flyEffects == 3 then
        sound.stopSound(self._flyEffects[1])
        table.remove(self._flyEffects, 1)
    end
end


-- ================================ 数字4 ================================ --
-- 加载数字4道具
function M:loadItemNum4()
    local wingCls   = require("app.numwrite.dooritem.N4Wing")
    local points    = {cc.p(self:cw()/2 - 64, 266), cc.p(self:cw()/2 + 64, 266),
                       cc.p(self:cw()/2 - 40, 178), cc.p(self:cw()/2 + 40, 178)}
    for i,v in ipairs(points) do
        local wing = wingCls.new({
            layer   = self._layer,
            id      = i,
            door    = self
        }):to(self, -10):p(v)
        table.insert(self._items, wing)
    end
end

-- 数字4回调
function M:fnNum4()
    -- for i,v in ipairs(self._items) do
    --     v:z(200):bindTouchLocate()
    -- end
end

function M:numBoneFlyAway()
    self:closeCurtain()
    self._layer._numBone:flyOut()
end



-- ================================ 数字5 ================================ --
-- 加载数字5道具
function M:loadItemNum5()
    local hookCls   = require("app.numwrite.dooritem.N5Hook")
    local points    = {cc.p(144, 326), cc.p(344, 326), cc.p(233 + 10, 233),
                       cc.p(144, 136), cc.p(344, 136)}
    for i,v in ipairs(points) do
        local hook = hookCls.new({
            layer   = self._layer,
            id      = i,
            door    = self
        }):to(self, -10):p(v):scale(1/3)
        table.insert(self._items, hook)
    end
    self._hookPoints = points
end

-- 数字5回调
function M:fnNum5()
    -- for i,v in ipairs(self._items) do
    --     v:z(200):bindTouchLocate()
    -- end
end

-- 数字骨骼跳到钩子上离开
function M:numBoneToHook()
    self:closeCurtain()
    table.sort(self._items, function (a, b)
        return a:px() < b:px()
    end)
    local pos   = cc.pAdd(self._items[1]:worldpoint(), cc.p(-32, -220))
    self._layer._numBone:jumpToHook(pos)
end

-- 整理钩子 -- 下方的钩子（3/4/5）用完后，把钩子1整理过去，避免被连接起来的钩子挡住
function M:arrangeHook()
    if self._arranged then return end

    if self._items[3]._isClicked and not self._items[1]._isClicked then
        self._arranged = true
        self._items[1]:line({
            {"moveTo", 0.3, self._hookPoints[3]},
        })
    elseif self._items[4]._isClicked and not self._items[1]._isClicked then
        self._arranged = true
        self._items[3]:line({
            {"moveTo", 0.3, self._hookPoints[4]},
        })
        self._items[1]:line({
            {"delay", 0.1},
            {"moveTo", 0.3, self._hookPoints[3]},
        })
    elseif self._items[5]._isClicked and not self._items[1]._isClicked then
        self._arranged = true
        self._items[3]:line({
            {"moveTo", 0.3, self._hookPoints[5]},
        })
        self._items[1]:line({
            {"delay", 0.1},
            {"moveTo", 0.3, self._hookPoints[3]},
        })
    end
end




-- ================================ 数字6 ================================ --
-- 加载数字6道具
function M:loadItemNum6()
    local trainCls   = require("app.numwrite.dooritem.N6Train")
    local height    = self:ch() / 4
    local points    = {cc.p(self:cw()/2 - 64, height * 3), cc.p(self:cw()/2 + 64, height * 3),
                       cc.p(self:cw()/2 - 64, height * 2), cc.p(self:cw()/2 + 64, height * 2),
                       cc.p(self:cw()/2 - 64, height * 1), cc.p(self:cw()/2 + 64, height * 1)}
    for i,v in ipairs(points) do
        local train = trainCls.new({
            layer   = self._layer,
            id      = i,
            door    = self,
            isLocomo = i == 1
        }):to(self, -10):p(v)
        table.insert(self._items, train)
    end
end

-- 数字6回调
function M:fnNum6()
    -- -- 火车头
    -- local locomotive = D.img("num/write/door/n6train/locomotive.png"):to(self, 200):p(480, 50):scale(1.5)
    -- self._locomotive = locomotive
end

-- 骨骼到火车上去
function M:numBoneToTrain()
    self:closeCurtain()
    local pos   = cc.pAdd(self._items[1]:worldpoint(), cc.p(-250, 130))
    self._layer._numBone:jumpToTrain(pos)
end




-- ================================ 数字7 ================================ --
-- 加载数字7道具
function M:loadItemNum7()
    local wormCls  = require("app.numwrite.dooritem.N7Glowworm")
    local cw        = self:cw() / 2
    local ch        = self:ch() / 4
    local yOff 		= -20
    local points    = {cc.p(cw - 100, ch * 3 + yOff), cc.p(cw, ch * 3 + yOff), cc.p(cw + 100, ch * 3 + yOff), 
                       cc.p(cw - 100, ch * 2 + yOff), cc.p(cw, ch * 2 + yOff), cc.p(cw + 100, ch * 2 + yOff),
                   	   cc.p(cw, ch + yOff)}
    for i,v in ipairs(points) do
        local worm = wormCls.new({
            layer   = self._layer,
            id      = i,
            door    = self
        }):to(self, -10):p(v)
        table.insert(self._items, worm)
    end
end

-- 数字一回调
function M:fnNum7()
end

-- 萤火虫飞出
function M:glowwormFlyOut()
    self:closeCurtain()

    for i,v in ipairs(self._items) do
        v:stopMove()
        v:flipX(true)
        self._layer:autoRemove(v)
        v:line({
            {"moveBy", 2, cc.p(1000, 0)},
            {"remove"},
        })
    end

    self._layer._numBone:jumpOut()
end


-- ================================ 数字8 ================================ --
-- 加载数字8道具
function M:loadItemNum8()
    local snowCls  = require("app.numwrite.dooritem.N8Snow")
    local cw        = self:cw() / 2
    local ch        = self:ch() / 4
    -- local points    = {cc.p(cw - 100, ch * 3), cc.p(cw, ch * 3), cc.p(cw + 100, ch * 3), 
    --                    cc.p(cw - 100, ch * 2), cc.p(cw + 100, ch * 2),
    --                    cc.p(cw - 100, ch * 1), cc.p(cw, ch * 1), cc.p(cw + 100, ch * 1)}
    local points    = {cc.p(178, 331), cc.p(308, 329), cc.p(113, 236), cc.p(243, 205), 
                        cc.p(377, 228), cc.p(129, 129), cc.p(244, 86), cc.p(355, 126)}
    for i,v in ipairs(points) do
        local snow = snowCls.new({
            layer   = self._layer,
            id      = i,
            door    = self
        }):to(self, -10):p(v)
        table.insert(self._items, snow)
    end
end

-- 数字一回调
function M:fnNum8()
    -- for i,v in ipairs(self._items) do
    --     v:bindTouchLocate()
    -- end
end

-- 开始下雪
function M:startSnow()
    if self._snowParticle then return end
    local particle = P.new("particle/xuehua.plist"):to(self._layer, 1000):p(V.w_2, V.h)
    self._snowParticle = particle
    self._snowSound = soundEffect:playEffectsfx14711103()
end

-- 下雪
function M:snow()
    self:closeCurtain()
    self._layer._numBone:beFrozen()
    sound.voiceFadeOut(self._snowSound)
end



-- ================================ 数字9 ================================ --
-- 加载数字9道具
function M:loadItemNum9()
    local balloonCls  = require("app.numwrite.dooritem.N9Balloon")
    local cw        = self:cw() / 2
    local ch        = self:ch() / 4
    local points    = {cc.p(87, 222), cc.p(155, 282), cc.p(232, 341), 
                       cc.p(259, 228), cc.p(337, 304), cc.p(400, 231),
                       cc.p(152, 139), cc.p(335, 132), cc.p(229, 113)}
    for i,v in ipairs(points) do
        local balloon = balloonCls.new({
            layer   = self._layer,
            id      = i,
            door    = self
        }):to(self, -10):p(v):scale(0.25)
        table.insert(self._items, balloon)
    end
end

-- 数字一回调
function M:fnNum9()
    -- for i,v in ipairs(self._items) do
    --     v:bindTouchLocate()
    -- end
end


function M:numBoneToBalloon()
    self:closeCurtain()
    table.sort(self._items, function (a, b)
        return a:px() < b:px()
    end)
    local pos   = cc.pAdd(self._items[1]:worldpoint(), cc.p(46, -235))
    self._layer._numBone:jumpToBalloon(pos)
end


-- 
function M:arrangeBalloon()
    -- body
end




-- ================================ 数字10 ================================ --
-- 加载数字10道具
function M:loadItemNum10()
    local boxCls  = require("app.numwrite.dooritem.N10Box")
    local points  = {cc.p(221, 341), cc.p(314, 342), cc.p(122, 312), cc.p(210, 228), cc.p(378, 256),
                    cc.p(96, 219), cc.p(245, 92), cc.p(327, 185), cc.p(140, 115), cc.p(359, 88)}
    for i,v in ipairs(points) do
        local box = boxCls.new({
            layer   = self._layer,
            id      = i,
            door    = self
        }):to(self, -10):p(v)
        table.insert(self._items, box)
    end
end

-- 数字一回调
function M:fnNum10()
    -- for i,v in ipairs(self._items) do
    --     v:z(200):bindTouchLocate()
    -- end
end

-- 开心走掉
function M:numBoneHappyOut()
    -- self:closeCurtain()
    self._layer._numBone:happyOut()
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end


return M