--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-20

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类 -- 菜单键
----------------------
local M     = classSpriteTouch("Menu")
M.TAG       = "Menu"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    self._main      = params.main
    self._num       = params.num or 1            -- 初始时的数字
    self._isExtend  = params.isExtend or false   -- 选项是否展开了
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()
    -- 加载显示栏
    self:loadBar()
    -- 加载菜单栏形状
    self:loadNumOption()
    -- 勾选数字按钮选项
    self:tickSelect(self._num)
end

-- 图片渲染
function M:loadDisplay()
    self:setTexture("common/ui/circle.png"):opacity(0)
    local showBtn = D.img("common/ui/circle.png"):to(self):p(self:cw()/2, self:ch()/2)

    -- 点击按钮上的数字
    local btnNum= D.img("num/puzzle/menu/"..self._num..".png"):to(showBtn):p(self:cw()/2, self:ch()/2):scale(0.9)
    self._showBtn = showBtn
    self._btnNum = btnNum
end

-- 加载菜单栏
function M:loadBar()
    self._barWidth  = 799
    -- 菜单栏
    local bar       = D.img("common/ui/bar.png"):anchor(ccp(0, 0.5))
    -- 按栏的最大区域进行矩形裁切
    local clippingRect = cc.ClippingRectangleNode:create(cc.rect(self:cw()/2, self:ch()/2 - bar:ch()/2, self._barWidth, bar:ch()))

    -- 按按钮图片样式进行裁切（按钮半透明，仅矩形裁切不够）
    local clipImg   = D.img("common/ui/circle.png"):p(self:cw()/2, self:ch()/2):scale(0.8)
    -- 创建遮罩蒙版裁切
    local clip      = CCNodeExtend.extend(cc.ClippingNode:create(clipImg)):to(clippingRect)
    -- 设置裁剪区域的显示，false为显示裁剪的区域，true为裁剪后的区域(很关键)
    clip:setInverted(true)
    -- 设置alpha阈值， 只有模板（stencil）的alpha像素大于alpha阈值（alphaThreshold）时内容才会被绘制。
    -- alpha阈值（threshold）范围应是0到1之间的浮点数。 alpha阈值（threshold）默认为1。
    clip:setAlphaThreshold(0)

    bar:to(clip, 10):p(self:cw()/2 - self._barWidth, self:ch()/2)

    if self._isExtend then
        bar:px(self:cw()/2)
    end
    self._bar          = bar
    self._clippingRect = CCNodeExtend.extend(clippingRect):to(self, -999)

    -- 绑定触控
    bar:bindTouch()
    clippingRect:setContentSize(cc.size(0.01, 0.01)):bindTouch()
    clip:setContentSize(cc.size(0.01, 0.01)):bindTouch()
    function clippingRect:onTouchBegan()
        return SIGN_TOUCH_BEGAN_NO_SWALLOWS
    end
    function bar:onTouchBegan()
        return SIGN_TOUCH_BEGAN_NO_SWALLOWS
    end
    function clip:onTouchBegan()
        return SIGN_TOUCH_BEGAN_NO_SWALLOWS
    end
end

-- 加载菜单栏数字
function M:loadNumOption()
    self._options   = {}
    local width     = self._barWidth / 11.1
    for i = 1 ,10 do
        local node  = D.img("num/puzzle/menu/"..i..".png"):to(self._bar):p(width * i + 20, self._bar:ch()/2):scale(0.8)
        node._tick = D.img("common/ui/tick.png"):to(node):p(node:cw()/2+20 , 15):hide()
        if self._num == i then
            node._tick:show()
        end
        table.insert(self._options, node)
        --点击数字如果与当前相同，则切换道具，否则跳转到对应数字的场景
        node:bindTouch({
            fnBegan = function (sender, x, y)
                if self._isInChange or not self._isExtend or D.getRunningScene()._inSwitching then return end
                -- 停止mainLayer全部动作
                self:getScene().mainLayer_:stopAllChildrenAction()
                -- 停止全部音效
                sound.stopAllEffects()
                -- 播放音效[sfx14700001]点击
                soundEffect:playEffectsfx14700001()
                self:barChange(false)
                self:playNumberVoice(i)
                self:setNumber(i)
                D.getRunningScene():switchNumber(i)
                return true
            end
        })
    end
end

function M:setNumber(num)
    self:setBtnCurNum(num)
    self:tickSelect(num)
end

-- 勾选数字按钮选项
function M:tickSelect(num)
    for i, v in pairs(self._options) do
        if num == i then
            v._tick:show()
            v:scale(1)
        else
            v._tick:hide()
            v:scale(0.8)
        end
    end
end

-- 播放数字台词
function M:playNumberVoice(num)
	soundVoice:stopNowSound()
	local voiceIndex = totable({
		"v147158", "v147159", "v147160", "v147161", "v147162",
		"v147163", "v147164", "v147165", "v147167", "v147168",
	})[num]

	self._numVoice = soundVoice["playEffect"..voiceIndex](soundVoice)
end

-- 设置菜单栏按钮的数字显示
function M:setBtnCurNum(num)
    self._num = num
    self._btnNum:setTexture("num/puzzle/menu/"..num..".png")
end

-- 菜单栏按钮显示打叉
function M:setBtnX()
    self._btnNum:setTexture("common/ui/cross.png")
end

-- 菜单栏状态切换(收起或展开选项),是否变成展开状态
function M:barChange(isExtend)
    if self._isExtend == isExtend then return end

    if isExtend == true then
        -- 播放音效[sfx14713001]左下角窗口打开
        soundEffect:playEffectsfx14713001()
    else
        -- 播放音效[sfx14713002]左下角窗口关闭
        soundEffect:playEffectsfx14713002()
    end

    self._isInChange    = true        -- 正在切换状态
    self._isExtend  = isExtend        -- 菜单栏状态切换布尔值

    if self._isExtend then
        self:setBtnX()
    else
        self:setBtnCurNum(self._num)
    end

    if self._changeAction then
        self:stopAction(self._changeAction)
    end

    -- 根据状态判断移动方向
    local num           = self._isExtend and 0 or -1
    self._changeAction = A.line({
        {"moveTo", 0.5, cc.p(self:cw()/2 + self._barWidth * num, self._bar:py())},
        {"fn", function()
            self._changeAction  = nil
            self._isInChange    = false
        end}
    }):at(self._bar)
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
    if self._isInChange then return end
    -- 播放音效[sfx14700001]点击
    soundEffect:playEffectsfx14700001()

    self._showBtn:line({
        {"scaleTo", 0.15, 0.85},
    })
    return true
end

function M:onTouchMoved(x, y, touches)

end

function M:onTouchEnded(x, y, touches)
    self._showBtn:stopAllActions():line({
        {"scaleTo", 0.2, 1},
    })
    self:barChange(not self._isExtend)
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end


return M