--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-10

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类 	-- 数字4骨骼
----------------------
local M    = class("NumBone4", require("app.numwrite.numbone.NumBoneBase"))
M.TAG    = "NumBone4"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 顺序 -- 左上 右上 左下 右下
local wingBoneName = {"sz4_chiban1", "sz4_chiban3", "sz4_chiban2", "sz4_chiban4"}




----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
end

-- 初始化通用动作名
function M:initCommonAnimationName()
    self._breathAni1    = "sz4_huxi_1"           -- 呼吸
    self._breathAni2    = "sz4_huxi_2"           -- 呼吸眨眼
    self._walkAni       = "sz4_likai"           -- 走路(跳)
    self._standbyAni    = "sz4_zhayan"           -- 待机-眨眼动画
    self._clickAni      = "sz4_fmaozi"           -- 点击反馈-翻转帽子的动画
end

-- 图片渲染
function M:loadDisplay()
	-- 隐藏帽子和翅膀
	self:getBone("sz4_maozi"):hide()
	for i,v in ipairs(wingBoneName) do
		self:getBone(v):hide()
	end
end

-- 写字完成高兴
function M:writeFinish()
	self:happyAct()
end

function M:moveToDoorCallback()
	self:z(1000)
end

function M:getWingPos(id)
	local pos = cc.p(self:getBone(wingBoneName[id]):getPosition())
	if pos.x == 0 or pos.y == 0 then
		pos = self:worldpoint()
	end
	return pos
end

function M:showWing(id)
	self:getBone(wingBoneName[id]):show()
end

-- 飞出场
function M:flyOut()
	soundEffect:playEffectsfx14711048()
	self:clapAct1()
end





-- 开心的笑动画-第二次书写后动画 -0
function M:happyAct()
	self:playAni("sz4_kaixin")
end

-- 鼓掌动画-反馈动画 -0
function M:clapAct1()
	self:playAni("sz4_guzhang")
end

-- 飞动画-反馈动画 -1
function M:flyAct1()
	self:playAni("sz4_fei")
end






function M:handlerOnDerived( __evtType, __moveId )
	if __evtType == 1 then
		if __moveId == "sz4_guzhang" then
			self:flyAct1()
			self:z(1000):line({
	            A.actionBezierTo({
	                startPosition = self:point(),
	                endPosition   = cc.pAdd(self:point(), cc.p(1300, 140)),
	                time          = 3
	            }),
				{"remove"},
			})

			local layer = self._layer
			layer:autoRemove(self)
			layer:line({
				{"delay", 3},
				{"fn", function()
					layer:ending() 
				end},
			})
		elseif __moveId == "sz4_kaixin" then
			self:walkAct()
			self._layer:sceneLeftMove()
		end
	end
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M