--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-10

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类    -- 数字5骨骼
----------------------
local M    = class("NumBone5", require("app.numwrite.numbone.NumBoneBase"))
M.TAG    = "NumBone5"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)

end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()
end

-- 初始化通用动作名
function M:initCommonAnimationName()
    self._breathAni1    = "sz5_huxi_1"          -- 呼吸
    self._breathAni2    = "sz5_huxi_2"          -- 呼吸眨眼
    self._walkAni       = "sz5_tiao"            -- 走路(跳)
    self._standbyAni    = "sz5_zhayan"          -- 待机-眨眼动画
    self._clickAni      = "sz5_chui"            -- 点击反馈-吹吹卷动画
end


-- 写字完成高兴
function M:writeFinish()
    soundEffect:playEffectsfx14711055()
    self:walkAct()
    self._layer:sceneLeftMove()
end


-- 跳到钩子上
function M:jumpToHook(pos)
    soundEffect:playEffectsfx14711057()
    self:jumpHookAct1()
    self:z(1000):line({
        A.union({
            {"moveTo", 0.3, pos},
            {"scaleTo", 0.3, 1.0},
        }),
        {"delay", 0.05},
        {"fn", function()
            self:leaveOut()
        end},
    })
end

-- 离开
function M:leaveOut()
    -- 跳上钩子后自身和第一个钩子做一个下沉动作 然后移出场景
    local hooks = self._layer._door._items
    self:p(cc.pAdd(hooks[1]:worldpoint(), cc.p(-32, -212)))
    hooks[1]:stopAllActions()
    -- 下沉动作
    local downUpAction = A.line({
        {"jumpBy", 0.3, cc.p(0, 0), -60, 1},
    }):at(hooks[1])
    self:line({
        downUpAction:clone(),
        {"fn", function()
            -- 移出场景动作
            local leaveAction = A.line({
                {"moveBy", 4, cc.p(1300, 0)},
                {"remove"},
            })
            -- 漂浮动作
            local floatAction = A.cycle({
                {"moveBy", math.random(10, 20)/10, cc.p(0, 30)},
                {"moveBy", math.random(10, 20)/10, cc.p(0, -30)},
            }):at(self)
            self:one(leaveAction)
            -- 所有钩子做漂浮动作同时移出场景
            for i,v in ipairs(hooks) do
                v:stopAllActions()
                v:runAction(leaveAction:clone())
                if i == 1 then
                    v:runAction(floatAction:clone())
                else
                    A.cycle({
                        {"moveBy", math.random(10, 20)/10, cc.p(0, 30)},
                        {"moveBy", math.random(10, 20)/10, cc.p(0, -30)},
                    }):at(v)
                end
            end

            local layer = self._layer
            self:cycle({
                {"delay", 1/60},
                {"fn", function()
                    if self:worldx() - self:cw()/2 > V.w then
                        self:remove()
                        for i,v in ipairs(layer._door._items) do
                            v:remove()
                        end
                    end
                end},
            })
            layer:line({
                {"delay", 4},
                {"fn", function()
                    layer:ending() 
                end},
            })
        end},
    })
end


-- 跳起来抓挂钩 - 0
function M:jumpHookAct1()
    self:playAni("sz5_tiao_1")
end

-- 滑走动画 - 1
function M:slideAwayAct1()
    self:playAni("sz5_zou")
end

function M:handlerOnDerived( __evtType, __moveId )
    if __evtType == 1 then
        if __moveId == "sz5_tiao_1" then
            self:slideAwayAct1()
        end
    end
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end




----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end


return M