--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-10

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类 	-- 数字6骨骼
----------------------
local M    = class("NumBone6", require("app.numwrite.numbone.NumBoneBase"))
M.TAG    = "NumBone6"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
end


-- 初始化通用动作名
function M:initCommonAnimationName()
    self._breathAni1    = "sz6_huxi_1"          -- 呼吸
    self._breathAni2    = "sz6_huxi_2"          -- 呼吸眨眼
    self._walkAni       = "sz6_likai"            -- 走路(跳)
    self._standbyAni    = "sz6_zhayan"          -- 待机-眨眼动画
    self._clickAni      = "sz6_dou"            -- 点击反馈-抖动身体动画
end

-- 写字完成高兴
function M:writeFinish()
	soundEffect:playEffectsfx14711063()
    self:walkAct()
    self._layer:sceneLeftMove()
end

function M:jumpToTrain(pos)
	soundEffect:playEffectsfx14711065()
	self:z(1000):line({
        A.union({
			{"jumpTo", 0.5, pos, 100, 1},
            {"scaleTo", 0.5, 1.0},
        }),
		{"fn", function()
			self:swingAct1()

			self:leaveOut()
		end},
	})
end

-- 离开场景
function M:leaveOut()
	local layer 	= self._layer
	local leaveTime = 4
	-- 粒子加在火车头上
	local locomotive = layer._door._items[1]
    local particle = P.new("particle/yanwu.plist"):to(locomotive):p(50, 100):scale(1/1.5)
    self._particle = particle
	-- 离开场景动作	        
	local leaveAction = A.line({
		{"easing", "SINEIN", {"moveBy", leaveTime, cc.p(1300, 0)}},
		{"remove"},
	})
	self:one(leaveAction)

	local scale 		= locomotive:scaleX()
	local scaleAction 	= A.cycle({
		A.union({
			{"scaleTo", 0.4, 0.95 * scale, 1.1 * scale},
			{"rotateBy", 0.4, 2},
		}),
		A.union({
			{"scaleTo", 0.4, 1.0 * scale, 1.0 * scale},
			{"rotateBy", 0.4, -2},
		}),
		A.union({
			{"scaleTo", 0.4, 0.95 * scale, 1.1 * scale},
			{"rotateBy", 0.4, -2},
		}),
		A.union({
			{"scaleTo", 0.4, 1.0 * scale, 1.0 * scale},
			{"rotateBy", 0.4, 2},
		}),
	})
	for i,v in ipairs(layer._door._items) do
		CUtil:changeAnchorP(v, cc.p(0.5, 0))
		v:runAction(leaveAction:clone())
		v:runAction(scaleAction:clone())
	end

	-- 4s后进入下一个场景
	layer:line({
		{"delay", leaveTime},
		{"fn", function()
			layer:ending() 
		end},
	})

	table.sort(layer._door._items, function (a, b)
		return a:px() < b:px()
	end)
	self:cycle({
		{"delay", 1/60},
		{"fn", function()
			if layer._door._items[1]:worldx() - layer._door._items[1]:cw()/2 > V.w then
				self:remove()
				for i,v in ipairs(layer._door._items) do
					v:remove()
				end
			end
		end},
	})
end



-- 摇摆动画-反馈动画 - 1
function M:swingAct1()
	self:playAni("sz6_yaobai")
end


function M:handlerOnDerived( __evtType, __moveId )

end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M