--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-10

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类 	-- 数字9骨骼
----------------------
local M    = class("NumBone9", require("app.numwrite.numbone.NumBoneBase"))
M.TAG    = "NumBone9"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
end


-- 初始化通用动作名
function M:initCommonAnimationName()
    self._breathAni1    = "sz9_huxi_1"          -- 呼吸
    self._breathAni2    = "sz9_huxi_2"          -- 呼吸眨眼
    self._walkAni       = "sz9_tiao"          -- 走路(跳)
    self._standbyAni    = "sz9_zhayan"          -- 待机-眨眼动画
    self._clickAni      = "sz9_pao"            -- 点击反馈-开心动画
end


-- 离开场景
function M:leaveScene()
	local leaveAction = A.line({
		{"moveBy", 4, cc.p(1300, 0)},
		{"remove"},
	})

	self:line({
		leaveAction
	})

	local floatAction = A.cycle({
		{"moveBy", math.random(10, 20)/10, cc.p(0, 30)},
		{"moveBy", math.random(10, 20)/10, cc.p(0, -30)},
	}):at(self)

	for i,v in ipairs(self._layer._door._items) do
		v:stopAllActions()
		v:runAction(leaveAction:clone())
		if i == 1 then
			v:runAction(floatAction:clone())
		else
			A.cycle({
				{"moveBy", math.random(10, 20)/10, cc.p(0, 30)},
				{"moveBy", math.random(10, 20)/10, cc.p(0, -30)},
			}):at(v)
		end
	end

    local layer = self._layer
    self:cycle({
        {"delay", 1/60},
        {"fn", function()
            if self:worldx() - self:cw()/2 > V.w then
                self:remove()
                for i,v in ipairs(layer._door._items) do
                    v:remove()
                end
            end
        end},
    })
    layer:line({
        {"delay", 4},
        {"fn", function()
            layer:ending() 
        end},
    })
end


-- 写字完成
function M:writeFinish()
	soundEffect:playEffectsfx14711090()
	self:walkAct()
	self._layer:sceneLeftMove()
end


-- 跳到气球上去
function M:jumpToBalloon(pos)
	soundEffect:playEffectsfx14711091()
	self:catchBalloonAct()
	self:z(1000):line({
        A.union({
            {"moveTo", 0.3, pos},
            {"scaleTo", 0.3, 1.0},
        }),
		{"fn", function()
			-- 
		end},
	})
end


-- 抓住气球动画-反馈动画 - 0
function M:catchBalloonAct()
	self:playAni("sz9_qiqiu")
end

-- 飞出屏幕动画-反馈动画 - 1
function M:flyAwayAct()
	self:playAni("sz9_fei")
end



function M:handlerOnDerived( __evtType, __moveId )
	if __evtType == 1 then
		if __moveId == "sz9_qiqiu" then
			self:flyAwayAct()
			self:leaveScene()
		end
	end
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M