--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-2-24

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类    -- 数字骨骼
----------------------
local M     = classArmatureTouch("NumBoneBase")
M.TAG       = "NumBoneBase"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 层
    self._layer         = params.layer
    -- 数字
    self._number        = params.number or 1
    -- 字段[骨骼资源文件名，不带文件后缀名]
    self._file          = "bone/sz"..self._number
    -- 字段[骨架名称]
    self._armatureName  = "sz"..self._number
    -- 触控锁定
    self._touchLock     = true
    -- 当前场景名
    self._sceneName    = params.sceneName or self:getScene().name
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()

    self:initCommonAnimationName()
end

-- 图片渲染
function M:loadDisplay()

end

-- 初始化通用动作名
function M:initCommonAnimationName()
    self._breathAni1    = nil           -- 呼吸
    self._breathAni2    = nil           -- 呼吸眨眼
    self._walkAni       = nil           -- 走路(跳)
    self._standbyAni    = nil           -- 待机
    self._clickAni      = nil           -- 点击反馈
end

function M:enter(callback)
    self:walkAct()
    self:line({
        {"moveTo", 0.8, cc.p(210 + X_OFFSET, self:y())},
        {"fn", function()
            self._walkStop = true
            if callback then
                callback()
            end
        end},
    })

    -- 数字跳进音效
    if self._number == 1 then
        soundEffect:playEffectsfx14711001()
    elseif self._number == 2 then
        soundEffect:playEffectsfx14711023()
    elseif self._number == 3 then
        soundEffect:playEffectsfx14711031()
    elseif self._number == 4 then
        soundEffect:playEffectsfx14711041()
    elseif self._number == 5 then
        soundEffect:playEffectsfx14711049()
    elseif self._number == 6 then
        soundEffect:playEffectsfx14711058()
    elseif self._number == 7 then
        soundEffect:playEffectsfx14711066()
    elseif self._number == 8 then
        soundEffect:playEffectsfx14711073()
    elseif self._number == 9 then
        soundEffect:playEffectsfx14711087()
    elseif self._number == 10 then
        soundEffect:playEffectsfx14711092()
    end
end

-- 
function M:playAni(ani, ...)
    if ani then
        if self._standbyAction then
            self:stopAction(self._standbyAction)
            self._standbyAction = nil
        end
        self:play(ani, ...)
        self._touchLock = true
    else
        print("播放动作为空")
    end
end

-- 呼吸
function M:breathAct()
    self:breathAct1()
    self._touchLock = false

    if self._sceneName == "NumpuzzleScene" then return end
    
    self._standbyAction = A.line({
        {"delay", math.random(30, 50) / 10},
        {"fn", function()
            self._standbyAction = nil
            self:standbyAct()
        end},
    }):at(self)
end

-- 普通呼吸1
function M:breathAct1()
    -- 记录呼吸动作播放次数，到达指定次数眨一下眼
    self._breathCount    = 0
    self:playAni(self._breathAni1 or self._breathAni)
end

-- 普通呼吸2(眨眼)
function M:breathAct2()
    self._breathCount    = 0
    self:playAni(self._breathAni2, nil, { loop = 0 })
end

-- 走路动作，即跳跳跳
function M:walkAct()
    self._touchLock = true
    self:playAni(self._walkAni)
end

-- 待机动作
function M:standbyAct()
    self:playAni(self._standbyAni)

    -- 数字待机音效
    if self._number == 1 then
        self._standbyEff = soundEffect:playEffectsfx14711002()
    elseif self._number == 2 then
        self._standbyEff = soundEffect:playEffectsfx14711027()
    elseif self._number == 3 then
        self._standbyEff = soundEffect:playEffectsfx14711037()
    elseif self._number == 4 then
        self._standbyEff = soundEffect:playEffectsfx14711047()
    elseif self._number == 5 then
        self._standbyEff = soundEffect:playEffectsfx14711054()
    elseif self._number == 6 then
        self._standbyEff = soundEffect:playEffectsfx14711062()
    elseif self._number == 7 then
        self._standbyEff = soundEffect:playEffectsfx14711067()
    elseif self._number == 8 then
        self._standbyEff = soundEffect:playEffectsfx14711084()
    elseif self._number == 9 then
        self._standbyEff = soundEffect:playEffectsfx14711089()
    elseif self._number == 10 then
        self._standbyEff = soundEffect:playEffectsfx14711100()
    end
end

-- 点击动作
function M:clickAct()
    self._touchLock = true
    self:playAni(self._clickAni, nil, {loop = 0})

    if self._standbyEff then
        sound.stopSound(self._standbyEff)
        self._standbyEff = nil
    end

    if self._sceneName == "NumpuzzleScene" then
        -- 拼数字场景，部分数字点击反馈时移除广告
        if self._number == 2 or self._number == 3 or self._number == 8 or self._number == 9 then
            CUtil:removeAd()
        end
        return
    end

    -- 数字点击音效(仅用于写数字场景)
    if self._clickEff then
        sound.stopSound(self._clickEff)
    end
	local effectIndex = totable({
		"sfx14713008", "sfx14713013", "sfx14713017", "sfx14713021", "sfx14713025",
        "sfx14713028", "sfx14713032", "sfx14713035", "sfx14713038", "sfx14713041",
	})[self._number]
	self._clickEff = soundEffect["playEffect"..effectIndex](soundEffect)
end


-- 移动到门边
function M:moveToDoor()
    if self.moveToDoorCallback then
        self:moveToDoorCallback()
    end
    self._walkStop = true
end

-- 写字完成
function M:writeFinish()
    print("派生类未重写写字完成方法")
end


-- 动画回调动作
function M:movementHandler(__evtType, __moveId)
    -- 派生类回调方法（如果返回值为true 此处的逻辑不再执行）
    local isReturn = self:handlerOnDerived(__evtType, __moveId)
    if isReturn then return end

    -- 根据场景名判断执行不同的动作回调
    if S.contains(self._sceneName, "NumwriteScene") then
        self:movementHandlerWrite(__evtType, __moveId)
    elseif S.contains(self._sceneName, "NumpuzzleScene") then
        self:movementHandlerPuzzle(__evtType, __moveId)
    end
end

-- 写数字场景回调
function M:movementHandlerWrite(__evtType, __moveId)
    if __evtType == 1 then
        if __moveId == self._clickAni then
            self:breathAct()
        elseif __moveId == self._standbyAni then
            self:breathAct()
        end

    elseif __evtType == 2 then
        if __moveId == self._walkAni then
            if self._walkStop then
                self._walkStop = false
                self:breathAct()
            end
        end
    end
end

-- 拼数字场景回调
function M:movementHandlerPuzzle(__evtType, __moveId)
    if __evtType == 1 then
        -- 眨眼呼吸后切回正常呼吸
        if __moveId == self._breathAni2 then
            self:breathAct1()
        -- 待机动画后切回正常呼吸
        elseif __moveId == self._standbyAni then
            self:breathAct1()
        -- 点击动画后切回正常呼吸
        elseif __moveId == self._clickAni then
            self:breathAct1()
            -- 拼数字场景，部分数字点击反馈结束时加回广告
            if self._number == 2 or self._number == 3 or self._number == 8 or self._number == 9 then
                NV.addAd()
            end
        end
    elseif __evtType == 2 then
        -- 每呼吸三下 眨一下眼睛
        if __moveId == self._breathAni1 then
            self._breathCount = self._breathCount + 1
            if self._breathCount == 3 then
                self:breathAct2()
            end
        end
    end
end


--------------------------
-- 触控
-------------------------- 

function M:onTouchBegan(x, y, touches)
    if self._touchLock then return end
    self:getScene()._menuBtn:barChange(false)
    self:clickAct()
    return SIGN_TOUCH_BEGAN_SWALLOWS 
end

function M:onTouchMoved(x, y, touches)
    --   
end

function M:onTouchEnded(x, y, touches)
    --  
end










----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


return M