--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-3-17

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字10用的奶油
----------------------
local M     = classSprite("N10Cream")
M.TAG       = "N10Cream"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
	-- 层
	self._layer 	= params.layer
	-- 数字节点
	self._numNode 	= params.numNode
    -- 在数字轨迹上坐标的索引
    self._posIndex  = params.posIndex
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
	self:display("num/write/num10/cream/c1.png")
    self._candleId  = math.random(1, 2)
    local candle    = D.img("num/write/num10/candle/candle"..self._candleId..".png"):to(self, 10)
                    :a(cc.p(0.5, 0)):p(self:cw()/2, self:ch()/2 - 10):scale(0.5):hide()
    local fire      = D.img("num/write/num10/candle/f1.png"):to(candle, 10)
                    :a(cc.p(0.5, 0.2)):p(candle:cw()/2, candle:ch()):hide()
    self._candle    = candle
    self._fire      = fire
end

-- 奶油显示出来
function M:showOut()
    if self._shown then return end
    self._shown = true
    self:scaleX(1.2):scaleY(1.5):show()

    self._numNode:playCreamEffect()

    if self._hideAction then
        self:stopAction(self._hideAction)
        self._hideAction = nil
    end

    local particle = P.new("particle/nienaiyou.plist"):to(self):p(self:cw()/2, 0)
        :scaleX(1/self:scaleX()):scaleY(1/self:scaleY())
        :setAutoRemoveOnFinish(true)

    self._showAction = A.line({
        -- {"scaleTo", 0.25, 1},
        {"image", "num/write/num10/cream/c", 6, 1/24},
        {"fn", function()
            self._showAction = nil
        end},
    }):at(self)
end

-- 隐藏回去
function M:hideBack()
    if not self._shown then return end
    self._shown = false

    if self._showAction then
        self:stopAction(self._showAction)
        self._showAction = nil
    end

    self._hideAction = A.line({
        {"scaleTo", 0.25, 0},
        {"hide"},
        {"fn", function()
            self._hideAction = nil
        end},
    }):at(self)
end


function M:showCandle()
    if not self._candleShowAct then
        self._numNode:playCandleEffect()
        local scale     = 0.5
        self._candlePy  = self._candle:py()
        self._candle:show():py(self._candlePy + 20):scale(scale)
        self._candleShowAct = A.line({
            {"jumpBy", 0.2, cc.p(0, -20), 10, 1},
            
        }):at(self._candle)

        self._numNode._candle:changeDisplay(self._candleId)
    end
end

function M:hideCanle()
    if self._candleShowAct then
        self._candle:stopAction(self._candleShowAct)
        self._candleShowAct = nil
        self._candle:hide():py(self._candlePy)
    end
end

-- 点火
function M:fireUp()
    self._fire:show()
    self._fire:cycle({
        {"image", "num/write/num10/candle/f", 10, math.random(12, 18)/ 100},
    })
end

-- 火扑灭
function M:firePutOut()
    self._fire:line({
        -- A.one({"image", "num/write/num10/candle/f", 10, math.random(12, 18)/ 100}):reverse(),
        {"scaleTo", 0.3, 0},
        {"hide"},
    })
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M