--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-12

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字10用用来拖的道具，挤奶油和蜡烛
----------------------
local M     = classSpriteTouch("N10Item")
M.TAG       = "N10Item"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._numNode 	= params.numNode
	-- 类型 1-奶油袋 2-蜡烛
	self._type 		= params.type_ or 1

	self._moveFunc 	= params.moveFunc
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
	if self._type == 1 then
		self:display("num/write/num10/cream/a1.png"):a(cc.p(0.4, 0.1))
		local particle = P.new("particle/naiyou.plist"):to(self):p(10, 10):hide()
		self._particle = particle
	else
		self:display("num/write/num10/candle/candle1.png")
	end
end

function M:enter()
	self._touchLock = true

	local scale = self._type == 1 and 1 or 2

	self:scale(0.3):line({
		A.union({
			{"rotateTo", 0.3, -40},
			{"scaleTo", 0.3, 1},
		}),
		{"fn", function()
			self._touchLock = false
			if self._type == 2 then
				self:bindTouch({
					rectFactorX = 2, rectFactorY = 2
				})
			end
		end},
	})
end


-- 播放动画
function M:playAni()
	if self._aniAction then return end
	if self._type == 1 then
		self:rotate(0)
		self._particle:show()
		self._aniAction = A.cycle({
			{"image", "num/write/num10/cream/a", 6, 1/24},
		}):at(self)
	else
		self:scale(1.3)
		-- self._aniAction = A.cycle({
		-- 	{"image", "num/write/num4/w", 2, 0.1},
		-- }):at(self)
	end
end

-- 
function M:changeDisplay(id)
	if self._type == 1 then return end
	self:display("num/write/num10/candle/candle"..id..".png")
end


-- 停止动画
function M:stopAni()
	if self._aniAction then
		self._particle:hide()
		self:rotate(-40)
		self:stopAction(self._aniAction)
		self._aniAction = nil
		self:loadDisplay()
	end

	if self._type == 2 then
		self:scale(1.0)
	end
end

function M:quit()
	self:remove()
end


function M:onTouchBegan(x, y, touches)
    if self._touchLock then return end
    self._touchLock = true
    self:playAni()
    self._numNode._canPlayEffect = true
    self._numNode:stopGuide()
    self:getScene()._menuBtn:barChange(false)
    return SIGN_TOUCH_BEGAN_SWALLOWS
end

function M:onTouchMoved(x, y, touches)
	if self._moveFunc then
		local dis = self._type == 1 and -50 or -30
		return self._moveFunc(x, y + dis)
	end
end

function M:onTouchEnded(x, y, touches)
	self:stopAni()
    self._touchLock = false
    if self._type == 1 then
    	self._numNode:creamBack()
    else
    	self._numNode:candleBack()
    end
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M