--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-1-21

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字1用的箱子
----------------------
local M     = classSpriteTouch("N1Box")
M.TAG       = "N1Box"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
	-- 显示的类型-1粉色箱子 -绿色箱子
	self._type 		= params.type_

	self._layer 	= params.layer

	self._numNode 	= params.numNode
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
	-- 拉链
	self:loadZipper()
	-- 拉链锁头
	self:loadFastener()

	self:openGuide()
end


-- 图片渲染
function M:loadDisplay()
	self:display("num/write/num1/box"..self._type.."/box.png")
end

-- 拉链
function M:loadZipper()
	local zPos 	= self._type == 1 and cc.p(121, 198) or cc.p(116, 150)

	local zipper = D.img("num/write/num1/box"..self._type.."/zipper1.png"):to(self, 20):p(zPos)
	self._zipper = zipper
end

-- 拉链锁头
function M:loadFastener()
	local fPos 		= self._type == 1 and cc.p(121 - 4, 280) or cc.p(111, 214)
	local fastener 	= D.img("num/write/num1/box"..self._type.."/fastener1.png"):to(self, 30):p(fPos)
	self._fastener 	= fastener

	local box 	= self
	fastener:bindTouch()
	--------------------------
	-- 触控
	-------------------------- 
	function fastener:onTouchBegan(x, y, touches)
    	self:getScene()._menuBtn:barChange(false)
		box:stopGuide()
		self._beganTouchY = y
		self._beganPointY = self:py()
		return SIGN_TOUCH_BEGAN_SWALLOWS
	end
	
	function fastener:onTouchMoved(x, y, touches)
		local dy 	= y - self._beganTouchY
		local newPy = box:convertToNodeSpace(cc.p(x, y)).y

		local maxY 	= box._type == 1 and 280 or 214
		local minY 	= box._type == 1 and 63 or 21

		newPy 		= newPy > self:py() and self:py() or ((newPy < minY) and minY or newPy)
		self:py(newPy)
		box:setZipper(newPy)
	end
	
	function fastener:onTouchEnded(x, y, touches)
		box:openGuide()
	end
end

-- 设置拉链的高度
function M:setZipper(py)
	local disPre 	= self._type == 1 and 22 or 20
	py 				= py + 10
	local index 	= 11 - math.ceil((py - 62) / disPre)
	index = index > 10 and 10 or index
	index = index < 1 and 1 or index
	self._zipper:display("num/write/num1/box"..self._type.."/zipper"..index..".png")


	self:playZipEffect(index)

	if index == 10 then
		self:zipperFinish()
	end
end

-- 拉拉链时的音效
function M:playZipEffect(index)
	if index == 10 then
		soundEffect:playEffectsfx14711009()
		if self._type == 2 then
			soundEffect:playEffectsfx14711014()
		end
	elseif index == 2 and self._zipEff ~= 1 then
		self._zipEff = 1
		soundEffect:playEffectsfx14711005()
		if self._type == 2 then
			soundEffect:playEffectsfx14711010()
		end
	elseif index == 4 and self._zipEff ~= 2 then
		self._zipEff = 2
		soundEffect:playEffectsfx14711006()
		if self._type == 2 then
			soundEffect:playEffectsfx14711011()
		end
	elseif index == 6 and self._zipEff ~= 3 then
		self._zipEff = 3
		soundEffect:playEffectsfx14711007()
		if self._type == 2 then
			soundEffect:playEffectsfx14711012()
		end
	elseif index == 8 and self._zipEff ~= 4 then
		self._zipEff = 4
		soundEffect:playEffectsfx14711008()
		if self._type == 2 then
			soundEffect:playEffectsfx14711013()
		end
	end
end


-- 拉链完成
function M:zipperFinish()
	self._fastener:unbindTouch()
	self._fastener:line({
		{"moveTo", 0.1, cc.p(self._fastener:px(), self._type == 1 and 63 or 21)},
		{"fn", function()
			self._zipper:display("num/write/num1/box"..self._type.."/zipper11.png")
			if self._type == 1 then
				self._numNode:drawZipperFinish()
			else
				self:loadCoin()
			end
		end},
	})
end

-- 金币
function M:loadCoin()
	local coin = D.img("num/write/num1/coin/coin1.png"):to(self, 40):p(self:cw()/2, 50)

	coin:line({
		{"image", "num/write/num1/coin/coin", 9, 1/12},
		{"fn", function()
			self._numNode:drawZipperFinish()
		end},
	})
end

-- 开启引导
function M:openGuide()
	self._numNode:loadFullMaskGuide()
    if not self._guideAct then
        local action = A.cycle({
            {"delay", 3},
            {"fn", function()
            	soundEffect:playEffectsfx14711003()
            	self._fastenerGuideAct = A.line({
            		{"image", "num/write/num1/box"..self._type.."/fastener", 8, 1/12},
            		{"fn", function()
        				self._fastener:display("num/write/num1/box"..self._type.."/fastener1.png")
            		end},
            	}):at(self._fastener)

            	local pos 		= self._fastener:point()
			    local finger    = require("app.shapefeed.node.GuideFinger").new({
			        startPos    = pos,
			        endPos      = cc.p(pos.x, pos.y - 100),
			    }):to(self, 1200):p(0, 0)
			    self._finger = finger
			    self._finger:openHandGuide(nil, nil, nil, 1)
			    self._finger:line({
			    	{"delay", 3},
			    	{"fn", function()
			    		if self._finger then
					        self._finger:closeHandGuide()
					        self._finger:remove()
					        self._finger = nil
					    end
			    	end},
			    })
            end},
            {"delay", 1},
        })
        self._guideAct  = action
        self:runAction(self._guideAct)
    end
end

-- 关闭引导
function M:stopGuide()
	self._numNode:stopFullMaskGuide()
    if self._guideAct then
        self:stopAction(self._guideAct)
        self._fastener:stopAction(self._fastenerGuideAct)
        self._fastener:display("num/write/num1/box"..self._type.."/fastener1.png")
        self._guideAct = nil
    end

    if self._finger then
        self._finger:closeHandGuide()
        self._finger:remove()
        self._finger = nil
    end
end



----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M