--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-1-21

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字2显示的灯管
----------------------
local M     = classSprite("N2Tube")
M.TAG       = "N2Tube"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 显示的类型-1直线 -2弧线
    self._type          = params.type_ or 1

    self._isHead        = params.isHead or false

    self._numNode       = params.numNode

    self._index         = params.index
    -- 灯光打开状态 0关闭 1打开
    self._showType      = 0
    -- 灯光颜色索引
    self._lightIndex    = 1

    self._posIndex      = 1
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()

    if self._isHead then
        self:setTexture("num/write/num2/tubehead.png")
        -- self:bindTouch()
    end
end


-- 图片渲染
function M:loadDisplay()
    self:setTexture("num/write/num2/tube"..self._type.."/tt.png")
end

function M:setTubeInfo(type_, pos, posIndex, rotate)
    if type_ ~= self._type and not self._isHead then
        self._type = type_
        self:loadDisplay()
    end
    self:setPosition(pos):setRotation(rotate)
    self._posIndex  = posIndex

    -- 头的角度单独计算
    if self._isHead then
        local rot = self._numNode._tubes[2]:rotate()
        if self._numNode._tubes[2]._type == 2 then rot = rot + 10 end
        self:setRotation(rot)
    end
end

-- 初始化灯光索引
function M:initLightIndex(index)
    self._lightIndex = index
end

-- 开灯
function M:onLight(isForce)
    if self._showType == 1 and not isForce then return end
    self._showType = 1
    if not isForce then
        self:playLightEffect()
    end

    local picIndex  = self._lightIndex <= 6 and self._lightIndex or (12-self._lightIndex)
    self:setTexture("num/write/num2/tube"..self._type.."/tl"..picIndex..".png")
end

-- 关灯
function M:offLight()
    if self._showType == 0 then return end
    self._showType = 0
    self:loadDisplay()
end

-- 跑马灯
function M:lightRun()
    self._lightIndex = self._lightIndex == 10 and 1 or (self._lightIndex + 1)
    self:onLight(true)
end

-- 点亮音效
function M:playLightEffect()
    if not self._numNode._canPlayEffect then return end

    local effectIndex
    if self._index >= 10 and self._index <= 22 then
        effectIndex = 23 - self._index
    elseif self._index >= 2 and self._index <= 6 then
        effectIndex = 20 - self._index
    end

    if effectIndex then
        if effectIndex < 10 then
            soundEffect:playSound("sfx14711101_0"..effectIndex)
        else
            soundEffect:playSound("sfx14711101_"..effectIndex)
        end
    end
end


--------------------------
-- 触控
-------------------------- 

function M:onTouchBegan(x, y, touches)
    if self._touchLock then return end
    self._touchLock = true
    self._numNode._canPlayEffect = true
    self._numNode:stopGuide()
    self:getScene()._menuBtn:barChange(false)

    return SIGN_TOUCH_BEGAN_SWALLOWS
end

function M:onTouchMoved(x, y, touches)
    return self._numNode:setTubePull(x, y)
end

function M:onTouchEnded(x, y, touches)
    self._numNode:tubeBack()
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end


return M