--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-12

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字4用的横条通道
----------------------
local M     = classSpriteTouch("N4Bar")
M.TAG       = "N4Bar"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..

local barPoints 	= {cc.p(105, 125), cc.p(138, 21), cc.p(28, 184)}

local barAnchors 	= {cc.p(0.5, 1), cc.p(0, 0.5), cc.p(0.5, 1)}



----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._numNode 	= params.numNode
	-- 横条 1-斜边 2-横边 3-竖边
	self._id 		= params.id
	-- 类型 1-泥土 2-水
	self._type 		= params.type_ or 1
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
	local showNode 	= nil
	if self._type == 1 then
		self:display("num/write/num4/bar/bar"..self._id..".png"):opacity(0):scale(0.985)
		showNode = D.img("num/write/num4/bar/bar"..self._id..".png"):to(self, 10)
	else
		self:display("num/write/num4/water/w"..self._id.."_1.png"):opacity(0):scale(0.95)
		showNode = D.img("num/write/num4/water/w"..self._id.."_1.png"):to(self, 10)
	end
	showNode:p(self:cw()/2, self:ch()/2)
	CUtil:changeAnchorP(showNode, barAnchors[self._id])

	self._showCw 	= showNode:cw()
	self._showCh 	= showNode:ch()

	if self._id == 2 then
		showNode:setTextureRect(cc.rect(0, 0, 0, self._showCh))
	else
		showNode:setTextureRect(cc.rect(0, 0, self._showCw, 0))
	end
	self._showNode = showNode
end

-- 设置显示的效果 #分点的世界坐标
function M:setDisplay(point)
	local showNode 	= self._showNode
	local lp 	= showNode:convertToNodeSpace(point)
	if self._id == 2 then
		showNode:setTextureRect(cc.rect(0, 0, lp.x, self._showCh))
	else
		showNode:setTextureRect(cc.rect(0, 0, self._showCw, showNode:ch() - lp.y))
	end
end

-- 显示面积设为零 不可见
function M:setRectZero()
	if self._id == 2 and self:cw() ~= 0 then
		self._showNode:setTextureRect(cc.rect(0, 0, 0, self._showCh))
	elseif self._id ~= 2 and self:ch() ~= 0 then
		self._showNode:setTextureRect(cc.rect(0, 0, self._showCw, 0))
	end
end

-- 全部显示
function M:setRectAll()
	if self._showNode:cw() ~= self._showCw or self._showNode:ch() ~= self._showCh then
		self._showNode:setTextureRect(cc.rect(0, 0, self._showCw, self._showCh))
	end
end

-- 水流动画
function M:waterRun()
	if self._type ~= 2 then return end
	self._showNode:setTextureRect(cc.rect(0, 0, self._showNode:cw(), self._showNode:ch()))
	self._showNode:cycle({
		{"image", "num/write/num4/water/w"..self._id.."_", 3, 0.1},
	})	
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M