--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-20

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字8用的鱼
----------------------
local M     = classArmatureTouch("N4Hamster")
M.TAG       = "N4Hamster"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..



-- 1打招呼(yanshu/dongkou) 2挖土的地鼠(pashu/pashu) 3钻出洞喷水的地鼠（dishu/tuboshu）
local files 	= {"bone/yanshu", "bone/pashu", "bone/dishu"}

local armatures = {"dongkou", "pashu", "tuboshu"}

--[[
	"yanshu_zuanchu", "yanshu_jindong"
	"watu", "yshu_huxi"
	"tuboshu_dong", "tuboshu_dong1", "tuboshu_dong2"
]]

----------------------
-- 构造方法
---------------------- 
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
	-- 层
	self._layer 	= params.layer
	-- 数字节点
	self._numNode 	= params.numNode


	self._id 		= params.id or 1

    self._file            = files[self._id]
    -- 字段[骨架名称]
    self._armatureName    = armatures[self._id]
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


function M:onEnterTransitionFinish()
	if self._id ~= 2 then
		self:unbindTouch()
	end
end


-- 出洞打招呼骨骼要裁切掉穿帮部分，重写创建骨骼对象方法
-- 创建骨骼对象
function M:createArmatureAndPlay()
    -- 其他骨骼无需裁切，调用基类方法
    if self._id ~= 1 then
        -- [超类调用]
        M.super.createArmatureAndPlay(self)
        return
    end

    local armature 
    if self._animName then
        armature     = DragonBone.createArmatureAndPlay(self._armatureName, self._animName)
    else
        armature     = DragonBone.createArmature(self._armatureName)
    end
    
    local size   = armature:getContentSize()
    local anchor = armature:getAnchorPoint()


    -- 正面裁切
    -- local clip     = cc.ClippingRectangleNode:create(cc.rect(0, 0, self:cw(), self:ch()))
    local clipSize = cc.rect(-20, 34, size.width + 40, size.height + 100)
    local clip     = cc.ClippingRectangleNode:create(clipSize)
    self._clip     = CCNodeExtend.extend(clip):to(self):bindTouch()

    armature:to(self._clip):p(size.width * anchor.x, size.height * anchor.y)

    self._armature = armature
end



-- 图片渲染
function M:loadDisplay()
	-- self:display("num/write/num5/leaf/l"..self._id..".png")
end


-- 钻出洞
function M:outHole()
	if self._id ~= 1 then return end
    soundEffect:playEffectsfx14711042()
	self:play("yanshu_zuanchu")
end

-- 钻进洞
function M:inHole()
	if self._id ~= 1 then return end
    soundEffect:playEffectsfx14711044()
	self:play("yanshu_jindong")
end



function M:breathAni()
	self:play("yshu_huxi")
end
M.stopAni = M.breathAni

function M:digAni()
	self:play("watu")
end


function M:outWater1()
	self:play("tuboshu_dong")
end

function M:outWater2()
	self:play("tuboshu_dong1")
end

function M:outWater3()
	self:play("tuboshu_dong2")
end


-- 动画回调动作
function M:movementHandler(__evtType, __moveId)
    if __evtType == 1 then
    	-- 鼹鼠两次钻出回调
        if __moveId == "yanshu_zuanchu" then
        	self._outCount = (self._outCount and self._outCount or 0) + 1
        	if self._outCount == 1 then
        		self._firstPos = self:point()
        		self._numNode:startDig()
        	elseif self._outCount == 2 then
        		self._numNode:secondDig()
        	end
        -- 鼹鼠两次进洞回调
        elseif __moveId == "yanshu_jindong" then
        	self._inCount = (self._inCount and self._inCount or 0) + 1
        	if self._inCount == 1 then
        		self:p(self._firstPos):hide()
        		self:line({
        			{"delay", 0.3},
        			{"show"},
        			{"fn", function()
        				self:outHole() 
        			end},
        		})
        	elseif self._inCount == 2 then
        		self:hide()
        		self._numNode:startPlayWater()
        	end


        elseif __moveId == "tuboshu_dong" then
        	self:outWater2()
        elseif __moveId == "tuboshu_dong1" then
        	self:outWater3()
        	self:line({
        		{"delay", 3},
        		{"fn", function()
					self._numNode._layer._numBone:writeFinish()
        		end},
        	})
        end
    end
end




function M:onTouchBegan(x, y, touches)
    if self._touchLock then return end
    self._touchLock = true
    self:digAni()
    self._numNode:stopGuide()
    self._numNode._canPlayEffect = true
    self:getScene()._menuBtn:barChange(false)
    return SIGN_TOUCH_BEGAN_SWALLOWS
end

function M:onTouchMoved(x, y, touches)
	return self._numNode:digging(x, y)
end

function M:onTouchEnded(x, y, touches)
	return self._numNode:hamsterBack(x, y)
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M