--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-12

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字4用用来拖的道具，地鼠和水花
----------------------
local M     = classSpriteTouch("N4Item")
M.TAG       = "N4Item"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._numNode 	= params.numNode
	-- 类型 1-地鼠 2-水花
	self._type 		= params.type_ or 1
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
	if self._type == 1 then
		self:display("num/write/num4/hamster/a1.png")
	else
		self:display("num/write/num4/w1.png")
	end
end

-- 播放动画
function M:playAni()
	if self._aniAction then return end
	if self._type == 1 then
		self._aniAction = A.cycle({
			{"image", "num/write/num4/hamster/a", 6, 0.1},
		}):at(self)
	else
		self._aniAction = A.cycle({
			{"image", "num/write/num4/w", 2, 0.1},
		}):at(self)
	end
end

-- 停止动画
function M:stopAni()
	if self._aniAction then
		self:stopAction(self._aniAction)
		self._aniAction = nil
		self:loadDisplay()
	end
end


function M:onTouchBegan(x, y, touches)
    if self._touchLock then return end
    self._touchLock = true
    self:playAni()
    self._numNode:stopGuide()
    self:getScene()._menuBtn:barChange(false)
    return SIGN_TOUCH_BEGAN_SWALLOWS
end

function M:onTouchMoved(x, y, touches)
	if self._type == 1 then
    	return self._numNode:digging(x, y)
    else
    	return self._numNode:pullWater(x, y)
	end
end

function M:onTouchEnded(x, y, touches)
	if self._type == 1 then
    	return self._numNode:hamsterBack(x, y)
    else
    	return self._numNode:waterBack(x, y)
	end
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M