--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-15

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字5用的叶子
----------------------
local M     = classSprite("N5Leaf")
M.TAG       = "N5Leaf"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
	-- 层
	self._layer 	= params.layer
	-- 数字节点
	self._numNode 	= params.numNode
	-- ID 3种叶子
	self._id 		= params.id
    -- 在数字轨迹上坐标的索引
    self._posIndex  = params.posIndex
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
	self:display("num/write/num5/leaf/l"..self._id..".png")
end

-- 长出
function M:showOut()
    if self._showed or self._posIndex == 1 then return end
    self._showed = true
    self:scale(0):show()
    self._numNode:playFlowerEffect()

    if self._retractAction then
        self:stopAction(self._retractAction)
        self._retractAction = nil
    end

    self._showAction = A.line({
        {"scaleTo", 0.25, 1},
        {"fn", function()
            self._showAction = nil
        end},
    }):at(self)
end

-- 缩回去
function M:retract()
    if not self._showed or self._posIndex == 1 then return end
    self._showed = false

    if self._showAction then
        self:stopAction(self._showAction)
        self._showAction = nil
    end

    self._retractAction = A.line({
        {"scaleTo", 0.25, 0},
        {"hide"},
        {"fn", function()
            self._retractAction = nil
        end},
    }):at(self)
end

-- 被吃
function M:eaten()
    if self._ate then return end
    self._ate = true
    self._numNode._wormHead:playEat()
    self._numNode:playEatEffect()
    if self._eatBackAction then
        self:stopAction(self._eatBackAction)
        self._eatBackAction = nil
    end

    self._eatAction = A.line({
        {"scaleTo", 0.25, 0},
        {"fn", function()
            self._eatAction = nil
        end},
        {"hide"},
    }):at(self)
end

-- 被吃复原
function M:eatBack()
    if not self._ate then return end
    self._ate = false

    if self._eatAction then
        self:stopAction(self._eatAction)
        self._eatAction = nil
    end

    self._eatBackAction = A.line({
        {"show"},
        {"scaleTo", 0.25, 1},
        {"fn", function()
            self._eatBackAction = nil
        end},
    }):at(self)
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M