--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-16

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字5用的虫子
----------------------
local M     = classSprite("N5Worm")
M.TAG       = "N5Worm"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
	self._layer 	= params.layer

	self._numNode 	= params.numNode

    -- self._file            = "bone/mmc"
    -- -- 字段[骨架名称]
    -- self._armatureName    = "Q2"

    self._id		= params.id or 1
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
	if self._id == 1 then
		DragonBone.addFile("bone/mmctou")
		local armature = DragonBone.createArmature("chon")
		local size   = armature:getContentSize()
		local anchor = armature:getAnchorPoint()

		armature:to(self):p(size.width * anchor.x, size.height * anchor.y)
		self:setContentSize(size)
		self._armature = armature

		self._armature.getAnimation():setMovementEventCallFunc(function (armature, movementType, movementID)
			if movementType == 1 then
				if movementID == "chi1" then
					self._armature:play("chi2", nil, {loop = 0})
				elseif movementID == "chi2" then
					self._armature:play("chi3")
				elseif movementID == "chi3" then
					self._eatting = false
				end
			end
		end)

	elseif self._id == 2 or self._id == 4 then
		self:display("num/write/num5/worm/mmc2.png")
	elseif self._id == 3 then
		self:display("num/write/num5/worm/mmc3.png")
	elseif self._id == 5 then
		self:display("num/write/num5/worm/mmc1.png")
	end
end

function M:playEat()
	if self._eatting then return end
	self._eatting = true
	self._armature:play("chi1")
end

-- 退出
function M:quit()
	for i=2, #self._numNode._worms do
		self._numNode._worms[i]:line({
			{"moveBy", 0.1 * (i-1), cc.p(20 * (i - 1), 0)},
			{"remove"},
		})
	end
	self:unscheduleUpdate()
	self._particle:remove()
	self:line({
		{"delay", 0.1 * (#self._numNode._worms-1)},
		{"scaleTo", 0.3, 0},
		{"remove"},
	})
end

function M:enter()
	self:scale(0):line({
		{"scaleTo", 0.3, 1},
		{"fn", function()
			self:bindTouch({rectFactorX = 2, rectFactorY = 2})
		end},
	})
end


function M:showParticle()
	if not self._particle then
	    local particle = P.new("particle/gemaizi.plist"):to(self):p(self:cw()/2, 0)
	    particle:setTexture(R.loadTexture("particle/10003.png"))
	    -- particle:setPositionType(cc.POSITION_TYPE_RELATIVE)
		-- self:scheduleUpdateWithPriorityLua(function ()
		-- 	if not self._posIndex then return end
		-- 	local points = self._numNode._allPathPts
		-- 	local index  = self._posIndex + 20 < #points and self._posIndex + 20 or #points
		-- 	particle:p(points[index]):rotate(self:rotate())
		-- end, 20)
		self._particle = particle
	else
		self._particle:show()
	end
end


function M:hideParticle()
	self._particle:hide()
end

--------------------------
-- 触控
-------------------------- 

function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	self._touchLock = true
	self._numNode._canPlayEffect = true
	self._numNode:stopGuide()
	self:showParticle()
    self:getScene()._menuBtn:barChange(false)
	return SIGN_TOUCH_BEGAN_SWALLOWS 
end

function M:onTouchMoved(x, y, touches)
	return self._numNode:wormEatLeaf(x, y)
end

function M:onTouchEnded(x, y, touches)
	self._numNode:wromBack()
end









----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M