--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-22

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字7用的光辉
----------------------
local M     = classSpriteTouch("N7GlowLight")
M.TAG       = "N7GlowLight"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..


----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._numNode 	= params.numNode
	-- 横条 1-横边 2-斜边
	self._id 		= params.id or 1
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


function M:onEnterTransitionFinish()
end


-- 图片渲染
function M:loadDisplay()
	self:setTexture("num/write/num7/light"..self._id..".png"):opacity(0)
end

function M:loadShowNode()
	local showNode  = D.img("num/write/num7/light"..self._id..".png"):to(self, 10):p(self:cw()/2, self:ch()/2)
	local lightNode = D.img("num/write/num7/light"..self._id.."b.png"):to(self:getParent(), self:z() - 10):p(self:p())
	if self._id == 1 then
		CUtil:changeAnchorP(showNode, cc.p(0, 0.5))
		CUtil:changeAnchorP(lightNode, cc.p(0, 0.5))
	else
		CUtil:changeAnchorP(showNode, cc.p(0.5, 1))
		CUtil:changeAnchorP(lightNode, cc.p(0.5, 1))
	end

	self._lCw = lightNode:cw()
	self._lCh = lightNode:ch()
	if self._id == 1 then
		showNode:setTextureRect(cc.rect(0, 0, 0, showNode:ch()))
		lightNode:setTextureRect(cc.rect(0, 0, 0, lightNode:ch()))
	else
		showNode:setTextureRect(cc.rect(0, 0, showNode:cw(), 0))
		lightNode:setTextureRect(cc.rect(0, 0, lightNode:cw(), 0))
	end
	self._showNode = showNode
	self._lightNode = lightNode
end



-- 设置显示的效果 #分点的世界坐标
function M:setDisplay(point)
	local lp1 		= self._showNode:convertToNodeSpace(point)
	local lp2 		= self._lightNode:convertToNodeSpace(point)
	if self._id == 1 then
		self._showNode:setTextureRect(cc.rect(0, 0, lp1.x, self._showNode:ch()))
		self._lightNode:setTextureRect(cc.rect(0, 0, lp2.x, self._lightNode:ch()))
	else
		self._showNode:setTextureRect(cc.rect(0, 0, self._showNode:cw(), self._showNode:ch() - lp1.y))
		self._lightNode:setTextureRect(cc.rect(0, 0, self._lightNode:cw(), self._lightNode:ch() - lp2.y))
	end
end

-- 显示全区域
function M:setRectAll()
	if self._showNode:cw() ~= self:cw() or self._showNode:ch() ~= self:ch() then
		self._showNode:setTextureRect(cc.rect(0, 0, self:cw(), self:ch()))
	end

	if self._lightNode:cw() ~= self._lCw or self._lightNode:ch() ~= self._lCh then
		self._lightNode:setTextureRect(cc.rect(0, 0, self._lCw, self._lCh))
	end
end

-- 显示面积设为0
function M:setRectZero()
	if self._id == 1 and self._showNode:cw() ~= 0 then
		self._showNode:setTextureRect(cc.rect(0, 0, 0, self:ch()))
		self._lightNode:setTextureRect(cc.rect(0, 0, 0, self._lCh))
	elseif self._id == 2 and self._showNode:ch() ~= 0 then
		self._showNode:setTextureRect(cc.rect(0, 0, self:cw(), 0))
		self._lightNode:setTextureRect(cc.rect(0, 0, self._lCw, 0))
	end
end

-- 水流动画
function M:waterRun()
	-- if self._type ~= 2 then return end
	-- self._showNode:setTextureRect(cc.rect(0, 0, self._showNode:cw(), self._showNode:ch()))
	-- self._showNode:cycle({
	-- 	{"image", "num/write/num4/water/w"..self._id.."_", 2, 0.2},
	-- })	
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M