--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-20

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字8用的鱼
----------------------
local M     = classArmatureTouch("N8Fish")
M.TAG       = "N8Fish"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
	-- 层
	self._layer 	= params.layer
	-- 数字节点
	self._numNode 	= params.numNode
    -- 在数字轨迹上坐标的索引
    self._posIndex  = params.posIndex
    -- 一开始所处的位置
    self._iniPos    = params.iniPos
    -- 所属水池的位置
    self._poolPos   = params.poolPos
    -- 所属索引位置 1-11
    self._index     = params.index

    self._file            = "bone/yuyuyu"
    -- 字段[骨架名称]
    self._armatureName    = "yu"
    -- 当前状态 1-水池外在路面上 2-水池里看不见 3-被吃了看不见
    self._state     = 2
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
    -- self:scale(0.5)
	-- self:display("num/write/num5/leaf/l"..self._id..".png")

end

-- 从水池跳出,显示在路上
function M:showInRoad()
    if self._state == 1 then return end
    local oldState  = self._state
    self._state = 1
    self:stopJumpAction()


    if oldState == 3 then
        self:p(self._iniPos):show()
        self._outAction = A.line({
            {"scaleTo", 0.2, 1},
            {"fn", function()
                self._outAction = nil
            end},
        }):at(self)
    else
        local particle = P.new("particle/tiaoyu.plist"):to(self:getParent(), self:z()):p(self._poolPos)
        particle:setAutoRemoveOnFinish(true)

        self:p(self._poolPos):show()
        self._outAction = A.line({
            {"jumpTo", 0.2, self._iniPos, 50, 1},
            {"fn", function()
                self._outAction = nil
            end},
        }):at(self)

        if self._index > 5 then
            local effectIndex = 14711071 + self._index
            soundEffect["playEffectsfx"..effectIndex](soundEffect)
        end
    end
end

-- 跳回水池
function M:inPool()
    if self._state == 2 then return end
    self._state = 2
    self:stopJumpAction()

    self:p(self._iniPos):scale(1):show()
    self._inAction = A.line({
        {"jumpTo", 0.2, self._poolPos, 50, 1},
        {"hide"},
        {"fn", function()
            self._inAction = nil
        end},
    }):at(self)
end

-- 被吃
function M:eaten()
    if self._state == 3 then return end
    self._state = 3
    self._numNode._penguin:pickAni()
    self:stopJumpAction()
    self._numNode:playFishEffect()
    self:hide()
    -- self._eatAction = A.line({
    --     {"scaleTo", 0.25, 0},
    --     {"fn", function()
    --         self._eatAction = nil
    --     end},
    --     {"hide"},
    -- }):at(self)
end

-- 停止动作
function M:stopJumpAction()
    if self._eatAction then
        self:stopAction(self._eatAction)
        self._eatAction = nil
    end

    if self._outAction then
        self:stopAction(self._outAction)
        self._outAction = nil
    end


    if self._inAction then
        self:stopAction(self._inAction)
        self._inAction = nil
    end
end

-- 被吃复原
-- function M:eatBack()
--     if not self._ate then return end
--     self._ate = false

--     if self._eatAction then
--         self:stopAction(self._eatAction)
--         self._eatAction = nil
--     end

--     self._eatBackAction = A.line({
--         {"scaleTo", 0.25, 1},
--         {"fn", function()
--             self._eatBackAction = nil
--         end},
--     }):at(self)
-- end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M