--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-3-16

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字9用的麦子
----------------------
local M     = classSprite("N9Wheat")
M.TAG       = "N9Wheat"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
	-- 层
	self._layer 	= params.layer
	-- 数字节点
	self._numNode 	= params.numNode
    -- 在数字轨迹上坐标的索引
    self._posIndex  = params.posIndex
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
	self:display("num/write/num9/wheat/a1.png"):a(cc.p(0.5, 0.2))
end

-- 麦子长出
function M:growUp()
    if self._grew then return end
    self._grew = true
    self:scale(0):show()

    if self._retractAction then
        self:stopAction(self._retractAction)
        self._retractAction = nil
    end

    self._numNode:playFlowerEffect()
    self._growAction = A.line({
        -- {"scaleTo", 0.25, 0.5},
        {"easing", "backout", {"scaleTo", 0.25, 1}},
        {"fn", function()
            self._growAction = nil
        end},
    }):at(self)
end

-- 麦子缩回去
function M:retract()
    if not self._grew then return end
    self._grew = false

    if self._growAction then
        self:stopAction(self._growAction)
        self._growAction = nil
    end

    -- 花缩回动作
    self._retractAction = A.line({
        {"scaleTo", 0.25, 0},
        {"hide"},
        {"fn", function()
            self._retractAction = nil
        end},
    }):at(self)
end

-- 被收割
function M:reaped()
    if self._reaped then return end
    self._reaped = true

    if self._noReapAction then
        self:stopAction(self._noReapAction)
        self._noReapAction = nil
    end

    -- 
    self._reapAction = A.line({
        {"image", "num/write/num9/wheat/a", 6, 1/24},
        {"fn", function()
            self._reapAction = nil
        end},
    }):at(self)
end

-- 逆收割
function M:notReap()
    if not self._reaped then return end
    self._reaped = false

    if self._reapAction then
        self:stopAction(self._reapAction)
        self._reapAction = nil
    end

    -- 
    self._noReapAction = A.line({
        A.one({"image", "num/write/num9/wheat/a", 6, 1/24}):reverse(),
        {"fn", function()
            self._noReapAction = nil
        end},
    }):at(self)
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M