--[[
Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-2-27

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类 -- 数字10
----------------------
local M    = class("Num10", require("app.numwrite.numpath.NumBase"))
M.TAG    = "Num10"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 数字
    self._number    = 10
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 写10的断点 第一条竖线后到了断点
    self._breakIndex    = #self._pathPoints[1]
end



-- 蛋糕显示出来
function M:loadWriteItem()
    local cake      = D.img("num/write/num10/cake.png"):to(self, 1):p(self:cw()/2, self:ch()/2 - 6):opacity(0)
    cake:line({
        {"fadeTo", 0.5, 255},
        {"fn", function()
            self._renderTexture:hide()
            self:loadCream()
            self:loadBag()
            self:openUpdate1()
            self:openGuide("_creamIndex", 80)
        end},
    })
end

-- 加载奶油
function M:loadCream()
    self._creams         = {}
    local interval      = 50
    local count         = math.ceil(#self._allPathPts / interval)
    local creamClass     = require("app.numwrite.numitem.N10Cream")
    for i=1,count do
        local index     = math.floor((i-1) * interval) + 1
        local point     = self._allPathPts[index]
        if index > self._breakIndex then
            point.y = point.y + 10
        end
        local cream     = creamClass.new({
            numNode     = self,
            posIndex    = index
        }):to(self, 500 - point.y):p(point):hide()
        table.insert(self._creams, cream)
    end
    self._creamIndex    = 1
end

-- 奶油袋
function M:loadBag()
    local bag = require("app.numwrite.numitem.N10Item").new({
        type_       = 1,
        numNode     = self,
        moveFunc    = function (x, y)
            return self:addCream(x, y)
        end
    }):to(self, 500):p(self._allPathPts[1])
    bag:enter()
    self._bag = bag
end

-- 添加奶油
function M:addCream(x, y)
    self._creamIndex, minDis = self:getNearestPtIndex(self._creamIndex, x, y)

    -- 离当前点最近的叶子索引和距离
    local newIndex, minDis = self:getNearestPtIndex(self._creamIndex, x, y)
    if not (self._addedOne and self._creamIndex <= self._breakIndex) then
        self._creamIndex = newIndex
    end

    -- 画完第一笔，走到竖线那里去
    if self._creamIndex >= self._breakIndex * 0.9 and not self._addedOne then
        self._addedOne = true
        self:bagToNextDot()
        return SIGN_TOUCH_MOVED_STOP
    elseif self._creamIndex >= #self._allPathPts * 0.9 then
        self:finishTask1()
        return SIGN_TOUCH_MOVED_STOP
    elseif minDis > 100 then
        self:creamBack()
        return SIGN_TOUCH_MOVED_STOP
    end
end

-- 任务一(挤奶油)更新
function M:openUpdate1()
    self:scheduleUpdateWithPriorityLua(function ()
        self._bag:p(cc.pAdd(self._allPathPts[self._creamIndex], cc.p(30, 20)))
        for i,v in ipairs(self._creams) do
            if v._posIndex <= self._creamIndex then
                v:showOut()
            else
                v:hideBack()
            end
        end
    end, 10)
end

-- 奶油音效
function M:playCreamEffect()
    if not self._canPlayEffect then return end
    -- 初始化
    if not self._creamEffects then
        self._creamEffects = {}
        self._cEffectIndex  = math.random(1, 3)
    end
    -- 声音的索引1-4循环
    self._cEffectIndex = self._cEffectIndex % 3 + 1
    -- 播放音效
    local soundEffects = {"sfx14711093", "sfx14711094", "sfx14711095"}
    local newSound     = soundEffect["playEffect"..soundEffects[self._cEffectIndex]](soundEffect)
    table.insert(self._creamEffects, newSound)

    -- 音效最多同时存在3个
    if #self._creamEffects == 3 then
        sound.stopSound(self._creamEffects[1])
        table.remove(self._creamEffects, 1)
    end
end


-- 奶油包到达下一个位置
function M:bagToNextDot()
    self:indexMoveTo("_creamIndex", self._breakIndex + 1, nil, true, function ()
        self._bag._touchLock = false
        self._canPlayEffect = false
        self._bag:stopAni()
        self:openGuide("_creamIndex", 80)
    end)
end


function M:creamBack()
    -- local endValue = self._addedOne and self._breakIndex + 1 or 1
    -- self:indexMoveTo("_creamIndex", endValue, nil, true, function ()
        self._bag._touchLock = false
        self._canPlayEffect = false
        self:openGuide("_creamIndex", 80)
    -- end)
end

-- 任务一(挤奶油)完成
function M:finishTask1()
    if self._finished1 then return end
    self._finished1 = true
    self:indexMoveTo("_creamIndex", #self._allPathPts, nil, true, function ()
        self:unscheduleUpdate()
        self._bag:quit()
        self:loadCandle()
        self:openUpdate2()
        self:openGuide("_candleIndex", 80)
    end)
end

-- 礼花粒子
function M:loadFireworksParticle()
    soundEffect:playEffectsfx14711102()
    local time      = 3
    local particle  = P.new("particle/lihua2.plist"):to(self._layer, 1000):p(V.w_2, V.h)
    particle:setAutoRemoveOnFinish(true)
    particle:line({
        {"delay", time},
        {"fn", function()
            particle:stopSystem() 
        end},
    })

    for i=3, 5 do
        local particle = P.new("particle/lihua2.plist"):to(self._layer, 1000):p(V.w_2, V.h)
        particle:setTexture(R.loadTexture("particle/lihua"..i..".png"))
        particle:setAutoRemoveOnFinish(true)
        particle:line({
            {"delay", time},
            {"fn", function()
                particle:stopSystem() 
            end},
        })
    end
end


-- 蜡烛
function M:loadCandle()
    self._candleIndex = 1
    local candle = require("app.numwrite.numitem.N10Item").new({
        type_       = 2,
        numNode     = self,
        moveFunc    = function (x, y)
            return self:addCandle(x, y)
        end
    }):to(self, 500):p(self._allPathPts[1])
    self._candle = candle
end

function M:addCandle(x, y)
    self._candleIndex, minDis = self:getNearestPtIndex(self._candleIndex, x, y)

    -- 离当前点最近的叶子索引和距离
    local newIndex, minDis = self:getNearestPtIndex(self._candleIndex, x, y)
    if not (self._candleOne and self._candleIndex <= self._breakIndex) then
        self._candleIndex = newIndex
    end

    -- 画完第一笔，走到竖线那里去
    if self._candleIndex >= self._breakIndex * 0.9 and not self._candleOne then
        self._candleOne = true
        self:candleToNextDot()
        return SIGN_TOUCH_MOVED_STOP
    elseif self._candleIndex >= #self._allPathPts * 0.9 then
        self:finishTask2()
        return SIGN_TOUCH_MOVED_STOP
    elseif minDis > 100 then
        self:candleBack()
        return SIGN_TOUCH_MOVED_STOP
    end
end

function M:candleBack()
    self._candle._touchLock = false
    self._canPlayEffect = false
    self:openGuide("_candleIndex", 80)
end



-- 任务一(挤奶油)更新
function M:openUpdate2()
    self:scheduleUpdateWithPriorityLua(function ()
        self._candle:p(cc.pAdd(self._allPathPts[self._candleIndex], cc.p(0, 10)))
        for i,v in ipairs(self._creams) do
            if v._posIndex <= self._candleIndex then
                v:showCandle()
            else
                v:hideCanle()
            end
        end
    end, 10)
end

-- 播放蜡烛出现音效
function M:playCandleEffect()
    if not self._canPlayEffect then return end
    if not self._candleEffects then
        self._candleEffects = {}
    end

    local newSound = soundEffect:playEffectsfx14711096()
    table.insert(self._candleEffects, newSound)
    if #self._candleEffects >= 5 then
        sound.stopSound(self._candleEffects[1])
        table.remove(self._candleEffects, 1)
    end


    -- if self._candleEffect then
    --     sound.voiceFadeOut(self._candleEffect, 0.3)
    -- end
    -- self._candleEffect = soundEffect:playEffectsfx14711096()
end


function M:candleToNextDot()
    self:indexMoveTo("_candleIndex", self._breakIndex + 1, nil, true, function ()
        self._candle._touchLock = false
        self._canPlayEffect = false
        self:openGuide("_candleIndex", 80)
    end)
end

function M:finishTask2()
    if self._finished2 then return end
    self._finished2 = true
    self:indexMoveTo("_candleIndex", #self._allPathPts, nil, true, function ()
        self:unscheduleUpdate()
        self._candle:quit()
        self:candleFire()
    end)
end

-- 点燃蜡烛
function M:candleFire()
    local index = 0
    self:line({
        A.cycle({
            {"delay", 0.1},
            {"fn", function()
                index = index + 1
                self._creams[index]:fireUp()
                if index == 1 then
                    soundEffect:playEffectsfx14711097()
                end
            end},
        }, #self._creams),
        {"fn", function()
            self:loadFireworksParticle()
            self._layer._numBone:writeFinish()
        end},
    })
end

-- 蜡烛熄灭
function M:candlePutOut()
    table.sort(self._creams, function (a, b)
        return a:px()< b:px()
    end)

    for i,v in ipairs(self._creams) do
        v:line({
            {"delay", 0.05 * i},
            {"fn", function()
                v:firePutOut()
            end},
        })
    end

end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)

    return true
end

function M:onTouchMoved(x, y, touches)


end

function M:onTouchEnded(x, y, touches)

end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end

return M