--[[
Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-2-27

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类 -- 数字2
----------------------
local M    = class("Num2", require("app.numwrite.numpath.NumBase"))
M.TAG    = "Num2"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..

local CalculatePoint    = require("app.numwrite.CalculatePoint")


----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 数字
    self._number    = 2
    -- 画笔的粗细
    self._strokeSize = 1
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
end


-- 重写轨迹表生成（去除第四段延长线）
function M:loadRenderOtherwise()
    -- 轨迹路径，分段
    self._pathPoints    = CalculatePoint["calculatePoint"..self._number](CalculatePoint)

    local newPts        = {self._pathPoints[1], self._pathPoints[2], self._pathPoints[3]}

    -- 将所有轨迹路径合并成一个表
    self._allPathPts    = CalculatePoint:mergeVectorTable(newPts)
    -- 笔头的纹理
    self._strokeTex     = R.loadTexture("num/write/stokehead.png")
end


-- 加载形似书写
function M:loadWriteItem()
    -- 重新计算生成轨迹表，包含延长段
    T.clear(self._allPathPts)
    self._allPathPts    = CalculatePoint:mergeVectorTable(self._pathPoints)

    self._renderSprite:line({
        {"delay", 0.5},
        {"fn", function()
            self._clip:bindTouch()
            self:loadTube()
            self:openTubeUpdate()
            self._effectIndex = self._pIndex
            self:indexMoveTo("_pIndex", 100, 200, false, function ()
                self._tubeHead:bindTouch({
                    rectFactorX = 2,
                    rectFactorY = 2
                })
                self:openGuide("_pIndex")
            end)
        end},
    })
end

-- 加载灯管
function M:loadTube()
    local dot   = D.img("num/write/num2/dot.png"):to(self, 200):p(38, 21):scale(1.2)
    local head  = D.img("num/write/num2/head.png"):to(self, 200):p(22, 193)
    -- 拐点索引
    self._dotIndex      = #self._pathPoints[1]

    self._tubes         = {}
    local scaleSize     = 1.05
    local interval      = 28 * scaleSize
    local count         = math.ceil(#self._allPathPts / interval)
    local tubeClass     = require("app.numwrite.numitem.N2Tube")
    for i=1,count do
        local tube      = tubeClass.new({
            type_       = 1,
            isHead      = i == 1 and true or false,
            numNode     = self,
            index       = i
        }):to(self._clip, 100):scale(scaleSize)
        table.insert(self._tubes, tube)
    end

    self._tubeHead  = self._tubes[1]
    self._interval  = interval
    self._pIndex    = 1
end

-- 开启灯管信息更新
function M:openTubeUpdate()
    local tubeLength    = #self._tubes
    self:scheduleUpdateWithPriorityLua(function ()
        for i,v in ipairs(self._tubes) do
            -- 当前灯管对应的坐标索引
            local index = math.floor((i - tubeLength) * self._interval + self._pIndex)
            local tube  = self._tubes[tubeLength - i + 1]
            if self._allPathPts[index] then
                -- 计算当前坐标角度
                local rot
                if self._allPathPts[index + 1] then
                    rot = -PT.quadAngle(self._allPathPts[index], self._allPathPts[index + 1])+90
                else
                    rot = -PT.quadAngle(self._allPathPts[index - 1], self._allPathPts[index])+90
                end
                -- 对应的类型
                local type_     = index < self._dotIndex and 2 or 1
                -- 设置类型、坐标、坐标索引、角度
                tube:show():setTubeInfo(type_, self._allPathPts[index], index, rot)
            else
                tube:hide()
            end
        end

        if math.abs(self._pIndex - self._effectIndex) >= self._interval and self._canPlayEffect then
            if self._tubeEffect then
                sound.stopSound(self._tubeEffect)
            end
            self._tubeEffect = soundEffect:playEffectsfx14711024()
            self._effectIndex = self._pIndex
        end
    end, 20)
end

-- 设置坐标
function M:setTubePull(x, y)
    self._pIndex, minDis = self:getNearestPtIndex(self._pIndex, x, y)

    if self._pIndex >= #self._allPathPts * 0.9 then
        self:pullTubeFinish()
        return SIGN_TOUCH_MOVED_STOP
    elseif minDis > 100 then
        self:tubeBack()
        return SIGN_TOUCH_MOVED_STOP
    end
end

-- 管转回
function M:tubeBack()
    self._tubeHead._touchLock = false
    self._canPlayEffect = false
    self:openGuide("_pIndex")
end

--拉出灯管完成
function M:pullTubeFinish()
    if self._isArrveTubeFinish then return end
    self._isArrveTubeFinish = true
    self._canPlayEffect = false

    self:indexMoveTo("_pIndex", #self._allPathPts, nil, true, function ()
        self:unscheduleUpdate()
        self:loadTubeLight()
        self._layer._numBone:eyeTwinkleAct()
        soundEffect:playEffectsfx14711025()
    end)
end

-- 加载灯管上的灯光
function M:loadTubeLight()
    self._clip:unbindTouch()
    self._inLight       = true                  -- 在点亮灯光的阶段
    self._lightIndex    = 1                     -- 灯光的索引
    self._nowLightTube  = 1                     -- 当前亮起的灯的索引
    for i,v in ipairs(self._tubes) do
        v:initLightIndex(self:getLightIndex())
    end

    self:openLightUpdate()
    self:indexMoveTo("_nowLightTube", 100, 200, false, function ()
        self:openGuide("_nowLightTube", 60)
    end)
end

-- 灯光更新
function M:openLightUpdate()
    self:scheduleUpdateWithPriorityLua(function ()
        for i,v in ipairs(self._tubes) do
            if v._posIndex <= self._nowLightTube then
                v:onLight()
            else
                v:offLight()
            end
        end
    end, 20)
end

-- 点亮坐标
function M:lightPosition(x, y)
    self._nowLightTube, minDis = self:getNearestPtIndex(self._nowLightTube, x, y)
    if self._nowLightTube >= #self._allPathPts * 0.9 then
        self:lightTubeFinish()
        return SIGN_TOUCH_MOVED_STOP
    elseif minDis > 100 then
        self:lightBack()
        return SIGN_TOUCH_MOVED_STOP
    end
end

-- 灯退回
function M:lightBack()
    self._lightTouchLock = false
    self._canPlayEffect = false
    self:openGuide("_nowLightTube", 60)
end


-- 循环顺序 1-2-3-4-5-6-5-4-3-2 一共10个一轮
function M:getLightIndex()
    self._lightIndex    = self._lightIndex == 10 and 1 or (self._lightIndex + 1)
    return self._lightIndex
end

-- 点亮灯完成
function M:lightTubeFinish()
    if self._lightedTube then return end
    self._lightedTube = true

    -- 点灯进度移动到终点
    self:indexMoveTo("_nowLightTube", #self._allPathPts, nil, true, function ()
        self:unscheduleUpdate()
        soundEffect:playEffectsfx14711026()
        self._layer._numBone:writeFinish()
        self:cycle({
            {"delay", 0.1},
            {"fn", function()
                for i,v in ipairs(self._tubes) do
                    v:lightRun()
                end
            end},
        })
    end)
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
-- 此处是执行点亮灯管的逻辑
function M:onTouchBegan(x, y, touches)
    if not self._inLight then return end
    if PT.d(self._allPathPts[self._nowLightTube], self:convertToNodeSpace(cc.p(x, y))) > 100 then return end
    if self._lightTouchLock then return end
    self._lightTouchLock = true
    self._canPlayEffect = true
    self:stopGuide()
    self:getScene()._menuBtn:barChange(false)

    return true
end

function M:onTouchMoved(x, y, touches)
    return self:lightPosition(x, y)
end

function M:onTouchEnded(x, y, touches)
    self:lightBack()
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end

return M