--[[
Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-2-27

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类 -- 数字3
----------------------
local M    = class("Num3", require("app.numwrite.numpath.NumBase"))
M.TAG    = "Num3"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 数字
    self._number    = 3

    self._flowers   = {}
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
end


-- 加载形似书写
function M:loadWriteItem()
    local soil = D.img("num/write/num3/stroke3.png"):to(self):p(self:cw()/2 - 2, self:ch()/2):opacity(0)
    for i=1,5 do
        local a = R.loadTexture("num/write/num3/flower/f"..i..".png")
    end
    soil:line({
        {"fadeTo", 0.5, 255},
        {"fn", function()
            self._renderTexture:hide()
            self:loadFlowers()
            self:loadPot()
        end},
    })
end

-- 加载花花们
function M:loadFlowers()
    -- local interval      = 30
    -- local count         = math.ceil(#self._allPathPts / interval)\

    local flowerClass   = require("app.numwrite.numitem.N3Flower")

    local fPoints = {
        cc.p(42, 228), cc.p(58, 244), cc.p(45, 275), cc.p(78, 261), cc.p(76, 297), 
        cc.p(53, 63), cc.p(103, 308), cc.p(144, 304), cc.p(120, 285), cc.p(192, 251), 
        cc.p(165, 228), cc.p(147, 252), cc.p(170, 280), cc.p(29, 100), cc.p(165, 180), 
        cc.p(134, 202), cc.p(187, 209), cc.p(28, 57), cc.p(83, 177), cc.p(107, 160), 
        cc.p(83, 40), cc.p(130, 166), cc.p(206, 123), cc.p(178, 139), cc.p(175, 107), 
        cc.p(150, 136), cc.p(159, 66), cc.p(193, 74), cc.p(185, 39), cc.p(133, 43), 
        cc.p(157, 17), cc.p(100, 5), cc.p(125, 20), cc.p(60, 21),
    }

    for i=1, #fPoints do
        local index     = self:getMinIndex(fPoints[i])
        local point     = fPoints[i] or cc.p(i* 20 - 300, 0)
        local flower    = flowerClass.new({
            numNode     = self,
            posIndex    = index,
            id          = math.random(1, 3)
        }):to(self, 500 - point.y):p(point):hide()
        table.insert(self._flowers, flower)
        -- flower:bindTouchLocate()
        -- flower:bloom()
    end
    self._flowerIndex   = 1
    self:openFlowerUpdate()

    self:openGuide("_flowerIndex", 50)

    -- local node = D.img("common/help.png"):to(self, 999):p(400, 1)

    -- node:bindTouch()
    -- function node:onTouchBegan()
    --     CUtil:printNodeTable(self:getParent()._flowers)
    -- end
end

-- 浇水壶
function M:loadPot()
    local pot = require("app.numwrite.numitem.N3Pot").new({
        numNode     = self
    }):to(self, 600):p(self._allPathPts[1])
    self._pot = pot
end

-- 画画坐标更新
function M:openFlowerUpdate()
    self._effectIndex = self._flowerIndex
    self:scheduleUpdateWithPriorityLua(function (dt)
        for i,v in ipairs(self._flowers) do
            if v._posIndex <= self._flowerIndex + 50 then
                v:bloom()
            else
                v:retract()
            end
        end
        self._pot:p(cc.pAdd(self._allPathPts[self._flowerIndex], cc.p(50, 40)))

        self:playWaterEffect(dt)
    end, 20)
end

-- 浇水音效
function M:playWaterEffect(dt)
    if not self._waterSoundEffect then
        self._waterSoundEffect = soundEffect:playEffectsfx14711032()
        self._waterVolume       = 0
    end

    if math.abs(self._flowerIndex - self._effectIndex) < 1 then
        self._waterVolume = self._waterVolume - dt * 2
    else
        self._waterVolume = self._waterVolume + dt * 2
    end

    if self._waterVolume < 0 then
        self._waterVolume = 0
    elseif self._waterVolume > 1 then
        self._waterVolume = 1
    end

    local volume = self._waterVolume
    if not self._canPlayEffect then
        volume = 0
    end
    sound.setVolume(self._waterSoundEffect, volume)

    self._effectIndex = self._flowerIndex
end

function M:stopWaterEffect()
    if self._waterSoundEffect then
        sound.stopSound(self._waterSoundEffect)
        self._waterSoundEffect = nil
    end
end



-- 浇花，#水的坐标
function M:waterFlowers(x, y)
    self._flowerIndex, minDis = self:getNearestPtIndex(self._flowerIndex, x - 40, y)
    if minDis > 100 then
        self:potGoBack()
        return SIGN_TOUCH_MOVED_STOP
    elseif self._flowerIndex >= #self._allPathPts * 0.9 then
        self:flowersBloomFinish()
        return SIGN_TOUCH_MOVED_STOP
    end
end

-- 水壶退回
function M:potGoBack()
    -- self:indexMoveTo("_flowerIndex", 1, nil, true, function ()
        self._pot:stopAni()
        self._pot._touchLock = false
        self._canPlayEffect = false
        self:openGuide("_flowerIndex", 50)
    -- end)
end

-- 花全开
function M:flowersBloomFinish()
    if self._flowersBloomed then return end
    self._flowersBloomed = true
    self._canPlayEffect = false

    self:indexMoveTo("_flowerIndex", #self._allPathPts, nil, true, function ()
        self:unscheduleUpdate()
        self:stopWaterEffect()
        self._flowers[#self._flowers]:bloom()
        self._pot:quit()
        self:line({
            {"delay", 1},
            {"fn", function()
                self:loadPluckHand()
                self:openHandUpdate()
                self:openGuide("_handIndex", 50)
            end},
        })
    end)
end

-- 摘花的手
function M:loadPluckHand()
    local hand      = D.img("num/write/num3/hand.png"):to(self, 600):p(self._allPathPts[1])
    self._hand      = hand
    self._handIndex = 1

    self:showHandParticle()
    self:loadSkyParticle()
    hand:bindTouch()
    function hand:onTouchBegan(x, y, touches)
        if self._touchLock then return end
        self:getParent()._canPlayEffect = true
        self:getScene()._menuBtn:barChange(false)
        self:getParent():stopGuide()
        self:scale(0.8)
        self:getParent():showHandParticle()
        self._touchLock = true
        return SIGN_TOUCH_BEGAN_SWALLOWS 
    end
    
    function hand:onTouchMoved(x, y, touches)
        return self:getParent():pluckFlowers(x, y)
    end
    
    function hand:onTouchEnded(x, y, touches)
        self:getParent():handGoBack()
    end
end

-- 手摘花调度更新
function M:openHandUpdate()
    self:scheduleUpdateWithPriorityLua(function ()
        for i,v in ipairs(self._flowers) do
            if v._posIndex <= self._handIndex then
                v:petalDrop()
            else
                v:petalRecovery()
            end
        end
        self._hand:p(cc.pAdd(self._allPathPts[self._handIndex], cc.p(20, -20)))
    end, 20)
end

-- 花花声音
function M:playFlowerEffect()
    if not self._canPlayEffect then
        return
    end

    -- 初始化
    if not self._flowerEffects then
        self._flowerEffects = {}
        self._fEffectIndex  = math.random(1, 4)
    end
    -- 声音的索引1-4循环
    self._fEffectIndex = self._fEffectIndex % 4 + 1
    -- 播放音效
    local soundEffects = {"sfx14711033", "sfx14711034", "sfx14711035", "sfx14711036"}
    local newSound     = soundEffect["playEffect"..soundEffects[self._fEffectIndex]](soundEffect)
    table.insert(self._flowerEffects, newSound)

    -- 音效最多同时存在3个
    if #self._flowerEffects == 4 then
        sound.stopSound(self._flowerEffects[1])
        table.remove(self._flowerEffects, 1)
    end
end


-- 摘花
function M:pluckFlowers(x, y)
    self._handIndex, minDis = self:getNearestPtIndex(self._handIndex, x - 20, y + 20)
    if minDis > 100 then
        self:handGoBack()        
        return SIGN_TOUCH_MOVED_STOP
    elseif self._handIndex >= #self._allPathPts * 0.9 then
        self:flowersPluckFinish()
        return SIGN_TOUCH_MOVED_STOP
    end
end

-- 手退回起点
function M:handGoBack()
    -- self:indexMoveTo("_handIndex", 1, nil, true, function ()
        self:openGuide("_handIndex")
        self._hand:scale(1.0)
        self:showHandParticle()
        self._hand._touchLock = false
        self._canPlayEffect = false
    -- end)
end

-- 花花采摘完成
function M:flowersPluckFinish()
    if self._flowersPlucked then return end
    self._flowersPlucked = true
    self._canPlayEffect = false
    self:indexMoveTo("_handIndex", #self._allPathPts, nil, true, function ()
        self:removeHandParticle()
        self:unscheduleUpdate()
        self._flowers[#self._flowers]:petalDrop()
        self._hand:line({
            {"scaleTo", 0.3, 0},
            {"fn", function()
                self._layer._numBone:writeFinish()
            end},
            {"remove"},
        })
    end)
end

-- 手上的粒子
function M:showHandParticle()
    if self._handParticles then
        for i=1,3 do
            self._handParticles[i]:show()
        end
    else
        self._handParticles = {}
        local ptl1  = P.new("particle/huaban.plist"):to(self._hand, -1):p(76, 145):hide()
        local ptl2  = P.new("particle/huaban.plist"):to(self._hand, -1):p(76, 145):hide()
        local ptl3  = P.new("particle/huaban.plist"):to(self._hand, -1):p(76, 145):hide()
        ptl2:setTexture(R.loadTexture("particle/10001.png"))
        ptl3:setTexture(R.loadTexture("particle/10002.png"))

        table.insert(self._handParticles, ptl1)
        table.insert(self._handParticles, ptl2)
        table.insert(self._handParticles, ptl3)
    end
end

function M:hideHandParticle()
    if self._handParticles then
        for i=1,3 do
            self._handParticles[i]:hide()
        end
    end
end

-- 移除手的粒子
function M:removeHandParticle()
    for i,v in ipairs(self._handParticles) do
        v:stopSystem()
        v:line({
            {"delay", 3},
            {"remove"},
        })
    end
    self._handParticles = nil
end

function M:loadSkyParticle()
    local ptl1  = P.new("particle/huaban2.plist"):to(self._layer, 9100):p(V.w_2, V.h)--:hide()
    local ptl2  = P.new("particle/huaban2.plist"):to(self._layer, 9100):p(V.w_2, V.h)--:hide()
    local ptl3  = P.new("particle/huaban2.plist"):to(self._layer, 9100):p(V.w_2, V.h)--:hide()
    ptl2:setTexture(R.loadTexture("particle/10001.png"))
    ptl3:setTexture(R.loadTexture("particle/10002.png"))
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)
    return true
end

function M:onTouchMoved(x, y, touches)


end

function M:onTouchEnded(x, y, touches)

end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end

return M