--[[
Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-2-27

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类 -- 数字4
----------------------
local M    = class("Num4", require("app.numwrite.numpath.NumBase"))
M.TAG    = "Num4"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 数字
    self._number 	= 4
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)

	-- 写4的断点
	self._breakIndex = #self._pathPoints[1] + #self._pathPoints[2]
end

-- 加载形似书写
function M:loadWriteItem()
    local soil = D.img("num/write/num4/soil.png"):to(self):p(self:cw()/2, self:ch()/2):opacity(0)

    soil:line({
        {"fadeTo", 0.5, 255},
        {"fn", function()
            self._renderTexture:hide()
            self:loadBar()
            self:loadHole()
            self:loadHamster()
        end},
    })
    self._soil = soil
end

-- 加载横条
function M:loadBar()
	self._bar 		= {}
	local barClass 	= require("app.numwrite.numitem.N4Bar")
	local points 	= {cc.p(99, 231), cc.p(133, 115), cc.p(197, 184)}
	for i=1,3 do
		local bar = barClass.new({
			id 		= i,
			numNode = self
		}):to(self, 100):p(points[i])--:bindTouchLocate()
		table.insert(self._bar, bar)
	end

	-- 三条线的分界点，用来判断操作那一条线
	self._barPartIndex = {}
	for i,v in ipairs(self._pathPoints) do
		table.insert(self._barPartIndex, #v + (self._barPartIndex[i-1] or 0))
	end
end

-- 孔
function M:loadHole()
	local points 	= {cc.p(183, 330 + 8), cc.p(233 + 7, 121 - 1), cc.p(194 + 2, 38 - 12)}
	self._holes 	= {}
	for i=1,3 do
		local hole 	= D.img("num/write/num4/hole/h1.png"):to(self, 200):p(points[i]):scale(1.5)
		local holeB = D.img("num/write/num4/hole/h2.png"):to(self, 300):p(points[i].x, points[i].y - 12 * 1.5):scale(1.5)
		hole._holeB = holeB
		table.insert(self._holes, hole)
	end
end

-- 加载地鼠
function M:loadHamster()
	-- 挖洞的鼹鼠
	local holeHamster = require("app.numwrite.numitem.N4Hamster").new({
		numNode 	= self,
		id 			= 1
	}):to(self, 400):p(182, 314 + 4 + 8):scale(0.92)
	self._holeHamster 	= holeHamster
	holeHamster:outHole()

	-- 趴土的鼹鼠
	local hamster = require("app.numwrite.numitem.N4Hamster").new({
		numNode 	= self,
		id 			= 2
	}):to(self, 400):p(self._allPathPts[1]):hide()
	self._hamster 	= hamster
end

function M:startDig()
	self._holeHamster:hide():bindTouch({rectFactorX = 1.5, rectFactorY = 1.5})
	self._hamster:show():breathAni()
	self._hIndex = 20
	self:openHamsterUpdate()
	self:openGuide("_hIndex")
end

-- 地鼠信息调度更新
function M:openHamsterUpdate()
	hamster = self._hamster
	-- 地鼠每帧刷新位置及角度
	self:scheduleUpdateWithPriorityLua(function (dt)
		local rotation 	
		if self._allPathPts[self._hIndex + 1] then
			rotation = -PT.quadAngle(self._allPathPts[self._hIndex], self._allPathPts[self._hIndex + 1]) + 90
		else
			rotation = -PT.quadAngle(self._allPathPts[self._hIndex - 1], self._allPathPts[self._hIndex]) + 90
		end
		hamster:p(self._allPathPts[self._hIndex]):rotate(rotation)

		if self._hIndex <= self._barPartIndex[1] then
			self._bar[1]:setDisplay(hamster:worldpoint())
			self._bar[2]:setRectZero()
			self._bar[3]:setRectZero()
		elseif self._hIndex <= self._barPartIndex[2] then
			self._bar[2]:setDisplay(hamster:worldpoint())
			self._bar[1]:setRectAll()
			self._bar[3]:setRectZero()
		else
			self._bar[3]:setDisplay(hamster:worldpoint())
			self._bar[1]:setRectAll()
			self._bar[2]:setRectAll()
		end

		self:playDigEffect(dt)
	end, 10)
end

-- 地鼠挖洞音效
function M:playDigEffect(dt)
    if not self._digSoundEffect then
        self._digSoundEffect = soundEffect:playEffectsfx14711043()
        self._digVolume      = 0
        self._dEffectIndex 	   = self._hIndex
    end

    if math.abs(self._hIndex - self._dEffectIndex) < 1 then
        self._digVolume = self._digVolume - dt * 2
    else
        self._digVolume = self._digVolume + dt * 10
    end

    if self._digVolume < 0 then
        self._digVolume = 0
    elseif self._digVolume > 1 then
        self._digVolume = 1
    end

    local volume = self._digVolume
    if not self._canPlayEffect then
    	volume = 0
    end

    sound.setVolume(self._digSoundEffect, volume)
    self._dEffectIndex = self._hIndex
end


-- 挖洞
function M:digging(x, y)
	local newIndex, minDis = self:getNearestPtIndex(self._hIndex, x, y)
	if not (self._diggedOne and self._hIndex <= self._barPartIndex[2]) then
		self._hIndex = newIndex
	end

	-- 画完第一笔，走到竖线那里去
	if self._hIndex >= self._barPartIndex[2] * 0.9 and not self._diggedOne then
		self._diggedOne = true
		self:hamsterMoveToNextHole()
		return SIGN_TOUCH_MOVED_STOP
	elseif self._hIndex >= self._barPartIndex[3] * 0.9 then
		self:digHoleFinish()
		return SIGN_TOUCH_MOVED_STOP
	elseif minDis > 100 then
		self:hamsterBack()
		return SIGN_TOUCH_MOVED_STOP
	end
end

-- 地鼠移动到下一个洞口
function M:hamsterMoveToNextHole()
	self:indexMoveTo("_hIndex", self._barPartIndex[2], nil, false, function ()
		self._canPlayEffect = false
		self._hamster:hide()
		self._holeHamster:p(233 + 7, 106 - 1):show()
		self._holeHamster:inHole()
	end)
end

-- 到第二画开挖
function M:secondDig()
	self._holeHamster:hide()
	self._hamster:show():breathAni()
	self._hamster._touchLock = false
	self._hIndex = self._barPartIndex[2] + 50
    self:openGuide("_hIndex")
end

-- 
function M:hamsterBack()
	self._hamster._touchLock = false
	self._canPlayEffect = false
	self._hamster:stopAni()
    self:openGuide("_hIndex")
end

-- 挖洞完成
function M:digHoleFinish()
	if self._diggedHole then return end
	self._diggedHole = true
	self:indexMoveTo("_hIndex", self._barPartIndex[3], nil, true, function ()
		self:unscheduleUpdate()
		self._hamster:hide()
		self._holeHamster:p(194 + 2, 23 - 12):show():inHole()
		sound.stopSound(self._digSoundEffect)
	end)
end

-- 开始玩水
function M:startPlayWater()
	self:loadWaters()
	self:loadWaterHead()
	self:openWaterUpdate()
	self:indexMoveTo("_waterIndex", 50, 200, false, function ()
		self:openGuide("_waterIndex")
	end)
end

-- 加载水水
function M:loadWaters()
	for i,v in ipairs(self._holes) do
		v:z(210)
		v._holeB:z(190)
	end

	self._water 	= {}

	local barClass 	= require("app.numwrite.numitem.N4Bar")
	local points 	= {cc.p(101, 229), cc.p(133, 116), cc.p(197, 185)}
	for i=1,3 do
		local water = barClass.new({
			id 		= i,
			type_ 	= 2,
			numNode = self
		}):to(self, 200 + 400):p(points[i])
		table.insert(self._water, water)
	end
	self._waterIndex 	= 1
end

-- 加载水头
function M:loadWaterHead()
	local waterHead = require("app.numwrite.numitem.N4Item").new({
		numNode 	= self,
		type_ 		= 2
	}):to(self, 500 + 400):p(self._allPathPts[1]):bindTouch({rectFactorX = 1.5, rectFactorY = 1.5})
	self._waterHead = waterHead
end


-- 水调度更新
function M:openWaterUpdate()
	-- 水流每帧刷新位置及角度
	self:scheduleUpdateWithPriorityLua(function (dt)
		local rotation 	
		if self._allPathPts[self._waterIndex + 1] then
			rotation = -PT.quadAngle(self._allPathPts[self._waterIndex], self._allPathPts[self._waterIndex + 1]) + 90
		else
			rotation = -PT.quadAngle(self._allPathPts[self._waterIndex - 1], self._allPathPts[self._waterIndex]) + 90
		end
		self._waterHead:p(self._allPathPts[self._waterIndex]):rotate(rotation)

		if self._waterIndex <= self._barPartIndex[1] then
			self._water[1]:setDisplay(self._waterHead:worldpoint())
			self._water[2]:setRectZero()
			self._water[3]:setRectZero()
		elseif self._waterIndex <= self._barPartIndex[2] then
			self._water[2]:setDisplay(self._waterHead:worldpoint())
			self._water[1]:setRectAll()
			self._water[3]:setRectZero()
		else
			self._water[3]:setDisplay(self._waterHead:worldpoint())
			self._water[1]:setRectAll()
			self._water[2]:setRectAll()
		end

		self:playWaterEffect(dt)
	end, 10)
end

-- 水花拖动音效
function M:playWaterEffect(dt)
    if not self._waterSoundEffect then
        self._waterSoundEffect = soundEffect:playEffectsfx14711045()
        self._waterVolume      = 0
        self._wEffectIndex 	   = self._waterIndex
    end

    if math.abs(self._waterIndex - self._wEffectIndex) < 1 then
        self._waterVolume = self._waterVolume - dt * 2
    else
        self._waterVolume = self._waterVolume + dt * 2
    end

    if self._waterVolume < 0 then
        self._waterVolume = 0
    elseif self._waterVolume > 1 then
        self._waterVolume = 1
    end

    sound.setVolume(self._waterSoundEffect, self._waterVolume)
    self._wEffectIndex = self._waterIndex
end


-- 拉出水流
function M:pullWater(x, y)
	local newIndex, minDis = self:getNearestPtIndex(self._waterIndex, x, y)
	if not (self._pulledOne and self._waterIndex <= self._barPartIndex[2]) then
		self._waterIndex = newIndex
	end

	-- 画完第一笔，走到竖线那里去
	if self._waterIndex >= self._barPartIndex[2] * 0.9 and not self._pulledOne then
		self._pulledOne = true
		self:waterMoveToNextHole()
		return SIGN_TOUCH_MOVED_STOP
	elseif self._waterIndex >= self._barPartIndex[3] * 0.9 then
		self:pullWaterFinish()
		return SIGN_TOUCH_MOVED_STOP
	elseif minDis > 100 then
		self:waterBack()
		return SIGN_TOUCH_MOVED_STOP
	end
end

-- 水移动到下一个洞口
function M:waterMoveToNextHole()
	self:indexMoveTo("_waterIndex", self._barPartIndex[2] + 50, nil, false, function ()
		self._waterHead._touchLock = false
		self._canPlayEffect =false
		self._waterHead:stopAni()
		self:openGuide("_waterIndex")
	end)
end

function M:waterBack()
	self._waterHead._touchLock = false
	self._canPlayEffect = false
	self._waterHead:stopAni()
	self:openGuide("_waterIndex")
end

-- 水拉出完成
function M:pullWaterFinish()
	if self._pulledWater then return end
	self._pulledWater = true
	self:indexMoveTo("_hIndex", self._barPartIndex[3], nil, true, function ()
		self:unscheduleUpdate()
		sound.stopSound(self._waterSoundEffect)
		for i,v in ipairs(self._water) do
			v:waterRun()
		end
		self._waterHead:line({
			{"fadeTo", 0.3, 0},
			{"remove"},
		})


		-- 玩水的鼹鼠
		local waterHamster = require("app.numwrite.numitem.N4Hamster").new({
			numNode 	= self,
			id 			= 3
		}):to(self, 400):p(400, 50)--:hide()
		self._waterHamster 	= waterHamster
		waterHamster:outWater1()
		soundEffect:playEffectsfx14711046()
	end)
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)

    return true
end

function M:onTouchMoved(x, y, touches)


end

function M:onTouchEnded(x, y, touches)

end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end

return M