--[[
Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-2-27

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类 -- 数字5
----------------------
local M    = class("Num5", require("app.numwrite.numpath.NumBase"))
M.TAG    = "Num5"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 数字
    self._number 	= 5
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 写5的断点 第一条竖线加弧线为第一笔 之后到了断点
    self._breakIndex    = #self._pathPoints[1] + #self._pathPoints[2]
end

-- 加载形似书写
function M:loadWriteItem()
    local wood = D.img("num/write/num5/stroke3.png"):to(self):p(self:cw()/2, self:ch()/2 + 6):opacity(0)
    wood:line({
        {"fadeTo", 0.5, 255},
        {"fn", function()
            self._renderTexture:hide()
            self:loadVines()
            self:loadLeaves()
            self:openLeafUpdate()
            self:indexMoveTo("_leafIndex", 60, nil, false, function ()
                self:openLeafGuide("_leafIndex", 80)
                self._inGrowLeaf    = true  -- 在叶子生长阶段
            end)
        end},
    })
end


-- 开启引导 #索引键值
function M:openLeafGuide(keyName, dis, speed, callback)
    if not self._guideAct then
        self._preGuideIndex = self[keyName]
        self._nowGuideKey   = keyName
        
        dis         = dis or 30
        speed       = speed or 150
        local time  = dis / speed
        local action = A.cycle({
            {"delay", 0.5},
            {"fn", function()
                self:loadGuideFinger(keyName, dis)
                if callback then
                    callback()
                else
                    soundEffect:playEffectsfx14700011()
                end
                -- 播放配音[v147169]写一写
                soundVoice:playEffectv147169()
                self._guideNode = self:indexMoveTo(keyName, self._preGuideIndex + dis, speed, true)
            end},
            {"delay", time},
            {"fn", function()
                self._guideNode = self:indexMoveTo(keyName, self._preGuideIndex, speed, true) 
            end},
            {"delay", time},
            {"fn", function()
                self._guideNode = self:indexMoveTo(keyName, self._preGuideIndex + dis, speed, true)
            end},
            {"delay", time},
            {"fn", function()
                self._guideNode = self:indexMoveTo(keyName, self._preGuideIndex, speed, true) 
            end},
            {"delay", time},
            {"delay", 2.5},
        })
        self._guideAct  = action
        self:runAction(self._guideAct)
    end
end



-- 藤蔓
function M:loadVines()
    self._vines  = {}
    local points = {cc.p(78, 270), cc.p(79, 195), cc.p(159, 187), cc.p(189, 92), cc.p(141, 40), cc.p(42, 57), cc.p(150, 310)}
    local rotate = {32, -44, -40, 40, 40, 130, 28}
    local scaleX = {0.9, 1.1, 0.9, 0.9, 0.9, 0.75, 1.35}

    for i,v in ipairs(points) do
        local vine = D.img("num/write/num5/vine.png"):to(self, 10)
            :p(v):rotate(rotate[i]):scaleX(scaleX[i]):hide()

        local pIndex = self:getMinIndex(v)
        vine._pIndex = pIndex
        table.insert(self._vines, vine)
    end
end


-- 叶子
function M:loadLeaves()
    self._clip:unbindTouch()
    self._leaves        = {}    -- 树叶表
    self._leafBreak     = 0     -- 中断处树叶索引

    local interval      = 25
    local count         = math.ceil(#self._allPathPts / interval)
    local leafClass     = require("app.numwrite.numitem.N5Leaf")
    for i=1,count do
        local index     = math.floor((i-1) * interval) + 1
        local point     = cc.pAdd(self._allPathPts[index], cc.p(math.random(-5, 5), math.random(-5, 5)))
        local rotation  = -PT.quadAngle(self._allPathPts[index], self._allPathPts[index + 1]) + 90

        local leaf      = leafClass.new({
            numNode     = self,
            posIndex    = index,
            id          = math.random(1, 3)
        }):to(self, 500 - point.y):p(point):rotate(math.random(1, 360)):hide()
        table.insert(self._leaves, leaf)

        if index < self._breakIndex then
            self._leafBreak = self._leafBreak + 1
        end
    end
    self._leafIndex   = 1
    self._leaves[1]:show()
end

-- 开启树叶调度更新
function M:openLeafUpdate()
    self:scheduleUpdateWithPriorityLua(function ()
        for i,v in ipairs(self._leaves) do
            if v._posIndex <= self._leafIndex then
                v:showOut()
            else
                v:retract()
            end
        end

        local vineIndex = 0
        for i,v in ipairs(self._vines) do
            if v._pIndex < self._leafIndex then
                vineIndex = i
            end
        end

        for i=1, vineIndex + 1 do
            if self._vines[i] then
                self._vines[i]:show()
            end
        end
    end, 10)
end

-- 虫子吃声音
function M:playEatEffect()
    if not self._canPlayEffect then return end
    -- 每三片叶子播一遍吃的声音
    self._callCount = (self._callCount or 0) + 1
    if self._callCount % 3 == 0 then
        soundEffect["playEffectsfx1471105"..math.random(1, 3)](soundEffect)
    end
end


-- 花花声音
function M:playFlowerEffect()
    if not self._canPlayEffect then return end
    -- 初始化
    if not self._flowerEffects then
        self._flowerEffects = {}
        self._fEffectIndex  = math.random(1, 4)
    end
    -- 声音的索引1-4循环
    self._fEffectIndex = self._fEffectIndex % 4 + 1
    -- 播放音效
    local soundEffects = {"sfx14711033", "sfx14711034", "sfx14711035", "sfx14711036"}
    local newSound     = soundEffect["playEffect"..soundEffects[self._fEffectIndex]](soundEffect)
    table.insert(self._flowerEffects, newSound)

    -- 音效最多同时存在3个
    if #self._flowerEffects == 4 then
        sound.stopSound(self._flowerEffects[1])
        table.remove(self._flowerEffects, 1)
    end
end


-- 长出叶子
function M:growLeaves(x, y)
    -- 离当前点最近的叶子索引和距离
    local newIndex, minDis = self:getNearestPtIndex(self._leafIndex, x, y)
    if not (self._grewOne and self._leafIndex <= self._breakIndex) then
        self._leafIndex = newIndex
    end

    -- 画完第一笔，走到竖线那里去
    if self._leafIndex >= self._breakIndex * 0.9 and not self._grewOne then
        self._grewOne = true
        self:leafToNextDot()
        return SIGN_TOUCH_MOVED_STOP
    elseif self._leafIndex >= #self._allPathPts * 0.9 then
        self:leafGrowFinish()
        return SIGN_TOUCH_MOVED_STOP
    elseif minDis > 100 then
        self:leafBack()        
        return SIGN_TOUCH_MOVED_STOP
    end
end

-- 叶子退回去
function M:leafBack()
    if not self._inGrowLeaf then return end
    self:openLeafGuide("_leafIndex", 80)
    self._canPlayEffect = false
end

-- 到下一笔开头
function M:leafToNextDot()
    self:indexMoveTo("_leafIndex", self._breakIndex + 40, nil, false, function ()
        self:openLeafGuide("_leafIndex", 80)
        self._canPlayEffect = false
    end)
end

-- 叶子生长完成
function M:leafGrowFinish()
    if self._grewLeaf then return end
    self:stopGuide()
    self._grewLeaf      = true
    self._inGrowLeaf    = false

    self:indexMoveTo("_leafIndex", #self._allPathPts, nil, false, function ()
        self:unscheduleUpdate()
        self:line({
            {"delay", 1},
            {"fn", function()
                self:loadWorm()
                self:openWormUpdate()
                self:openGuide("_wromIndex")
            end},
        })
    end)
end

-- 吃树叶的虫子
function M:loadWorm()

    self._worms         = {}
    local interval      = 20
    local wormClass     = require("app.numwrite.numitem.N5Worm")
    for i=1,5 do
        local tube      = wormClass.new({
            id          = i,
            numNode     = self
        }):to(self, 500 - i):p(1, 1)
        table.insert(self._worms, tube)
    end
    self._wormHead  = self._worms[1]
    self._wormHead:enter()
    self._wromIndex = 1

    soundEffect:playEffectsfx14711050()
end

-- 开启虫子调度更新
function M:openWormUpdate()
    self:scheduleUpdateWithPriorityLua(function ()
        -- 当前索引-最大取全路径的前一个点
        local nowIndex = self._allPathPts[self._wromIndex + 1] and self._wromIndex or #self._allPathPts - 1
        local rotation = -PT.quadAngle(self._allPathPts[nowIndex], self._allPathPts[nowIndex + 1]) + 90
        self._wormHead:p(self._allPathPts[nowIndex]):rotate(rotation)
        self._wormHead._posIndex = nowIndex

        for i=2, #self._worms do
            local index = nowIndex - ((i-1) * 20)
            if self._allPathPts[index] and not (self._ateOne and index <= self._breakIndex) then
                local rotation = -PT.quadAngle(self._allPathPts[index], self._allPathPts[index + 1]) + 90
                self._worms[i]:show():p(self._allPathPts[index]):rotate(rotation)
            else
                self._worms[i]:hide()
            end
        end

        for i,v in ipairs(self._leaves) do
            if v._posIndex <= self._wromIndex then
                v:eaten()
            else
                v:eatBack()
            end
        end
    end, 10)
end


-- 虫子吃叶子
function M:wormEatLeaf(x, y)
    -- 离当前点最近的叶子索引和距离
    local newIndex, minDis = self:getNearestPtIndex(self._wromIndex, x, y)
    if not (self._ateOne and self._wromIndex <= self._breakIndex) then
        self._wromIndex = newIndex
    end

    -- 画完第一笔，走到竖线那里去
    if self._wromIndex > self._breakIndex * 0.9 and not self._ateOne then
        self._ateOne = true
        self:wromToNextDot()
        return SIGN_TOUCH_MOVED_STOP
    elseif self._wromIndex >= #self._allPathPts * 0.95 then
        self:wromEatFinish()
        return SIGN_TOUCH_MOVED_STOP
    elseif minDis > 100 then
        self:wromBack()
        return SIGN_TOUCH_MOVED_STOP
    end
end



-- 虫子退回
function M:wromBack()
    -- local endValue = self._ateOne and self._breakIndex + 40 or 1
    -- self:indexMoveTo("_wromIndex", endValue, nil, true, function ()
        self:openGuide("_wromIndex", 80)
        self._wormHead:hideParticle()
        self._wormHead._touchLock = false
        self._canPlayEffect = false
    -- end)
end

-- 
function M:wromToNextDot()
    self:indexMoveTo("_wromIndex", self._breakIndex + 40, 200, false, function ()
        self._wormHead:hideParticle()
        self:openGuide("_wromIndex", 80)
        self._wormHead._touchLock = false
        self._canPlayEffect = false
    end)
end

-- 虫子吃完
function M:wromEatFinish()
    if self._ateFinish then return end
    self._ateFinish = true

    self:indexMoveTo("_wromIndex", #self._allPathPts + 60, 200, false, function ()
        self:unscheduleUpdate()
        self._wormHead:quit()
        self:line({
            {"delay", 1},
            {"fn", function()
                self._layer._numBone:writeFinish()
            end},
        })
    end)
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)
    if self._inGrowLeaf and self._allPathPts[self._leafIndex] 
        and PT.d(self._allPathPts[self._leafIndex], self:convertToNodeSpace(cc.p(x, y))) < 100 then
            self:stopGuide()
            self._canPlayEffect = true
            self:getScene()._menuBtn:barChange(false)
            return true
    end
end

function M:onTouchMoved(x, y, touches)
    return self:growLeaves(x, y)
end

function M:onTouchEnded(x, y, touches)
    self:leafBack()
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end

return M