--[[
Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-2-27

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类 -- 数字6
----------------------
local M    = class("Num6", require("app.numwrite.numpath.NumBase"))
M.TAG    = "Num6"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 数字
    self._number 	= 6
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
end


-- 加载形似书写
function M:loadWriteItem()
    local stone = D.img("num/write/num6/stroke3.png"):to(self, 1):p(self:cw()/2, self:ch()/2 + 8):opacity(0)
    stone:line({
        {"fadeTo", 0.5, 255},
        {"fn", function()
            self._renderTexture:hide()
            self:loadRails()
            self:loadStationAndMoutain()
            self:openRailUpdate()
            self:indexMoveTo("_rIndex", 100, 300, true, function ()
                self._railHead:bindTouch({rectFactorX = 2, rectFactorY = 3})
                self:openGuide("_rIndex")
            end)
        end},
    })
end

-- 加载铁路
function M:loadRails()
    self._rails         = {}
    -- 拐点索引
    self._dotIndex      = #self._pathPoints[1]

    local interval      = 28
    local count         = math.ceil(#self._allPathPts / interval)
    local tubeClass     = require("app.numwrite.numitem.N6Rail")
    for i=1,count do
        local tube      = tubeClass.new({
            type_       = 1,
            isHead      = i == 1 and true or false,
            numNode     = self
        }):to(self, 100):p(self._allPathPts[1])
        table.insert(self._rails, tube)
    end

    self._rIndex    = 1
    self._railHead  = self._rails[1]
    self._interval  = interval

end

-- 铁轨信息更新
function M:openRailUpdate()
    local railLength    = #self._rails
    self._preRailIndex  = self._rIndex
    self:scheduleUpdateWithPriorityLua(function ()
        for i,v in ipairs(self._rails) do
            -- 当前灯管对应的坐标索引
            local index = math.floor((i - railLength) * self._interval + self._rIndex)
            local rail  = self._rails[railLength - i + 1]
            if self._allPathPts[index] and index < #self._allPathPts - 60 then
                -- 计算当前坐标角度
                local rot
                if self._allPathPts[index + 1] then
                    rot = -PT.quadAngle(self._allPathPts[index], self._allPathPts[index + 1])+90
                else
                    rot = -PT.quadAngle(self._allPathPts[index - 1], self._allPathPts[index])+90
                end
                -- 对应的类型
                local type_     = index < self._dotIndex and 2 or 1
                -- 设置类型、坐标、坐标索引、角度
                rail:show():setRailInfo(type_, self._allPathPts[index], index, rot)
            else
                rail:hide()
            end
        end

        if self._preRailIndex ~= self._rIndex then
            self:playRailEffect()
            self._preRailIndex = self._rIndex
        end
    end, 20)
end

-- 铁路声音
function M:playRailEffect()
    if not self._canPlayEffect then return end
    if not self._railEffect then
        self._railEffect = soundEffect:playEffectsfx14711059()
        self:line({
            {"delay", sound:soundDuration("sfx14711059", 2)},
            {"fn", function()
                self._railEffect = nil
            end},
        })
    end
end

-- 加载火车站和山
function M:loadStationAndMoutain()
	local station = D.img("num/write/num6/station.png")
		:to(self, 990):p(139, 331):scale(1.1)--:bindTouchLocate()

	local moutain = D.img("num/write/num6/moutain.png")
		:to(self, 990):p(159, 205):scale(1.1)--:bindTouchLocate()
end


function M:buildRail(x, y)
    self._rIndex, minDis = self:getNearestPtIndex(self._rIndex, x, y)

    if self._rIndex >= #self._allPathPts * 0.8 then
        self:railBuildFinish()
        return SIGN_TOUCH_MOVED_STOP
    elseif minDis > 100 then
        self:railBack()
        return SIGN_TOUCH_MOVED_STOP
    end
end

-- 铁路倒回
function M:railBack()
    self._railHead._touchLock = false
    self._canPlayEffect = false
    self:openGuide("_rIndex")
end

-- 铺铁路完成
function M:railBuildFinish()
    if self._builtRail then return end
    self._builtRail = true
    self._canPlayEffect = false
    self:indexMoveTo("_rIndex", #self._allPathPts, nil, true, function ()
        self:unscheduleUpdate()
        self:loadTrain()
        self:openTrainUpdate()
        self:indexMoveTo("_trainIndex", 100, 300, true, function ()
            self._trains[1]:bindTouch({rectFactorX = 1.5, rectFactorY = 1.5})
            self:openGuide("_trainIndex")
        end)
    end)
end

-- 加载火车
function M:loadTrain()
    self._trains = {}
    local trainClass = require("app.numwrite.numitem.N6Train")
    for i=1,5 do
        local train = trainClass.new({
            id      = i,
            isHead  = i == 1 and true or false,
            numNode = self,
        }):to(self, 110):p(self._allPathPts[1])
        table.insert(self._trains, train)
    end
    self._trainIndex = 1
end

-- 火车跑
function M:trainRun(x, y)
    self._trainIndex, minDis = self:getNearestPtIndex(self._trainIndex, x, y)

    if self._trainIndex >= #self._allPathPts * 0.8 then
        self:trainFinish()
        return SIGN_TOUCH_MOVED_STOP
    elseif minDis > 100 then
        self:trainBack()
        return SIGN_TOUCH_MOVED_STOP
    end
end

-- 火车倒回
function M:trainBack()
    self._trains[1]._touchLock = false
    self._canPlayEffect = false
    self:openGuide("_trainIndex")
end

-- 火车泡完
function M:trainFinish()
    if self._trained then return end
    self._trained = true
    self:indexMoveTo("_trainIndex", #self._allPathPts + 500, nil, true, function ()
        self:unscheduleUpdate()
        self._trains[1]._particle:stopSystem()
        self:line({
            {"delay", 0.5},
            {"fn", function()
                self._layer._numBone:writeFinish()
            end},
        })
    end)
end

-- 开启火车更新
function M:openTrainUpdate()
    local trainLength   = #self._trains
    local trainInterval = 60
    self._preTrainIndex  = self._trainIndex
    self:scheduleUpdateWithPriorityLua(function ()
        for i,v in ipairs(self._trains) do
            -- 当前灯管对应的坐标索引
            local index = math.floor((i - trainLength) * trainInterval + self._trainIndex)
            local train  = self._trains[trainLength - i + 1]
            if self._allPathPts[index] and index < #self._allPathPts - 60 then
                -- 计算当前坐标角度
                local rot
                if self._allPathPts[index + 1] then
                    rot = -PT.quadAngle(self._allPathPts[index], self._allPathPts[index + 1])+90
                else
                    rot = -PT.quadAngle(self._allPathPts[index - 1], self._allPathPts[index])+90
                end
                -- 对应的类型
                local type_     = index < self._dotIndex and 2 or 1
                -- 设置类型、坐标、坐标索引、角度
                train:show():p(self._allPathPts[index]):rotate(rot)
            else
                train:hide()
            end
        end

        if self._preTrainIndex ~= self._trainIndex then
            self:playTrainEffect()
            self._preTrainIndex = self._trainIndex
        end
    end, 20)
end


-- 铁路声音
function M:playTrainEffect()
    if not self._canPlayEffect then return end
    if not self._trainEffect then
        self._trainEffect = soundEffect:playEffectsfx14711061()
        self:line({
            {"delay", sound:soundDuration("sfx14711061", 2)},
            {"fn", function()
                self._trainEffect = nil
            end},
        })
    end
end





----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)

    return true
end

function M:onTouchMoved(x, y, touches)


end

function M:onTouchEnded(x, y, touches)

end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end

return M