--[[
Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-2-27

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类 -- 数字7
----------------------
local M    = class("Num7", require("app.numwrite.numpath.NumBase"))
M.TAG    = "Num7"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 数字
    self._number 	= 7
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
end


-- 加载形似书写
function M:loadWriteItem()
    local soil = D.img("num/write/num7/stroke3.png"):to(self, 1):p(self:cw()/2, self:ch()/2):opacity(0)
    soil:line({
        {"fadeTo", 0.5, 255},
        {"fn", function()
            self._renderTexture:hide()
            self:loadStones()
            self:loadSpade()
            self:openStoneUpdate()
            self:openGuide("_stoneIndex", 110)
        end},
    })
end

-- 加载石头
function M:loadStones()
    self._stones     = {}

    local stonePos   = {cc.p(51, 312), cc.p(120, 317), cc.p(184, 309), cc.p(170, 252),
                        cc.p(129, 197), cc.p(119, 139), cc.p(97, 95), cc.p(60, 39)}
    local stoneScale = {0.75, 0.6, 0.6, 0.6, 0.7, 0.55, 0.5, 0.6}
    local isFlippedX = {false, false, true, false, true, false, false, true}

    local stoneClass = require("app.numwrite.numitem.N7Stone")

    for i,v in ipairs(stonePos) do
        local stone = stoneClass.new({
            numNode     = self,
        }):to(self, 500 - v.y):p(v):scale(stoneScale[i]):flipX(isFlippedX[i])--:bindTouchLocate()
        local index = self:getMinIndex(cc.pAdd(v, cc.p(0, 40)))
        stone._posIndex = index
        table.insert(self._stones, stone)
    end       
    self._stoneIndex    = 1
    self._tubeHead      = self._stones[1]

    -- local interval      = 70
    -- local count         = math.ceil(#self._allPathPts / interval)
    -- local stoneClass    = require("app.numwrite.numitem.N7Stone")
    -- for i=1,count do
    --     local index     = math.floor((i-1) * interval) + 10
    --     local point     = cc.pAdd(self._allPathPts[index], cc.p(0, -20))
    --     local stone     = stoneClass.new({
    --         numNode     = self,
    --         id          = i % 2 + 1,
    --         posIndex    = index
    --     }):to(self, 500 - point.y):p(point):scale(0.75)
    --     table.insert(self._stones, stone)
    -- end
    -- self._stoneIndex    = 1
    -- self._tubeHead      = self._stones[1]
end

-- 加载铲子
function M:loadSpade()
    local spade = require("app.numwrite.numitem.N7Spade").new({
        numNode    = self
    }):to(self, 500):p(self._allPathPts[1]):scale(0.6)
    self._spade = spade
end

function M:openStoneUpdate()
    self:scheduleUpdateWithPriorityLua(function ()
        if self._allPathPts[self._stoneIndex] then
            self._spade:p(cc.pAdd(self._allPathPts[self._stoneIndex], cc.p(30, 30)))
        end
        for i,v in ipairs(self._stones) do
            if v._posIndex <= self._stoneIndex then
                v:breakUp()
            else
                v:restore()
            end
        end
    end, 10)
end

function M:knockStone(x, y)
    self._stoneIndex, minDis = self:getNearestPtIndex(self._stoneIndex, x, y)
    if minDis > 100 then
        self:spadeBack()        
        return SIGN_TOUCH_MOVED_STOP
    elseif self._stoneIndex >= #self._allPathPts * 0.9 then
        self:knockFinish()
        return SIGN_TOUCH_MOVED_STOP
    end
end

-- 铲子回去
function M:spadeBack()
    -- self:indexMoveTo("_stoneIndex", 1, nil, true, function ()
        self._spade:stopAni()
        self:openGuide("_stoneIndex", 110)
        self._canPlayEffect = false
    -- end)
end

-- 石头破碎声音 
function M:playBreakEffect()
    if not self._canPlayEffect then return end
    -- 初始化
    if not self._breakEffects then
        self._breakEffects = {}
    end

    if #self._breakEffects < 3 then
        local newSound     = soundEffect:playEffectsfx14711068()
        table.insert(self._breakEffects, newSound)
        self:line({
            {"delay", sound:soundDuration("sfx14711068", 1)},
            {"fn", function()
                T.removeOrder(self._breakEffects, newSound)
            end},
        })
    end
end

-- 敲石头完成
function M:knockFinish()
    if self._knocked then return end
    self._knocked = true
    self._canPlayEffect = false
    self:indexMoveTo("_stoneIndex", #self._allPathPts, nil, true, function ()
        self:unscheduleUpdate()
        self:line({
            {"delay", 0.5},
            {"fn", function()
                self._spade:quit()
                self:loadGlowworm()
                self:loadGlowLight()
                self:openGlowUpdate()
                self:openGuide("_glowIndex", 80)
            end},
        })
    end)
end

-- 萤火虫
function M:loadGlowworm()
    local worm = require("app.numwrite.numitem.N7Glowworm").new({
        numNode     = self,
    }):to(self, 600):p(self._allPathPts[1]):flipX(true):scale(1.5)
    self._glowworm = worm
end

-- 光辉粒子
function M:loadGlowParticl()
    if self._glowParticle then return end
    local particle = P.new("particle/yhc2.plist"):to(self._layer, 1000):p(V.w_2, V.h_2)
    self._glowParticle = particle
end

-- 光辉
function M:loadGlowLight()
    self._lights        = {}
    local lightClass    = require("app.numwrite.numitem.N7GlowLight")
    local points        = {cc.p(114, 303), cc.p(131, 166)}
    for i=1,2 do
        local light = lightClass.new({
            numNode = self,
            layer   = self._layer,
            id      = i
        }):to(self, 500 - i):p(points[i])--:bindTouchLocate()
        light:loadShowNode()
        table.insert(self._lights, light)
    end
    self._glowIndex = 1
end

-- 点亮火
function M:fireLight(x, y)
    self._glowIndex, minDis = self:getNearestPtIndex(self._glowIndex, x, y)

    -- 画完第一笔，走到竖线那里去
    if self._glowIndex >= #self._allPathPts * 0.9 then
        self:lightFinish()
        return SIGN_TOUCH_MOVED_STOP
    elseif minDis > 100 then
        self:lightBack()
        return SIGN_TOUCH_MOVED_STOP
    end
end

function M:openGlowUpdate()
    self._preGlowIndex = self._glowIndex
    -- 萤火虫每帧刷新位置及角度
    self:scheduleUpdateWithPriorityLua(function (dt)
        -- local rotation
        -- if self._allPathPts[self._glowIndex + 1] then
        --     rotation = -PT.quadAngle(self._allPathPts[self._glowIndex], self._allPathPts[self._glowIndex + 1]) - 90
        -- else
        --     rotation = -PT.quadAngle(self._allPathPts[self._glowIndex - 1], self._allPathPts[self._glowIndex]) - 90
        -- end
        self._glowworm:p(cc.pAdd(self._allPathPts[self._glowIndex], cc.p(0, 10)))
        if self._allPathPts[self._glowIndex - 1] then
            if self._allPathPts[self._glowIndex - 1].x < self._allPathPts[self._glowIndex].x then
                self._glowworm:flipX(true)
            else
                self._glowworm:flipX(false)
            end
        end

        if self._glowIndex <= #self._pathPoints[1] then
            self._lights[1]:setDisplay(self._glowworm:getLightPoint())
            self._lights[2]:setRectZero()
        else
            self._lights[2]:setDisplay(self._glowworm:getLightPoint())
            self._lights[1]:setRectAll()
        end
        self:playWormEffect(dt)
    end, 10)
end


function M:playWormEffect(dt)
    if not self._canPlayEffect then return end
    if not self._wormSoundEffect then
        self._wormSoundEffect   = soundEffect:playEffectsfx14711069()
        self._wormVolume       = 0
    end

    if math.abs(self._glowIndex - self._preGlowIndex) < 1 then
        self._wormVolume = self._wormVolume - dt * 2
    else
        self._wormVolume = 1
    end

    if self._wormVolume < 0 then
        self._wormVolume = 0
    elseif self._wormVolume > 1 then
        self._wormVolume = 1
    end
    sound.setVolume(self._wormSoundEffect, self._wormVolume)

    self._preGlowIndex = self._glowIndex
end


-- 点亮光结束
function M:lightFinish()
    if self._lighted then return end
    self._lighted = true
    self:indexMoveTo("_glowIndex", #self._allPathPts, nil, true, function ()
        self._lights[2]:setRectAll()
        self:unscheduleUpdate()
        self._glowworm:line({
            A.actionBezierTo({
                startPosition = self._glowworm:point(),
                endPosition   = cc.pAdd(self._glowworm:point(), cc.p(-450, 280)),
                time          = 2
            }),
            {"fn", function()
                sound.stopSound(self._wormSoundEffect)
                self._layer._numBone:writeFinish()
            end},
            {"remove"},
        })
    end)
end

-- 
function M:lightBack()
    -- self:indexMoveTo("_glowIndex", 1, nil, true, function ()
        self._glowworm._touchLock = false
        self:openGuide("_glowIndex", 80)
        self._canPlayEffect = false
    -- end)
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)

    return true
end

function M:onTouchMoved(x, y, touches)


end

function M:onTouchEnded(x, y, touches)

end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end

return M