--[[
Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-2-27

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类 -- 数字8
----------------------
local M    = class("Num8", require("app.numwrite.numpath.NumBase"))
M.TAG    = "Num8"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 数字
    self._number 	= 8
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)

    -- local penguin = require("app.numwrite.numitem.N8Penguin").new({
    --     numNode = self,
    -- }):to(self, 600):p(400, 0):scale(1.5):a(cc.p(0.5, 0.2))
    -- penguin:pickAni()
end



-- 加载形似书写
function M:loadWriteItem()
	local snow 		= D.img("num/write/num8/snow.png"):to(self, 1):p(self:cw()/2, self:ch()/2):opacity(0)
    local cover = D.img("num/write/num8/cover.png"):to(self, 100):p(self:cw()/2, self:ch()/2):opacity(0)
	-- local cover 	= D.img("num/write/num8/cover.png"):to(self, 100):p(self:cw()/2, self:ch()/2):opacity(0)
    cover:one({"fadeTo", 0.5, 255})
	snow:line({
		{"fadeTo", 0.5, 255},
		{"fn", function()
            self._renderTexture:hide()
            self:loadDrawGround()
            self:loadSnowPlow()
            self:openShovelUpdate()
            soundEffect:playEffectsfx14711074()
            self:indexMoveTo("_snowIndex", 20, nil, false, function ()
            	self:openGuide("_snowIndex")
            end)
		end},
	})
end

-- 画出地面
function M:loadDrawGround()
	self._grounds 	= {}
	local ptNum 	= #self._allPathPts
	local interval 	= 2
	local dotNum 	= math.ceil(ptNum / interval) + 1
	local color 	= cc.c3b(140, 140, 140)
	local size 		= 1.0
	local texture 	= self._strokeTex
	for i=1,dotNum do
		if self._allPathPts[i*2] then
			local dot 	= D.img(texture):to(self, 10):color(color):scale(size):p(self._allPathPts[i*2]):hide()
			dot._posIndex = i*2
			table.insert(self._grounds, dot)
		end
	end
	self._arrowInterval 	= interval
end

-- 铲雪车
function M:loadSnowPlow()
	local snowPlow = require("app.numwrite.numitem.N8SnowPlow").new({
		numNode = self
	}):to(self, 200):p(self._allPathPts[1])
	self._snowPlow = snowPlow
	self._snowIndex = 1
end

-- 铲雪调度更新
function M:openShovelUpdate()
    self._preSnowIndex = self._snowIndex
	self:scheduleUpdateWithPriorityLua(function ()
		if self._allPathPts[self._snowIndex + 1] then
			local rotation = -PT.quadAngle(self._allPathPts[self._snowIndex], self._allPathPts[self._snowIndex + 1]) - 90
			self._snowPlow:rotate(rotation)
		end
		self._snowPlow:p(self._allPathPts[self._snowIndex])

		for i,v in ipairs(self._grounds) do
			if v._posIndex <= self._snowIndex + 12 then
				v:show()
			else
				v:hide()
			end
		end
	end, 10)
end

-- 铲雪音效
function M:playShovelEffect(x, y)
    if not self._canPlayEffect then return end
    if not self._effPrePos then
        self._effPrePos = cc.p(x, y)
    end

    if PT.d(self._effPrePos, cc.p(x, y)) > 1 then
        if self._checkStopEff then
            self:stopAction(self._checkStopEff)
            self._checkStopEff = nil
        end
        self._effPrePos.x = x
        self._effPrePos.y = y
    end

    if not self._checkStopEff then
        self._checkStopEff = A.line({
            {"delay", 0.1},
            {"fn", function()
                self._checkStopEff = nil
                self:stopShovelEffect()
            end},
        }):at(self)
    end

    if not self._shovelEffect then
        self._shovelEffect = soundEffect:playEffectsfx14711075()
        self._delayStopShovelEffect = A.line({
            {"delay", sound:soundDuration("sfx14711075", 5)},
            {"fn", function()
                self._delayStopShovelEffect = nil
                self._shovelEffect = nil
            end},
        }):at(self)
    end
end

-- 停止铲雪音效
function M:stopShovelEffect()
    if self._checkStopEff then
        self:stopAction(self._checkStopEff)
        self._checkStopEff = nil
    end

    if self._delayStopShovelEffect then
        self:stopAction(self._delayStopShovelEffect)
        self._delayStopShovelEffect = nil
    end

    if self._shovelEffect then
        sound.stopSound(self._shovelEffect)
        self._shovelEffect = nil
    end
end


-- 铲雪
function M:shovelSnow(x, y)
    self._snowIndex, minDis = self:getNearestPtIndex(self._snowIndex, x, y)
    self:playShovelEffect(x, y)
    if minDis > 100 then
        self:plowBack()
        return SIGN_TOUCH_MOVED_STOP
    elseif self._snowIndex >= #self._allPathPts * 0.9 then
        self:snowShovelFinish()
        return SIGN_TOUCH_MOVED_STOP
    end
end

-- 
function M:plowBack()
    self:stopShovelEffect()
    self:openGuide("_snowIndex")
    self._canPlayEffect = false
    self._snowPlow:stopAni()
end

-- 完成铲雪
function M:snowShovelFinish()
	if self._shoveled then return end
	self._shoveled = true
    self:indexMoveTo("_snowIndex", #self._allPathPts, nil, true, function ()
    	self._snowPlow:quit()
        self:unscheduleUpdate()
        self:stopShovelEffect()
        self:loadFish()
        self:loadPenguin()

        local count = 0
        self:line({
        	A.cycle({
        		{"delay", 0.3},
        		{"fn", function()
        			count = count + 1
        			self._fishes[count]:showInRoad()
                    if count == 1 then
                        soundEffect:playEffectsfx14711076()
                    end
        		end},
        	}, 2),
            {"delay", 0.9},
        	{"fn", function()
    			self:openFishUpdate()
                self:openGuide("_fishIndex")
        	end},
        })
    end)
end

-- 鱼鱼
function M:loadFish()
	self._fishes 		= {}

	-- 鱼分布的位置
    local indexes 		= {40, 125, 210, 295, 380, 465, 550, 635, 720, 800, 880}
    -- 两个水池的坐标
    local poolPos 		= {cc.p(118, 256), cc.p(118, 111)}
    -- 鱼的类
    local fishClass   = require("app.numwrite.numitem.N8Fish")
    -- for i=1,count do
    for i,index in ipairs(indexes) do
        local point     = self._allPathPts[index]
        local fish    = fishClass.new({
            numNode     = self,
            posIndex    = index,
            poolPos 	= i <= 5 and poolPos[1] or poolPos[2],
            iniPos 	 	= point,
            index       = i
        }):to(self, 500 - point.y):p(point):hide()
        table.insert(self._fishes, fish)
    end
    self._fishIndex   = 1
end

-- 加载企鹅
function M:loadPenguin()
	local penguin = require("app.numwrite.numitem.N8Penguin").new({
		numNode = self,
	}):to(self, 600):p(self._allPathPts[1]):scale(1.5):a(cc.p(0.5, 0.2))
	penguin:penguinEnter()
	self._penguin = penguin
end

function M:pickUpFish(x, y)
    self._fishIndex, minDis = self:getNearestPtIndex(self._fishIndex, x, y)
    if minDis > 100 then
        self:fishBack()
        return SIGN_TOUCH_MOVED_STOP
    elseif self._fishIndex >= #self._allPathPts * 0.9 then
        self:fishFinish()
        return SIGN_TOUCH_MOVED_STOP
    end
end


function M:openFishUpdate()
	self:scheduleUpdateWithPriorityLua(function ()
		self._penguin:p(self._allPathPts[self._fishIndex])

		for i,v in ipairs(self._fishes) do
			if v._posIndex <= self._fishIndex then
				v:eaten()
			elseif v._posIndex <= self._fishIndex + 160 then
				v:showInRoad()
			else
				v:inPool()
			end
		end
	end, 10)
end

-- 抓鱼鱼音效 -- 最多播放三个
function M:playFishEffect()
    self._fishEffectCount = self._fishEffectCount or 0
    if self._fishEffectCount < 3 then
        self._fishEffectCount = self._fishEffectCount + 1
        soundEffect:playEffectsfx14711083()
        self:line({
            {"delay", sound:soundDuration("sfx14711083", 1)},
            {"fn", function()
                self._fishEffectCount = self._fishEffectCount - 1
            end},
        })
    end
end

function M:fishBack()
    self:openGuide("_fishIndex")
    self._canPlayEffect = false
end


-- 捡鱼完成
function M:fishFinish()
	if self._fished then return end
	self._fished = true
    self:stopGuide()

    self:indexMoveTo("_fishIndex", #self._allPathPts, nil, true, function ()
        self:unscheduleUpdate()
        self:line({
            {"delay", 1},
            {"fn", function()
                self._penguin:quit()
                self._layer._numBone:writeFinish()
            end},
        })
    end)
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)

    return true
end

function M:onTouchMoved(x, y, touches)


end

function M:onTouchEnded(x, y, touches)

end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end

return M