--[[
Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-2-27

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类 -- 数字9
----------------------
local M    = class("Num9", require("app.numwrite.numpath.NumBase"))
M.TAG    = "Num9"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)
    -- 数字
    self._number 	= 9
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
end



-- 加载形似书写
function M:loadWriteItem()
	local soil 		= D.img("num/write/num9/stroke3.png"):to(self, 1):p(self:cw()/2, self:ch()/2 - 5):opacity(0)
	soil:line({
		{"fadeTo", 0.5, 255},
		{"fn", function()
            self._renderTexture:hide()
            self:loadWheat()
            self:loadBag()
            self:openSowUpdate()
            self:openGuide("_wheatIndex")
		end},
	})
end

-- 麦子
function M:loadWheat()
	self._wheats         = {}
    local interval      = 30
    local count         = math.ceil(#self._allPathPts / interval)
    local wheatClass     = require("app.numwrite.numitem.N9Wheat")
    for i=1,count do
        local index     = math.floor((i-1) * interval) + 1
        local point     = self._allPathPts[index]
        local wheat 	= wheatClass.new({
        	numNode 	= self,
        	posIndex 	= index,
        }):to(self, 500 - point.y):p(point):hide()
        table.insert(self._wheats, wheat)
    end

    self._wheatIndex    = 1
    self._interval  = interval
end

-- 种子袋
function M:loadBag()
    local bag = require("app.numwrite.numitem.N9Bag").new({
        numNode     = self,
    }):to(self, 500):p(self._allPathPts[1]):bindTouch({rectFactorX = 1.5, rectFactorY = 1.5})
    self._bag = bag
end

-- 播种调度更新
function M:openSowUpdate()
    self:scheduleUpdateWithPriorityLua(function ()
        self._bag:p(self._allPathPts[self._wheatIndex])

        for i,v in ipairs(self._wheats) do
            if v._posIndex <= self._wheatIndex then
                v:growUp()
            else
                v:retract()
            end
        end
    end, 10)
end

-- 花花声音
function M:playFlowerEffect()
    if not self._canPlayEffect then return end
    -- 初始化
    if not self._flowerEffects then
        self._flowerEffects = {}
        self._fEffectIndex  = math.random(1, 4)
    end
    -- 声音的索引1-4循环
    self._fEffectIndex = self._fEffectIndex % 4 + 1
    -- 播放音效
    local soundEffects = {"sfx14711033", "sfx14711034", "sfx14711035", "sfx14711036"}
    local newSound     = soundEffect["playEffect"..soundEffects[self._fEffectIndex]](soundEffect)
    table.insert(self._flowerEffects, newSound)

    -- 音效最多同时存在3个
    if #self._flowerEffects == 4 then
        sound.stopSound(self._flowerEffects[1])
        table.remove(self._flowerEffects, 1)
    end
end



-- 种麦子
function M:wheatSow(x, y)
    self._wheatIndex, minDis = self:getNearestPtIndex(self._wheatIndex, x, y)
    if minDis > 100 then
        self:sowBagBack()
        return SIGN_TOUCH_MOVED_STOP
    elseif self._wheatIndex >= #self._allPathPts * 0.9 then
        self:sowWheatFinish()
        return SIGN_TOUCH_MOVED_STOP
    end
end

function M:sowBagBack()
    self:openGuide("_wheatIndex")
    self._canPlayEffect = false
end


-- 种麦子完成
function M:sowWheatFinish()
    if self._sowed then return end
    self._sowed = true
    self._canPlayEffect = false
    self:indexMoveTo("_wheatIndex", #self._allPathPts, nil, true, function ()
        self:line({
            {"delay", 0.1},
            {"fn", function()
                self:unscheduleUpdate()
                self._bag:quit()
                self:loadReaper()
                self:openReapUpdate()
                self:openGuide("_reapIndex")
            end},
        })
    end)
end

-- 加载收割机
function M:loadReaper()
    local reaper = require("app.numwrite.numitem.N9Reaper").new({
        numNode     = self,
    }):to(self, 500):p(self._allPathPts[1])
    self._reaper = reaper

    self._reapIndex = 1
end


function M:openReapUpdate()
    self:scheduleUpdateWithPriorityLua(function ()
        local rotation  
        if self._allPathPts[self._reapIndex + 1] then
            rotation = -PT.quadAngle(self._allPathPts[self._reapIndex], self._allPathPts[self._reapIndex + 1]) - 90
        else
            rotation = -PT.quadAngle(self._allPathPts[self._reapIndex - 1], self._allPathPts[self._reapIndex]) - 90
        end
        self._reaper:p(self._allPathPts[self._reapIndex]):rotate(rotation)

        for i,v in ipairs(self._wheats) do
            if v._posIndex <= self._reapIndex + 60 then
                v:reaped()
            else
                v:notReap()
            end
        end
    end, 10)
end

-- 播放收割音效
function M:playReapEffect(x, y)
    if not self._canPlayEffect then return end
    if not self._effPrePos then
        self._effPrePos = cc.p(x, y)
    end

    if PT.d(self._effPrePos, cc.p(x, y)) > 1 then
        if self._checkStopEff then
            self:stopAction(self._checkStopEff)
            self._checkStopEff = nil
        end
        self._effPrePos.x = x
        self._effPrePos.y = y
    end

    if not self._checkStopEff then
        self._checkStopEff = A.line({
            {"delay", 0.1},
            {"fn", function()
                self._checkStopEff = nil
                self:stopReapEffect()
            end},
        }):at(self)
    end

    if not self._reapEffect then
        self._reapEffect = soundEffect:playEffectsfx14711088()
        self._delayStopReapEffect = A.line({
            {"delay", sound:soundDuration("sfx14711088", 3)},
            {"fn", function()
                self._delayStopReapEffect = nil
                self._reapEffect = nil
            end},
        }):at(self)
    end
end

-- 停止收割音效
function M:stopReapEffect()
    if self._checkStopEff then
        self:stopAction(self._checkStopEff)
        self._checkStopEff = nil
    end

    if self._delayStopReapEffect then
        self:stopAction(self._delayStopReapEffect)
        self._delayStopReapEffect = nil
    end

    if self._reapEffect then
        sound.stopSound(self._reapEffect)
        self._reapEffect = nil
    end
end


-- 种麦子
function M:wheatReap(x, y)
    self._reapIndex, minDis = self:getNearestPtIndex(self._reapIndex, x, y)
    self:playReapEffect(x, y)
    if minDis > 100 then
        self:reapBack()
        return SIGN_TOUCH_MOVED_STOP
    elseif self._reapIndex >= #self._allPathPts * 0.9 then
        self:reapWheatFinish()
        return SIGN_TOUCH_MOVED_STOP
    end
end

function M:reapBack()
    self._reaper:stopAni()
    self:openGuide("_reapIndex")
    self._canPlayEffect = false
    self:stopReapEffect()
end


function M:reapWheatFinish()
    self:indexMoveTo("_reapIndex", #self._allPathPts, nil, true, function ()
        self:unscheduleUpdate()
        self:stopReapEffect()
        self._reaper:quit()
        self._layer._numBone:writeFinish()
    end)
end



----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)

    return true
end

function M:onTouchMoved(x, y, touches)


end

function M:onTouchEnded(x, y, touches)

end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end

return M