--[[

Copyright (c) 2012-2020 Baby-Bus.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[!--

场景类，定义场景相关操作方法及逻辑实现。

-   定义场景功能方法。

]]

----------------------
-- 类 -- 写数字场景
----------------------
local M = classScene("Numwrite")


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..

-- [操作变量]
-- ..








----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义视图实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    M.super.ctor(self, params)
    -- [超类调用]
    self:assertParameters(params)

    -- 起始数字 1和6 轮切
    self._number        = params.number
    if not self._number then
        self._number = G_numwrite_beganNumer == 1 and 1 or 6
        G_numwrite_beganNumer = 3 - G_numwrite_beganNumer
    end
    -- 已完成的关卡树
    self._finishCount   = 0
end




----------------------
-- 视图渲染
----------------------
--[[--

视图渲染，处理视图结点加载、事件绑定等相关操作

]]
function M:onRender()
    G_addEnter("numwrite")
    -- 重写骨骼API，用来获取骨骼的坐标
    bb.si.DragonBone.resetBoneInterface = true
end


function M:onEnterTransitionFinish()
    self:getMainLayer():enterFinish()

    if self._number <= 3 then
        sound:playbgm14705()
    elseif self._number <= 6 then
        sound:playbgm14706()
    else
        sound:playbgm14707()
    end
end


-- 点击返回按键的处理函数
function M:onKeypadBackHandler()
    local buttonLayer = self:getButtonLayer()
    -- 逻辑与当前场景的左上角点击返回一致
    if buttonLayer and buttonLayer.btnBackEvent then
        buttonLayer:btnBackEvent()
    end
end

function M:enterNextNumber()
    self._number = self._number < 10 and (self._number + 1) or 1
    self:getButtonLayer()._menuBtn:setNumber(self._number)
    self:switchNumber(self._number, true)
end

-- 切换数字 #目标数字 #是否停止所有音效、动作
function M:switchNumber(number, isStopAll)
    if self._inSwitching then return end
    -- bool值锁定操作
    self._inSwitching = true
    -- 记录新数字
    self._number      = number                      
    -- 停止所有音效、动作
    if isStopAll then
        self.mainLayer_:stopAllChildrenAction()
        sound.stopAllEffects()
        self._menuBtn:barChange(false)
    end
    -- 播放切换音效
    soundEffect:playEffectsfx14711022()             
    -- 旧的mainLayer
    local oldLayer      = self.mainLayer_
    local mainZorder    = oldLayer:z()
    oldLayer:z(mainZorder):unbindTouch()
    -- 移除骨骼避免冲突
    DragonBone.removeAll()
    -- 新的mainLayer
    local newLayer = require("app.numwrite.layer.MainLayer").new({
        number = self._number
    }):to(self, mainZorder):p(1200, 0)
    self.mainLayer_ = newLayer
    -- 移动的时间和相对距离
    local moveTime  = 1
    local movePoint = cc.p(-1200, 0)
    -- 背景、旧layer、新layer同时移动
    self:getBackgroundLayer():bgMoveBy(moveTime, movePoint)
    A.line({
        {"moveBy", moveTime, movePoint},
        {"remove"},
        {"fn", function()
            GLOBAL_TOUCH_LOCK = false
        end},
    }):at(oldLayer)
    A.line({
        {"moveBy", moveTime, movePoint},
        {"fn", function()
            self._inSwitching = false
            newLayer:enterFinish()
        end},
    }):at(newLayer)
end


----------------------
-- 统计时长
----------------------
function M:beginPage()
    local fnName = "record"..string.sub(M.name, 1, -6).."Begin"
    if umengOP[fnName] then
        umengOP[fnName]()
    else
        print("统计时长开始-没有这个方法名", fnName)
    end
end

function M:endPage()
    local fnName = "record"..string.sub(M.name, 1, -6).."End"
    if umengOP[fnName] then
        umengOP[fnName]()
    else
        print("统计时长结束-没有这个方法名", fnName)
    end
end



-- 拖动场景忽略广告
function M:isIgnoreAdForDrag()
    return false
end
----------------------
-- 模板方法
----------------------
--[[--

获得层名称集合, 用于动态定义该场景需要加载的层

### Returns: 
-   string...           层名称1, 层名称2, ...

]]
function M:getLayerNames()
    return "background", "main","button"
end

--[[--

获得获得信息集合, 用于动态定义该场景需要加载的资源(纹理, 帧等)

### Returns: 
-   string|table...     帧名称1|{资源名称,类型,自动清理,图片后缀(包含.)}, ...

]]
function M:getResourceNames()
    return 
end




-- 是否是开场动画
function M:isWelcomeScene()
    return false
end

--选择页面
function M:isSelectScene()
    return false
end
 
--loading界面
function M:isLoadingScene()
    return false
end

----------------------
-- 验证
----------------------
-- 验证参数
function M:assertParameters(params)

end




----------------------
-- 结点析构
----------------------
--[[--

视图析构，处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    M.super.onDestructor(self)
end


return M