--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]

--[[!--

场景层类，定义层相关操作方法及逻辑实现。

-   定义场景层功能方法。

]]

----------------------
-- 类
----------------------
local M = classLayerTouch("Main")


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..

-- 形状的类:1-三角形 2-正方形 3-圆形 4-长方形 5-爱心形 6-星形
local ShapeClasses = {
	require("app.common.shape.Shape1Triangle"),
	require("app.common.shape.Shape2Square"),
	require("app.common.shape.Shape3Circle"),
	require("app.common.shape.Shape4Rectangle"),
	require("app.common.shape.Shape5Heart"),
	require("app.common.shape.Shape6Star"),
}


-- [操作变量]
-- ..








----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义视图实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
	-- 形状ID 1-三角形 2-正方形 3-圆形 4-长方形 5-爱心形 6-星形
	self._shapeId 	= 1

	self._level 	= 1

	self._shapes 	= {}

	self._eatNumber = 0

	self._finishNum = 1
end

----------------------
-- 结点渲染
----------------------
--[[--

视图渲染，处理视图结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- 加载渲染以外的其他操作
	self:loadRenderOtherwise()
	-- 背景
	self:loadBg()

	self:loadShapeAndFoodBar()

	-- CUtil:loadSubtitle():to(self, 9999)
	-- -- 形状
	-- self:loadShape()
	-- -- 食物
	-- self:loadFoodBar()
end

function M:onEnterTransitionFinish()
	sound:playbgm14704()
	-- 播放配音[v147087]形状精灵好饿呀！快给它们吃饼干~
	self._beganVoice = soundVoice:playEffectv147087()
end

function M:stopBeganVoice()
	if self._beganVoice then
		sound.stopSound(self._beganVoice)
		self._beganVoice = nil
	end
end


-- 加载其他
function M:loadRenderOtherwise()
	-- 精灵ID表
	self._spriteIdTab = {1, 2, 3, 4, 5, 6}
	self._spriteIdTab = {1, 3, 5, 6, 2, 4}

	-- T.shuffle(self._spriteIdTab)
	-- 正方形、长方形（ID2和4）不能同时出现，如果在第3和第4出现了，就让第三个与1、2、5、6随机一个交换
	if (self._spriteIdTab[3] == 2 and self._spriteIdTab[4] == 4) or 
		(self._spriteIdTab[3] == 4 and self._spriteIdTab[4] == 2) then
			local randomN 				= math.random() < 0.5 and math.random(1, 2) or math.random(5, 6)
			local temp 					= self._spriteIdTab[randomN]
			self._spriteIdTab[randomN]  = self._spriteIdTab[3]
			self._spriteIdTab[3] 		= temp

	elseif (self._spriteIdTab[5] == 2 and self._spriteIdTab[6] == 4) or 
		(self._spriteIdTab[5] == 4 and self._spriteIdTab[6] == 2) then
			local randomN 				= math.random(1, 4)
			local temp 					= self._spriteIdTab[randomN]
			self._spriteIdTab[randomN]  = self._spriteIdTab[5]
			self._spriteIdTab[5] 		= temp
	end
end

-- 背景
function M:loadBg()
	local bg = U.loadBg("shape/feed/bg.png"):to(self)
end

-- 形状、食物栏
function M:loadShapeAndFoodBar()
	local shapeInfo = self:getSpriteInfo()
	local foodInfo 	= self:getFoodInfo(shapeInfo)

	local width 	= 960 / (#shapeInfo + 1)
	for i,v in ipairs(shapeInfo) do
		local shape = ShapeClasses[v].new({
			layer 	= self,
			scale 	= 0.6
		}):to(self, 10):p(width * i + X_OFFSET, 330 + Y_OFFSET)
		shape:feedEnter()
		table.insert(self._shapes, shape)
	end

	local foodBar = require("app.shapefeed.node.FoodBar").new({
		layer 		= self,
		foodIds 	= foodInfo
	}):to(self, 100):p(V.w_2, - 50 - 200)
	self:line({
		{"delay", 0.3},
		{"fn", function()
			soundEffect:playEffectsfx14705001()
		end},
		{"delay", 0.2},
		{"fn", function()
			foodBar:enter(function ()
				if self._level == 1 then
					self:loadHoleGuide()
				else
					self:stopGuide()
					self:openGuide()
				end
			end)
		end},
	})
	self._foodBar = foodBar
end

-- 洞窥遮罩引导
function M:loadHoleGuide()
	local guideMask = require("app.shapefeed.node.GuideMask").new({
		layer 	= self
	}):to(self, 150):p(V.w_2, V.h_2)

	local food 	= self._foodBar._foods[1]
	local pos1 	= food:worldpoint()
	local pos2 	= self._shapes[1]:point()

	guideMask:loadMask(pos1, pos2)
	self._guideMask = guideMask
end

-- 移除洞窥遮罩引导
function M:removeHoleGuide()
	if self._guideMask then
		self._guideMask:removeSelf()
		self._guideMask = nil
	end
end

-- 开启洞窥遮罩手指引导
function M:openHoleFingerGuide()
	if self._guideMask then
		self._guideMask._finger:openHandGuide(2)
	end
end

-- 停止洞窥遮罩手指引导
function M:stopHoleFingerGuide()
	if self._guideMask then
		self._guideMask:stopFingerGuide()
	end
end

-- 开启引导
function M:openGuide()
	if self._openedGuide or self._level == 1 then return end
	self._openedGuide = true
	CAction:openGuideItems(self._foodBar._foods, nil, 5, nil, nil, function ()
		self._guideVoiceIndex = (self._guideVoiceIndex or math.random(1, 2)) + 1
		if self._guideVoiceIndex % 2 == 0 then
			soundVoice:playEffectv147147()
		elseif self._guideVoiceIndex % 2 == 1 then
			soundVoice:playEffectv147148()
		end
	end)
end

function M:stopGuide()
	if self._openedGuide then
		self._openedGuide = false
		CAction:stopGuideItems(self._foodBar._foods)
	end
end


-- 正确的形状张开嘴
function M:shapeOpenMouth(id)
	for i,v in ipairs(self._shapes) do
		if v._id == id then
			v:openMouthAct1()
		end
	end
	self:stopHoleFingerGuide()
end

-- 闭嘴
function M:closeMouth(id)
	for i,v in ipairs(self._shapes) do
		if v._id == id then
			v:breathAct1()
		end
	end
	self:openHoleFingerGuide()
end

-- 吃了的数量加一，判断已完成解除其他食物的触控
function M:eatPlusOne()
	self._eatNumber = self._eatNumber + 1
	if self._eatNumber == self._finishNum then
		self._foodBar:unbindTouch()
	end
end

-- 吃完一个食物，判断是否进入下一关
function M:eatOneFood()
	if self._eatNumber == self._finishNum then
		self:levelFinish()
	end
end

-- 关卡完成
function M:levelFinish()
	self._eatNumber = 0
	self._level 	= self._level + 1
	self._finishNum = totable({1, 1, 2, 3})[self._level]

	soundEffect:playEffectsfx14705003()
	self._foodBar:quit()
	self:line({
		{"delay", 1},
		{"fn", function()
			-- 精灵全部退场，保存退场音效去重
			local effectTab = {}
			for i,v in ipairs(self._shapes) do
				local index = v:feedQuit()
				table.insert(effectTab, index) 
			end
			T.clear(self._shapes)
			-- 音效去重
			local newTab = T.unique(effectTab)
			for i,v in ipairs(newTab) do
				soundEffect["playEffect"..v](soundEffect)
			end

			soundEffect:playEffectsfx14713016()
			local particle = P.new("particle/bingganjl.plist"):to(self, 999):p(V.w_2, 20)
			particle:setAutoRemoveOnFinish(true)
			for i=2,4 do
				local particle = P.new("particle/bingganjl.plist"):to(self, 999):p(V.w_2, 20)
				particle:setTexture(R.loadTexture("particle/bingganjl"..i..".png"))
				particle:setAutoRemoveOnFinish(true)
			end
		end},
		{"delay", 2},
		{"fn", function()
			if self._level > 4 then
				local encourage = CUtil:loadEncourage(self, 4, function ()
					G_addFinish("shapefeed")
					SceneController:enterNextMoudleScene()
				end)
			else
				self:loadShapeAndFoodBar()
			end
		end},
	})
end

-- 加载形状
function M:loadShape()
	for i=1,6 do
		local shape = ShapeClasses[i].new({
			layer 	= self,
		}):to(self, 10):p(960 / 12 * (i * 2 - 1), 300):scale(0.5)
	end
end


function M:loadFoodBar()
	local foodBar = require("app.shapefeed.node.FoodBar").new({
		layer 	= self
	}):to(self, 100):p(V.w_2, -48)
	self._foodBar = foodBar
end

-- 精灵信息
-- 第一关、第二关一个精灵、第三关2个精灵、第四关三个精灵
function M:getSpriteInfo()
	if self._level == 1 then
		return {self._spriteIdTab[1]}
	elseif self._level == 2 then
		return {self._spriteIdTab[2]}
	elseif self._level == 3 then
		return {self._spriteIdTab[3], self._spriteIdTab[4]}
	elseif self._level == 4 then
		local idTab  	= {}
		local com 		= T.shuffle({1, 3, 5, 6})
		table.insert(idTab, com[1])
		table.insert(idTab, com[2])
		table.insert(idTab, T.shuffle({2, 4})[1])
		return idTab
	end
end

-- 食物信息
function M:getFoodInfo(idTab)
	local foodTab 	= {}
	local foodNum 	= self._level ~= 1 and 3 or 1
	for i=1,foodNum do
		if i <= #idTab then
			table.insert(foodTab, idTab[i])
		else
			-- 如果有了2或者4则不再出现这两个
			if T.contains(foodTab, 2) or T.contains(foodTab, 4) then
				table.insert(foodTab, self:randomExcept(1, 6, {2, 4, unpack(foodTab)}))
			else
				table.insert(foodTab, self:randomExcept(1, 6, foodTab))
			end
		end
	end
	return T.shuffle(foodTab)
end

-- 取随机数排除(区间下限, 区间上限, 排除数字或集合)
function M:randomExcept(min, max, excepts)
	if max - min > 10000 then print("太大了不要不要") return 0 end
	if isnumber(excepts) then
		excepts 	= {excepts}
	end
    local numTab 	= {}
	for i=min,max do
		if not T.contains(excepts, i) then
			table.insert(numTab, i)
		end
	end
    local num 		= numTab[math.random(1, #numTab)]
    numTab 			= nil
    return num
end



-- 播放形状台词
function M:playShapeVoice(shapeId)
	-- 1-三角形 2-正方形 3-圆形 4-长方形 5-爱心形 6-星形
	soundVoice:stopNowSound()
	if shapeId == 1 then
		-- 播放配音[v147084]三角形
		soundVoice:playEffectv147084()
	elseif shapeId == 2 then
		-- 播放配音[v147081]正方形
		soundVoice:playEffectv147081()
	elseif shapeId == 3 then
		-- 播放配音[v147082]圆形
		soundVoice:playEffectv147082()
	elseif shapeId == 4 then
		-- 播放配音[v147083]长方形
		soundVoice:playEffectv147083()
	elseif shapeId == 5 then
		-- 播放配音[v147085]爱心形
		soundVoice:playEffectv147085()
	elseif shapeId == 6 then
		-- 播放配音[v147086]星型
		soundVoice:playEffectv147086()
	end
end



----------------------
-- 结点析构
----------------------
--[[--

视图析构，处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end

function M:onExitTransitionStart()
    -- R.removeAllTextures()
    -- R.removeAllFrames()
    -- DragonBone.removeAll()
end

return M