--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-1-26

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类 -- 食物
----------------------
local M     = classSpriteTouch("Food")
M.TAG       = "Food"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
	-- 层
	self._layer 	= params.layer
	-- ID
	self._id 		= params.id or 1
	-- 初始坐标
	self._iniPos 	= params.iniPos

	self._touchLock = false

	self._scale 	= self._id == 4 and 0.8 or 1
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end


-- 图片渲染
function M:loadDisplay()
	self:setTexture("shape/feed/food/f"..self._id..".png"):scale(self._scale)
end

-- 检查是否可以吃
function M:checkIsCanEat()
	local shape = nil
	for i,v in ipairs(self._layer._shapes) do
		-- if PT.d(v:point(), cc.pAdd(self._moveItem:point(), cc.p(0, self._moveItem:ch()/2))) < 120 then
		if PT.d(v:point(), self._moveItem:point()) < 120 then
			if self._id == v._id then
				self:enterMouth(v)
			else
				self:playRefuseVoice()

				self:jumpBack()
				self:line({
					{"delay", 0.1},
					{"fn", function()
						v:refuseEat()
					end},
				})
			end
			return
		end
	end
	self:jumpBack()
end

-- 播放拒绝吃语音
function M:playRefuseVoice()
	if self._layer._refuseVoice then return end
	self._layer._refuseCount = (self._layer._refuseCount or 0) + 1

	local time
	if self._layer._refuseCount == 1 then
		-- 播放配音[v147088]不行不行~形状精灵只吃得下同样形状的饼干哦！
		self._layer._refuseVoice = soundVoice:playEffectv147088()
		time = sound:soundDuration("v147088", 4)
	elseif self._layer._refuseCount % 2 == 0 then
		-- 播放配音[v147150]它不想吃这个！
		self._layer._refuseVoice = soundVoice:playEffectv147150()
		time = sound:soundDuration("v147150", 2)
	elseif self._layer._refuseCount % 2 == 1 then
		-- 播放配音[v147151]它不喜欢这种饼干~
		self._layer._refuseVoice = soundVoice:playEffectv147151()
		time = sound:soundDuration("v147151", 2)
	end

	local layer = self._layer
	layer:line({
		{"delay", time},
		{"fn", function()
			layer._refuseVoice = nil
		end},
	})
end

-- 跳回食物栏
function M:jumpBack()
	soundEffect:playEffectsfx14700003()
	-- 形状闭上嘴
	self._layer:closeMouth(self._id)

	self._moveItem:line({
		A.union({
			{"jumpTo", 0.3, self:worldpoint(), 100, 1},
			{"scaleTo", 0.3, 1.0 * self._scale},
		}),
		{"fn", function()
			self:show()
			CAction:feedback(self)
			self._layer:openGuide()
			self._touchLock = false
			local particle = P.new("particle/bgth.plist"):to(self):p(self:cw()/2, 0)
    		CUtil:changeParticleColor(particle, cc.c3b(255, 234, 110))
			particle:setAutoRemoveOnFinish(true)
		end},
		{"hide"},
	})
end



-- 进入形状嘴里
function M:enterMouth(shape)
	soundEffect:playEffectsfx14705034()
	self._layer:removeHoleGuide()
	self._layer:eatPlusOne()
	T.removeOrder(self._layer._foodBar._foods, self)

	local mouthPos = shape:convertToWorldSpace(shape:getMouthPoint())
	self._moveItem:line({
		A.union({
			{"jumpTo", 0.2, mouthPos, 50, 1},
			{"scaleTo", 0.2, 0},
		}),
		{"hide"},
		{"fn", function()
			shape:eatFood()
			self._layer:openGuide()
		end},
	})
end

function M:moveItemShow(x, y)
	if not self._moveItem then
		local moveItem = D.img("shape/feed/food/f"..self._id..".png"):to(self._layer, 200)
		self._moveItem = moveItem
	end
	self._moveItem:show():p(x - 20, y + 20):scale(1.1 * self._scale)
end

function M:moveItemMove(x, y)
	self._moveItem:p(x - 20, y + 20)
	CUtil:limitRectMove({node = self._moveItem})
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	soundEffect:playEffectsfx14700001()
	self._touchLock = true
	self._layer:stopGuide()
	self._layer:shapeOpenMouth(self._id)
	self:hide()
	self:moveItemShow(x, y)
	return true
end

function M:onTouchMoved(x, y, touches)
	self:moveItemMove(x, y)
	-- self:p(self:getParent():convertToNodeSpace(cc.p(x - 20, y - 10)))
	-- CUtil:limitRectMove({node = self})
end

function M:onTouchEnded(x, y, touches)
	self:checkIsCanEat()
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M