--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSprite("FoodBar")
M.TAG       = "FoodBar"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._layer 	= params.layer

	self._foodIds 	= params.foodIds or {1, 3}

	self._foods 	= {}
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()

	self:loadFoods()
end


-- 图片渲染
function M:loadDisplay()
	self:setTexture("shape/feed/bar.png")
end

-- 加载食物
function M:loadFoods()
	local foodClass 	= require("app.shapefeed.node.Food")
	local width 		= self:cw()
	local number 		= #self._foodIds

	for i,v in ipairs(self._foodIds) do
		local foodPos 	= cc.p(width / (number + 1) * i, 120)
		local food 		= foodClass.new({
			layer 		= self._layer,
			id 			= v,
			iniPos 		= foodPos
		}):to(self):p(foodPos)
		table.insert(self._foods, food)
	end
end

function M:enter(callback)
	self:line({
		{"easing", "backOut", {"moveBy", 0.5, cc.p(0, 320)}},
		{"fn", function()
			self:bindTouch()
			if callback then callback() end
		end},
	})
end

function M:quit()
	self:unbindTouch()
	self:line({
		{"easing", "backIn", {"moveBy", 0.5, cc.p(0, -320)}},
		{"remove"},
	})
end




----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M