--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类 -- 引导手指
----------------------
local M     = classSprite("GuideFinger")
M.TAG       = "GuideFinger"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..






----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    --J.assert(M.TAG, params.images ~= nil and #params.images == 4,"images size must be 4")
    -- [超类调用]
    M.super.ctor(self, params)
    self._mainLayer     = params.layer
    self._startPos      = ifnil(params.startPos, ccp(V.w_4, V.h_2))
    self._endPos        = ifnil(params.endPos, ccp(V.w_2, V.h_2))
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)

end

-- 加载手指
function M:loadFinger()
    self:setTexture("common/blank.png"):opacity(0)
    local hand = D.img("common/finger/1.png"):p(self._startPos):to(self, 800):anchor(cc.p(0, 1))
    self._hand = hand 
end

--手指引导 #首次出现延迟时间 #每次华动静间隔时间 #滑动时回调 #回调 #循环次数
function M:openHandGuide(time, delayTime, fn, cycleTime)
    time = time or 0
    delayTime = delayTime or 3
    if not self._fingerGuide then
        self._fingerGuide = A.line({
            {"delay", time},
            {"fn", function()
                self._fingerGuide = nil

                if not self._hand then
                    self:loadFinger()
                end
                if fn and isfunction(fn) then
                    fn()
                end
                self._hand:cycle({
                    {"place", self._startPos},
                    {"fadeIn", 0.3},
                    {"image", "common/finger/", 3, 0.05},
                    {"fn", function()
                        soundEffect:playEffectsfx14705002()
                        -- 拖尾动作
                        self:motionStreakAct(self._hand)
                    end},
                    {"moveTo", .8, self._endPos},
                    {"fn", function()
                        -- 删除拖尾
                        self:deleteMotionStreak()
                    end},
                    A.reverse({"image", "common/finger/", 3, 0.1}),
                    {"fadeOut", 0.5},
                    {"delay", .5},
                    {"delay", delayTime},
                }, cycleTime)
            end},
        })
        self:runAction(self._fingerGuide)
    end
end



--关闭手指引导
function M:closeHandGuide()
    if self._fingerGuide then
        self:stopAction(self._fingerGuide)
        self._fingerGuide = nil
    end
    -- 移除拖尾刷新
    if self._emptyForStreak then
        self._emptyForStreak:stopAllActions()
        self._emptyForStreak:remove()
        self._emptyForStreak = nil
    end
    -- 移除拖尾
    if self._motionStreak then
        self._motionStreak:remove()
        self._motionStreak = nil
    end
    -- 移除手指
    if self._hand then
        self._hand:remove()
        self._hand = nil
    end
end

-- 拖尾动作
function M:motionStreakAct(node)
    self._motionStreak = self:loadMotionStreak(self, 700, .5)
    self._emptyForStreak = U.loadNode({contentSize = cc.size(0.1, 0.1)}):to(self)
    self._emptyForStreak:cycle({
        {"fn", function()
            if self._motionStreak then
                self._motionStreak:setPosition(node:px() + 10 , node:py() - 35)
            end
        end},
        {"delay", 1/60},
    })
end

-- 删除拖尾
function M:deleteMotionStreak()
    if not tolua.isnull(self._emptyForStreak) then
        self._emptyForStreak:stopAllActions()
        self._emptyForStreak:remove()
        self._emptyForStreak = nil
    end
end

--拖尾，1.父节点 2.层级 3.渐隐时间
function M:loadMotionStreak(toWho, zorder, time)
    time                = time or 1
    zorder              = zorder or 1000
    local pointDistance = 1.0
    local size          = 30
    local imgName       = "common/finger/lightx.png"
    
    -- 拖尾渐隐时间(秒), 最小的片段长度, 渐隐条带的宽度, 片段颜色值, 纹理图片的文件名。
    local motionStreak = cc.MotionStreak:create(time, pointDistance, size, cc.c3b(255,255,255), imgName)
    CCNodeExtend.extend(motionStreak)
    motionStreak:to(toWho, zorder)

    motionStreak:setBlendFunc(GL_SRC_ALPHA, GL_ONE)

    return motionStreak
end





----------------------
-- 结点析构
----------------------
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 过度动画开始之前的调用
----------------------
--[[--

1.资源释放，退出场景前的操作准备
2.处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onExitTransitionStart()

end



----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    
end


return M