--[[
	类 UI栏引导遮罩
--]]


local M 	= classSpriteTouch("GuideMask")




--------------------------
-- 构造函数
--------------------------

function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
 	-- [本类调用]
 	self._layer = params.layer
end

--------------------------
-- 渲染
--------------------------

function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
 	-- [本类调用]
	-- 加载节点
	self:loadDisplay() 
end 

-- 加载节点
function M:loadDisplay()
	self:setContentSize(cc.size(V.w, V.h)):bindTouch()

	-- 吞噬触控
	-- local absorbTouchNode = D.img("common/blank.png"):scaleX(V.w):scaleY(V.h)
	-- :to(self._layer, 29999):pc():opacity(0):bindTouch()
	-- self._absorbTouchNode = absorbTouchNode

    local render = CCNodeExtend.extend(cc.RenderTexture:create(V.w, V.h))
    render:to(self):p(V.w_2, V.h_2)--:hide()
    self._render = render
end 

-- 加载渲染以外的其他操作
function M:onEnterTransitionFinish()
	--  
end

-- 加载2个引导遮罩 #引导起始点 #引导终点 #起点洞窥的大小 #终点洞窥的大小
function M:loadMask2(pos1, pos2, scale1, scale2)
    pos1        = pos1 or cc.p(V.w_2, 100)
    pos2        = pos2 or cc.p(V.w_2, V.h_2)
    scale1      = scale1 or 2.5
    scale2      = scale2 or 2.5
    local opa   = 200 / 255
    self._render:beginWithClear(0, 0, 0, opa)
    self._render:endToLua()
    local hole1 = D.img("common/focusguide/2.png"):p(pos1):to(self):scale(0):hide()
    local hole2 = D.img("common/focusguide/2.png"):p(pos2):to(self):scale(0):hide()

    hole1:line({
        {"easing", "backOut", {"scaleTo", 0.5, scale1}},
        {"fn", function()
            self:unbindTouch()
            self:loadFingerGuide(pos1, pos2)
        end},
    })

    hole2:line({
        {"easing", "backOut", {"scaleTo", 0.5, scale2},},
    })

    self:scheduleUpdateWithPriorityLua(function ()
        self._render:beginWithClear(0, 0, 0, opa)
        local node1 = D.img("common/focusguide/3.png"):p(hole1:p()):scaleX(hole1:scaleX()):scaleY(hole1:scaleY())
        node1:setBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_ALPHA)
        node1:visit()

        local node2 = D.img("common/focusguide/3.png"):p(hole2:p()):scaleX(hole2:scaleX()):scaleY(hole2:scaleY())
        node2:setBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_ALPHA)
        node2:visit()
        self._render:endToLua()
    end, 0)
end


-- 加载引导遮罩（1个）#两个目标的位置 #洞窥X缩放 洞窥Y缩放
function M:loadMask(pos1, pos2, scaleX, scaleY)
    pos1        = pos1 or cc.p(V.w_2, 100)
    pos2        = pos2 or cc.p(V.w_2, V.h_2)
    local pos   = cc.pMidpoint(pos1, pos2)
    scaleX      = scaleX or 2.5
    scaleY      = scaleY or 5 * Y_FACTOR
    local opa   = 200 / 255
    self._render:beginWithClear(0, 0, 0, opa)
    self._render:endToLua()
    local hole = D.img("common/focusguide/2.png"):p(pos):to(self):scale(0):hide()

    hole:line({
        {"easing", "backOut", {"scaleTo", 0.5, scaleX, scaleY}},
        {"fn", function()
            self:unbindTouch()
            self:loadFingerGuide(pos1, pos2)
        end},
    })

    self:scheduleUpdateWithPriorityLua(function ()
        self._render:beginWithClear(0, 0, 0, opa)
        local node1 = D.img("common/focusguide/3.png"):p(hole:p()):scaleX(hole:scaleX()):scaleY(hole:scaleY())
        node1:setBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_ALPHA)
        node1:visit()
        self._render:endToLua()
    end, 0)
end


-- 加载手指引导
function M:loadFingerGuide(pos1, pos2)
    if not self._finger then
        local finger    = require("app.shapefeed.node.GuideFinger").new({
            startPos    = pos1,
            endPos      = pos2,
        }):to(self):p(0, 0)
        self._finger = finger
    end
    self._finger:openHandGuide()
end

-- 停止手指引导
function M:stopFingerGuide()
    if self._finger then
        self._finger:closeHandGuide()
    end
end

-- 移除自身
function M:removeSelf()
    self:stopFingerGuide()
    self:remove()
end

-- -- 开始引导
-- function M:beginGuide()
-- 	local boxPos = self._boxPos
--     self._render:beginWithClear(0, 0, 0, 220/255)
--     self._render:endToLua()
--     local hole1 = D.img("common/uibar.png"):p(boxPos):to(self):scale(0):hide()
--     -- local hole2 = D.img("common/blank.png"):p(V.w_2, 120):to(self):scale(0):hide()

--     local t1 = soundVoice:soundDuration("v255097", 3)
--     local t2 = soundVoice:soundDuration("v255098", 3)


--     -- 提示框先放大
--     hole1:line({
--         {"scaleTo", 0.3, 1, 1},
--         {"scaleTo", 0.15, 1 * 1.1, 1 * 1.1},
--         {"scaleTo", 0.1, 1 * 1.0, 1 * 1.0},
--         {"fn", function()
--         	-- 这里是翼龙想要的鱼
--         	soundVoice:playEffectv255097()
--         end},
--         {"delay", t1},
--         {"delay", 0.5},
--         {"fn", function()
--             -- 去海里找找吧
--             soundVoice:playEffectv255098()
--         end},
--         {"delay", t2},
--         {"fn", function()
-- 			self._absorbTouchNode:remove()
--             self._layer:removeMaskGuide()
--         end},
--     })

--     self:scheduleUpdateWithPriorityLua(function ()
--         self._render:beginWithClear(0, 0, 0, 220/255)
--         local node1 = D.img("common/uibar.png"):p(hole1:p()):scaleX(hole1:scaleX()):scaleY(hole1:scaleY())
--         node1:setBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_ALPHA)
--         node1:visit()
--         self._render:endToLua()
--     end, 0)
-- end



--------------------------
-- 触控
-------------------------- 

function M:onTouchBegan(x, y, touches)
	--  触控有效
    return SIGN_TOUCH_BEGAN_SWALLOWS 
end

function M:onTouchMoved(x, y, touches)
	--   
end

function M:onTouchEnded(x, y, touches)
	--  
end


--------------------------
-- 析构
--------------------------
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
 	-- [本类调用]

end



return M
