--[[
Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

场景层类，定义层相关操作方法及逻辑实现。

-   定义场景层功能方法。

]]
----------------------
-- 类
----------------------
local M = classLayerTouch("Main")


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
local Config = require("app.shapeline.Config")
local Menu = require("app.shapeline.node.Menu")
local Prop = require("app.shapeline.node.Prop")
local TargetNode = require("app.shapeline.node.TargetNode")
local TipNode = require("app.shapeline.node.TipNode")
local GuideFinger = require("app.common.GuideFinger")
local DevY = -50  -- 形状的Y轴偏移值

-- [操作变量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义视图实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)

    -- 出现的提示点的集合
    self._tipNodeTab = {}
    -- 提示点的索引   
    self._tipIndex   = 0
    -- 提示点点击的次序
    self._clickIndex = 0
end

----------------------
-- 结点渲染
----------------------
--[[--

视图渲染，处理视图结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- 加载背景图
    self:loadBg()
    -- 获取当前轮次的关卡组合表
    self:getCurLevelTab()
    -- 加载手指引导
    self:loadGuideFinger()
    -- 加载菜单栏
    self:loadMenu(false)
	-- 道具入场
    self:enterNextProp()
    -- 播放开局语音
    self:playStartVoice()

    -- CUtil:loadTestNode(self, 20, pos)
end

-- 加载其他
function M:loadRenderOtherwise()

end

function M:onEnterTransitionFinish()
    -- 播放背景音乐[bgm14704]
    sound:playbgm14704()
end

-- 播放开局语音
function M:playStartVoice()
    -- 播放配音[v147089]点一点星星，帮形状精灵变身！
    soundVoice:playEffectv147089()
end

-- 加载背景图
function M:loadBg()
    local bg = U.loadBg("shape/line/bg.png"):to(self)
end

-- 获取当前轮次的关卡组合表
function M:getCurLevelTab()
	if (not G_shapeline_turnIndex) or G_shapeline_turnIndex == 0 or G_shapeline_turnIndex >= #G_shapelineLevelTab then
		G_shapeline_turnIndex = 1
	else
		G_shapeline_turnIndex = G_shapeline_turnIndex + 1
	end
	-- 关卡组合表
	self._shapeIdTab = G_shapelineLevelTab[G_shapeline_turnIndex]
	-- 关卡索引
	self._shapeIdIndex = 1
	-- 初始化关卡ID
	self._curShapeId = self._shapeIdTab[self._shapeIdIndex]
end

-- 加载菜单栏
function M:loadMenu(isExtend)
    local menu = Menu.new({
		main = self,
        shapeId = self._curShapeId,
        isExtend = isExtend
	}):to(self, 400):p(100, 100)
	
	self._menu = menu
end

-- 下一个道具进场
function M:enterNextProp()
    self._tipIndex = 0       -- 提示点的索引
    self._clickIndex = 0     -- 提示点点击的次序

	if not tolua.isnull(self._prop) then
        self._prop:remove()
	end

    local prop = self:createProp()
    prop:enterEffect()
    self._prop = prop
end

-- 创建道具
function M:createProp()
    local prop = Prop.new({
        main = self,
        shapeId = self._curShapeId,
        shapeName = Config.shapeInfo[self._curShapeId].shapeName
    }):to(self):p(V.w_2, 270 + DevY + Y_OFFSET)

    return prop
end

-- 当前道具退场，接着下一个道具进场(正常按顺序操作)
function M:quitAndEnterNextProp()
    if self._shapeIdIndex >= #self._shapeIdTab then
        self:enterNextScene()
        return 
    end
    self._shapeIdIndex = self._shapeIdIndex + 1

    -- 菜单栏手动切换形状后，则不按照预设的形状ID表顺序进行
    if self._isOperateByHand then
        if self._curShapeId <6 then
            self._curShapeId = self._curShapeId + 1
        else
            self._curShapeId = 1
        end
    else
        self._curShapeId = self._shapeIdTab[self._shapeIdIndex]
    end
    self._menu:tickSelect(self._curShapeId)

    soundEffect:playEffectsfx14713016()
    -- 激励粒子
    local particle = P.new("particle/tuwujl.plist"):to(self, 999):p(V.w_2, 20)
    particle:setAutoRemoveOnFinish(true)
    for i=2,3 do
        local particle = P.new("particle/tuwujl.plist"):to(self, 999):p(V.w_2, 20)
        particle:setAutoRemoveOnFinish(true):setTexture(R.loadTexture("particle/tuwujl"..i..".png"))
    end

    self:allTipNodeGetOut()
    self._prop:quitSubtitle()

    local callback = function ()
        self:enterNextProp()
    end
    self._prop:quitEffect(callback)
    -- 播放音效[sfx14706015]一组道具消失/吐完，形状向中间缩小消失的时候播放
    soundEffect:playEffectsfx14706015()
end

-- 当前道具退场，接着下一个道具进场(菜单栏手动切换)
function M:quitAndEnterNextPropByHand(shapeId)
    self._prop:clearPaint()
    if not tolua.isnull(self._prop._title) then
        self._prop._title:remove()
    end
    if not tolua.isnull(self._prop) then
        self._prop:remove()
    end
    self:allTipNodeGetOut()

    -- 是否手动操作
    self._isOperateByHand = true
    self._curShapeId = shapeId
    self:enterNextProp()
end

-- 进入下一个场景
function M:enterNextScene()
    self._prop._bone:stopGuide()
    self._prop._bone:stopAllActions()
    local encourage = CUtil:loadEncourage(self, 4, function ()
        G_addFinish("shapeline")
        SceneController:enterNextMoudleScene()
    end)
end

-- 创建移动点
function M:createTargetNode()
    local index = Config.shapeInfo[self._curShapeId].pauseIndexTab[1]
    local pos = Config.shapeInfo[self._curShapeId].points[index]

    local targetNode = TargetNode.new({
         main = self
    }):to(self, 300):p(pos):hide()
    
    targetNode:loadMoveClass()
    targetNode:enterEffect()
    self._targetNode = targetNode
end

-- 显示下一个提示点
function M:showNextTipNode()
    self._tipIndex = self._tipIndex + 1
    local pauseIndex = Config.shapeInfo[self._curShapeId].pauseIndexTab[self._tipIndex]
    local pos = Config.shapeInfo[self._curShapeId].points[pauseIndex]

    local tipNode = self:createTipNode(pos)
    table.insert(self._tipNodeTab, tipNode)
    tipNode:enterEffect()
    self._tipNode = tipNode
end

-- 创建提示点
function M:createTipNode()
    local index = Config.shapeInfo[self._curShapeId].pauseIndexTab[self._tipIndex]   -- 停顿的点坐标的索引
    local pos = Config.shapeInfo[self._curShapeId].points[index]                     -- 停顿的点坐标

    if self._tipIndex == #Config.shapeInfo[self._curShapeId].pauseIndexTab then
        self._tipNodeTab[1]:hide()
    end

    local tipNode = TipNode.new({
         main    = self,
         shapeId = self._curShapeId
    }):to(self, 300):p(pos)
    return tipNode
end

-- 提示点和移动点触控锁
function M:tipAndTargetTouchLock(bool)
    self._targetNode._touchLock = bool
    self._tipNode._touchLock = bool
end 

-- 获取描边轮廓的路径信息（点坐标，停顿点索引）
function M:getOutlinePathInfo()
    local points = Config.shapeInfo[self._curShapeId].points
    local pauseIndexTab = Config.shapeInfo[self._curShapeId].pauseIndexTab
    return points, pauseIndexTab
end

-- 所有提示点消失(包括targetNode)
function M:allTipNodeGetOut()
    -- -- 播放音效[sfx14706009]连完所有点消失/连线完成，所有的圆点消失的时候播放
    -- soundEffect:playEffectsfx14706009()
    for i, v in pairs(self._tipNodeTab) do
        v:remove()
    end
    self._tipNodeTab = {}

    if tolua.isnull(self._targetNode) then
        self._targetNode:remove()
    end
end

-- 加载手指引导
function M:loadGuideFinger(pos1, pos2)
    if not self._finger then
        local finger    = GuideFinger.new({
            startPos    = pos1,
            endPos      = pos2,
        }):to(self, 10000):p(0, 0)
        self._finger = finger
    end
end

-- 播放手指点击引导（跟随律动引导的line动作，非cycle）
function M:playFingerClickGuide(node)
	local wpos = node:worldpoint()
	local anchorY = node:getAnchorPoint().y
    wpos.y = wpos.y + node:ch()/2 - node:ch()*anchorY + 30
	self._finger:playFingerClickGuide(wpos)
end

-- 停止手指引导(拖动/点击通用)
function M:stopFingerGuide()
	self._finger:closeHandGuide()
end

----------------------
-- 结点析构
----------------------
--[[--

视图析构，处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

function M:onExitTransitionStart()
    -- R.removeAllTextures()
    -- R.removeAllFrames()
    -- DragonBone.removeAll()
end

return M