--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-20

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类 -- 菜单键
----------------------
local M     = classSpriteTouch("Menu")
M.TAG       = "Menu"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
    M.super.ctor(self, params)

	self._main     = params.main
	-- 初始时的形状Id
    self._shapeId  = params.shapeId
	-- 选项是否展开
	self._isExtend = params.isExtend or false	 
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
    self:loadDisplay()
    -- 加载显示栏
	self:loadBar()
	-- 加载菜单栏形状
	self:loadShapeOption()
end

-- 图片渲染
function M:loadDisplay()
	self:setTexture("common/ui/circle.png"):opacity(0)
    local showBtn = D.img("common/ui/circle.png"):to(self):p(self:cw()/2, self:ch()/2)
	-- 点击按钮上的形状
	local devY
	if self._shapeId == 1 then
		devY = 7
	elseif self._shapeId == 4 then
		devY = 1
	elseif self._shapeId == 5 then
		devY = 3
	else
		devY = 0
	end
	local pos = ccp(self:cw()/2, self:ch()/2 + devY)
    local btnShape = D.img("shape/line/menu/shape"..self._shapeId..".png"):to(showBtn):p(pos):scale(0.9)
	self._showBtn = showBtn
    self._btnShape = btnShape
end

-- 加载菜单栏
function M:loadBar()
    -- 菜单栏
    local bar = D.img("common/ui/bar.png"):anchor(ccp(0, 0.5))
    -- 按栏的最大区域进行矩形裁切
    local clippingRect = cc.ClippingRectangleNode:create(cc.rect(self:cw()/2, self:ch()/2 - bar:ch()/2, bar:cw(), bar:ch()))

    -- 按按钮图片样式进行裁切（按钮半透明，仅矩形裁切不够）
    local clipImg   = D.img("common/ui/circle.png"):p(self:cw()/2, self:ch()/2):scale(0.8)
    -- 创建遮罩蒙版裁切
    local clip      = CCNodeExtend.extend(cc.ClippingNode:create(clipImg)):to(clippingRect)
    -- 设置裁剪区域的显示，false为显示裁剪的区域，true为裁剪后的区域(很关键)
    clip:setInverted(true)
    -- 设置alpha阈值， 只有模板（stencil）的alpha像素大于alpha阈值（alphaThreshold）时内容才会被绘制。
    -- alpha阈值（threshold）范围应是0到1之间的浮点数。 alpha阈值（threshold）默认为1。
    clip:setAlphaThreshold(0)

    bar:to(clip, 10):p(self:cw()/2 - bar:cw(), self:ch()/2)

    if self._isExtend then
        bar:px(self:cw()/2)
    end
    self._bar          = bar
    self._clippingRect = CCNodeExtend.extend(clippingRect):to(self, -999)

    -- 绑定触控
    bar:bindTouch()
    clippingRect:setContentSize(cc.size(0.01, 0.01)):bindTouch()
    clip:setContentSize(cc.size(0.01, 0.01)):bindTouch()
    function clippingRect:onTouchBegan()
        return SIGN_TOUCH_BEGAN_NO_SWALLOWS
    end
    function bar:onTouchBegan()
        return SIGN_TOUCH_BEGAN_NO_SWALLOWS
    end
    function clip:onTouchBegan()
        return SIGN_TOUCH_BEGAN_NO_SWALLOWS
    end
end

-- 加载菜单栏形状
function M:loadShapeOption()
	self._options 	= {}
	local width 	= self._bar:cw() / 6.7
	for i = 1 ,6 do
		local node 	= D.img("shape/line/menu/shape"..i..".png"):to(self._bar):p(width * i, self._bar:ch()/2)
		node._tick = D.img("common/ui/tick.png"):to(node):p(node:cw()/2+25 , 15):hide()
		if self._shapeId == i then
			node._tick:show()
		end
		table.insert(self._options, node)
		--点击形状如果与当前相同，则切换道具，否则跳转到对应形状的场景
		local menu = self
		node:bindTouch({
			fnBegan = function (sender, x, y)
				if menu._isInChange or not menu._isExtend then return end
				    -- 播放音效[sfx14700001]点击
					soundEffect:playEffectsfx14700001()
					
					self:setBtnCurShape(i)
					self:tickSelect(i)
					self:playShapeVoice(i)
					self:barChange(false)
					self._main:quitAndEnterNextPropByHand(i)
			end
		})
	end
end

-- 勾选形状按钮选项
function M:tickSelect(id)
	for i, v in pairs(self._options) do
		if id == i then
			v._tick:show()
		else
			v._tick:hide()
		end
	end
end

-- 播放形状台词
function M:playShapeVoice(id)
	soundVoice:stopNowSound()
	if id == 1 then
		-- 播放配音[v147084]三角形
		soundVoice:playEffectv147084()
	elseif id == 2 then
		-- 播放配音[v147081]正方形
		soundVoice:playEffectv147081()
	elseif id == 3 then
		-- 播放配音[v147083]长方形
		soundVoice:playEffectv147083()
	elseif id == 4 then
		-- 播放配音[v147082]圆形
		soundVoice:playEffectv147082()
	elseif id == 5 then
		-- 播放配音[v147086]星型
		soundVoice:playEffectv147086()
	elseif id == 6 then
		-- 播放配音[v147085]爱心形
		soundVoice:playEffectv147085()
	end
end

-- 设置菜单栏按钮的形状显示
function M:setBtnCurShape(id)
	self._shapeId = id
	self._btnShape:setTexture("shape/line/menu/shape"..id..".png")
	local devY
	if id == 1 then
		devY = 7
	elseif id == 4 then
		devY = 1
	elseif id == 5 then
		devY = 3
	else
		devY = 0
	end
	self._btnShape:p(self:cw()/2, self:ch()/2 + devY)
end

-- 菜单栏按钮显示打叉
function M:setBtnX()
	self._btnShape:setTexture("common/ui/cross.png")
	self._btnShape:p(self:cw()/2, self:ch()/2)
end

-- 菜单栏状态切换(收起或展开选项),是否变成展开状态
function M:barChange(isExtend)
	if self._isExtend == isExtend then return end
	self._isInChange 	= true	      -- 正在切换状态
	self._isExtend 	= isExtend 	      -- 菜单栏状态切换布尔值

	if isExtend == true then
		self:setBtnX()
		-- 播放音效[sfx14713001]左下角窗口打开
		soundEffect:playEffectsfx14713001()
	else
		self:setBtnCurShape(self._shapeId)
		-- 播放音效[sfx14713002]左下角窗口关闭
		soundEffect:playEffectsfx14713002()
	end

	-- 根据状态判断移动方向
	local num 			= self._isExtend and 1 or -1
	self._bar:line({
		{"moveBy", 0.5, cc.p(num * self._bar:cw(), 0)},
		{"fn", function()
			self._isInChange 	= false
		end}
	})
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
	if self._isInChange then return end
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()

	self._showBtn:line({
		{"scaleTo", 0.15, 0.85},
	})
	return true
end

function M:onTouchMoved(x, y, touches)

end

function M:onTouchEnded(x, y, touches)
	self._showBtn:stopAllActions():line({
		{"scaleTo", 0.2, 1},
	})
	self:barChange(not self._isExtend)
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M