--
-- Author: Babybus Coder
--   Date: 2017-10-16 11:42:52
--
--[[Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2020-9-23

]]
--[[	类 跟随路径移动类
--]]
local M    = class("MoveClass")

--------------------------
-- 构造函数
--------------------------
function M:ctor(params)
    -- 目标精灵
    self._targetNode = params.targetNode

    self:initMovePoint()
end

-- 初始化参数
function M:initMovePoint()
    -- 当前组成笔画的点集索引
    self.currentPointIndex = 1
    -- 移动的前后点
    self.point1 = 1
    self.point2 = 1
    -- 当前笔画走向
    -- isX:0  无走向， 大于0 有走向，值越大，优先级越高
    -- isIncreasing:true 坐标递增，false坐标递减
    self.direction    = { isX = 0, isIncreasingX = false, isY = 0, isIncreasingY = false }
    -- 下一笔画走向
    self.nextDirection = { isX = 0, isIncreasingX = false, isY = 0, isIncreasingY = false }
    -- 模糊判断
    self.openFuzzy    = false
    -- 模糊坐标
    self.fuzzyPoint    = nil
    self._point        = nil
    -- 当前垃圾移动的路径
    self.crtMovePoints = {}
    self._finishAll = false
end

-- 是否将近拐弯处
function M:iscorner()
    local num = #self.crtMovePoints
    -- if not self.point2 then return end
    local dis = cc.pGetDistance(self._targetNode:point(), self.point2)
    if dis < 40 and (num - self.currentPointIndex) > 1 then
        self.openFuzzy    = true
    else
        self.openFuzzy    = false
        self.fuzzyPoint = nil
        self._point    = nil
    end
end

-- 判断移动x, y
function M:judgeDrawMove(targetPoint, disx, disy)
    local autoSpeed = 5
    if self.direction.isX >= self.direction.isY then  -- x优先级高
        targetPoint = self:priorityX(targetPoint, disx, disy, autoSpeed)
        return targetPoint
    elseif self.direction.isY > self.direction.isX then  -- y优先级高
        targetPoint = self:priorityY(targetPoint, disx, disy, autoSpeed)
        return targetPoint
    end
end

-- 优先x
function M:priorityX(targetPoint, disx, disy, autoSpeed)
    -- 模糊写法,只要在拐角处，且笔画xy走向不同的时候使用
    local isFuzzy = false
    if self.openFuzzy and math.abs(disy) > math.abs(disx) and self.nextDirection.isX < self.nextDirection.isY then
        if (self.nextDirection.isIncreasingY and disy > 0) or (not self.nextDirection.isIncreasingY and disy < 0) then
            if self.direction.isIncreasingX then
                targetPoint.x = targetPoint.x + autoSpeed
            else
                targetPoint.x = targetPoint.x - autoSpeed
            end
            isFuzzy = true
        end
    end

    -- 非模糊写法
    if not isFuzzy then
        if (self.direction.isIncreasingX and disx > 0) or (not self.direction.isIncreasingX and disx < 0) then
            targetPoint.x = targetPoint.x + disx
        end
    end

    -- 得到按钮坐标
    if self.point2.x > self.point1.x then
        if targetPoint.x >= self.point2.x then
            targetPoint = self.point2
            self:completePtoP()
        elseif targetPoint.x <= self.point1.x then
            targetPoint = self.point1
        else
            targetPoint.y = self:getDrawY(targetPoint.x)
        end
        return targetPoint
    elseif self.point2.x < self.point1.x then
        if targetPoint.x <= self.point2.x then
            targetPoint = self.point2
            self:completePtoP()
        elseif targetPoint.x >= self.point1.x then
            targetPoint = self.point1
        else
            targetPoint.y = self:getDrawY(targetPoint.x)
        end
        return targetPoint
    end
end

-- 优先y
function M:priorityY(targetPoint, disx, disy, autoSpeed)
    -- 模糊写法
    local isFuzzy = false
    if self.openFuzzy and math.abs(disx) > math.abs(disy) and self.nextDirection.isX > self.nextDirection.isY then
        if (self.nextDirection.isIncreasingX and disx > 0) or (not self.nextDirection.isIncreasingX and disx < 0) then
            if self.direction.isIncreasingY then
                targetPoint.y = targetPoint.y + autoSpeed
            else
                targetPoint.y = targetPoint.y - autoSpeed
            end
            isFuzzy = true
        end
    end
    -- 非模糊写法
    if not isFuzzy then
        if self.direction.isIncreasingY and disy > 0 then
            targetPoint.y = targetPoint.y + disy
        elseif not self.direction.isIncreasingY and disy < 0 then
            targetPoint.y = targetPoint.y + disy
        end
    end

    -- 得到按钮坐标
    if self.point2.y > self.point1.y then
        if targetPoint.y >= self.point2.y then
            targetPoint = self.point2
            self:completePtoP()
        elseif targetPoint.y <= self.point1.y then
            targetPoint = self.point1
        else
            targetPoint.x = self:getDrawX(targetPoint.y)
        end
        return targetPoint
    elseif self.point2.y < self.point1.y then
        if targetPoint.y <= self.point2.y then
            targetPoint = self.point2
            self:completePtoP()
        elseif targetPoint.y >= self.point1.y then
            targetPoint = self.point1
        else
            targetPoint.x = self:getDrawX(targetPoint.y)
        end
        return targetPoint
    end
end

-- 获得连线两节点
function M:getPoint12()
    -- 新的节点
    self.point1 = self.crtMovePoints[self.currentPointIndex]
    self.point2 = self.crtMovePoints[self.currentPointIndex + 1]
    -- 判断   当前笔画走向
    -- 重置数据
    self.direction        = { isX = 0, isIncreasingX = false, isY = 0, isIncreasingY = false }
    self.nextDirection    = { isX = 0, isIncreasingX = false, isY = 0, isIncreasingY = false }
    self.openFuzzy        = false

    local disx = self.point2.x - self.point1.x
    local disy = self.point2.y - self.point1.y

    if disx ~= 0 then
        self.direction.isX = 1
        self.direction.isIncreasingX = disx > 0 and true or false
    end
    if disy ~= 0 then
        self.direction.isY = 1
        self.direction.isIncreasingY = disy > 0 and true or false
    end

    if math.abs(disx) >= math.abs(disy) then
        self.direction.isX = 2
    else
        self.direction.isY = 2
    end

    local totalIndex = #self.crtMovePoints

    if self.currentPointIndex + 2 > totalIndex then return end
    -- 判断   下一笔画走向
    local point2 = self.crtMovePoints[self.currentPointIndex + 2]
    local disx1 = point2.x - self.point2.x
    local disy1 = point2.y - self.point2.y
    if disx1 ~= 0 then
        self.nextDirection.isX = 1
    end
    if disy1 ~= 0 then
        self.nextDirection.isY = 1
    end

    if math.abs(disx1) >= math.abs(disy1) then
        self.nextDirection.isX = 2
        self.nextDirection.isIncreasingX = disx1 > 0 and true or false
    else
        self.nextDirection.isY = 2
        self.nextDirection.isIncreasingY = disy1 > 0 and true or false
    end
end

-- 完成两点连接
function M:completePtoP()
    self.currentPointIndex = self.currentPointIndex + 1
    if self.currentPointIndex == #self.crtMovePoints then
        local node = self._targetNode
        node:paintFinish()
        print("成功到达终点！！！")
        self._finishAll = true
        self._notFirstTouch = false
    else
        self:getPoint12()
    end
end

-- 设置轨迹坐标
function M:setMovePoints(points, pauseIndexTab)
    if points then
        self.crtMovePoints = points
        self._pauseIndexTab = pauseIndexTab   -- 停顿点的索引集合
        self:getPoint12()
    end
end

-- 检查是否结束
function M:checkFinish()
    if self._finishAll then return end
    if self.currentPointIndex >= #self.crtMovePoints then
        self._finishAll = true
        self._notFirstTouch = false
        self._targetNode:paintFinish()
    end
end

-- 自动移动
function M:autoMove()
    local node = self._targetNode
    node._touchLock = true
    self.currentPointIndex = self.currentPointIndex + 1
    self._nextPoint = self.crtMovePoints[self.currentPointIndex]
    self._time = ccpDistance(node:point(), self._nextPoint) / 360
    node:cyclePaint()
    node:line({
        { "moveTo", self._time, self._nextPoint},
        { "fn", function()
            self:checkPauseOrContinue(node)
        end }
    })
end

-- 检查是停顿还是继续移动
function M:checkPauseOrContinue(node)
    local bool1 = false
    local bool2 = false

    if self.currentPointIndex >= self._pauseIndexTab[#self._pauseIndexTab] then
        bool1 = true
    end

    for i, v in pairs(self._pauseIndexTab) do
        if v == self.currentPointIndex then
            bool2 = true
        end
    end

    if bool1 and bool2 then   -- 到达终点
        print("成功到达终点！！！")
        node:pausePaint()
        self._finishAll = true
        self._notFirstTouch = false
        node:paintFinish()
        return
    elseif bool2 then   -- 暂停
        node:pausePaint()
        self:getPoint12()
        -- node._main:tipAndTargetTouchLock(false)
        node._main:showNextTipNode()
        return
    end

    -- 继续移动到下一个描边点
    self.currentPointIndex = self.currentPointIndex + 1
    self._nextPoint = self.crtMovePoints[self.currentPointIndex]
    local time = ccpDistance(node:point(), self._nextPoint) / 360
    node:line({
        { "moveTo", time, self._nextPoint},
        { "fn", function()
            self:checkPauseOrContinue(node)
        end }
    })
end

-- 设置写字按钮的位置和角度
function M:setWordBtn()
    local angle = ccpToAngle(ccpSub(self.point2, self.point1)) / math.pi * 180 - 90
    if self._targetNode.setScissorsRotation then
        self._targetNode:setScissorsRotation(-angle)
    end
end

-- 根据y获得x, 线性方程
function M:getDrawX(y)
    local x = (y - self.point1.y) * (self.point2.x - self.point1.x) / (self.point2.y - self.point1.y) + self.point1.x
    return x
end

-- 根据x获得y, 线性方程
function M:getDrawY(x)
    local y = (x - self.point1.x) * (self.point2.y - self.point1.y) / (self.point2.x - self.point1.x) + self.point1.y
    return y
end

return M