

--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("Prop")
M.TAG       = "Prop"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
local Config = require("app.shapeline.Config")
-- 形状的类:1-三角形 2-正方形 3-长方形 4-圆形 5-星形 6-爱心形
local ShapeClasses = {
    require("app.common.shape.Shape1Triangle"),
    require("app.common.shape.Shape2Square"),
    require("app.common.shape.Shape4Rectangle"),
	require("app.common.shape.Shape3Circle"),
	require("app.common.shape.Shape6Star"),
    require("app.common.shape.Shape5Heart"),
}
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
	
	self._main       = params.main
	self._shapeId    = params.shapeId
	self._shapeName  = params.shapeName
	self._scale      = 0.8
	self._itemZIndex = 1000
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
	-- 创建形状骨骼
	self:createBone()
end

-- 图片渲染
function M:loadDisplay()
	self:display("")
	self:scale(0)

	-- 这个outline1虚线，只用来道具入场显示，入场完成就隐藏掉了
	local outline1 = D.img("shape/line/outline/shape"..self._shapeId.."/outline1.png")
	:to(self):p(self:cw()/2, self:ch()/2)
	self._outline1 = outline1
end

-- 道具入场效果
function M:enterEffect()
	self:line({
		{ "easing", "backOut", { "scaleTo", 0.5, 1 } },
		{"fn", function ()
			self._outline1:hide()
			self:loadOthers()
			self:enterHandler()
		end}
	})
end

-- 道具入场后的回调
function M:enterHandler()
	self._main:createTargetNode()
	self._main:showNextTipNode()

	if self._main._menu._isInChange then
		return
	else
		self._main._menu:setBtnCurShape(self._shapeId)
	end
end

-- 道具退场效果
function M:quitEffect(callback)
	self:line({
		{ "easing", "backIn", { "scaleTo", 0.5, 0 } },
		{"fn", function ()
			if callback then
				callback()
			end
		end}
	})
end

-- 加载其他
function M:loadOthers()
	local shape = D.img("shape/line/outline/shape"..self._shapeId.."/shape.png")
	:to(self):p(self:cw()/2, self:ch()/2):opacity(0)
	self._shape = shape

	self:createPaint()
end

-- 形状轮廓完成后的回调
function M:onPaintFinishHandler()
	self:clearPaint()
	-- 粒子
	local particle = CUtil:newParticle("particle/twbs.plist"):to(self, 100):p(self:cw()/2, self:ch()/2)
	-- 播放音效[sfx14706033]形状出现之后中心粒子
	soundEffect:playEffectsfx14706033()
	self._shape:line({
		{"fadeTo", 0.3, 255},
		{"delay", 1},
		{"fn", function ()
			self:playBecomeBoneAni()
		end}
	})
end

-- 播放变成骨骼精灵动画
function M:playBecomeBoneAni()
	self:setCascadeOpacityEnabled(true)
	CUtil:changeAnchorP(self, ccp(0.5, 0))
	-- CUtil:adjustAnchor(self)  -- 锚点调试
	self:playVioceAndSubtitle()

	-- 播放音效[sfx14706010]形状变身/形状变身的时候播放
	soundEffect:playEffectsfx14706010()

	self:line({
		A.union({
			{"scaleTo", 0.3, self._scale},
			{"jumpBy", 0.3, ccp(0, 0), 100, 1},
		}),
		{"fn", function ()
			self._shape:hide()
			self._outline1:hide()
			self._outline2:hide()
			self._bone:show()
			self._bone:changeItemSkin()
			self._bone:transformationAct()
			local particle = CUtil:newParticle("particle/bianshen.plist"):to(self, 1000):p(self:cw()/2, self:ch()/2)
			self._bone:onBecomeBoneHandler()
		end},
	})
end

-- 播放道具语音和字幕
function M:playVioceAndSubtitle()
	if isZhLanguage() then
		local subtitlePath = self:getSubtitlePath()
		local title = D.img(subtitlePath):to(self._main, 150):p(self:px(), self:py() + 170):scale(0)
		title:line({
			A.union({
				{ "easing", "backOut", { "scaleTo", 0.6, 1.2 } },
				{ "jumpBy", 0.3, cc.p(0, 0), 50, 1 },
			}),
		})
		self._title = title
	end

	self:playShapeVoice()
end

-- 播放形状台词
function M:playShapeVoice()
	soundVoice:stopNowSound()
	if self._shapeId == 1 then
		-- 播放配音[v147084]三角形
		soundVoice:playEffectv147084()
	elseif self._shapeId == 2 then
		-- 播放配音[v147081]正方形
		soundVoice:playEffectv147081()
	elseif self._shapeId == 3 then
		-- 播放配音[v147083]长方形
		soundVoice:playEffectv147083()
	elseif self._shapeId == 4 then
		-- 播放配音[v147082]圆形
		soundVoice:playEffectv147082()
	elseif self._shapeId == 5 then
		-- 播放配音[v147086]星型
		soundVoice:playEffectv147086()
	elseif self._shapeId == 6 then
		-- 播放配音[v147085]爱心形
		soundVoice:playEffectv147085()
	end
end

-- 字幕退场
function M:quitSubtitle()
	if tolua.isnull(self._title) then
		return
	end
	self._title:line({
		{"easing", "backIn", {"scaleTo", 0.6, 0}},
		{"remove"}
	})
end

-- 获取字幕文件路径
function M:getSubtitlePath()
	local subtitlePath
	if self._shapeId == 1 then
		subtitlePath = "1401.png"
	elseif self._shapeId == 2 then 
		subtitlePath = "1402.png"
	elseif self._shapeId == 3 then 
		subtitlePath = "1403.png"
	elseif self._shapeId == 4 then 
		subtitlePath = "1404.png"
	elseif self._shapeId == 5 then 
		subtitlePath = "1405.png"
	elseif self._shapeId == 6 then 
		subtitlePath = "1406.png"
	end
	return subtitlePath
end

-- 道具移动位置(吐物时)
function M:propMoveWhenSpit()
	if self._isMoved then return end   -- 是否移动过一次
	self._isMoved = true
	self:line({
		{"delay", 0.2},
		{"moveTo", 0.3, ccp(335+X_OFFSET, self:py())},
	})
	if self._title and not tolua.isnull(self._title)then
		self._title:line({
			{ "delay", 0.2 },
			{ "moveTo", 0.3, ccp(335 + X_OFFSET, self._title:py()) },
		})
	end
end

-- 创建形状骨骼
function M:createBone()
	local bone = ShapeClasses[self._shapeId].new({
		layer   = self._main,
		parent  = self,
		shapeId = self._shapeId
	}):to(self, 300):p(self:cw()/2, self:ch()/2):hide()
	bone._touchLock = true
	self._bone = bone
	
	local parent = self
	-- 赋予触控事件
	function bone:onTouchBegan(x, y)
		if bone._touchLock then return end
		bone._touchLock = true
		parent._main._menu:barChange(false)
		soundVoice:stopNowSound()
		-- 播放音效[sfx14700001]点击
		soundEffect:playEffectsfx14700001()
		
		bone:stopGuide()
		bone._rhythmCount = 0
		bone:checkAndSpitOutItem()
		parent:propMoveWhenSpit()
		return SIGN_TOUCH_BEGAN_SWALLOWS
	end
end

-- 创建实物节点（用于后续的物品退出/展示效果）
function M:createItem()
	self._itemZIndex = self._itemZIndex - 1
	local pos = Config.itemInfo.pos[self._shapeId][self._bone._itemId]
	local item = D.img("shape/line/item/shape"..self._shapeId.."/item"..self._bone._itemId..".png")
	:to(self, self._itemZIndex):p(pos)
	return item
end

-- 创建画板
function M:createPaint()
	-- 路径
	local path 		= "shape/line/outline/shape"..self._shapeId.."/"
	-- 道具的坐标
	local point 	= cc.p(self:cw()/2, self:ch()/2)
	-- 最终画出来的道具
	local outline2 		= D.img(path.."outline2.png"):to(self, 10):p(point)
	local color     = Config.shapeInfo[self._shapeId].color
	outline2:color(color)
	-- 画布
	local canvas 	= CCNodeExtend.extend(cc.RenderTexture:create(V.w, V.h))
	canvas:to(self._main, 200):p(V.w_2, V.h_2)
	canvas:begin()
	-- 把画板和道具阴影画在画布上做擦除，显示出来的就是画出来的效果
	local node1  	= U.loadBg("shape/line/bg.png"):visit()
	local node2 	= D.img(path.."outline1.png"):p(self:worldpoint()):visit()
	canvas:endToLua()

	self._outline2 	= outline2
	self._canvas 	= canvas
end

-- 形状轮廓描绘
function M:paint(startPos, endPos)
	if tolua.isnull(self._canvas) then return end
		
	local p1 = startPos --self:convertToNodeSpace(startPos)
	local p2 = endPos   --self:convertToNodeSpace(endPos)
    -- 绘图
    self._canvas:begin()
    local size 			= 8	
    local distance 		= PT.distance(p1, p2) 		-- 两点之间距离
    local distanceX 	= p2.x - p1.x 				-- x轴距离
    local distanceY 	= p2.y - p1.y 				-- y轴距离
    for i = 0, distance, size do
        local rate 		= i / distance 						-- 比例
        local pos 		= cc.p(p1.x + (distanceX * rate), p1.y + (distanceY * rate))
	    local renderNode 	= D.img("common/help.png"):scale(1.5/self:scale())
	    renderNode:setBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_ALPHA)
        renderNode:p(pos)
        renderNode:visit()
    end
    self._canvas:endToLua()
end

-- 清除画板
function M:clearPaint()
	if not tolua.isnull(self._canvas) then
		self._canvas:remove()
		self._canvas = nil
	end
end

-- 实物退出场
function M:itemGetOutAction(item, callback)
	local getOutTypeId = Config.itemInfo.getOutTypeTab[self._shapeId][self._bone._showItemId]
	if getOutTypeId == 1 then
		-- 音效写在动作里
		self:frogJumpOutAction(item, callback)
	elseif getOutTypeId == 2 then
		-- 播放音效[sfx14700004]物品离场-弹跳走
		soundEffect:playEffectsfx14700004()
		self:bounceAction(item, callback)
	elseif getOutTypeId == 3 then
		-- 播放音效[sfx14706032]矩形吐物离场/吐出来的东西离场的时候播放
		soundEffect:playEffectsfx14706032()
		self:jumpOutAction(item, callback)
	elseif getOutTypeId == 4 then
		-- 播放音效[sfx14700005]物品离场-滚走
		soundEffect:playEffectsfx14700005()
		self:rollOutAction(item, callback)
	elseif getOutTypeId == 5 then
		-- 播放音效[sfx14700010]物品离场-摇摇晃晃
		soundEffect:playEffectsfx14700010()
		self:shakeOutAction(item, callback)
	elseif getOutTypeId == 6 then
		-- 播放音效[sfx14700006]物品离场-飘走
		soundEffect:playEffectsfx14700006()
		self:flyOutAction(item, callback)
	end
end

-- 往右下角跳出去动作（类似涂色的青蛙跳）
function M:frogJumpOutAction(item, callback)
	-- 放慢动作时间的倍数
	local multiplier = 1.2
	item:line({
		A.union({
			{"jumpBy", 0.4 * multiplier, cc.p(100, -85 * Y_FACTOR), 100, 1},
			A.line({
				{"scaleTo", 0.15 * multiplier, 0.9 * 1, 1.15 * 1},
				{"scaleTo", 0.2 * multiplier, 1.15 * 1, 0.9 * 1},
				{"scaleTo", 0.1 * multiplier, 1 * 1},
			})
		}),
		{"fn", function ()
			-- 播放音效[sfx14700004]物品离场-弹跳走
			self._effect = soundEffect:playEffectsfx14700004()
		end},
		{"delay", 0.05},
		A.union({
			{"jumpBy", 0.4 * multiplier, cc.p(-160, -85 * Y_FACTOR), 120, 1},
			A.line({
				{"scaleTo", 0.15 * multiplier, 0.9 * 1.13, 1.15 * 1.13},
				{"scaleTo", 0.2 * multiplier, 1.15 * 1.13, 0.9 * 1.13},
				{"scaleTo", 0.1 * multiplier, 1 * 1.13},
			})
		}),
		{"delay", 0.05},
		A.union({
			{"jumpBy", 0.4 * multiplier, cc.p(160, -210 * Y_FACTOR), 100, 1},
			A.line({
				{"scaleTo", 0.15 * multiplier, 0.9 * 1.3, 1.15 * 1.3},
				{"scaleTo", 0.2 * multiplier, 1.15 * 1.3, 0.9 * 1.3},
				{"scaleTo", 0.1 * multiplier, 1 * 1.3},
			})
		}),
		{"fn", function()
			sound.stopSound(self._effect)
			if callback then
				callback()
			end
		end},
		{"remove"},
	})
end

-- 弹跳出去动作
function M:bounceAction(item, callback)
	-- 放慢动作时间的倍数
	local multiplier = 1.2
	item:line({
		A.cycle({
			A.union({
				{"jumpBy", 0.4 * multiplier, cc.p(200, 0), 50, 1},
				A.line({
					{"scaleTo", 0.1 * multiplier, 0.9, 1.15},
					{"scaleTo", 0.2 * multiplier, 1.1, 0.95},
					{"scaleTo", 0.1 * multiplier, 1},
				})
			})
		}, 4),
		{"fn", function()
			if callback then
				callback()
			end
		end},
		{"remove"},
	})
end

-- 向右前方一步跳出动作
function M:jumpOutAction(item, callback)
	-- 跳跃的动作参数
	local multiplier = 1.2                      -- 放慢动作时间的倍数
	local devPos 	 = ccp(500+X_OFFSET, -100)  -- 落地位置和距离初始位置的偏移向量坐标
	local height 	 = 150						-- 跳动高度
	local number  	 = 1			        	-- 跳动次数
	local multiplier = 1.2                      -- 放慢动作时间的倍数
	item:line({
		A.cycle({
			A.union({
				{"jumpBy", 0.48 * multiplier, devPos, height, 1},
				A.line({
			        {"scaleTo", 0.12 * multiplier, 0.9, 1.15},
			        {"scaleTo", 0.24 * multiplier, 1.1, 0.95},
			        {"scaleTo", 0.12 * multiplier, 1},
				})
			})
		}, number),
		{"fn", function()
			if callback then
				callback()
			end
		end},
		{"remove"},
	})
end

-- 滚动出去动作
function M:rollOutAction(item, callback)
	-- 放慢动作时间的倍数
	local multiplier = 1.2
	item:line({
		A.union({
			{"moveBy", 1.6 * multiplier, cc.p(800, 0)},
			{"rotateBy", 1.6 * multiplier, 720},
		}),
		{"fn", function ()
			item:hide()

			if callback then
				callback()
			end
		end},
		{"remove"},
	})
end

-- 摇晃出去动作
function M:shakeOutAction(item, callback)
	local multiplier = 1.2
	CUtil:changeAnchorP(self, cc.p(0.5, 0.2))
	item:line({
		{"moveBy", 1.6 * multiplier, cc.p(800, 0)},
		{"fn", function()
			if callback then
				callback()
			end
		end},
		{"remove"},
	})

	item:cycle({
		{"easing", "SINEOUT", {"rotateBy", 0.2 * multiplier, 16}},
		{"easing", "SINEIN", {"rotateBy", 0.2 * multiplier, -16}},
		{"easing", "SINEOUT", {"rotateBy", 0.2 * multiplier, -16}},
		{"easing", "SINEIN", {"rotateBy", 0.2 * multiplier, 16}},
	})

	item:cycle({
		{"scaleTo", 0.2 * multiplier, 0.95},
		{"scaleTo", 0.2 * multiplier, 1},
	})
end

-- 飘出去动作
function M:flyOutAction(item, callback)
	local points1 = {
		cc.p(586, 267), cc.p(613, 224), cc.p(644, 195), cc.p(687, 166), cc.p(733, 145), 
		cc.p(785, 131), cc.p(839, 125), cc.p(911, 123), cc.p(978, 132), cc.p(1037, 144), 
		cc.p(1101, 162), cc.p(1170, 188), cc.p(1270, 200)
	}
	local points2 = CatmullRomeCurvePoints(points1, 8)

	-- 生成动作表
	local index 	= 0
	local actionTab = {}
	local speed 	= 600
	for i,v in ipairs(points2) do
		local sPos 	= points2[i-1] and cc.pAdd(points2[i-1], cc.p(-105 + X_OFFSET, Y_OFFSET)) or self:point()
		local ePos 	= cc.pAdd(points2[i], cc.p(-105 + X_OFFSET, Y_OFFSET))
		speed 		= speed + 5
		table.insert(actionTab, {"moveTo", PT.d(sPos, ePos) / speed, ePos})
	end
	table.insert(actionTab, {"fn", function()
		if callback then
			callback()
		end
		item:remove()
	end})
	item:line(actionTab)

	local node = item
	local pos1 = node:point()
	local pos2 = node:point()
	local endAngle = node:rotate()
	-- 物品的初始角度
	local angleDis = 180
	node:scheduleUpdateWithPriorityLua(function (dt)
		pos2 = node:point()
		if pos1.x ~= pos2.x or pos1.y ~= pos2.y then
			local angle = -PT.quadAngle(pos1, pos2) + angleDis
			endAngle = angle
			pos1 = pos2
		end

		local angDis = endAngle - node:rotate()
		angDis = angDis % 360
		if angDis < -180 then
			angDis = 360 + angDis
		elseif angDis > 180 then
			angDis = angDis - 360
		end
		node:rotate(node:rotate() + angDis * dt * 5)
	end, 1)
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)

	return true
end

function M:onTouchMoved(x, y, touches)

end

function M:onTouchEnded(x, y, touches)
	
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M