--[[Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类
----------------------
local M    = classSpriteTouch("TargetNode")
M.TAG    = "TargetNode"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
local MoveClass = require("app.shapeline.node.MoveClass")
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)

    self._main      = params.main
    self._touchLock = true
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()
end

-- 图片渲染
function M:loadDisplay()
    self:display("shape/line/outline/tip.png"):scale(0)
end

-- 入场效果
function M:enterEffect()
    self:line({
        {"easing", "backOut", {"scaleTo", 0.5, 1}},
        {"fn", function ()
            self._touchLock = false
        end}
    })
end

-- 退场效果
function M:quitEffect()
    -- self._touchLock = true
    -- self:line({
    --     {"easing", "backIn", {"scaleTo", 0.5, 0}},
    --     {"remove"}
    -- })
    self:remove()
end

-- 加载移动类
function M:loadMoveClass()
    self._isInPaint    = true

    if not self._moveClass then
        self._moveClass = MoveClass.new({ targetNode = self })
    end

    local points, pauseIndexTab = self._main:getOutlinePathInfo()
    self._moveClass:setMovePoints(points, pauseIndexTab)
end

-- 描边结束
function M:paintFinish()
    self._isTouchEnded = true
    self._touchLock = true
    self._isInPaint    = false

    self._main:allTipNodeGetOut()
    self._main._prop:onPaintFinishHandler()
end

-- 不断描边(点击操作触发)
function M:cyclePaint()
    self._cyclePaint = A.cycle({
        {"delay", 1/60},
        {"fn", function ()
            self._main._prop:paint(self._startPoint, self:point())
            self._startPoint = self:point()
        end}
    }):at(self)
end

-- 暂停描边(点击操作，暂停点自动触发)
function M:pausePaint()
    self:stopAction(self._cyclePaint)
    self._cyclePaint = nil
end

-- 描边轮廓
function M:paintMove(x, y)
    -- 移动点按轨迹移动逻辑
    self._moveClass:iscorner()
    local disx = x - self._touchPoint.x
    local disy = y - self._touchPoint.y
    local targetPoint = self:point()
    local pos = self._moveClass:judgeDrawMove(targetPoint, disx, disy)
    self:p(pos)
    self._main._prop:paint(self._startPoint, pos)
    self._startPoint = self:point()
    self._touchPoint = cc.p(x, y)

    if ccpDistance(self:point(), self._main._tipNode:point())<=5 then
        self._main:changeNextTip()
    end
end

-- 向下一个提示点移动（点击提示点操作）
function M:moveToNextTip(tipIndex)
    if tipIndex == 1 then
        self._main:showNextTipNode()
    else
        -- 播放音效[sfx14706008]连线c/实线开始出现的时候播放
        soundEffect:playEffectsfx14706008()
        self._startPoint    = self:point()
        self._moveClass:autoMove()
    end
end

--------------------------
-- 触控
-------------------------- 
function M:onTouchBegan(x, y, touches)
    if self._touchLock then return end

    self._main._tipNode._touchLock = true
    self._main._menu:barChange(false)
    -- 解除触控移动锁
    self._isTouchEnded    = false
    self._moveLock        = false
    -- 记录触控开始时移动点的状态（避免触控结束时状态与开始时不同）
    self._beganInPaint    = self._isInPaint

    self._startPoint    = self:point()
    self._touchPoint    = cc.p(x, y)
    self._touchX        = x
    self._touchY        = y

    --  触控有效
    return SIGN_TOUCH_BEGAN_SWALLOWS
end

function M:onTouchMoved(x, y, touches)
    -- 用来中断描边完成后的触控
    if self._isTouchEnded then return SIGN_TOUCH_MOVED_STOP end
    if self._isInPaint then
        self:paintMove(x, y)
    else
        -- 拖动移动
        local dx = x - self._touchX
        local dy = y - self._touchY
        self:p(self._startPoint.x + dx, self._startPoint.y + dy)
        CUtil:limitRectMove({ node = self })
        self._touchPoint = cc.p(x, y)
    end
end

function M:onTouchEnded(x, y, touches)
    if self._beganInPaint and self._isInPaint then
        if PT.d(self._touchX, self._touchY, x, y) < 30 then
            self._moveClass:autoMove()
        else
            self._main._tipNode._touchLock = false
        end
    end
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end

return M