--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("CameraNode")
M.TAG       = "CameraNode"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
local Config = require("app.shapepuzzle.Config")                     	 -- 配置表
local Outline = require("app.shapepuzzle.node.Outline")                  -- 形状虚线框
local Icecream = require("app.shapepuzzle.node.ObjIcecream")  			 -- 冰激凌
local ChristmasTree = require("app.shapepuzzle.node.ObjChristmasTree")   -- 圣诞树
local Car = require("app.shapepuzzle.node.ObjCar")                		 -- 小汽车
local Tortoise = require("app.shapepuzzle.node.ObjTortoise")			 -- 乌龟
local Castle = require("app.shapepuzzle.node.ObjCastle")			     -- 城堡
local Cake = require("app.shapepuzzle.node.ObjCake")					 -- 蛋糕
local ObjTab = {
	Icecream, ChristmasTree, Car, Tortoise, Castle, Cake,
}

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

    self._main 		 = params.main
	-- 最外层级（使虚线出现时置顶）
	self._mostZIndex = 1  
	-- 储存拼好的的形状块   
	self._shapeTab 	 = {}      
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
end

-- 下一个形状虚线框进场
function M:enterNextOutline(curObjId, shapeId, shapeTag, curShapeIndex)
	local imgPath = "shape/puzzle/object"..curObjId.."/shape/line"..shapeId..".png"
	local pos = self:getOutlinePos(curObjId, curShapeIndex)
	local outline = self:createOutline(shapeTag, imgPath, pos)
	self._curOutline = outline

	local scale = self:getScale(curObjId, curShapeIndex)
	self:moveToCenterOutline(scale)
end

-- 获取虚线框的坐标
function M:getOutlinePos(curObjId, curShapeIndex)
	local pos = Config.objectInfo[curObjId].shapePosTab[curShapeIndex]
	return pos
end

-- 获取镜头缩放倍数
function M:getScale(curObjId, curShapeIndex)
	local scale = Config.objectInfo[curObjId].cameraScaleTab[curShapeIndex]
	return scale
end

-- 移动镜头使虚线框显示在中心
function M:moveToCenterOutline(scale)
	local x = V.w_2 - (self._curOutline:px()*scale + 0)
	local y = V.h_2 - self._curOutline:py()*scale
	self:line({
		A.union({
			{"scaleTo", 0.3, scale},
			{"moveTo", 0.3, ccp(x, y)},
		}),
		{"fn", function ()
			self._curOutline:enterEffect()
		end}
	})
end

-- 镜头归位
function M:cameraReturn()
	self:line({
		A.union({
			{"scaleTo", 0.3, 1},
			{"moveTo", 0.3, ccp(0, 0)},
		}),
		{"fn", function ()
			self:line({
				{"delay", 0.3},
				{"fn", function ()
					self:shapeToObject()
				end}
			})
			-- 车子关卡/乌龟关卡拼好后加广告
			if self._main._curLevel == 3 or self._main._curLevel == 4 then
				NV.addAd()
			end
		end}
	})
end

-- 形状转变为对象
function M:shapeToObject()
	local object = self:createObject()
	self._object = object

	for i,v in pairs(self._shapeTab) do
		CUtil:resetParent(v, object, 100)
	end

	local callback = function ()                            -- 实物弹动完成后触发的回调
		CUtil:changeAnchorP(object, ccp(0.5, 0.5))
		self:line({
			{"delay", 0.3},
			{"fn", function ()
				-- 开始物品的互动动画
				object:beganItemEffect()
				self._main:playActiveParticle()
				-- 播放音效[sfx14707006]黄色星星粒子/变形后出现黄色星星粒子的时候播放
				soundEffect:playEffectsfx14707006()
			end}
		})
	end
	
	local hight  -- 跳跃高度
	if     self._main._curLevel == 1 then hight = 40
	elseif self._main._curLevel == 2 then hight = 30
	elseif self._main._curLevel == 3 then hight = 50
	elseif self._main._curLevel == 4 then hight = 30
	elseif self._main._curLevel == 5 then hight = 30
	elseif self._main._curLevel == 6 then hight = 30
	end

	-- 播放音效[sfx14707005]完整图形变形/完成拼图后，抖动一下的时候播放
	soundEffect:playEffectsfx14707005()
	object:line({
		{"jumpTo", 0.3, object:point(), hight, 1},
		{"fn", function ()
			-- 粒子
			local particle = CUtil:newParticle("particle/bianshen.plist"):to(object, 10000):p(object:cw()/2, object:ch()/2)
			object:showItem()
			CUtil:changeAnchorP(object, ccp(0.5, 0))
			CAction:feedback2(object, 2, callback)
		end}
	})
end

-- 创建形状虚线框
function M:createOutline(shapeTag, imgPath, pos)
	local outline = Outline.new({
		camera   = self,
		shapeTag = shapeTag,
		imgPath  = imgPath,
		initPos  = pos
	}):to(self, self._mostZIndex):p(pos):scale(0)

	self:changeMostZIndex()
	return outline
end

-- 创建对象
function M:createObject()
	local object = ObjTab[self._main._curLevel]
	local objPos = Config.objectInfo[self._main._curLevel].objPos

	local obj = object.new({
		camera = self
	}):to(self):p(objPos):opacity(0)
	
	-- 蛋糕场景扩大触控范围
	if self._main._curLevel == 6 then
		obj:bindTouch({
			rectFactorX = 1,
			rectFactorY = 1.5
		})
	end
	return obj
end

-- 改变最外层级数
function M:changeMostZIndex()
	self._mostZIndex = self._mostZIndex + 1
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)

	return true
end

function M:onTouchMoved(x, y, touches)


end

function M:onTouchEnded(x, y, touches)
	
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M