--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("ObjCar")
M.TAG       = "ObjCar"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 自动事件类
local AutoEvent = require("app.shapepuzzle.node.AutoEvent")

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	-- 摄像机节点
	self._camera    = params.camera
	-- 触控锁
	self._touchLock = true
end


----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)

	-- 图片渲染
	self:loadDisplay()
	--  加载尾气图
	self:loadGas()
	--  加载鸣笛图
	self:loadHorn()
	-- 加载自动事件
	self:loadAutoEvent()
end

-- 加载其他
function M:loadRenderOtherwise()

end

-- 图片渲染
function M:loadDisplay()
	self:display("shape/puzzle/object3/item/car.png")
end

-- 加载自动事件
function M:loadAutoEvent()
	local autoEvent = AutoEvent.new({
		itemId    = 3, 	  -- 实物ID
		delayTime = 2,	  -- 自动事件的间隔时间
		itemNode  = self  -- 物品节点
	}):to(self):p(10000, 10000):unbindTouch()

	self._autoEvent = autoEvent
end

-- 加载尾气图
function M:loadGas()
	local gas = D.img("shape/puzzle/object3/item/gas.png"):to(self, 1010):p(-50, 70):hide()
	self._gas = gas
end

-- 加载鸣笛图
function M:loadHorn()
	local horn = D.img("shape/puzzle/object3/item/horn/horn1.png"):to(self, 1030):p(500, 180):hide()
	self._horn = horn
end

-- 显示实物图
function M:showItem()
	for i, v in pairs(self._camera._shapeTab) do
		if not tolua.isnull(v) then
			v:hide()
		end
	end
	
	self:opacity(255)
end

-- 点击缩放效果
function M:playClickScale()
	self:line({
		{"scaleTo", 0.05, 0.9},
		{"scaleTo", 0.15, 1.1},
		{"scaleTo", 0.1, 1},
	})
end

-- 开始实物动作互动
function M:beganItemEffect()
	self:playCarActive()
end

-- 激活自动事件
function M:activeAutoEvent()
	-- 开启自动事件
	self._autoEvent:openAutoEvent()
end

-- 播放车子启动
function M:playCarActive()
	CUtil:changeAnchorP(self, cc.p(0.5, 0))
	self:playCarShake()
	self:line({
		{"delay", 0.5},
		{"fn", function ()
			-- 播放配音[v147124]“嘟嘟嘟~”变成小汽车开走啦~
			soundVoice:playEffectv147124()
			local callback = function ()
				local time = sound:soundDuration("v147124")
				A.line({
					{"delay", time},
					{"fn", function ()
						self:activeAutoEvent()
					end}
				}):at(self)
			end
			self:playHornAni(callback)
		end}
	})
end

-- 播放开车退场动画
function M:quitEffect()
	-- 播放音效[sfx14707012]汽车/汽车开始行驶的时候播放
	soundEffect:playEffectsfx14707012()
	self:playHornAni()
	self:playGasAni()
	self:line({
		A.union({
			{"moveBy", 3, ccp(800+X_OFFSET, 0)},
			A.line({
				{"delay", 1},
				{"fn", function ()
					self:playHornAni()
				end}
			})
		}),
		{"fn", function ()
			self._camera._main:clearActiveParticle()
			self._camera._main:enterNextLevel()
		end},
		{"remove"}
	})
end

-- 播放汽车颠簸抖动方法
function M:playCarShake()
	local callback = function ()
		self:playCarShake()
	end
	local strength = math.random(10, 30)/100
	CAction:feedback2(self, strength, callback)
end

-- 播放鸣笛的帧动画方法
function M:playHornAni(callback)
	self._horn:line({
		{"fn", function ()
			self._horn:show()
		end},
		{"image", "shape/puzzle/object3/item/horn/horn", 5, 0.05},
		{"fn", function ()
			self._horn:hide()
			if callback then
				callback()
			end
		end},
	})
end

-- 播放尾气的方法
function M:playGasAni()
	self._gas:show()
	self._gas:cycle({
		A.union({
			{"scaleTo", 0.3, 1.2},
			{"rotateBy", 0.3, 90}
		}),
		A.union({
			{"scaleTo", 0.3, 1},
			{"rotateBy", 0.3, 90}
		}),
	})
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end

function M:onExitTransitionStart()

end
----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()

	self:playClickScale()
	self._autoEvent:clickEvent()
	return true
end

function M:onTouchMoved(x, y, touches)

end

function M:onTouchEnded(x, y, touches)
	
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M