--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("ObjIcecream")
M.TAG       = "ObjIcecream"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 自动事件类
local AutoEvent = require("app.shapepuzzle.node.AutoEvent")

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	-- 摄像机节点
	self._camera    = params.camera
	-- 触控锁
	self._touchLock = true
	-- 第几次咬冰激凌
	self._eatIndex  = 0
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)

	-- 图片渲染
	self:loadDisplay()
	-- 加载自动事件
	self:loadAutoEvent()
end

-- 图片渲染
function M:loadDisplay()
	self:display("shape/puzzle/object1/item/icecream/icecream1.png")
end

-- 加载自动事件
function M:loadAutoEvent()
	local autoEvent = AutoEvent.new({
		itemId    = 1, 	  -- 实物ID
		delayTime = 1.5,  -- 自动事件的间隔时间
		itemNode  = self  -- 物品节点
	}):to(self):p(10000, 10000):unbindTouch()

	self._autoEvent = autoEvent
end

-- 显示实物图
function M:showItem()
	for i, v in pairs(self._camera._shapeTab) do
		if not tolua.isnull(v) then
			v:hide()
		end
	end
	
	self:opacity(255)
end

-- 点击缩放效果
function M:playClickScale()
	self:line({
		{"scaleTo", 0.05, 0.9},
		{"scaleTo", 0.15, 1.1},
		{"scaleTo", 0.1, 1},
	})
end

-- 播放吃冰激凌随机音效
function M:playRanEatEffect()
	local ranNum = math.random(1, 2)
	if ranNum == 1 then
	-- 播放音效[sfx14707011]吃冰淇淋1
		soundEffect:playEffectsfx14707011()
	elseif ranNum == 2 then
		-- 播放音效[sfx14707013]吃冰淇淋2
		soundEffect:playEffectsfx14707013()
	end
end

-- 开始实物动作互动
function M:beganItemEffect()
	-- 播放配音[v147122]哇！是冰淇淋~把它吃掉！
	soundVoice:playEffectv147122()
	local time = sound:soundDuration("v147122")
	A.line({
		{"delay", time},
		{"fn", function ()
			self:activeAutoEvent()
		end}
	}):at(self)
end

-- 激活自动事件
function M:activeAutoEvent()
	-- 开启自动事件
	self._autoEvent:openAutoEvent()
end

-- 播放冰激凌被吃一口动画（前面4口）
function M:playEatAni1(event)
	self._eatIndex = self._eatIndex + 1
	self:playRanEatEffect()
	self:display("shape/puzzle/object1/item/icecream/icecream"..(self._eatIndex+1)..".png")
	if event then
		event()
	end
end

-- 播放冰淇淋剩下全吃掉动画
function M:playEatAni2()
	self:line({
		{"fn", function ()
			self:display("shape/puzzle/object1/item/icecream/icecream6.png")
			self:playRanEatEffect()
		end},
		{"delay", 0.1},
		{"fn", function ()
			self:display("shape/puzzle/object1/item/icecream/icecream7.png")
			self:playRanEatEffect()
		end},
		{"delay", 0.1},
		{"fn", function ()
			self:display("shape/puzzle/object1/item/icecream/icecream8.png")
			self:playRanEatEffect()
		end},
		{"delay", 0.1},
		{"fn", function ()
			self._camera._main:clearActiveParticle()
			self._camera._main:enterNextLevel()
		end},
		{"remove"}
	})
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end

----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()
	
	self:playClickScale()
	self._autoEvent:clickEvent()
	return true
end

function M:onTouchMoved(x, y, touches)


end

function M:onTouchEnded(x, y, touches)
	
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M