--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classArmatureTouch("ObjTortoise")
M.TAG       = "ObjTortoise"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 自动事件类
local AutoEvent = require("app.shapepuzzle.node.AutoEvent")
-- 乌龟爬行路径
local CrawlPathTab = {
	cc.p(441, 123), cc.p(383, 180), cc.p(385, 259), cc.p(453, 318), cc.p(549, 335),
	cc.p(652, 308), cc.p(733, 262), cc.p(780, 191), cc.p(812, 117), cc.p(817, -221),
}
CrawlPathTab = CatmullRomeCurvePoints(CrawlPathTab, 32)

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	-- 骨骼文件路径
	self._file         = "bone/wg"
	-- 骨骼名称
	self._armatureName = "wugui"
	-- 摄像机节点
	self._camera        = params.camera
	-- 乌龟当前要爬行路径的索引
	self._curCrawlIndex = 0
	-- 触控锁
	self._touchLock     = true
end


----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)

	-- 加载自动事件
	self:loadAutoEvent()

	self._armature:opacity(0)
end

-- 骨骼动画衔接
function M:movementHandler( __evtType, __moveId )
	if __evtType == 1 then
		-- 伸出头后接呼吸动画
		if __moveId == "wg_shentou" then
			self:playBreathAni()
		end
	elseif __evtType == 2 then

	end
end

-- 乌龟随机呼吸效果
function M:playBreathAni()
	self:play("wg_huxi_1")
	self._isCrawling = false
	if self._crawlSoundEffect then
		sound.stopSound(self._crawlSoundEffect)
		self._crawlSoundEffect = nil
	end
end

-- 播放乌龟缩头动画
function M:playHideHeadAni()
	-- 播放音效[sfx14707009]乌龟-缩身体
	soundEffect:playEffectsfx14707009()
	self:play("wg_suohui")
end

-- 播放乌龟伸出头动画
function M:playShowHeadAni()
	-- 播放音效[sfx14707014]乌龟-身体重新伸出来
	soundEffect:playEffectsfx14707014()
	self:play("wg_shentou")
end

-- 播放乌龟爬行的骨骼动画
function M:playCrawlAni()
	if self._isCrawling then return end
	self._isCrawling = true
	self:play("wg_pa")
	if self._crawlSoundEffect then
		sound.stopSound(self._crawlSoundEffect)
		self._crawlSoundEffect = nil
	end
	-- 播放音效[sfx14707015]乌龟-爬行
	self._crawlSoundEffect = soundEffect:playEffectsfx14707015()
end


-----------------------------------------------------
-- 加载自动事件
function M:loadAutoEvent()
	local autoEvent = AutoEvent.new({
		itemId    = 4, 	  -- 实物ID
		delayTime = 2,	  -- 自动事件的间隔时间
		itemNode  = self  -- 物品节点
	}):to(self):p(10000, 10000):unbindTouch()

	self._autoEvent = autoEvent
end

-- 显示实物图
function M:showItem()
	for i, v in pairs(self._camera._shapeTab) do
		if not tolua.isnull(v) then
			v:hide()
		end
	end
	
	self:opacity(255)
	self._armature:opacity(255)
end

-- 点击缩放效果
function M:playClickScale()
	self:line({
		{"scaleTo", 0.05, 0.55*0.9},
		{"scaleTo", 0.15, 0.55*1.1},
		{"scaleTo", 0.1, 0.55*1},
	})
end

-- 开始实物动作互动
function M:beganItemEffect()
	-- 播放配音[v147125]小乌龟爬呀爬~
	soundVoice:playEffectv147125()
	self:playBreathAni()
	self:tortoiseSmaller()
end

-- 激活自动事件
function M:activeAutoEvent()
	-- 开启自动事件
	self._autoEvent:openAutoEvent()
end

-- 仅仅是乌龟缩小(模拟镜头缩小，使乌龟在屏幕内有空间四处爬行)
function M:tortoiseSmaller()
	self:line({
		A.union({
			{"scaleTo", 0.3, 0.55},
			{"moveBy", 0.3, ccp(0, -100)}
		}),
		{"fn", function ()
			CUtil:changeAnchorP(self, cc.p(0.5, 0.358))
			-- CUtil:adjustAnchor(self)  -- 锚点调试
			self:activeAutoEvent()
		end}
	})
end

-- 播放乌龟缩头
function M:playHideHead(event)
	self:playHideHeadAni()
	self:line({
		{"delay", 1},
		{"fn", function ()
			self:playShowHeadAni()
			self:line({
				{"delay", 1.5},
				{"fn", function ()
					if event then
						event()
					end
				end}
			})
		end}
	})
end

-- 乌龟第一次爬行
function M:playTortoiseCrawl1(event)
	-- 乌龟爬到一半时停住
	if self._curCrawlIndex >= #CrawlPathTab/2 then
		self:playBreathAni()
		if event then
			event()
		end
		return
	end
	self:playTortoiseCrawl(function ()
		self:playTortoiseCrawl1(event)
	end)
end

-- 乌龟第二次爬行退场
function M:playTortoiseCrawl2()
	if self._curCrawlIndex >= #CrawlPathTab then
		self:playBreathAni()
		self:remove()
		self._camera._main:clearActiveParticle()
		self._camera._main:enterNextLevel()
		return
	end
	self:playTortoiseCrawl(function ()
		self:playTortoiseCrawl2()
	end)
end

-- 乌龟在屏幕爬行的方法
function M:playTortoiseCrawl(callback)
	self._curCrawlIndex = self._curCrawlIndex + 1
	local startPos = self:point()
	local endPos = CrawlPathTab[self._curCrawlIndex]
	local dist = ccpDistance(startPos, endPos)

	local time = dist/300
	local angle = self:getAngleByPos(startPos, endPos)
	self:playCrawlAni()
	self:rotation(90 - angle)
	self:line({
		{"moveTo", time, endPos},
		{"fn", function ()
			if callback then
				callback()
			end
		end}
	})
end

-- 通过两点求得乌龟的角度方法
function M:getAngleByPos(startPos, endPos)
	local x = endPos.x - startPos.x  
	local y = endPos.y - startPos.y  
	local angle = math.atan2(y, x)*180/math.pi
	return angle 
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()
	
	self:playClickScale()
	self._autoEvent:clickEvent()
	return true
end

function M:onTouchMoved(x, y, touches)


end

function M:onTouchEnded(x, y, touches)
	
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M