--[[
Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类
----------------------
local M    = classSpriteTouch("Shape")
M.TAG    = "Shape"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)

    self._main = params.main
	-- 形状标识
	self._shapeTag = params.shapeTag 
	-- 渲染图片的文件路径
    self._imgPath = params.imgPath 
	-- 初始坐标位置  
	self._initPos = params.initPos 
	-- 初始化倍数，配合镜头效果  
	self._scale = params.scale       
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()
end

-- 图片渲染
function M:loadDisplay()
    self:display(self._imgPath)
end

-- 检查形状是否靠近当前虚线框
function M:checkIsNearOutline()
    self._touchLock = true

    local dist = self._main:getShapeToOutlineDist()
    if dist <= 100 then
        self:onSuccessHandler()
    else
        self:onFailHandler()
    end
end

-- 拼放正确的回调
function M:onSuccessHandler()
	local pos = self._main._camera._curOutline:worldpoint()

	self:playShapeVoice()
	-- 播放音效[sfx14707004]图形嵌入/图形放入虚线框的时候播放
	soundEffect:playEffectsfx14707004()
	self:line({
		A.union({
			{"moveTo", 0.25, ccp(pos.x, pos.y+35)},
			{"scaleTo", 0.25, self._scale * 1.3}
		}),
		{"fn", function ()
			-- 粒子
			local particle = CUtil:newParticle("particle/pintu.plist"):to(self:getParent(), 100):p(self:point())
			particle:scheduleUpdateWithPriorityLua(function ()
				particle:p(self:point())
			end, 0)
		end},
		A.union({
			{"moveTo", 0.02, pos},
			{"scaleTo", 0.02, self._scale}
		}),
		{"fn", function ()
			CUtil:resetParent(self, self._main._camera, self._main._camera._mostZIndex)
			table.insert(self._main._camera._shapeTab, self)
			self._main._camera:changeMostZIndex()
			self._main._camera._curOutline:quitEffect()
			self._main._globalTouchLock = false
			self:line({
				{"delay", 0.3}, 
				{"fn", function ()
					self._main:enterNextShape()
				end}
			})
		end}
	})
end

-- 播放形状台词
function M:playShapeVoice()
	soundVoice:stopNowSound()
	if self._shapeTag == 1 then
		-- 播放配音[v147084]三角形
		soundVoice:playEffectv147084()
	elseif self._shapeTag == 2 then
		-- 播放配音[v147081]正方形
		soundVoice:playEffectv147081()
	elseif self._shapeTag == 3 then
		-- 播放配音[v147083]长方形
		soundVoice:playEffectv147083()
	elseif self._shapeTag == 4 then
		-- 播放配音[v147082]圆形
		soundVoice:playEffectv147082()
	elseif self._shapeTag == 5 then
		-- 播放配音[v147086]星型
		soundVoice:playEffectv147086()
	elseif self._shapeTag == 6 then
		-- 播放配音[v147085]爱心形
		soundVoice:playEffectv147085()
	end
end

-- 拼放错误的回调(回到原来位置)
function M:onFailHandler()
	-- 播放音效[sfx14700002]错误提示
	soundEffect:playEffectsfx14700002()
	
	if self._shapeTag == 4 then         -- 形状为圆形时触发
		self:line({
			{"moveBy", 0.05, ccp(self:cw()/5, 0)},
			{"moveBy", 0.1, ccp(-2*self:cw()/5, 0)},
			{"moveBy", 0.05, ccp(self:cw()/5, 0)},
			{"delay", 0.05},
			{"fn", function ()
				-- 播放音效[sfx14700003]错误弹回
				soundEffect:playEffectsfx14700003()
			end},
			CUtil:jumpToLimitHeight(self, 0.3, self._initPos, 100, 1),
			{"fn", function ()
				self._touchLock = false
				CAction:feedback2(self,1)
			end}
		})
	else
		self:line({
			{"rotateBy", 0.05, 15},
			{"rotateBy", 0.1, -30},
			{"rotateBy", 0.05, 15},
			{"fn", function ()
				-- 播放音效[sfx14700003]错误弹回
				soundEffect:playEffectsfx14700003()
			end},
			CUtil:jumpToLimitHeight(self, 0.3, self._initPos, 100, 1),
			{"fn", function ()
				self._touchLock = false
				CAction:feedback2(self,1)
			end}
		})
	end
end

-- 开启缩放引导
function M:openGuide()
	if self._isOpenGuide then return end
	self._isOpenGuide = true
	CAction:openScaleGuide(self, nil, 4, function ()
		self:playGuideVoice()
		self._main:playFingerMoveGuide()
	end)
end

-- 播放引导台词
function M:playGuideVoice()
	if self._guideVoiceTag then
		self._guideVoiceTag = false
		-- 播放配音[v147155]把形状拼上去吧！~
		soundVoice:playEffectv147155()
	else
		self._guideVoiceTag = true
		-- 播放配音[v147121]拼一拼~看它会变成什么呢？
		soundVoice:playEffectv147121()
	end
end

-- 关闭缩放引导
function M:stopGuide()
	if self._isOpenGuide then
		self._isOpenGuide = false
		CAction:stopScaleGuide(self)
		self._main:stopFingerGuide()
	end
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)
    if self._touchLock or self._main._globalTouchLock then return end
	soundVoice:stopNowSound()
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()

	self:stopGuide()
	self:scale(self._scale * 1.1)
    return true
end

function M:onTouchMoved(x, y, touches)
	self:p(x - 30, y + 30)
	CUtil:limitRectMove({node = self})
	return true
end

function M:onTouchEnded(x, y, touches)
	self:scale(self._scale)
	self:checkIsNearOutline()
	self:openGuide()
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end


return M