--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("Tray")
M.TAG       = "Tray"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
end

-- 图片渲染
function M:loadDisplay()
	self:display("shape/puzzle/tray.png")
end

-- 托盘从右边滑入屏幕(形状道具也跟着)
function M:getInto(shape, callback)
	self:line({
		{"delay", 0.75},
		{"fn", function ()
			-- 播放音效[sfx14707002]右下角弹窗/弹出的时候播放
			soundEffect:playEffectsfx14707002()
		end},
		{"easing", "backout", {"moveBy", 0.5, ccp(-400, 0)}},
	})

	shape:line({
		{"delay", 0.75},
		{"easing", "backout", {"moveBy", 0.5, ccp(-400, 0)}},
		{"fn", function ()
			if callback then
				callback()
			end
		end}
	})
end

-- 托盘滑出屏幕
function M:getOut(callback)
	-- 播放音效[sfx14707003]右下角弹窗收起/收起的时候播放
	soundEffect:playEffectsfx14707003()
	self:line({
		{"easing", "backin", {"moveBy", 0.5, ccp(400, 0)}},
		{"fn",function ()
			if callback then
				callback()
			end
		end}
	})
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
	return true
end

function M:onTouchMoved(x, y, touches)

end

function M:onTouchEnded(x, y, touches)
	
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M